/****************************************************************************
 *
 *  Rohde & Schwarz Audio Video Platform instrument driver include file
 *
 ****************************************************************************/

#ifndef __RSAVP_HEADER
#define __RSAVP_HEADER

#include "rsidr_core.h"
#include "rsavp_attributes.h"
#include "rsavp_utility.h"

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#if defined(__cplusplus) || defined(__cplusplus__)
extern "C" {
#endif

/*****************************************************************************
 *- Useful Macros & Constants -----------------------------------------------*
 *****************************************************************************/
#define RSAVP_SIMULATION_ID_QUERY      "Rohde&Schwarz,R&S VTE,2115.7300k02/900000,1.45.0.0"  /* Default definition of ID Query Response for simulation */
#define RSAVP_VALID_ID_RESPONSE_STRING "Rohde&Schwarz,R&S VT"      /* Valid response for identification query */
#define RSAVP_SIMULATION_OPT_QUERY     ""      /* Simulated response for *OPT? command */

#define RSAVP_IO_BUFFER_SIZE           1024L   /* I/O buffer size */
#define RSAVP_OPC_TIMEOUT              5000L   /* Maximum time to wait for OPC in milliseconds */
#define IEEE_488_2_ERROR_MASK             0x4    /* 488.2 STB Bits */   
#define IEEE_488_2_ESE_MASK               0x3D    /* 488.2 Event Status Register (ESE) Bits */
#define RSAVP_NAN                 9.91e+37
#define RSAVP_NAN_VI_INT32             0x80000000     
/****************************************************************************
 *----------------- Instrument Driver Revision Information -----------------*
 ****************************************************************************/

/* Class Driver Identification */

#define RSAVP_CLASS_DRIVER_DESCRIPTION           ""
#define RSAVP_CLASS_DRIVER_PREFIX                ""
#define RSAVP_CLASS_DRIVER_VENDOR                ""
#define RSAVP_CLASS_DRIVER_REVISION              ""
#define RSAVP_CLASS_SPEC_MAJOR_VERSION           1L    /* Class specification major version */
#define RSAVP_CLASS_SPEC_MINOR_VERSION           0L    /* Class specification minor version */

/* Driver Identification */

#define RSAVP_SPECIFIC_DRIVER_DESCRIPTION        ""
#define RSAVP_SPECIFIC_DRIVER_PREFIX             "RSAVP"
#define RSAVP_SPECIFIC_DRIVER_LOCATOR            ""
#define RSAVP_SPECIFIC_DRIVER_VENDOR             ""
#define RSAVP_MAJOR_VERSION                      1L    /* Instrument driver major version          */
#define RSAVP_MINOR_VERSION                      0L    /* Instrument driver minor version          */
#define RSAVP_MINOR_MINOR_VERSION                0L    /* Instrument driver minor minor version    */

/* Driver Capabilities */          

#define RSAVP_SUPPORTED_INSTRUMENT_MODELS        ""    /* Instrument driver supported model(s)     */
#define RSAVP_GROUP_CAPABILITIES                 ""
#define RSAVP_FUNCTION_CAPABILITIES              ""

/****************************************************************************
 *---------------------------- Attribute Defines ---------------------------*
 ****************************************************************************/

    /*- Inherent Instrument Specific Attributes ----------------------------*/

        /* User Options */

#define RSAVP_ATTR_RANGE_CHECK                                           RS_ATTR_RANGE_CHECK                                     /* ViBoolean */
#define RSAVP_ATTR_QUERY_INSTRUMENT_STATUS                               RS_ATTR_QUERY_INSTRUMENT_STATUS                         /* ViBoolean */
#define RSAVP_ATTR_CACHE                                                 RS_ATTR_CACHE                                           /* ViBoolean */
#define RSAVP_ATTR_SIMULATE                                              RS_ATTR_SIMULATE                                        /* ViBoolean */
#define RSAVP_ATTR_RECORD_COERCIONS                                      RS_ATTR_RECORD_COERCIONS                                /* ViBoolean */
#define RSAVP_ATTR_INTERCHANGE_CHECK                                     RS_ATTR_INTERCHANGE_CHECK                               /* ViBoolean */
#define RSAVP_ATTR_SPY                                                   RS_ATTR_SPY                                             /* ViBoolean */
#define RSAVP_ATTR_USE_SPECIFIC_SIMULATION                               RS_ATTR_USE_SPECIFIC_SIMULATION                         /* ViBoolean */

        /* Class Driver Identification */

#define RSAVP_ATTR_CLASS_DRIVER_DESCRIPTION                              RS_ATTR_CLASS_DRIVER_DESCRIPTION                        /* ViString, read-only */
#define RSAVP_ATTR_CLASS_DRIVER_PREFIX                                   RS_ATTR_CLASS_DRIVER_PREFIX                             /* ViString, read-only */
#define RSAVP_ATTR_CLASS_DRIVER_VENDOR                                   RS_ATTR_CLASS_DRIVER_VENDOR                             /* ViString, read-only */
#define RSAVP_ATTR_CLASS_DRIVER_REVISION                                 RS_ATTR_CLASS_DRIVER_REVISION                           /* ViString, read-only */
#define RSAVP_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION           /* ViInt32,  read-only */
#define RSAVP_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION           /* ViInt32,  read-only */

        /* Driver Identification */

#define RSAVP_ATTR_SPECIFIC_DRIVER_DESCRIPTION                           RS_ATTR_SPECIFIC_DRIVER_DESCRIPTION                     /* ViString, read-only */
#define RSAVP_ATTR_SPECIFIC_DRIVER_PREFIX                                RS_ATTR_SPECIFIC_DRIVER_PREFIX                          /* ViString, read-only */
#define RSAVP_ATTR_SPECIFIC_DRIVER_LOCATOR                               RS_ATTR_SPECIFIC_DRIVER_LOCATOR                         /* ViString, read-only */
#define RSAVP_ATTR_SPECIFIC_DRIVER_VENDOR                                RS_ATTR_SPECIFIC_DRIVER_VENDOR                          /* ViString, read-only */
#define RSAVP_ATTR_SPECIFIC_DRIVER_REVISION                              RS_ATTR_SPECIFIC_DRIVER_REVISION                        /* ViString, read-only */
#define RSAVP_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION        /* ViInt32,  read-only */
#define RSAVP_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION        /* ViInt32,  read-only */

        /* Driver Capabilities */

#define RSAVP_ATTR_SUPPORTED_INSTRUMENT_MODELS                           RS_ATTR_SUPPORTED_INSTRUMENT_MODELS                     /* ViString, read-only */
#define RSAVP_ATTR_GROUP_CAPABILITIES                                    RS_ATTR_GROUP_CAPABILITIES                              /* ViString, read-only */
#define RSAVP_ATTR_FUNCTION_CAPABILITIES                                 RS_ATTR_FUNCTION_CAPABILITIES                           /* ViString, read-only */
#define RSAVP_ATTR_CHANNEL_COUNT                                         RS_ATTR_CHANNEL_COUNT                                   /* ViInt32,  read-only */

        /* Driver Setup */

#define RSAVP_ATTR_DRIVER_SETUP                                          RS_ATTR_DRIVER_SETUP                                    /* ViString            */

        /* Instrument Identification */

#define RSAVP_ATTR_INSTRUMENT_MANUFACTURER                               RS_ATTR_INSTRUMENT_MANUFACTURER                         /* ViString, read-only */
#define RSAVP_ATTR_INSTRUMENT_MODEL                                      RS_ATTR_INSTRUMENT_MODEL                                /* ViString, read-only */
#define RSAVP_ATTR_INSTRUMENT_FIRMWARE_REVISION                          RS_ATTR_INSTRUMENT_FIRMWARE_REVISION                    /* ViString, read-only */

        /* Advanced Session Information */

#define RSAVP_ATTR_IO_RESOURCE_DESCRIPTOR                                RS_ATTR_IO_RESOURCE_DESCRIPTOR                          /* ViString, read-only */
#define RSAVP_ATTR_LOGICAL_NAME                                          RS_ATTR_LOGICAL_NAME

/* ViString, read-only */

    /*- Instrument-Specific Attributes -------------------------------------*/

//#define RSAVP_ATTR_ID_QUERY_RESPONSE                                     (RS_SPECIFIC_PUBLIC_ATTR_BASE + 1L)                     /* ViString (Read Only)              */

/* BEGIN GENERATE */
#define RSAVP_ATTR_FILE_CHANGE_DRIVE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1
#define RSAVP_ATTR_FILE_CHANGE_DIRECTORY                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2
#define RSAVP_ATTR_DRIVES                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3
#define RSAVP_ATTR_FILE_CREATE_DIRECTORY                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 4
#define RSAVP_ATTR_FILE_REMOVE_DIRECTORY                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5
#define RSAVP_ATTR_FILE_REMOVE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6
#define RSAVP_ATTR_UNIT_ANGLE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7
#define RSAVP_ATTR_UNIT_CAPACITY                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8
#define RSAVP_ATTR_UNIT_CHARGE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 9
#define RSAVP_ATTR_UNIT_CONDUCTANCE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10
#define RSAVP_ATTR_UNIT_CURRENT                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 11
#define RSAVP_ATTR_UNIT_ENERGY                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 12
#define RSAVP_ATTR_UNIT_FREQUENCY                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 13
#define RSAVP_ATTR_UNIT_LENGTH                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 14
#define RSAVP_ATTR_UNIT_POWER                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 15
#define RSAVP_ATTR_UNIT_RESISTANCE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 16
#define RSAVP_ATTR_UNIT_TEMPERATURE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 17
#define RSAVP_ATTR_UNIT_TIME                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 18
#define RSAVP_ATTR_UNIT_VOLTAGE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 19
#define RSAVP_ATTR_DISPLAY_LANGUAGE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 20
#define RSAVP_ATTR_DISPLAY_FRONT_SIZE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 21
#define RSAVP_ATTR_DISPLAY_COLOR_SET                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 22
#define RSAVP_ATTR_KEY_LOCK                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 23
#define RSAVP_ATTR_MARKER_SOURCE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 24
#define RSAVP_ATTR_SELF_TEST_QUERY                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 25
#define RSAVP_ATTR_SYST_VERSION                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 31
#define RSAVP_ATTR_SYSTEM_RESET_ALL                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 32
#define RSAVP_ATTR_SYSTEM_RESET_BASE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 33
#define RSAVP_ATTR_SYSTEM_PRESET_ALL                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 34
#define RSAVP_ATTR_SYSTEM_PRESET_BASE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 35
#define RSAVP_ATTR_REFERENCE_OSCILLATOR_OUTPUT                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 36
#define RSAVP_ATTR_REFERENCE_OSCILLATOR_SOURCE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 37
#define RSAVP_ATTR_REFERENCE_OSCILLATOR_INTERNAL_ADJUSTMENT_STATE           RS_SPECIFIC_PUBLIC_ATTR_BASE + 38
#define RSAVP_ATTR_REFERENCE_OSCILLATOR_INTERNAL_ADJUSTMENT_VALUE           RS_SPECIFIC_PUBLIC_ATTR_BASE + 39
#define RSAVP_ATTR_REFERENCE_OSCILLATOR_EXTERNAL_FREQUENCY                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 40
#define RSAVP_ATTR_REFERENCE_OSCILLATOR_EXTERNAL_BANDWIDTH                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 41
#define RSAVP_ATTR_LOCAL_OSCILLATOR_OUTPUT_STATE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 42
#define RSAVP_ATTR_LOCAL_OSCILLATOR_MODE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 43
#define RSAVP_ATTR_PRESET_STATUS_REGISTERS                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 30
#define RSAVP_ATTR_QUESTIONABLE_REGISTER_ENABLE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 47
#define RSAVP_ATTR_QUESTIONABLE_REGISTER_NTR                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 26
#define RSAVP_ATTR_QUESTIONABLE_REGISTER_PTR                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 48
#define RSAVP_ATTR_QUESTIONABLE_REGISTER_CONDITION                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 49
#define RSAVP_ATTR_QUESTIONABLE_REGISTER_EVENT                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 50
#define RSAVP_ATTR_OPERATION_REGISTER_ENABLE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 28
#define RSAVP_ATTR_OPERATION_REGISTER_NTR                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 51
#define RSAVP_ATTR_OPERATION_REGISTER_PTR                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 52
#define RSAVP_ATTR_OPERATION_REGISTER_CONDITION                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 53
#define RSAVP_ATTR_OPERATION_REGISTER_EVENT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 54
#define RSAVP_ATTR_ID_QUERY_RESPONSE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 44
#define RSAVP_ATTR_SYSTEM_ERROR                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 45
#define RSAVP_ATTR_DISPLAY_UPDATE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 46

// rsavp_rngAngleUnit
#define RSAVP_VAL_ANGLE_DEG                                                 0
#define RSAVP_VAL_ANGLE_RAD                                                 1
#define RSAVP_VAL_ANGLE_GRAD                                                2

// rsavp_rngCapacityUnit
#define RSAVP_VAL_CAPACITY_ATTO                                             0
#define RSAVP_VAL_CAPACITY_FEMTO                                            1
#define RSAVP_VAL_CAPACITY_PICO                                             2
#define RSAVP_VAL_CAPACITY_NANO                                             3
#define RSAVP_VAL_CAPACITY_MICRO                                            4
#define RSAVP_VAL_CAPACITY_MILI                                             5
#define RSAVP_VAL_CAPACITY_FARAD                                            6
#define RSAVP_VAL_CAPACITY_KILO                                             7
#define RSAVP_VAL_CAPACITY_MEGA                                             8
#define RSAVP_VAL_CAPACITY_GIGA                                             9
#define RSAVP_VAL_CAPACITY_TERA                                             10
#define RSAVP_VAL_CAPACITY_PETA                                             11
#define RSAVP_VAL_CAPACITY_EXA                                              12

// rsavp_rngUnitCharge
#define RSAVP_VAL_CHARGE_ATTO                                               0
#define RSAVP_VAL_CHARGE_FEMTO                                              1
#define RSAVP_VAL_CHARGE_PICO                                               2
#define RSAVP_VAL_CHARGE_NANO                                               3
#define RSAVP_VAL_CHARGE_MICRO                                              4
#define RSAVP_VAL_CHARGE_MILI                                               5
#define RSAVP_VAL_CHARGE_COULOMB                                            6
#define RSAVP_VAL_CHARGE_KILO                                               7
#define RSAVP_VAL_CHARGE_MEGA                                               8
#define RSAVP_VAL_CHARGE_GIGA                                               9
#define RSAVP_VAL_CHARGE_TERA                                               10
#define RSAVP_VAL_CHARGE_PETA                                               11
#define RSAVP_VAL_CHARGE_EXA                                                12

// rsavp_rngUnitConductance
#define RSAVP_VAL_CONDUCTANCE_ATTO                                          0
#define RSAVP_VAL_CONDUCTANCE_FEMTO                                         1
#define RSAVP_VAL_CONDUCTANCE_PICO                                          2
#define RSAVP_VAL_CONDUCTANCE_NANO                                          3
#define RSAVP_VAL_CONDUCTANCE_MICRO                                         4
#define RSAVP_VAL_CONDUCTANCE_MILI                                          5
#define RSAVP_VAL_CONDUCTANCE_SIEMENS                                       6
#define RSAVP_VAL_CONDUCTANCE_KILO                                          7
#define RSAVP_VAL_CONDUCTANCE_MEGA                                          8
#define RSAVP_VAL_CONDUCTANCE_GIGA                                          9
#define RSAVP_VAL_CONDUCTANCE_TERA                                          10
#define RSAVP_VAL_CONDUCTANCE_PETA                                          11
#define RSAVP_VAL_CONDUCTANCE_EXA                                           12

// rsavp_rngUnitCurrent
#define RSAVP_VAL_CURRENT_ATTO                                              0
#define RSAVP_VAL_CURRENT_FEMTO                                             1
#define RSAVP_VAL_CURRENT_PICO                                              2
#define RSAVP_VAL_CURRENT_NANO                                              3
#define RSAVP_VAL_CURRENT_MICRO                                             4
#define RSAVP_VAL_CURRENT_MILI                                              5
#define RSAVP_VAL_CURRENT_AMPERE                                            6
#define RSAVP_VAL_CURRENT_KILO                                              7
#define RSAVP_VAL_CURRENT_MEGA                                              8
#define RSAVP_VAL_CURRENT_GIGA                                              9
#define RSAVP_VAL_CURRENT_TERA                                              10
#define RSAVP_VAL_CURRENT_PETA                                              11
#define RSAVP_VAL_CURRENT_EXA                                               12
#define RSAVP_VAL_CURRENT_DBPA                                              13
#define RSAVP_VAL_CURRENT_DBNA                                              14
#define RSAVP_VAL_CURRENT_DBUA                                              15
#define RSAVP_VAL_CURRENT_DBMA                                              16
#define RSAVP_VAL_CURRENT_DBA                                               17

// rsavp_rngUnitEnergy
#define RSAVP_VAL_ENERGY_ATTO                                               0
#define RSAVP_VAL_ENERGY_FEMTO                                              1
#define RSAVP_VAL_ENERGY_PICO                                               2
#define RSAVP_VAL_ENERGY_NANO                                               3
#define RSAVP_VAL_ENERGY_MICRO                                              4
#define RSAVP_VAL_ENERGY_MILI                                               5
#define RSAVP_VAL_ENERGY_JOULE                                              6
#define RSAVP_VAL_ENERGY_KILO                                               7
#define RSAVP_VAL_ENERGY_MEGA                                               8
#define RSAVP_VAL_ENERGY_GIGA                                               9
#define RSAVP_VAL_ENERGY_TERA                                               10
#define RSAVP_VAL_ENERGY_PETA                                               11
#define RSAVP_VAL_ENERGY_EXA                                                12

// rsavp_rngUnitFrequency
#define RSAVP_VAL_FREQ_ATTO                                                 0
#define RSAVP_VAL_FREQ_FEMTO                                                1
#define RSAVP_VAL_FREQ_PICO                                                 2
#define RSAVP_VAL_FREQ_NANO                                                 3
#define RSAVP_VAL_FREQ_MICRO                                                4
#define RSAVP_VAL_FREQ_MILI                                                 5
#define RSAVP_VAL_FREQ_HERTZ                                                6
#define RSAVP_VAL_FREQ_KILO                                                 7
#define RSAVP_VAL_FREQ_MEGA                                                 8
#define RSAVP_VAL_FREQ_GIGA                                                 9
#define RSAVP_VAL_FREQ_TERA                                                 10
#define RSAVP_VAL_FREQ_PETA                                                 11
#define RSAVP_VAL_FREQ_EXA                                                  12

// rsavp_rngUnitLength
#define RSAVP_VAL_LENGTH_ATTO                                               0
#define RSAVP_VAL_LENGTH_FEMTO                                              1
#define RSAVP_VAL_LENGTH_PICO                                               2
#define RSAVP_VAL_LENGTH_NANO                                               3
#define RSAVP_VAL_LENGTH_MICRO                                              4
#define RSAVP_VAL_LENGTH_MILI                                               5
#define RSAVP_VAL_LENGTH_METER                                              6
#define RSAVP_VAL_LENGTH_KILO                                               7
#define RSAVP_VAL_LENGTH_MEGA                                               8
#define RSAVP_VAL_LENGTH_GIGA                                               9
#define RSAVP_VAL_LENGTH_TERA                                               10
#define RSAVP_VAL_LENGTH_PETA                                               11
#define RSAVP_VAL_LENGTH_EXA                                                12

// rsavp_rngUnitPower
#define RSAVP_VAL_POWER_ATTO                                                0
#define RSAVP_VAL_POWER_FEMTO                                               1
#define RSAVP_VAL_POWER_PICO                                                2
#define RSAVP_VAL_POWER_NANO                                                3
#define RSAVP_VAL_POWER_MICRO                                               4
#define RSAVP_VAL_POWER_MILI                                                5
#define RSAVP_VAL_POWER_WATT                                                6
#define RSAVP_VAL_POWER_KILO                                                7
#define RSAVP_VAL_POWER_MEGA                                                8
#define RSAVP_VAL_POWER_GIGA                                                9
#define RSAVP_VAL_POWER_TERA                                                10
#define RSAVP_VAL_POWER_PETA                                                11
#define RSAVP_VAL_POWER_EXA                                                 12
#define RSAVP_VAL_POWER_DBPW                                                13
#define RSAVP_VAL_POWER_DBNW                                                14
#define RSAVP_VAL_POWER_DBUW                                                15
#define RSAVP_VAL_POWER_DBMW                                                16
#define RSAVP_VAL_POWER_DBW                                                 17

// rsavp_rngUnitResistance
#define RSAVP_VAL_RESISTANCE_ATTO                                           0
#define RSAVP_VAL_RESISTANCE_FEMTO                                          1
#define RSAVP_VAL_RESISTANCE_PICO                                           2
#define RSAVP_VAL_RESISTANCE_NANO                                           3
#define RSAVP_VAL_RESISTANCE_MICRO                                          4
#define RSAVP_VAL_RESISTANCE_MILI                                           5
#define RSAVP_VAL_RESISTANCE_OHM                                            6
#define RSAVP_VAL_RESISTANCE_KILO                                           7
#define RSAVP_VAL_RESISTANCE_MEGA                                           8
#define RSAVP_VAL_RESISTANCE_GIGA                                           9
#define RSAVP_VAL_RESISTANCE_TERA                                           10
#define RSAVP_VAL_RESISTANCE_PETA                                           11
#define RSAVP_VAL_RESISTANCE_EXA                                            12

// rsavp_rngUnitTemperature
#define RSAVP_VAL_TEMPERATURE_CELSIUS                                       0
#define RSAVP_VAL_TEMPERATURE_FAHRENHEIT                                    1
#define RSAVP_VAL_TEMPERATURE_KELVIN                                        2

// rsavp_rngUnitTime
#define RSAVP_VAL_TIME_ATTO                                                 0
#define RSAVP_VAL_TIME_FEMTO                                                1
#define RSAVP_VAL_TIME_PICO                                                 2
#define RSAVP_VAL_TIME_NANO                                                 3
#define RSAVP_VAL_TIME_MICRO                                                4
#define RSAVP_VAL_TIME_MILI                                                 5
#define RSAVP_VAL_TIME_SECOND                                               6
#define RSAVP_VAL_TIME_KILO                                                 7
#define RSAVP_VAL_TIME_MEGA                                                 8
#define RSAVP_VAL_TIME_GIGA                                                 9
#define RSAVP_VAL_TIME_TERA                                                 10
#define RSAVP_VAL_TIME_PETA                                                 11
#define RSAVP_VAL_TIME_EXA                                                  12
#define RSAVP_VAL_TIME_MINUTE                                               13
#define RSAVP_VAL_TIME_HOUR                                                 14

// rsavp_rngUnitVoltage
#define RSAVP_VAL_VOLTAGE_ATTO                                              0
#define RSAVP_VAL_VOLTAGE_FEMTO                                             1
#define RSAVP_VAL_VOLTAGE_PICO                                              2
#define RSAVP_VAL_VOLTAGE_NANO                                              3
#define RSAVP_VAL_VOLTAGE_MICRO                                             4
#define RSAVP_VAL_VOLTAGE_MILI                                              5
#define RSAVP_VAL_VOLTAGE_VOLT                                              6
#define RSAVP_VAL_VOLTAGE_KILO                                              7
#define RSAVP_VAL_VOLTAGE_MEGA                                              8
#define RSAVP_VAL_VOLTAGE_GIGA                                              9
#define RSAVP_VAL_VOLTAGE_TERA                                              10
#define RSAVP_VAL_VOLTAGE_PETA                                              11
#define RSAVP_VAL_VOLTAGE_EXA                                               12
#define RSAVP_VAL_VOLTAGE_DBPV                                              13
#define RSAVP_VAL_VOLTAGE_DBNV                                              14
#define RSAVP_VAL_VOLTAGE_DBUV                                              15
#define RSAVP_VAL_VOLTAGE_DBMV                                              16
#define RSAVP_VAL_VOLTAGE_DBV                                               17

// rsavp_rngDisplayLanguage
#define RSAVP_VAL_LANG_EN                                                   0
#define RSAVP_VAL_LANG_KO                                                   1
#define RSAVP_VAL_LANG_JA                                                   2
#define RSAVP_VAL_LANG_ZH                                                   3

// rsavp_rngDisplayFrontSize
#define RSAVP_VAL_FONT_SIZE_SMALL                                           0
#define RSAVP_VAL_FONT_SIZE_LARGE                                           1

// rsavp_rngDisplayColorSet
#define RSAVP_VAL_COLOR_SET_DEFAULT                                         0
#define RSAVP_VAL_COLOR_SET_OLD                                             1
#define RSAVP_VAL_COLOR_SET_BW                                              2
#define RSAVP_VAL_COLOR_SET_ASIA                                            3

// rsavp_rngReferenceOscillatorSource
#define RSAVP_VAL_INTERNAL                                                  0
#define RSAVP_VAL_EXTERNAL                                                  1
/* END GENERATE */

/*****************************************************************************
 *- Hidden Attribute Declarations -------------------------------------------*
 *****************************************************************************/

#define RSAVP_ATTR_OPC_TIMEOUT               (RS_ATTR_OPC_TIMEOUT)           /* ViInt32 */
#define RSAVP_ATTR_IO_SESSION                (RS_ATTR_IO_SESSION)            /* ViSession */
#define RSAVP_ATTR_OPC_CALLBACK              (RS_ATTR_OPC_CALLBACK)          /* ViAddr */
#define RSAVP_ATTR_CHECK_STATUS_CALLBACK     (RS_ATTR_CHECK_STATUS_CALLBACK) /* ViAddr */

/****************************************************************************
 *---------------- Constant definition  ------------------------------------*
 ****************************************************************************/
#define RSAVP_VAL_ANALOG   0
#define RSAVP_VAL_DIGITAL  1

#define RSAVP_VAL_FILE_MAN_CDRIVE    0
#define RSAVP_VAL_FILE_MAN_CDIR      1
#define RSAVP_VAL_FILE_MAN_MDIR      2
#define RSAVP_VAL_FILE_MAN_RDIR      3
#define RSAVP_VAL_FILE_MAN_COPY      4
#define RSAVP_VAL_FILE_MAN_MOVE      5
#define RSAVP_VAL_FILE_MAN_DELETE    6

#define	RSAVP_VAL_REG_COND   3
#define RSAVP_VAL_REG_EVENT  4

#define RSAVP_VAL_MEM_SAVE     0
#define RSAVP_VAL_MEM_RECALL   1

#define	RSAVP_VAL_RST_BASE  0
#define RSAVP_VAL_RST_ALL   1
/****************************************************************************
*---------------- Instrument Driver Function Declarations -----------------*
 ****************************************************************************/

ViStatus _VI_FUNC rsavp_init (ViRsrc resourceName, ViBoolean IDQuery,
                              ViBoolean resetDevice, ViSession* instrumentHandle);
ViStatus _VI_FUNC rsavp_InitWithOptions (ViRsrc resourceName, ViBoolean IDQuery,
                                         ViBoolean resetDevice,
                                         ViChar _VI_FAR optionString[],
                                         ViSession* instrumentHandle);
ViStatus _VI_FUNC rsavp_FileManager (ViSession instrumentHandle,
                                     ViInt32 operationToBePerformed,
                                     ViString source, ViString destination);
ViStatus _VI_FUNC rsavp_GetCurrentDirectory (ViSession instrumentHandle,
                                             ViInt32 bufferSize,
                                             ViChar currentDirectory[]);
ViStatus _VI_FUNC rsavp_FileCount (ViSession instrumentHandle, ViString directory,
                                   ViInt32* numberOfFilesAndFolders);
ViStatus _VI_FUNC rsavp_FileCatalog (ViSession instrumentHandle, ViString directory,
                                     ViInt32 bufferSize, ViChar _VI_FAR catalog[]);
ViStatus _VI_FUNC rsavp_FileSubdirectoriesCount (ViSession instrumentHandle,
                                                 ViString directory,
                                                 ViInt32* numberOfSubdirectories);
ViStatus _VI_FUNC rsavp_FileSubdirectories (ViSession instrumentHandle,
                                            ViString directory, ViInt32 bufferSize,
                                            ViChar _VI_FAR output[]);
ViStatus _VI_FUNC rsavp_FileDrives (ViSession instrumentHandle, ViInt32 bufferSize,
                                    ViChar _VI_FAR output[]);
ViStatus _VI_FUNC rsavp_ReadToFileFromInstrument (ViSession instrumentHandle,
                                                  ViString source,
                                                  ViString destination);
ViStatus _VI_FUNC rsavp_WriteFromFileToInstrument (ViSession instrumentHandle,
                                                   ViString source,
                                                   ViString destination);
ViStatus _VI_FUNC rsavp_DataSetFileOperations (ViSession instrumentHandle,
                                               ViInt32 operation,
                                               ViInt32 stateNumber, ViString path);
ViStatus _VI_FUNC rsavp_DataSetApplicationOperations (ViSession instrumentHandle,
                                                      ViInt32 operation,
                                                      ViString itemName,
                                                      ViString path);
ViStatus _VI_FUNC rsavp_ConfigureDisplay (ViSession instrumentHandle,
                                          ViInt32 language, ViInt32 fontSize,
                                          ViInt32 colorSet);
ViStatus _VI_FUNC rsavp_ConfigureMarkerSource (ViSession instrumentHandle,
                                               ViInt32 board,
                                               ViString markerSource);
ViStatus _VI_FUNC rsavp_SetStatusRegister (ViSession instrumentHandle,
                                           ViInt32 registerOperation,
                                           ViInt32 enable, ViInt32 PTransition,
                                           ViInt32 NTransition);
ViStatus _VI_FUNC rsavp_GetStatusRegister (ViSession instrumentHandle,
                                           ViInt32 registerOperation,
                                           ViInt32 registerPart,
                                           ViInt32 *registerValue);
ViStatus _VI_FUNC rsavp_ConfigureReferenceOscillatorOut (ViSession instrumentHandle,
                                                         ViBoolean state);
ViStatus _VI_FUNC rsavp_ConfigureReferenceOscillatorSource
             (ViSession instrumentHandle, ViInt32 source);
ViStatus _VI_FUNC rsavp_ConfigureReferenceOscillatorInternalAdjustment
             (ViSession instrumentHandle, ViInt32 board, ViBoolean adjustmentState,
              ViInt32 adjustmentValue);
ViStatus _VI_FUNC rsavp_ConfigureReferenceOscillatorExternalSettings
             (ViSession instrumentHandle, ViInt32 board, ViInt32 frequency,
              ViInt32 sBandwidth);
ViStatus _VI_FUNC rsavp_ConfigureLocalOscillator (ViSession instrumentHandle,
                                                  ViInt32 board,
                                                  ViBoolean outputState,
                                                  ViInt32 mode);
ViStatus _VI_FUNC rsavp_reset (ViSession instrumentHandle);
ViStatus _VI_FUNC rsavp_Preset (ViSession instrumentHandle, ViInt32 reference);
ViStatus _VI_FUNC rsavp_ResetSystem (ViSession instrumentHandle, ViInt32 reference);
ViStatus _VI_FUNC rsavp_self_test (ViSession instrumentHandle,
                                   ViInt16* selfTestResult,
                                   ViChar _VI_FAR selfTestMessage[]);
ViStatus _VI_FUNC rsavp_revision_query (ViSession instrumentHandle,
                                        ViChar _VI_FAR instrumentDriverRevision[],
                                        ViChar _VI_FAR firmwareRevision[]);
ViStatus _VI_FUNC rsavp_error_query (ViSession instrumentHandle, ViInt32* errorCode,
                                     ViChar _VI_FAR errorMessage[]);
ViStatus _VI_FUNC rsavp_error_message (ViSession instrumentHandle,
                                       ViStatus errorCode,
                                       ViChar _VI_FAR errorMessage[]);
ViStatus _VI_FUNC rsavp_SystemVersion (ViSession instrumentHandle, ViInt32 length,
                                       ViChar _VI_FAR systemVersion[]);
ViStatus _VI_FUNC rsavp_setCheckStatus (ViSession instrumentHandle,
                                        ViBoolean state);
ViStatus _VI_FUNC rsavp_ConfigureDisplayUpdate (ViSession instrumentHandle,
                                                ViBoolean displayUpdate);
ViStatus _VI_FUNC rsavp_GetError (ViSession instrumentHandle, ViStatus* code,
                                  ViInt32 bufferSize, ViChar _VI_FAR description[]);
ViStatus _VI_FUNC rsavp_ClearError (ViSession instrumentHandle);
ViStatus _VI_FUNC rsavp_WriteInstrData (ViSession instrumentHandle,
                                        ViChar _VI_FAR writeBuffer[]);
ViStatus _VI_FUNC rsavp_ReadInstrData (ViSession instrumentHandle,
                                       ViInt32 numberOfBytesToRead,
                                       ViChar _VI_FAR readBuffer[],
                                       ViUInt32* numBytesRead);
ViStatus _VI_FUNC rsavp_QueryViBoolean (ViSession instrumentHandle,
                                        ViString command, ViBoolean* value);
ViStatus _VI_FUNC rsavp_QueryViInt32 (ViSession instrumentHandle, ViString command,
                                      ViInt32* value);
ViStatus _VI_FUNC rsavp_QueryViReal64 (ViSession instrumentHandle, ViString command,
                                       ViReal64* value);
ViStatus _VI_FUNC rsavp_QueryViString (ViSession instrumentHandle, ViString command,
                                       ViInt32 bufferSize, ViChar _VI_FAR value[]);
ViStatus _VI_FUNC rsavp_SetAttributeViInt32 (ViSession instrumentHandle,
                                             ViChar _VI_FAR channelName[],
                                             ViAttr attributeID,
                                             ViInt32 attributeValue);
ViStatus _VI_FUNC rsavp_SetAttributeViInt64 (ViSession instrumentHandle,
                                             ViChar _VI_FAR repeatedCapabilityName[],
                                             ViAttr attributeID,
                                             ViInt64 attributeValue);
ViStatus _VI_FUNC rsavp_SetAttributeViReal64 (ViSession instrumentHandle,
                                              ViChar _VI_FAR channelName[],
                                              ViAttr attributeID,
                                              ViReal64 attributeValue);
ViStatus _VI_FUNC rsavp_SetAttributeViString (ViSession instrumentHandle,
                                              ViChar _VI_FAR channelName[],
                                              ViAttr attributeID,
                                              ViChar _VI_FAR attributeValue[]);
ViStatus _VI_FUNC rsavp_SetAttributeViBoolean (ViSession instrumentHandle,
                                               ViChar _VI_FAR channelName[],
                                               ViAttr attributeID,
                                               ViBoolean attributeValue);
ViStatus _VI_FUNC rsavp_SetAttributeViSession (ViSession instrumentHandle,
                                               ViChar _VI_FAR channelName[],
                                               ViAttr attributeID,
                                               ViSession attributeValue);
ViStatus _VI_FUNC rsavp_GetAttributeViInt32 (ViSession instrumentHandle,
                                             ViChar _VI_FAR channelName[],
                                             ViAttr attributeID,
                                             ViInt32* attributeValue);
ViStatus _VI_FUNC rsavp_GetAttributeViInt64 (ViSession instrumentHandle,
                                             ViChar _VI_FAR repeatedCapabilityName[],
                                             ViAttr attributeID,
                                             ViInt64* attributeValue);
ViStatus _VI_FUNC rsavp_GetAttributeViReal64 (ViSession instrumentHandle,
                                              ViChar _VI_FAR channelName[],
                                              ViAttr attributeID,
                                              ViReal64* attributeValue);
ViStatus _VI_FUNC rsavp_GetAttributeViString (ViSession instrumentHandle,
                                              ViChar _VI_FAR channelName[],
                                              ViAttr attributeID,
                                              ViInt32 bufferSize,
                                              ViChar _VI_FAR attributeValue[]);
ViStatus _VI_FUNC rsavp_GetAttributeViBoolean (ViSession instrumentHandle,
                                               ViChar _VI_FAR channelName[],
                                               ViAttr attributeID,
                                               ViBoolean* attributeValue);
ViStatus _VI_FUNC rsavp_GetAttributeViSession (ViSession instrumentHandle,
                                               ViChar _VI_FAR channelName[],
                                               ViAttr attributeID,
                                               ViSession* attributeValue);
ViStatus _VI_FUNC rsavp_CheckAttributeViInt32 (ViSession instrumentHandle,
                                               ViChar _VI_FAR channelName[],
                                               ViAttr attributeID,
                                               ViInt32 attributeValue);
ViStatus _VI_FUNC rsavp_CheckAttributeViInt64 (ViSession instrumentHandle,
                                               ViChar _VI_FAR channelName[],
                                               ViAttr attributeID,
                                               ViInt64 attributeValue);
ViStatus _VI_FUNC rsavp_CheckAttributeViReal64 (ViSession instrumentHandle,
                                                ViChar _VI_FAR channelName[],
                                                ViAttr attributeID,
                                                ViReal64 attributeValue);
ViStatus _VI_FUNC rsavp_CheckAttributeViString (ViSession instrumentHandle,
                                                ViChar _VI_FAR channelName[],
                                                ViAttr attributeID,
                                                ViChar _VI_FAR attributeValue[]);
ViStatus _VI_FUNC rsavp_CheckAttributeViBoolean (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViBoolean attributeValue);
ViStatus _VI_FUNC rsavp_CheckAttributeViSession (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViSession attributeValue);
ViStatus _VI_FUNC rsavp_GetAttributeRepeatedCapabilityIds
             (ViSession instrumentHandle, ViUInt32 attributeID, ViInt32 bufferSize,
              ViChar _VI_FAR repeatedCapabilityId_s[]);
ViStatus _VI_FUNC rsavp_GetAttributeRepeatedCapabilityIdNames
             (ViSession instrumentHandle, ViUInt32 attributeID,
              ViString repeatedCapabilityId, ViInt32 bufferSize,
              ViChar _VI_FAR repeatedCapabilityIdName_s[]);
ViStatus _VI_FUNC rsavp_close (ViSession instrumentHandle);

/****************************************************************************
 *------------------------ Error And Completion Codes ----------------------*
 ****************************************************************************/

#define RSAVP_WARN_MEASURE_UNCALIBRATED    (RS_CLASS_WARN_BASE + 0x0001L)
#define RSAVP_WARN_OVER_RANGE              (RS_CLASS_WARN_BASE + 0x0002L)

#define RSAVP_ERROR_MARKER_NOT_ENABLED     (RS_CLASS_ERROR_BASE + 0x0003L)
#define RSAVP_ERROR_NOT_DELTA_MARKER       (RS_CLASS_ERROR_BASE + 0x0002L)
#define RSAVP_ERROR_MAX_TIME_EXCEEDED      (RS_CLASS_ERROR_BASE + 0x0001L)

#define RSAVP_WARNMSG_MEASURE_UNCALIBRATED "Uncalibrated measurement"
#define RSAVP_WARNMSG_OVER_RANGE           "Measurement overrange"

#define RSAVP_ERRMSG_MARKER_NOT_ENABLED    "Marker Not Enabled"
#define RSAVP_ERRMSG_NOT_DELTA_MARKER      "Not Delta Marker"
#define RSAVP_ERRMSG_MAX_TIME_EXCEEDED     "Max Time Exceeded"

#define RSAVP_ERROR_CODES_AND_MSGS \
    {RSAVP_WARN_MEASURE_UNCALIBRATED,  RSAVP_WARNMSG_MEASURE_UNCALIBRATED},\
    {RSAVP_WARN_OVER_RANGE,            RSAVP_WARNMSG_OVER_RANGE},\
    {RSAVP_ERROR_MARKER_NOT_ENABLED,   RSAVP_ERRMSG_MARKER_NOT_ENABLED},\
    {RSAVP_ERROR_NOT_DELTA_MARKER,     RSAVP_ERRMSG_NOT_DELTA_MARKER},\
    {RSAVP_ERROR_MAX_TIME_EXCEEDED,    RSAVP_ERRMSG_MAX_TIME_EXCEEDED}

/****************************************************************************
 *---------------------------- End Include File ----------------------------*
 ****************************************************************************/

#if defined(__cplusplus) || defined(__cplusplus__)
}
#endif
#endif /* __RSAVP_HEADER */



