/****************************************************************************
 *
 *  Rohde & Schwarz Audio Video Platform Audio Analyzer instrument driver include file
 *
 ****************************************************************************/

#ifndef __RSAVPAA_HEADER
#define __RSAVPAA_HEADER

#include "rsidr_core.h"
#include "rsavpaa_attributes.h"
#include "rsavpaa_utility.h"

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#if defined(__cplusplus) || defined(__cplusplus__)
extern "C" {
#endif

/*****************************************************************************
 *- Useful Macros & Constants -----------------------------------------------*
 *****************************************************************************/
#define RSAVPAA_SIMULATION_ID_QUERY      "Rohde&Schwarz,R&S VTE,2115.7300k02/900000,1.45.0.0"  /* Default definition of ID Query Response for simulation */
#define RSAVPAA_VALID_ID_RESPONSE_STRING "Rohde&Schwarz"      /* Valid response for identification query */
#define RSAVPAA_SIMULATION_OPT_QUERY     ""      /* Simulated response for *OPT? command */

#define RSAVPAA_IO_BUFFER_SIZE           1024L   /* I/O buffer size */
#define RSAVPAA_OPC_TIMEOUT              5000L   /* Maximum time to wait for OPC in milliseconds */
#define IEEE_488_2_ERROR_MASK             0x4    /* 488.2 STB Bits */   
#define IEEE_488_2_ESE_MASK               0x3D    /* 488.2 Event Status Register (ESE) Bits */
#define RSAVPAA_NAN                 9.91e+37
#define RSAVPAA_NAN_VI_INT32             0x80000000     
/****************************************************************************
 *----------------- Instrument Driver Revision Information -----------------*
 ****************************************************************************/

/* Class Driver Identification */

#define RSAVPAA_CLASS_DRIVER_DESCRIPTION           ""
#define RSAVPAA_CLASS_DRIVER_PREFIX                ""
#define RSAVPAA_CLASS_DRIVER_VENDOR                ""
#define RSAVPAA_CLASS_DRIVER_REVISION              ""
#define RSAVPAA_CLASS_SPEC_MAJOR_VERSION           1L    /* Class specification major version */
#define RSAVPAA_CLASS_SPEC_MINOR_VERSION           0L    /* Class specification minor version */

/* Driver Identification */

#define RSAVPAA_SPECIFIC_DRIVER_DESCRIPTION        ""
#define RSAVPAA_SPECIFIC_DRIVER_PREFIX             "RSAVPAA"
#define RSAVPAA_SPECIFIC_DRIVER_LOCATOR            ""
#define RSAVPAA_SPECIFIC_DRIVER_VENDOR             ""
#define RSAVPAA_MAJOR_VERSION                      1L    /* Instrument driver major version          */
#define RSAVPAA_MINOR_VERSION                      0L    /* Instrument driver minor version          */
#define RSAVPAA_MINOR_MINOR_VERSION                0L    /* Instrument driver minor minor version    */

/* Driver Capabilities */          

#define RSAVPAA_SUPPORTED_INSTRUMENT_MODELS        ""    /* Instrument driver supported model(s)     */
#define RSAVPAA_GROUP_CAPABILITIES                 ""
#define RSAVPAA_FUNCTION_CAPABILITIES              ""

/****************************************************************************
 *---------------------------- Attribute Defines ---------------------------*
 ****************************************************************************/

    /*- Inherent Instrument Specific Attributes ----------------------------*/

        /* User Options */

#define RSAVPAA_ATTR_RANGE_CHECK                                           RS_ATTR_RANGE_CHECK                                     /* ViBoolean */
#define RSAVPAA_ATTR_QUERY_INSTRUMENT_STATUS                               RS_ATTR_QUERY_INSTRUMENT_STATUS                         /* ViBoolean */
#define RSAVPAA_ATTR_CACHE                                                 RS_ATTR_CACHE                                           /* ViBoolean */
#define RSAVPAA_ATTR_SIMULATE                                              RS_ATTR_SIMULATE                                        /* ViBoolean */
#define RSAVPAA_ATTR_RECORD_COERCIONS                                      RS_ATTR_RECORD_COERCIONS                                /* ViBoolean */
#define RSAVPAA_ATTR_INTERCHANGE_CHECK                                     RS_ATTR_INTERCHANGE_CHECK                               /* ViBoolean */
#define RSAVPAA_ATTR_SPY                                                   RS_ATTR_SPY                                             /* ViBoolean */
#define RSAVPAA_ATTR_USE_SPECIFIC_SIMULATION                               RS_ATTR_USE_SPECIFIC_SIMULATION                         /* ViBoolean */

        /* Class Driver Identification */

#define RSAVPAA_ATTR_CLASS_DRIVER_DESCRIPTION                              RS_ATTR_CLASS_DRIVER_DESCRIPTION                        /* ViString, read-only */
#define RSAVPAA_ATTR_CLASS_DRIVER_PREFIX                                   RS_ATTR_CLASS_DRIVER_PREFIX                             /* ViString, read-only */
#define RSAVPAA_ATTR_CLASS_DRIVER_VENDOR                                   RS_ATTR_CLASS_DRIVER_VENDOR                             /* ViString, read-only */
#define RSAVPAA_ATTR_CLASS_DRIVER_REVISION                                 RS_ATTR_CLASS_DRIVER_REVISION                           /* ViString, read-only */
#define RSAVPAA_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION           /* ViInt32,  read-only */
#define RSAVPAA_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION           /* ViInt32,  read-only */

        /* Driver Identification */

#define RSAVPAA_ATTR_SPECIFIC_DRIVER_DESCRIPTION                           RS_ATTR_SPECIFIC_DRIVER_DESCRIPTION                     /* ViString, read-only */
#define RSAVPAA_ATTR_SPECIFIC_DRIVER_PREFIX                                RS_ATTR_SPECIFIC_DRIVER_PREFIX                          /* ViString, read-only */
#define RSAVPAA_ATTR_SPECIFIC_DRIVER_LOCATOR                               RS_ATTR_SPECIFIC_DRIVER_LOCATOR                         /* ViString, read-only */
#define RSAVPAA_ATTR_SPECIFIC_DRIVER_VENDOR                                RS_ATTR_SPECIFIC_DRIVER_VENDOR                          /* ViString, read-only */
#define RSAVPAA_ATTR_SPECIFIC_DRIVER_REVISION                              RS_ATTR_SPECIFIC_DRIVER_REVISION                        /* ViString, read-only */
#define RSAVPAA_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION        /* ViInt32,  read-only */
#define RSAVPAA_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION        /* ViInt32,  read-only */

        /* Driver Capabilities */

#define RSAVPAA_ATTR_SUPPORTED_INSTRUMENT_MODELS                           RS_ATTR_SUPPORTED_INSTRUMENT_MODELS                     /* ViString, read-only */
#define RSAVPAA_ATTR_GROUP_CAPABILITIES                                    RS_ATTR_GROUP_CAPABILITIES                              /* ViString, read-only */
#define RSAVPAA_ATTR_FUNCTION_CAPABILITIES                                 RS_ATTR_FUNCTION_CAPABILITIES                           /* ViString, read-only */
#define RSAVPAA_ATTR_CHANNEL_COUNT                                         RS_ATTR_CHANNEL_COUNT                                   /* ViInt32,  read-only */

        /* Driver Setup */

#define RSAVPAA_ATTR_DRIVER_SETUP                                          RS_ATTR_DRIVER_SETUP                                    /* ViString            */

        /* Instrument Identification */

#define RSAVPAA_ATTR_INSTRUMENT_MANUFACTURER                               RS_ATTR_INSTRUMENT_MANUFACTURER                         /* ViString, read-only */
#define RSAVPAA_ATTR_INSTRUMENT_MODEL                                      RS_ATTR_INSTRUMENT_MODEL                                /* ViString, read-only */
#define RSAVPAA_ATTR_INSTRUMENT_FIRMWARE_REVISION                          RS_ATTR_INSTRUMENT_FIRMWARE_REVISION                    /* ViString, read-only */

        /* Advanced Session Information */

#define RSAVPAA_ATTR_IO_RESOURCE_DESCRIPTOR                                RS_ATTR_IO_RESOURCE_DESCRIPTOR                          /* ViString, read-only */
#define RSAVPAA_ATTR_LOGICAL_NAME                                          RS_ATTR_LOGICAL_NAME

/* ViString, read-only */

    /*- Instrument-Specific Attributes -------------------------------------*/

//#define RSAVPAA_ATTR_ID_QUERY_RESPONSE                                     (RS_SPECIFIC_PUBLIC_ATTR_BASE + 1L)                     /* ViString (Read Only)              */

/* BEGIN GENERATE */
#define RSAVPAA_ATTR_AUDIO_ANALYZER_TAB                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1
#define RSAVPAA_ATTR_QUERY_AUDIO_ANALYZER_SIGNAL                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2
#define RSAVPAA_ATTR_AUDIO_ANALYZER_INPUT                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3
#define RSAVPAA_ATTR_AUDIO_ANALYZER_ZOOM                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 4
#define RSAVPAA_ATTR_AUDIO_ANALYZER_CHANNEL_COUNT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5
#define RSAVPAA_ATTR_AUDIO_ANALYZER_SAMPLE_RATE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6
#define RSAVPAA_ATTR_AUDIO_ANALYZER_SAMPLE_FORMAT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7
#define RSAVPAA_ATTR_AUDIO_ANALYZER_INPUTS_LIST                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 13
#define RSAVPAA_ATTR_AUDIO_ANALYZER_ANALOG_REFERENCE_DBRA                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8
#define RSAVPAA_ATTR_AUDIO_ANALYZER_ANALOG_REFERENCE_DBRA_OFFSET            RS_SPECIFIC_PUBLIC_ATTR_BASE + 9
#define RSAVPAA_ATTR_AUDIO_ANALYZER_ANALOG_REFERENCE_DBRB                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10
#define RSAVPAA_ATTR_AUDIO_ANALYZER_ANALOG_REFERENCE_DBRB_OFFSET            RS_SPECIFIC_PUBLIC_ATTR_BASE + 11
#define RSAVPAA_ATTR_AUDIO_ANALYZER_ANALOG_REFERENCE_DBSPL2                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 12
#define RSAVPAA_ATTR_AUDIO_ANALYZER_ANALOG_REFERENCE_DBSPL1                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 74
#define RSAVPAA_ATTR_AUDIO_ANALYZER_DIGITAL_REFERENCE_DBRA                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 75
#define RSAVPAA_ATTR_AUDIO_ANALYZER_DIGITAL_REFERENCE_DBRA_OFFSET           RS_SPECIFIC_PUBLIC_ATTR_BASE + 76
#define RSAVPAA_ATTR_AUDIO_ANALYZER_DIGITAL_REFERENCE_DBRB                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 77
#define RSAVPAA_ATTR_AUDIO_ANALYZER_DIGITAL_REFERENCE_DBRB_OFFSET           RS_SPECIFIC_PUBLIC_ATTR_BASE + 78
#define RSAVPAA_ATTR_AUDIO_ANALYZER_DIGITAL_REFERENCE_DBSPL1                RS_SPECIFIC_PUBLIC_ATTR_BASE + 79
#define RSAVPAA_ATTR_AUDIO_ANALYZER_DIGITAL_REFERENCE_DBSPL2                RS_SPECIFIC_PUBLIC_ATTR_BASE + 80
#define RSAVPAA_ATTR_AUDIO_ANALYZER_RMS_LEVEL_UNIT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 14
#define RSAVPAA_ATTR_AUDIO_ANALYZER_ANALOG_RMS_LEVEL_UNIT                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 15
#define RSAVPAA_ATTR_AUDIO_ANALYZER_FREQUENCY_UNIT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 16
#define RSAVPAA_ATTR_AUDIO_ANALYZER_LEVEL_FILTERS                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 17
#define RSAVPAA_ATTR_AUDIO_ANALYZER_LEVEL_PROGRESS                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 18
#define RSAVPAA_ATTR_AUDIO_ANALYZER_LEVEL_AVERAGE_STATE                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 19
#define RSAVPAA_ATTR_AUDIO_ANALYZER_LEVEL_AVERAGING_ALGORITHM               RS_SPECIFIC_PUBLIC_ATTR_BASE + 20
#define RSAVPAA_ATTR_AUDIO_ANALYZER_LEVEL_AVERAGE_TIME                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 21
#define RSAVPAA_ATTR_AUDIO_ANALYZER_LEVEL_AVERAGING_TOLERANCE               RS_SPECIFIC_PUBLIC_ATTR_BASE + 22
#define RSAVPAA_ATTR_AUDIO_ANALYZER_LEVEL_AVERAGING_SETTLING_TIME           RS_SPECIFIC_PUBLIC_ATTR_BASE + 23
#define RSAVPAA_ATTR_AUDIO_ANALYZER_THDN_UNIT                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 24
#define RSAVPAA_ATTR_AUDIO_ANALYZER_THDN_FREQUENCY_RANGE_HIGH               RS_SPECIFIC_PUBLIC_ATTR_BASE + 25
#define RSAVPAA_ATTR_AUDIO_ANALYZER_THDN_FREQUENCY_RANGE_LOW                RS_SPECIFIC_PUBLIC_ATTR_BASE + 26
#define RSAVPAA_ATTR_AUDIO_ANALYZER_THDN_PROGRESS                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 27
#define RSAVPAA_ATTR_AUDIO_ANALYZER_THDN_FILTERS                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 28
#define RSAVPAA_ATTR_AUDIO_ANALYZER_THDN_AVERAGE_STATE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 29
#define RSAVPAA_ATTR_AUDIO_ANALYZER_THDN_AVERAGING_ALGORITHM                RS_SPECIFIC_PUBLIC_ATTR_BASE + 30
#define RSAVPAA_ATTR_AUDIO_ANALYZER_THDN_AVERAGE_TIME                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 31
#define RSAVPAA_ATTR_AUDIO_ANALYZER_THDN_AVERAGING_TOLERANCE                RS_SPECIFIC_PUBLIC_ATTR_BASE + 32
#define RSAVPAA_ATTR_AUDIO_ANALYZER_THDN_AVERAGING_SETTLING_TIME            RS_SPECIFIC_PUBLIC_ATTR_BASE + 33
#define RSAVPAA_ATTR_AUDIO_ANALYZER_FREQUENCY_RESPONSE_DISPLAY_UNIT         RS_SPECIFIC_PUBLIC_ATTR_BASE + 34
#define RSAVPAA_ATTR_AUDIO_ANALYZER_FREQUENCY_RESPONSE_FFT_SIZE             RS_SPECIFIC_PUBLIC_ATTR_BASE + 35
#define RSAVPAA_ATTR_AUDIO_ANALYZER_FREQUENCY_RESPONSE_AVERAGE_COUNT        RS_SPECIFIC_PUBLIC_ATTR_BASE + 36
#define RSAVPAA_ATTR_AUDIO_ANALYZER_FREQUENCY_RESPONSE_PROGRESS             RS_SPECIFIC_PUBLIC_ATTR_BASE + 37
#define RSAVPAA_ATTR_AUDIO_ANALYZER_FREQUENCY_RESPONSE_FILTERS              RS_SPECIFIC_PUBLIC_ATTR_BASE + 38
#define RSAVPAA_ATTR_AUDIO_ANALYZER_SIGNAL_TO_NOISE_UNIT                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 39
#define RSAVPAA_ATTR_AUDIO_ANALYZER_SIGNAL_LENGTH                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 40
#define RSAVPAA_ATTR_AUDIO_ANALYZER_NOISE_LENGTH                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 41
#define RSAVPAA_ATTR_AUDIO_ANALYZER_SIGNAL_TO_NOISE_PROGRESS                RS_SPECIFIC_PUBLIC_ATTR_BASE + 42
#define RSAVPAA_ATTR_AUDIO_ANALYZER_SIGNAL_TO_NOISE_FILTERS                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 43
#define RSAVPAA_ATTR_AUDIO_ANALYZER_CROSSTALK_UNIT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 44
#define RSAVPAA_ATTR_AUDIO_ANALYZER_CROSSTALK_REFERENCE_CHANNEL             RS_SPECIFIC_PUBLIC_ATTR_BASE + 45
#define RSAVPAA_ATTR_AUDIO_ANALYZER_CROSSTALK_PROGRESS                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 46
#define RSAVPAA_ATTR_AUDIO_ANALYZER_CROSSTALK_FILTERS                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 47
#define RSAVPAA_ATTR_AUDIO_ANALYZER_CROSSTALK_AVERAGE_STATE                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 48
#define RSAVPAA_ATTR_AUDIO_ANALYZER_CROSSTALK_AVERAGING_ALGORITHM           RS_SPECIFIC_PUBLIC_ATTR_BASE + 49
#define RSAVPAA_ATTR_AUDIO_ANALYZER_CROSSTALK_AVERAGE_TIME                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 50
#define RSAVPAA_ATTR_AUDIO_ANALYZER_CROSSTALK_AVERAGING_TOLERANCE           RS_SPECIFIC_PUBLIC_ATTR_BASE + 51
#define RSAVPAA_ATTR_AUDIO_ANALYZER_CROSSTALK_AVERAGING_SETTLING_TIME       RS_SPECIFIC_PUBLIC_ATTR_BASE + 52
#define RSAVPAA_ATTR_AUDIO_ANALYZER_INTERCHANNEL_PHASE_UNIT                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 53
#define RSAVPAA_ATTR_AUDIO_ANALYZER_INTERCHANNEL_PHASE_REFERENCE_CHANNEL    RS_SPECIFIC_PUBLIC_ATTR_BASE + 54
#define RSAVPAA_ATTR_AUDIO_ANALYZER_INTERCHANNEL_PHASE_PROGRESS             RS_SPECIFIC_PUBLIC_ATTR_BASE + 55
#define RSAVPAA_ATTR_AUDIO_ANALYZER_INTERCHANNEL_PHASE_FILTERS              RS_SPECIFIC_PUBLIC_ATTR_BASE + 56
#define RSAVPAA_ATTR_AUDIO_ANALYZER_INTERCHANNEL_PHASE_AVERAGE_STATE        RS_SPECIFIC_PUBLIC_ATTR_BASE + 57
#define RSAVPAA_ATTR_AUDIO_ANALYZER_INTERCHANNEL_PHASE_AVERAGING_ALGORITHM  RS_SPECIFIC_PUBLIC_ATTR_BASE + 58
#define RSAVPAA_ATTR_AUDIO_ANALYZER_INTERCHANNEL_PHASE_AVERAGE_TIME         RS_SPECIFIC_PUBLIC_ATTR_BASE + 59
#define RSAVPAA_ATTR_AUDIO_ANALYZER_INTERCHANNEL_PHASE_AVERAGING_TOLERANCE  RS_SPECIFIC_PUBLIC_ATTR_BASE + 60
#define RSAVPAA_ATTR_AUDIO_ANALYZER_INTERCHANNEL_PHASE_AVERAGING_SETTLING_TIME RS_SPECIFIC_PUBLIC_ATTR_BASE + 61
#define RSAVPAA_ATTR_AUDIO_ANALYZER_MEASUREMENT_INIT                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 62
#define RSAVPAA_ATTR_AUDIO_ANALYZER_MEASUREMENT_ABORT                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 63
#define RSAVPAA_ATTR_QUERY_AUDIO_ANALYZER_MEASUREMENT_STATUS                RS_SPECIFIC_PUBLIC_ATTR_BASE + 64
#define RSAVPAA_ATTR_SELF_TEST_QUERY                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 65
#define RSAVPAA_ATTR_SYST_VERSION                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 66
#define RSAVPAA_ATTR_SYSTEM_RESET_ALL                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 67
#define RSAVPAA_ATTR_SYSTEM_RESET_BASE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 68
#define RSAVPAA_ATTR_SYSTEM_PRESET_ALL                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 69
#define RSAVPAA_ATTR_SYSTEM_PRESET_BASE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 70
#define RSAVPAA_ATTR_ID_QUERY_RESPONSE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 71
#define RSAVPAA_ATTR_SYSTEM_ERROR                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 72
#define RSAVPAA_ATTR_DISPLAY_UPDATE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 73

// rsavpaa_rngAudioAnalyzerTab
#define RSAVPAA_VAL_TAB_INPUT                                               0
#define RSAVPAA_VAL_TAB_LEVEL                                               1
#define RSAVPAA_VAL_TAB_THDN                                                2
#define RSAVPAA_VAL_TAB_FREQUENCY_RESPONSE                                  3
#define RSAVPAA_VAL_TAB_SIGNAL_TO_NOISE_RATIO                               4
#define RSAVPAA_VAL_TAB_CROSSTALK                                           5
#define RSAVPAA_VAL_TAB_INTERCHANNEL_PHASE                                  6

// rsavpaa_rngAudioAnalyzerZoom
#define RSAVPAA_VAL_AUDIO_ANALYZER_ZOOM_MINIMUM                             0
#define RSAVPAA_VAL_AUDIO_ANALYZER_ZOOM_MEDIUM                              1
#define RSAVPAA_VAL_AUDIO_ANALYZER_ZOOM_MAXIMUM                             2

// rsavpaa_rngAudioAnalyzerRMSLevelUnit
#define RSAVPAA_VAL_RMS_UNIT_FS                                             0
#define RSAVPAA_VAL_RMS_UNIT_PCTF                                           1
#define RSAVPAA_VAL_RMS_UNIT_DBFS                                           2
#define RSAVPAA_VAL_RMS_UNIT_DBRA                                           3
#define RSAVPAA_VAL_RMS_UNIT_DBRB                                           4
#define RSAVPAA_VAL_RMS_UNIT_DB1S                                           5
#define RSAVPAA_VAL_RMS_UNIT_DB2S                                           6

// rsavpaa_rngAudioAnalyzerFrequencyUnit
#define RSAVPAA_VAL_FREQUENCY_HZ                                            0
#define RSAVPAA_VAL_FREQUENCY_KHZ                                           1

// rsavpaa_rngAudioAnalyzerLevelAveragingAlgorithm
#define RSAVPAA_VAL_ALGORITHM_AVERAGE                                       0
#define RSAVPAA_VAL_ALGORITHM_FLAT                                          1

// rsavpaa_rngAudioAnalyzerTHDNUnit
#define RSAVPAA_VAL_THDN_UNIT_XY                                            0
#define RSAVPAA_VAL_THDN_UNIT_PCT                                           1
#define RSAVPAA_VAL_THDN_UNIT_PPM                                           2
#define RSAVPAA_VAL_THDN_UNIT_DB                                            3

// rsavpaa_rngAudioAnalyzerFrequencyResponseDisplayUnit
#define RSAVPAA_VAL_DISPLAY_UNIT_VRMS                                       0
#define RSAVPAA_VAL_DISPLAY_UNIT_DBV                                        1
#define RSAVPAA_VAL_DISPLAY_UNIT_DBU                                        2
#define RSAVPAA_VAL_DISPLAY_UNIT_DBRA                                       3
#define RSAVPAA_VAL_DISPLAY_UNIT_DBRB                                       4
#define RSAVPAA_VAL_DISPLAY_UNIT_DB1S                                       5
#define RSAVPAA_VAL_DISPLAY_UNIT_DB2S                                       6

// rsavpaa_rngAudioAnalyzerFrequencyResponseFFTSize
#define RSAVPAA_VAL_FFT_SIZE_256                                            0
#define RSAVPAA_VAL_FFT_SIZE_512                                            1
#define RSAVPAA_VAL_FFT_SIZE_1K                                             2
#define RSAVPAA_VAL_FFT_SIZE_2K                                             3
#define RSAVPAA_VAL_FFT_SIZE_4K                                             4
#define RSAVPAA_VAL_FFT_SIZE_8K                                             5
#define RSAVPAA_VAL_FFT_SIZE_16K                                            6
#define RSAVPAA_VAL_FFT_SIZE_32K                                            7
#define RSAVPAA_VAL_FFT_SIZE_64K                                            8
#define RSAVPAA_VAL_FFT_SIZE_128K                                           9
#define RSAVPAA_VAL_FFT_SIZE_256K                                           10
#define RSAVPAA_VAL_FFT_SIZE_512K                                           11
#define RSAVPAA_VAL_FFT_SIZE_1M                                             12

// rsavpaa_rngAudioAnalyzerSignalToNoiseUnit
#define RSAVPAA_VAL_SNR_DB                                                  0
#define RSAVPAA_VAL_SNR_XY                                                  1

// rsavpaa_rngAudioAnalyzerSignalLength
#define RSAVPAA_VAL_SIGNAL_LENGTH_S1                                        1
#define RSAVPAA_VAL_SIGNAL_LENGTH_S2                                        2
#define RSAVPAA_VAL_SIGNAL_LENGTH_S3                                        3
#define RSAVPAA_VAL_SIGNAL_LENGTH_S4                                        4
#define RSAVPAA_VAL_SIGNAL_LENGTH_S5                                        5

// rsavpaa_rngAudioAnalyzerNoiseLength
#define RSAVPAA_VAL_NOISE_LENGTH_S1                                         1
#define RSAVPAA_VAL_NOISE_LENGTH_S2                                         2
#define RSAVPAA_VAL_NOISE_LENGTH_S3                                         3
#define RSAVPAA_VAL_NOISE_LENGTH_S4                                         4
#define RSAVPAA_VAL_NOISE_LENGTH_S5                                         5

// rsavpaa_rngAudioAnalyzerCrosstalkUnit
#define RSAVPAA_VAL_CROSSTALK_UNIT_XY                                       0
#define RSAVPAA_VAL_CROSSTALK_UNIT_PCT                                      1
#define RSAVPAA_VAL_CROSSTALK_UNIT_PPM                                      2
#define RSAVPAA_VAL_CROSSTALK_UNIT_DB                                       3

// rsavpaa_rngAudioAnalyzerCrosstalkReferenceChannel
#define RSAVPAA_VAL_REFERENCE_CHANNEL1                                      1
#define RSAVPAA_VAL_REFERENCE_CHANNEL2                                      2
#define RSAVPAA_VAL_REFERENCE_CHANNEL3                                      3
#define RSAVPAA_VAL_REFERENCE_CHANNEL4                                      4
#define RSAVPAA_VAL_REFERENCE_CHANNEL5                                      5
#define RSAVPAA_VAL_REFERENCE_CHANNEL6                                      6
#define RSAVPAA_VAL_REFERENCE_CHANNEL7                                      7
#define RSAVPAA_VAL_REFERENCE_CHANNEL8                                      8

// rsavpaa_rngAudioAnalyzerInterchannelPhaseUnit
#define RSAVPAA_VAL_PHASE_UNIT_DEG                                          0
#define RSAVPAA_VAL_PHASE_UNIT_RAD                                          1

// rsavpaa_rngQueryAudioAnalyzerMeasurementStatus
#define RSAVPAA_VAL_MEASUREMENT_NONE                                        0
#define RSAVPAA_VAL_MEASUREMENT_LEVEL                                       1
#define RSAVPAA_VAL_MEASUREMENT_THDN                                        2
#define RSAVPAA_VAL_MEASUREMENT_FREQUENCY_RESPONSE                          3
#define RSAVPAA_VAL_MEASUREMENT_SNR                                         4
#define RSAVPAA_VAL_MEASUREMENT_CROSSTALK                                   5
#define RSAVPAA_VAL_MEASUREMENT_INTERCHANNEL_PHASE                          6

// rsavpaa_rngQueryAudioAnalyzerSignal
#define RSAVPAA_VAL_AUDIO_SIGNAL_UNSYNCHRONIZED                             0
#define RSAVPAA_VAL_AUDIO_SIGNAL_SYNCHRONIZED                               1

// rsavpaa_rngAudioAnalyzerFilters
#define RSAVPAA_VAL_FILTER_DISABLED                                         0
#define RSAVPAA_VAL_FILTER_A_WEIGHTING                                      1
#define RSAVPAA_VAL_FILTER_C_MESSAGE                                        2
#define RSAVPAA_VAL_FILTER_CCITT                                            3
#define RSAVPAA_VAL_FILTER_DE_EMPHASIS1550                                  4
#define RSAVPAA_VAL_FILTER_DE_EMPHASIS50                                    5
#define RSAVPAA_VAL_FILTER_DE_EMPHASIS75                                    6
#define RSAVPAA_VAL_FILTER_IEC_TUNER                                        7
#define RSAVPAA_VAL_FILTER_DE_JITTW                                         8
#define RSAVPAA_VAL_FILTER_PRE_EMPHASIS1550                                 9
#define RSAVPAA_VAL_FILTER_PRE_EMPHASIS50                                   10
#define RSAVPAA_VAL_FILTER_PRE_EMPHASIS75                                   11
#define RSAVPAA_VAL_FILTER_HIGH_PASS_FILTER_22                              12
#define RSAVPAA_VAL_FILTER_HIGH_PASS_FILTER_400                             13
#define RSAVPAA_VAL_FILTER_LOW_PASS_FILTER_22                               14
#define RSAVPAA_VAL_FILTER_LOW_PASS_FILTER_30                               15

// rsavpaa_rngAudioAnalyzerAnalogRMSLevelUnit
#define RSAVPAA_VAL_RMS_UNIT_VRMS                                           0
#define RSAVPAA_VAL_RMS_UNIT_DBV                                            1
#define RSAVPAA_VAL_RMS_UNIT_DBU                                            2
/* END GENERATE */

/*****************************************************************************
 *- Hidden Attribute Declarations -------------------------------------------*
 *****************************************************************************/

#define RSAVPAA_ATTR_OPC_TIMEOUT               (RS_ATTR_OPC_TIMEOUT)           /* ViInt32 */
#define RSAVPAA_ATTR_IO_SESSION                (RS_ATTR_IO_SESSION)            /* ViSession */
#define RSAVPAA_ATTR_OPC_CALLBACK              (RS_ATTR_OPC_CALLBACK)          /* ViAddr */
#define RSAVPAA_ATTR_CHECK_STATUS_CALLBACK     (RS_ATTR_CHECK_STATUS_CALLBACK) /* ViAddr */

/****************************************************************************
 *---------------- Constant definition  ------------------------------------*
 ****************************************************************************/

#define RSAVPAA_VAL_ANALOG   0
#define RSAVPAA_VAL_DIGITAL  1

#define RSAVPAA_VAL_RST_BASE  0
#define RSAVPAA_VAL_RST_ALL   1
/****************************************************************************
*---------------- Instrument Driver Function Declarations -----------------*
 ****************************************************************************/

ViStatus _VI_FUNC rsavpaa_init (ViRsrc resourceName, ViBoolean IDQuery,
                                ViBoolean resetDevice, ViSession* instrumentHandle);
ViStatus _VI_FUNC rsavpaa_InitWithOptions (ViRsrc resourceName, ViBoolean IDQuery,
                                           ViBoolean resetDevice,
                                           ViChar _VI_FAR optionString[],
                                           ViSession* instrumentHandle);
ViStatus _VI_FUNC rsavpaa_SelectAudioAnalyzerTab (ViSession instrumentHandle,
                                                   ViInt32 board, ViInt32 tab);
ViStatus _VI_FUNC rsavpaa_QueryAudioAnalyzerSignal (ViSession instrumentHandle,
                                                    ViInt32 board, ViInt32* signal);
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerInput (ViSession instrumentHandle,
                                                       ViInt32 board,
                                                       ViString input,
                                                       ViInt32 zoom);
ViStatus _VI_FUNC rsavpaa_QueryAudioAnalyzerInput (ViSession instrumentHandle,
                                                   ViInt32 board,
                                                   ViInt32* channelCount,
                                                   ViReal64* sampleRate,
                                                   ViInt32* sampleFormat);
ViStatus _VI_FUNC rsavpaa_QueryAudioAnalyzerInputsList (ViSession instrumentHandle,
                                                        ViInt32 board,
                                                        ViInt32 bufferSize,
                                                        ViChar _VI_FAR input[]);
ViStatus _VI_FUNC rsavpaa_QueryAudioAnalyzerLevelProgress
             (ViSession instrumentHandle, ViInt32 board, ViInt32* progress);
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerLevelUnit
             (ViSession instrumentHandle, ViInt32 board, ViInt32 signalType,
              ViInt32 digitalRMSLevel, ViInt32 analogRMSLevel, ViInt32 frequency);
ViStatus _VI_FUNC rsavpaa_ReadAudioAnalyzerRMSLevelMeasurementResults
             (ViSession instrumentHandle, ViInt32 board, ViInt32 channelNumber,
              ViReal64* RMSLevel, ViReal64* frequency);
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerLevelFilters
             (ViSession instrumentHandle, ViInt32 board, ViInt32 index,
              ViInt32 filter);
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerAveragingLevel (ViSession instrumentHandle,
                                                                ViInt32 board,
                                                                ViBoolean average,
                                                                ViInt32 algorithm,
                                                                ViReal64 averageTime,
                                                                ViReal64 tolerance,
                                                                ViReal64 settlingTime);
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerTHDNUnit (ViSession instrumentHandle,
                                                          ViInt32 board,
                                                          ViInt32 signalType,
                                                          ViInt32 THDN,
                                                          ViInt32 digitalRMSLevel,
                                                          ViInt32 analogRMSLevel,
                                                          ViInt32 frequency);
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerTHDNFrequencyRange (ViSession instrumentHandle,
                                                                    ViInt32 board,
                                                                    ViInt32 frequencyRangeHigh,
                                                                    ViInt32 frequencyRangeLow);
ViStatus _VI_FUNC rsavpaa_ReadAudioAnalyzerTHDNMeasurementResults
             (ViSession instrumentHandle, ViInt32 board, ViInt32 channelNumber,
              ViReal64* RMSLevel, ViReal64* frequency, ViReal64* THDN);
ViStatus _VI_FUNC rsavpaa_QueryAudioAnalyzerTHDNProgress (ViSession instrumentHandle,
                                                          ViInt32 board,
                                                          ViInt32* progress);
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerTHDNFilters
             (ViSession instrumentHandle, ViInt32 board, ViInt32 index,
              ViInt32 filter);
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerAveragingTHDN (ViSession instrumentHandle,
                                                               ViInt32 board,
                                                               ViBoolean average,
                                                               ViInt32 algorithm,
                                                               ViReal64 averageTime,
                                                               ViReal64 tolerance,
                                                               ViReal64 settlingTime);
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerFrequencyResponseDisplayUnit
             (ViSession instrumentHandle, ViInt32 board, ViInt32 display);
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerFrequencyResponseDisplayState (ViSession instrumentHandle,
                                                                               ViInt32 board,
                                                                               ViInt32 arraySize,
                                                                               ViBoolean displayState[]);
ViStatus _VI_FUNC rsavpaa_ReadAudioAnalyzerFrequencyResponseDisplayTraces
             (ViSession instrumentHandle, ViInt32 board, ViInt32 channelNumber,
              ViReal64 _VI_FAR xTrace[], ViReal64 _VI_FAR yTrace[]);
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerFrequencyResponseSettings
             (ViSession instrumentHandle, ViInt32 board, ViInt32 FFTSize,
              ViInt32 averageCount);
ViStatus _VI_FUNC rsavpaa_QueryAudioAnalyzerFrequencyResponseProgress
             (ViSession instrumentHandle, ViInt32 board, ViInt32* progress);
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerFrequencyResponseFilters
             (ViSession instrumentHandle, ViInt32 board, ViInt32 index,
              ViInt32 filter);
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerSignalToNoiseUnit
             (ViSession instrumentHandle, ViInt32 board, ViInt32 SNR);
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerSignalToNoiseLength
             (ViSession instrumentHandle, ViInt32 board, ViInt32 signalLength,
              ViInt32 noiseLength);
ViStatus _VI_FUNC rsavpaa_ReadAudioAnalyzerSignalToNoiseMeasurementResults
             (ViSession instrumentHandle, ViInt32 board, ViInt32 channelNumber,
              ViReal64* SNR);
ViStatus _VI_FUNC rsavpaa_QueryAudioAnalyzerSignalToNoiseProgress
             (ViSession instrumentHandle, ViInt32 board, ViInt32* progress);
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerSignalToNoiseFilters
             (ViSession instrumentHandle, ViInt32 board, ViInt32 index,
              ViInt32 filter);
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerCrosstalkUnit
             (ViSession instrumentHandle, ViInt32 board, ViInt32 signalType,
              ViInt32 crosstalk, ViInt32 digitalRMSLevel, ViInt32 analogRMSLevel,
              ViInt32 frequency);
ViStatus _VI_FUNC rsavpaa_ReadAudioAnalyzerCrosstalkMeasurementResults
             (ViSession instrumentHandle, ViInt32 board, ViInt32 channelNumber,
              ViReal64* crosstalk, ViReal64* RMSLevel, ViReal64* frequency);
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerCrosstalkReferenceChannel
             (ViSession instrumentHandle, ViInt32 board, ViInt32 referenceChannel);
ViStatus _VI_FUNC rsavpaa_QueryAudioAnalyzerCrosstalkProgress
             (ViSession instrumentHandle, ViInt32 board, ViInt32* progress);
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerCrosstalkFilters
             (ViSession instrumentHandle, ViInt32 board, ViInt32 index,
              ViInt32 filter);
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerAveragingCrosstalk (ViSession instrumentHandle,
                                                                    ViInt32 board,
                                                                    ViBoolean average,
                                                                    ViInt32 algorithm,
                                                                    ViReal64 averageTime,
                                                                    ViReal64 tolerance,
                                                                    ViReal64 settlingTime);
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerInterchannelPhaseUnit
             (ViSession instrumentHandle, ViInt32 board, ViInt32 signalType,
              ViInt32 phase, ViInt32 digitalRMSLevel, ViInt32 analogRMSLevel,
              ViInt32 frequency);
ViStatus 
    _VI_FUNC rsavpaa_ReadAudioAnalyzerInterchannelPhaseMeasurementResults
        (ViSession instrumentHandle, ViInt32 board, ViInt32 channelNumber,
         ViReal64* phase, ViReal64* RMSLevel, ViReal64* frequency);
ViStatus 
    _VI_FUNC rsavpaa_ConfigureAudioAnalyzerInterchannelPhaseReferenceChannel
        (ViSession instrumentHandle, ViInt32 board, ViInt32 referenceChannel);
ViStatus _VI_FUNC rsavpaa_QueryAudioAnalyzerInterchannelPhaseProgress
             (ViSession instrumentHandle, ViInt32 board, ViInt32* progress);
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerInterchannelPhaseFilters
             (ViSession instrumentHandle, ViInt32 board, ViInt32 index,
              ViInt32 filter);
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerAveragingInterchannelPhase (ViSession instrumentHandle,
                                                                            ViInt32 board,
                                                                            ViBoolean average,
                                                                            ViInt32 algorithm,
                                                                            ViReal64 averageTime,
                                                                            ViReal64 tolerance,
                                                                            ViReal64 settlingTime);
ViStatus _VI_FUNC rsavpaa_AudioAnalyzerMeasurementInit (ViSession instrumentHandle,
                                                        ViInt32 board,
                                                        ViInt32 measurement);
ViStatus _VI_FUNC rsavpaa_AudioAnalyzerMeasurementAbort (ViSession instrumentHandle,
                                                         ViInt32 board);
ViStatus _VI_FUNC rsavpaa_QueryAudioAnalyzerMeasurementStatus
             (ViSession instrumentHandle, ViInt32 board, ViInt32* measurementState);
ViStatus _VI_FUNC rsavpaa_reset (ViSession instrumentHandle);

ViStatus _VI_FUNC rsavpaa_ResetSystem (ViSession instrumentHandle,
                                       ViInt32 reference);

ViStatus _VI_FUNC rsavpaa_Preset (ViSession instrumentHandle, ViInt32 reference);
ViStatus _VI_FUNC rsavpaa_self_test (ViSession instrumentHandle,
                                     ViInt16* selfTestResult,
                                     ViChar _VI_FAR selfTestMessage[]);
ViStatus _VI_FUNC rsavpaa_revision_query (ViSession instrumentHandle,
                                          ViChar _VI_FAR instrumentDriverRevision[],
                                          ViChar _VI_FAR firmwareRevision[]);
ViStatus _VI_FUNC rsavpaa_error_query (ViSession instrumentHandle,
                                       ViInt32* errorCode,
                                       ViChar _VI_FAR errorMessage[]);
ViStatus _VI_FUNC rsavpaa_error_message (ViSession instrumentHandle,
                                         ViStatus errorCode,
                                         ViChar _VI_FAR errorMessage[]);
ViStatus _VI_FUNC rsavpaa_SystemVersion (ViSession instrumentHandle, ViInt32 length,
                                         ViChar _VI_FAR systemVersion[]);
ViStatus _VI_FUNC rsavpaa_setCheckStatus (ViSession instrumentHandle,
                                          ViBoolean state);
ViStatus _VI_FUNC rsavpaa_ConfigureDisplayUpdate (ViSession instrumentHandle,
                                                  ViBoolean displayUpdate);
ViStatus _VI_FUNC rsavpaa_GetError (ViSession instrumentHandle, ViStatus* code,
                                    ViInt32 bufferSize,
                                    ViChar _VI_FAR description[]);
ViStatus _VI_FUNC rsavpaa_ClearError (ViSession instrumentHandle);
ViStatus _VI_FUNC rsavpaa_WriteInstrData (ViSession instrumentHandle,
                                          ViChar _VI_FAR writeBuffer[]);
ViStatus _VI_FUNC rsavpaa_ReadInstrData (ViSession instrumentHandle,
                                         ViInt32 numberOfBytesToRead,
                                         ViChar _VI_FAR readBuffer[],
                                         ViUInt32* numBytesRead);
ViStatus _VI_FUNC rsavpaa_QueryViBoolean (ViSession instrumentHandle,
                                          ViString command, ViBoolean* value);
ViStatus _VI_FUNC rsavpaa_QueryViInt32 (ViSession instrumentHandle,
                                        ViString command, ViInt32* value);
ViStatus _VI_FUNC rsavpaa_QueryViReal64 (ViSession instrumentHandle,
                                         ViString command, ViReal64* value);
ViStatus _VI_FUNC rsavpaa_QueryViString (ViSession instrumentHandle,
                                         ViString command, ViInt32 bufferSize,
                                         ViChar _VI_FAR value[]);
ViStatus _VI_FUNC rsavpaa_SetAttributeViInt32 (ViSession instrumentHandle,
                                               ViChar _VI_FAR channelName[],
                                               ViAttr attributeID,
                                               ViInt32 attributeValue);
ViStatus _VI_FUNC rsavpaa_SetAttributeViInt64 (ViSession instrumentHandle,
                                               ViChar _VI_FAR repeatedCapabilityName[],
                                               ViAttr attributeID,
                                               ViInt64 attributeValue);
ViStatus _VI_FUNC rsavpaa_SetAttributeViReal64 (ViSession instrumentHandle,
                                                ViChar _VI_FAR channelName[],
                                                ViAttr attributeID,
                                                ViReal64 attributeValue);
ViStatus _VI_FUNC rsavpaa_SetAttributeViString (ViSession instrumentHandle,
                                                ViChar _VI_FAR channelName[],
                                                ViAttr attributeID,
                                                ViChar _VI_FAR attributeValue[]);
ViStatus _VI_FUNC rsavpaa_SetAttributeViBoolean (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViBoolean attributeValue);
ViStatus _VI_FUNC rsavpaa_SetAttributeViSession (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViSession attributeValue);
ViStatus _VI_FUNC rsavpaa_GetAttributeViInt32 (ViSession instrumentHandle,
                                               ViChar _VI_FAR channelName[],
                                               ViAttr attributeID,
                                               ViInt32* attributeValue);
ViStatus _VI_FUNC rsavpaa_GetAttributeViInt64 (ViSession instrumentHandle,
                                               ViChar _VI_FAR repeatedCapabilityName[],
                                               ViAttr attributeID,
                                               ViInt64* attributeValue);
ViStatus _VI_FUNC rsavpaa_GetAttributeViReal64 (ViSession instrumentHandle,
                                                ViChar _VI_FAR channelName[],
                                                ViAttr attributeID,
                                                ViReal64* attributeValue);
ViStatus _VI_FUNC rsavpaa_GetAttributeViString (ViSession instrumentHandle,
                                                ViChar _VI_FAR channelName[],
                                                ViAttr attributeID,
                                                ViInt32 bufferSize,
                                                ViChar _VI_FAR attributeValue[]);
ViStatus _VI_FUNC rsavpaa_GetAttributeViBoolean (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViBoolean* attributeValue);
ViStatus _VI_FUNC rsavpaa_GetAttributeViSession (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViSession* attributeValue);
ViStatus _VI_FUNC rsavpaa_CheckAttributeViInt32 (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViInt32 attributeValue);
ViStatus _VI_FUNC rsavpaa_CheckAttributeViInt64 (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViInt64 attributeValue);
ViStatus _VI_FUNC rsavpaa_CheckAttributeViReal64 (ViSession instrumentHandle,
                                                  ViChar _VI_FAR channelName[],
                                                  ViAttr attributeID,
                                                  ViReal64 attributeValue);
ViStatus _VI_FUNC rsavpaa_CheckAttributeViString (ViSession instrumentHandle,
                                                  ViChar _VI_FAR channelName[],
                                                  ViAttr attributeID,
                                                  ViChar _VI_FAR attributeValue[]);
ViStatus _VI_FUNC rsavpaa_CheckAttributeViBoolean (ViSession instrumentHandle,
                                                   ViChar _VI_FAR channelName[],
                                                   ViAttr attributeID,
                                                   ViBoolean attributeValue);
ViStatus _VI_FUNC rsavpaa_CheckAttributeViSession (ViSession instrumentHandle,
                                                   ViChar _VI_FAR channelName[],
                                                   ViAttr attributeID,
                                                   ViSession attributeValue);
ViStatus _VI_FUNC rsavpaa_GetAttributeRepeatedCapabilityIds
             (ViSession instrumentHandle, ViUInt32 attributeID, ViInt32 bufferSize,
              ViChar _VI_FAR repeatedCapabilityId_s[]);
ViStatus _VI_FUNC rsavpaa_GetAttributeRepeatedCapabilityIdNames
             (ViSession instrumentHandle, ViUInt32 attributeID,
              ViString repeatedCapabilityId, ViInt32 bufferSize,
              ViChar _VI_FAR repeatedCapabilityIdName_s[]);
ViStatus _VI_FUNC rsavpaa_close (ViSession instrumentHandle);


/****************************************************************************
 *------------------------ Error And Completion Codes ----------------------*
 ****************************************************************************/

#define RSAVPAA_WARN_MEASURE_UNCALIBRATED    (RS_CLASS_WARN_BASE + 0x0001L)
#define RSAVPAA_WARN_OVER_RANGE              (RS_CLASS_WARN_BASE + 0x0002L)

#define RSAVPAA_ERROR_MARKER_NOT_ENABLED     (RS_CLASS_ERROR_BASE + 0x0003L)
#define RSAVPAA_ERROR_NOT_DELTA_MARKER       (RS_CLASS_ERROR_BASE + 0x0002L)
#define RSAVPAA_ERROR_MAX_TIME_EXCEEDED      (RS_CLASS_ERROR_BASE + 0x0001L)

#define RSAVPAA_WARNMSG_MEASURE_UNCALIBRATED "Uncalibrated measurement"
#define RSAVPAA_WARNMSG_OVER_RANGE           "Measurement overrange"

#define RSAVPAA_ERRMSG_MARKER_NOT_ENABLED    "Marker Not Enabled"
#define RSAVPAA_ERRMSG_NOT_DELTA_MARKER      "Not Delta Marker"
#define RSAVPAA_ERRMSG_MAX_TIME_EXCEEDED     "Max Time Exceeded"

#define RSAVPAA_ERROR_CODES_AND_MSGS \
    {RSAVPAA_WARN_MEASURE_UNCALIBRATED,  RSAVPAA_WARNMSG_MEASURE_UNCALIBRATED},\
    {RSAVPAA_WARN_OVER_RANGE,            RSAVPAA_WARNMSG_OVER_RANGE},\
    {RSAVPAA_ERROR_MARKER_NOT_ENABLED,   RSAVPAA_ERRMSG_MARKER_NOT_ENABLED},\
    {RSAVPAA_ERROR_NOT_DELTA_MARKER,     RSAVPAA_ERRMSG_NOT_DELTA_MARKER},\
    {RSAVPAA_ERROR_MAX_TIME_EXCEEDED,    RSAVPAA_ERRMSG_MAX_TIME_EXCEEDED}

/****************************************************************************
 *---------------------------- End Include File ----------------------------*
 ****************************************************************************/

#if defined(__cplusplus) || defined(__cplusplus__)
}
#endif
#endif /* __RSAVPAA_HEADER */



