/****************************************************************************
 *
 *  Rohde & Schwarz Audio Video Platform AV Distorsion Analyzer instrument driver include file
 *
 ****************************************************************************/

#ifndef __RSAVPAVDA_HEADER
#define __RSAVPAVDA_HEADER

#include "rsidr_core.h"
#include "rsavpavda_attributes.h"
#include "rsavpavda_utility.h"

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#if defined(__cplusplus) || defined(__cplusplus__)
extern "C" {
#endif

/*****************************************************************************
 *- Useful Macros & Constants -----------------------------------------------*
 *****************************************************************************/
#define RSAVPAVDA_SIMULATION_ID_QUERY      "Rohde&Schwarz,R&S VTE,2115.7300k02/900000,1.45.0.0"  /* Default definition of ID Query Response for simulation */
#define RSAVPAVDA_VALID_ID_RESPONSE_STRING "Rohde&Schwarz"      /* Valid response for identification query */
#define RSAVPAVDA_SIMULATION_OPT_QUERY     ""      /* Simulated response for *OPT? command */

#define RSAVPAVDA_IO_BUFFER_SIZE           1024L   /* I/O buffer size */
#define RSAVPAVDA_OPC_TIMEOUT              5000L   /* Maximum time to wait for OPC in milliseconds */
#define IEEE_488_2_ERROR_MASK             0x4    /* 488.2 STB Bits */   
#define IEEE_488_2_ESE_MASK               0x3D    /* 488.2 Event Status Register (ESE) Bits */
#define RSAVPAVDA_NAN                 9.91e+37
#define RSAVPAVDA_NAN_VI_INT32             0x80000000     
/****************************************************************************
 *----------------- Instrument Driver Revision Information -----------------*
 ****************************************************************************/

/* Class Driver Identification */

#define RSAVPAVDA_CLASS_DRIVER_DESCRIPTION           ""
#define RSAVPAVDA_CLASS_DRIVER_PREFIX                ""
#define RSAVPAVDA_CLASS_DRIVER_VENDOR                ""
#define RSAVPAVDA_CLASS_DRIVER_REVISION              ""
#define RSAVPAVDA_CLASS_SPEC_MAJOR_VERSION           1L    /* Class specification major version */
#define RSAVPAVDA_CLASS_SPEC_MINOR_VERSION           0L    /* Class specification minor version */

/* Driver Identification */

#define RSAVPAVDA_SPECIFIC_DRIVER_DESCRIPTION        ""
#define RSAVPAVDA_SPECIFIC_DRIVER_PREFIX             "RSAVPAVDA"
#define RSAVPAVDA_SPECIFIC_DRIVER_LOCATOR            ""
#define RSAVPAVDA_SPECIFIC_DRIVER_VENDOR             ""
#define RSAVPAVDA_MAJOR_VERSION                      1L    /* Instrument driver major version          */
#define RSAVPAVDA_MINOR_VERSION                      0L    /* Instrument driver minor version          */
#define RSAVPAVDA_MINOR_MINOR_VERSION                0L    /* Instrument driver minor minor version    */

/* Driver Capabilities */          

#define RSAVPAVDA_SUPPORTED_INSTRUMENT_MODELS        ""    /* Instrument driver supported model(s)     */
#define RSAVPAVDA_GROUP_CAPABILITIES                 ""
#define RSAVPAVDA_FUNCTION_CAPABILITIES              ""

/****************************************************************************
 *---------------------------- Attribute Defines ---------------------------*
 ****************************************************************************/

    /*- Inherent Instrument Specific Attributes ----------------------------*/

        /* User Options */

#define RSAVPAVDA_ATTR_RANGE_CHECK                                           RS_ATTR_RANGE_CHECK                                     /* ViBoolean */
#define RSAVPAVDA_ATTR_QUERY_INSTRUMENT_STATUS                               RS_ATTR_QUERY_INSTRUMENT_STATUS                         /* ViBoolean */
#define RSAVPAVDA_ATTR_CACHE                                                 RS_ATTR_CACHE                                           /* ViBoolean */
#define RSAVPAVDA_ATTR_SIMULATE                                              RS_ATTR_SIMULATE                                        /* ViBoolean */
#define RSAVPAVDA_ATTR_RECORD_COERCIONS                                      RS_ATTR_RECORD_COERCIONS                                /* ViBoolean */
#define RSAVPAVDA_ATTR_INTERCHANGE_CHECK                                     RS_ATTR_INTERCHANGE_CHECK                               /* ViBoolean */
#define RSAVPAVDA_ATTR_SPY                                                   RS_ATTR_SPY                                             /* ViBoolean */
#define RSAVPAVDA_ATTR_USE_SPECIFIC_SIMULATION                               RS_ATTR_USE_SPECIFIC_SIMULATION                         /* ViBoolean */

        /* Class Driver Identification */

#define RSAVPAVDA_ATTR_CLASS_DRIVER_DESCRIPTION                              RS_ATTR_CLASS_DRIVER_DESCRIPTION                        /* ViString, read-only */
#define RSAVPAVDA_ATTR_CLASS_DRIVER_PREFIX                                   RS_ATTR_CLASS_DRIVER_PREFIX                             /* ViString, read-only */
#define RSAVPAVDA_ATTR_CLASS_DRIVER_VENDOR                                   RS_ATTR_CLASS_DRIVER_VENDOR                             /* ViString, read-only */
#define RSAVPAVDA_ATTR_CLASS_DRIVER_REVISION                                 RS_ATTR_CLASS_DRIVER_REVISION                           /* ViString, read-only */
#define RSAVPAVDA_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION           /* ViInt32,  read-only */
#define RSAVPAVDA_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION           /* ViInt32,  read-only */

        /* Driver Identification */

#define RSAVPAVDA_ATTR_SPECIFIC_DRIVER_DESCRIPTION                           RS_ATTR_SPECIFIC_DRIVER_DESCRIPTION                     /* ViString, read-only */
#define RSAVPAVDA_ATTR_SPECIFIC_DRIVER_PREFIX                                RS_ATTR_SPECIFIC_DRIVER_PREFIX                          /* ViString, read-only */
#define RSAVPAVDA_ATTR_SPECIFIC_DRIVER_LOCATOR                               RS_ATTR_SPECIFIC_DRIVER_LOCATOR                         /* ViString, read-only */
#define RSAVPAVDA_ATTR_SPECIFIC_DRIVER_VENDOR                                RS_ATTR_SPECIFIC_DRIVER_VENDOR                          /* ViString, read-only */
#define RSAVPAVDA_ATTR_SPECIFIC_DRIVER_REVISION                              RS_ATTR_SPECIFIC_DRIVER_REVISION                        /* ViString, read-only */
#define RSAVPAVDA_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION        /* ViInt32,  read-only */
#define RSAVPAVDA_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION        /* ViInt32,  read-only */

        /* Driver Capabilities */

#define RSAVPAVDA_ATTR_SUPPORTED_INSTRUMENT_MODELS                           RS_ATTR_SUPPORTED_INSTRUMENT_MODELS                     /* ViString, read-only */
#define RSAVPAVDA_ATTR_GROUP_CAPABILITIES                                    RS_ATTR_GROUP_CAPABILITIES                              /* ViString, read-only */
#define RSAVPAVDA_ATTR_FUNCTION_CAPABILITIES                                 RS_ATTR_FUNCTION_CAPABILITIES                           /* ViString, read-only */
#define RSAVPAVDA_ATTR_CHANNEL_COUNT                                         RS_ATTR_CHANNEL_COUNT                                   /* ViInt32,  read-only */

        /* Driver Setup */

#define RSAVPAVDA_ATTR_DRIVER_SETUP                                          RS_ATTR_DRIVER_SETUP                                    /* ViString            */

        /* Instrument Identification */

#define RSAVPAVDA_ATTR_INSTRUMENT_MANUFACTURER                               RS_ATTR_INSTRUMENT_MANUFACTURER                         /* ViString, read-only */
#define RSAVPAVDA_ATTR_INSTRUMENT_MODEL                                      RS_ATTR_INSTRUMENT_MODEL                                /* ViString, read-only */
#define RSAVPAVDA_ATTR_INSTRUMENT_FIRMWARE_REVISION                          RS_ATTR_INSTRUMENT_FIRMWARE_REVISION                    /* ViString, read-only */

        /* Advanced Session Information */

#define RSAVPAVDA_ATTR_IO_RESOURCE_DESCRIPTOR                                RS_ATTR_IO_RESOURCE_DESCRIPTOR                          /* ViString, read-only */
#define RSAVPAVDA_ATTR_LOGICAL_NAME                                          RS_ATTR_LOGICAL_NAME

/* ViString, read-only */

    /*- Instrument-Specific Attributes -------------------------------------*/

//#define RSAVPAVDA_ATTR_ID_QUERY_RESPONSE                                     (RS_SPECIFIC_PUBLIC_ATTR_BASE + 1L)                     /* ViString (Read Only)              */

/* BEGIN GENERATE */
#define RSAVPAVDA_ATTR_START_AVDA_MEASUREMENT                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1
#define RSAVPAVDA_ATTR_ABORT_AVDA_MEASUREMENT                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2
#define RSAVPAVDA_ATTR_CLEAR_AVDA_MEASUREMENT_RESULTS                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3
#define RSAVPAVDA_ATTR_AVDA_REFERENCE_STATE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4
#define RSAVPAVDA_ATTR_AVDA_SYNCHRONIZATION                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5
#define RSAVPAVDA_ATTR_AVDA_LOOP_DETECTION_TIME                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6
#define RSAVPAVDA_ATTR_AVDA_ALLOWED_MISSING_FRAMES                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7
#define RSAVPAVDA_ATTR_AVDA_REFERENCE_OPERATION                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8
#define RSAVPAVDA_ATTR_AVDA_LOAD_REFERENCE_SEQUENCE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 9
#define RSAVPAVDA_ATTR_AVDA_SAVE_REFERENCE_SEQUENCE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10
#define RSAVPAVDA_ATTR_QUERY_AVDA_REFERENCE_STATE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 11
#define RSAVPAVDA_ATTR_QUERY_AVDA_REFERENCE_PLAYBACK_PROGRESS               RS_SPECIFIC_PUBLIC_ATTR_BASE + 12
#define RSAVPAVDA_ATTR_AVDA_REFERENCE_SEQUENCE_LOAD                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 13
#define RSAVPAVDA_ATTR_AVDA_REFERENCE_SEQUENCE_STORE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 14
#define RSAVPAVDA_ATTR_QUERY_AVDA_CHARACTERISTICS_NOTE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 15
#define RSAVPAVDA_ATTR_QUERY_AVDA_CHARACTERISTICS_DATE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 16
#define RSAVPAVDA_ATTR_QUERY_AVDA_CHARACTERISTICS_TIME                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 17
#define RSAVPAVDA_ATTR_QUERY_AVDA_CHARACTERISTICS_FILE_SIZE                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 18
#define RSAVPAVDA_ATTR_QUERY_AVDA_CHARACTERISTICS_VIDEO_RESOLUTION          RS_SPECIFIC_PUBLIC_ATTR_BASE + 19
#define RSAVPAVDA_ATTR_QUERY_AVDA_CHARACTERISTICS_LOOP_LENGTH               RS_SPECIFIC_PUBLIC_ATTR_BASE + 20
#define RSAVPAVDA_ATTR_QUERY_AVDA_CHARACTERISTICS_NUMBER_OF_FRAMES          RS_SPECIFIC_PUBLIC_ATTR_BASE + 21
#define RSAVPAVDA_ATTR_QUERY_AVDA_CHARACTERISTICS_FITNESS_FOR_SYNCHRONIZATION RS_SPECIFIC_PUBLIC_ATTR_BASE + 22
#define RSAVPAVDA_ATTR_QUERY_AVDA_CHARACTERISTICS_MAX_SYNC_TIME_CLEAR_SIGNAL RS_SPECIFIC_PUBLIC_ATTR_BASE + 23
#define RSAVPAVDA_ATTR_QUERY_AVDA_CHARACTERISTICS_MAX_SYNC_TIME_NOISY_SIGNAL RS_SPECIFIC_PUBLIC_ATTR_BASE + 24
#define RSAVPAVDA_ATTR_QUERY_AVDA_REFERENCE_INPUT_SYNCHRONIZATION           RS_SPECIFIC_PUBLIC_ATTR_BASE + 25
#define RSAVPAVDA_ATTR_QUERY_AVDA_REFERENCE_INPUT_RESOLUTION                RS_SPECIFIC_PUBLIC_ATTR_BASE + 26
#define RSAVPAVDA_ATTR_QUERY_AVDA_REFERENCE_INPUT_FRAME_RATE                RS_SPECIFIC_PUBLIC_ATTR_BASE + 27
#define RSAVPAVDA_ATTR_QUERY_AVDA_REFERENCE_INPUT_COLOR_CODING              RS_SPECIFIC_PUBLIC_ATTR_BASE + 28
#define RSAVPAVDA_ATTR_QUERY_AVDA_REFERENCE_INPUT_COLOR_DEPTH               RS_SPECIFIC_PUBLIC_ATTR_BASE + 29
#define RSAVPAVDA_ATTR_QUERY_AVDA_REFERENCE_INPUT_COLOR_SPACE               RS_SPECIFIC_PUBLIC_ATTR_BASE + 30
#define RSAVPAVDA_ATTR_QUERY_AVDA_REFERENCE_INPUT_ASPECT_RATIO              RS_SPECIFIC_PUBLIC_ATTR_BASE + 31
#define RSAVPAVDA_ATTR_QUERY_AVDA_REFERENCE_INPUT_AUDIO_CHANNELS            RS_SPECIFIC_PUBLIC_ATTR_BASE + 32
#define RSAVPAVDA_ATTR_QUERY_AVDA_REFERENCE_INPUT_FILE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 33
#define RSAVPAVDA_ATTR_QUERY_AVDA_REFERENCE_INPUT_FILE_DATE_TIME            RS_SPECIFIC_PUBLIC_ATTR_BASE + 34
#define RSAVPAVDA_ATTR_QUERY_AVDA_REFERENCE_INPUT_COMMENT                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 35
#define RSAVPAVDA_ATTR_QUERY_AVDA_REFERENCE_INPUT_LOOP_LENGTH               RS_SPECIFIC_PUBLIC_ATTR_BASE + 36
#define RSAVPAVDA_ATTR_QUERY_AVDA_REFERENCE_INPUT_NUMBER_OF_FRAMES          RS_SPECIFIC_PUBLIC_ATTR_BASE + 37
#define RSAVPAVDA_ATTR_QUERY_AVDA_REFERENCE_INPUT_FITNESS_VIDEO_SYNC        RS_SPECIFIC_PUBLIC_ATTR_BASE + 38
#define RSAVPAVDA_ATTR_QUERY_AVDA_REFERENCE_INPUT_FITNESS_AV_DELAY          RS_SPECIFIC_PUBLIC_ATTR_BASE + 39
#define RSAVPAVDA_ATTR_QUERY_AVDA_REFERENCE_INPUT_MAX_TIME_SIGNAL           RS_SPECIFIC_PUBLIC_ATTR_BASE + 40
#define RSAVPAVDA_ATTR_AVDA_LIST_SIGNAL_STATE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 41
#define RSAVPAVDA_ATTR_AVDA_LOOP_DETECTION_STATE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 42
#define RSAVPAVDA_ATTR_AVDA_LOOP_DETECTION_TIME_STATE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 43
#define RSAVPAVDA_ATTR_AVDA_LOOP_DETECTION_TIME_LIMIT                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 44
#define RSAVPAVDA_ATTR_AVDA_FRAME_COUNT_STATE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 45
#define RSAVPAVDA_ATTR_AVDA_CYCLE_NUMBER_STATE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 46
#define RSAVPAVDA_ATTR_AVDA_VISIBLE_ERROR_THRESHOLD_SSIM                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 47
#define RSAVPAVDA_ATTR_AVDA_VISIBLE_ERROR_THRESHOLD_PSNR_Y                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 48
#define RSAVPAVDA_ATTR_AVDA_VISIBLE_ERROR_DURATION                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 49
#define RSAVPAVDA_ATTR_AVDA_VISIBLE_ERROR_DEFAULT_SETTINGS                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 50
#define RSAVPAVDA_ATTR_AVDA_VISIBLE_ERROR_PICTURE_LENGTH_OF_PERIOD          RS_SPECIFIC_PUBLIC_ATTR_BASE + 51
#define RSAVPAVDA_ATTR_AVDA_VISIBLE_ERROR_PICTURE_TOTAL_NUMBER_OF_PERIODS   RS_SPECIFIC_PUBLIC_ATTR_BASE + 52
#define RSAVPAVDA_ATTR_AVDA_VISIBLE_ERROR_PICTURE_GAP_BETWEEN_PERIODS       RS_SPECIFIC_PUBLIC_ATTR_BASE + 53
#define RSAVPAVDA_ATTR_AVDA_VISIBLE_ERROR_PICTURE_ALLOWED_PERIODS           RS_SPECIFIC_PUBLIC_ATTR_BASE + 54
#define RSAVPAVDA_ATTR_AVDA_VISIBLE_ERROR_PICTURE_DEFAULT                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 55
#define RSAVPAVDA_ATTR_AVDA_VISIBLE_ERROR_AUDIO_DURATION                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 56
#define RSAVPAVDA_ATTR_AVDA_VISIBLE_ERROR_AUDIO_DEFAULT                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 57
#define RSAVPAVDA_ATTR_AVDA_VIDEO_DROPPED_FRAMES_LIMIT                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 58
#define RSAVPAVDA_ATTR_AVDA_VIDEO_PIXEL_ERROR_LIMIT_Y                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 59
#define RSAVPAVDA_ATTR_AVDA_VIDEO_PIXEL_ERROR_LIMIT_CB                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 90
#define RSAVPAVDA_ATTR_AVDA_VIDEO_PIXEL_ERROR_LIMIT_CR                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 91
#define RSAVPAVDA_ATTR_AVDA_VIDEO_PSNR_LIMIT_Y                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 60
#define RSAVPAVDA_ATTR_AVDA_VIDEO_PSNR_LIMIT_CB                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 92
#define RSAVPAVDA_ATTR_AVDA_VIDEO_PSNR_LIMIT_CR                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 93
#define RSAVPAVDA_ATTR_AVDA_VIDEO_SSIM_LIMIT                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 61
#define RSAVPAVDA_ATTR_AVDA_VIDEO_MOSV_LIMIT                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 62
#define RSAVPAVDA_ATTR_AVDA_VIDEO_HORIZONTAL_SHIFT_STATE                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 63
#define RSAVPAVDA_ATTR_AVDA_VIDEO_VERTICAL_SHIFT_STATE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 64
#define RSAVPAVDA_ATTR_AVDA_INPUT_MODULE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 65
#define RSAVPAVDA_ATTR_AVDA_INPUT_EXTERNAL_SYNCHRONIZATION                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 66
#define RSAVPAVDA_ATTR_AVDA_INPUT_COLOR_ANALOG                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 67
#define RSAVPAVDA_ATTR_AVDA_INPUT_COLOR_DIGITAL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 68
#define RSAVPAVDA_ATTR_QUERY_AVDA_MODULE_CATALOG                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 69
#define RSAVPAVDA_ATTR_AVDA_INPUT_ASPECT_RATIO                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 70
#define RSAVPAVDA_ATTR_AVDA_INPUT_AUDIO_CHANNELS_COUNT                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 71
#define RSAVPAVDA_ATTR_AVDA_INPUT_COLOR_DEPTH                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 72
#define RSAVPAVDA_ATTR_AVDA_INPUT_COLOR_SPACE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 73
#define RSAVPAVDA_ATTR_AVDA_INPUT_COLOR_CODING                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 74
#define RSAVPAVDA_ATTR_AVDA_INPUT_FRAME_RATE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 75
#define RSAVPAVDA_ATTR_AVDA_INPUT_RESOLUTION                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 76
#define RSAVPAVDA_ATTR_AVDA_INPUT_SIGNAL_SYNCHRONISATION                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 77
#define RSAVPAVDA_ATTR_QUERY_AVDA_DETECTION_STATUS                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 78
#define RSAVPAVDA_ATTR_QUERY_AVDA_DETECTION_TIME                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 79
#define RSAVPAVDA_ATTR_SAVE_AVDA_LOG                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 80
#define RSAVPAVDA_ATTR_SELF_TEST_QUERY                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 81
#define RSAVPAVDA_ATTR_SYST_VERSION                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 82
#define RSAVPAVDA_ATTR_SYSTEM_PRESET_ALL                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 83
#define RSAVPAVDA_ATTR_SYSTEM_PRESET_BASE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 84
#define RSAVPAVDA_ATTR_SYSTEM_RESET_ALL                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 85
#define RSAVPAVDA_ATTR_SYSTEM_RESET_BASE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 86
#define RSAVPAVDA_ATTR_ID_QUERY_RESPONSE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 87
#define RSAVPAVDA_ATTR_SYSTEM_ERROR                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 88
#define RSAVPAVDA_ATTR_DISPLAY_UPDATE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 89

// rsavpavda_rngQueryAVDAReferenceState
#define RSAVPAVDA_VAL_REFERENCE_STATE_IDLE                                  0
#define RSAVPAVDA_VAL_REFERENCE_STATE_LEARNING                              1
#define RSAVPAVDA_VAL_REFERENCE_STATE_LOOP_DETECTED                         2
#define RSAVPAVDA_VAL_REFERENCE_STATE_ACCEPTED                              3

// rsavpavda_rngAVDASynchronization
#define RSAVPAVDA_VAL_AVDA_SYNC_LOOP_APL                                    0
#define RSAVPAVDA_VAL_AVDA_SYNC_LOOP_APL_SECTION                            1
#define RSAVPAVDA_VAL_AVDA_SYNC_LOOP_TIME_CODE                              2
#define RSAVPAVDA_VAL_AVDA_SYNC_STILL_PICTURE                               3
#define RSAVPAVDA_VAL_AVDA_SYNC_SELF_REFERENCED                             4

// rsavpavda_rngAVDAReferenceOperation
#define RSAVPAVDA_VAL_AVDA_REF_LEARN                                        0
#define RSAVPAVDA_VAL_AVDA_REF_ACCEPT                                       1
#define RSAVPAVDA_VAL_AVDA_REF_ABORT                                        2
#define RSAVPAVDA_VAL_AVDA_REF_CLEAR                                        3

// rsavpavda_rngQueryAVDACharacteristicsFitnessForSynchronization
#define RSAVPAVDA_VAL_AVDA_FITNESS_NAV                                      -1
#define RSAVPAVDA_VAL_AVDA_FITNESS_EXCELLENT                                0
#define RSAVPAVDA_VAL_AVDA_FITNESS_GOOD                                     1
#define RSAVPAVDA_VAL_AVDA_FITNESS_FEASIBLE                                 2
#define RSAVPAVDA_VAL_AVDA_FITNESS_POOR                                     3
#define RSAVPAVDA_VAL_AVDA_FITNESS_INSUFFICIENT                             4

// rsavpavda_rngQueryAVDAReferenceInputSynchronization
#define RSAVPAVDA_VAL_SYNC_NAV                                              -1
#define RSAVPAVDA_VAL_SYNC_FAIL                                             0
#define RSAVPAVDA_VAL_SYNC_OK                                               1

// rsavpavda_rngQueryAVDAReferenceInputColorCoding
#define RSAVPAVDA_VAL_COLOR_CODING_NAV                                      -1
#define RSAVPAVDA_VAL_COLOR_CODING_RGB                                      0
#define RSAVPAVDA_VAL_COLOR_CODING_YCBCR                                    1
#define RSAVPAVDA_VAL_COLOR_CODING_YPBPR                                    2

// rsavpavda_rngQueryAVDAReferenceInputColorDepth
#define RSAVPAVDA_VAL_COLOR_DEPTH_NAV                                       -1
#define RSAVPAVDA_VAL_COLOR_DEPTH_4_2_0                                     0
#define RSAVPAVDA_VAL_COLOR_DEPTH_4_2_2                                     1
#define RSAVPAVDA_VAL_COLOR_DEPTH_4_4_4                                     2

// rsavpavda_rngQueryAVDAReferenceInputAspectRatio
#define RSAVPAVDA_VAL_ASPECT_RATIO_NAV                                      -1
#define RSAVPAVDA_VAL_ASPECT_RATIO_4_3                                      0
#define RSAVPAVDA_VAL_ASPECT_RATIO_16_9                                     1

// rsavpavda_rngQueryAVDAReferenceInputFitness
#define RSAVPAVDA_VAL_AVDA_FITNESS_ACCEPTABLE                               1

// rsavpavda_rngAVDAVisibleErrorDefaultSettings
#define RSAVPAVDA_VAL_AVDA_VISIBLE_ERROR_DEFAULT_ANALOG                     0
#define RSAVPAVDA_VAL_AVDA_VISIBLE_ERROR_DEFAULT_DIGITAL                    1

// rsavpavda_rngAVDAVisibleErrorPictureDefault
#define RSAVPAVDA_VAL_AVDA_PICTURE_FAILURE_POINT_DEFAULT_QMP_1              0
#define RSAVPAVDA_VAL_AVDA_PICTURE_FAILURE_POINT_DEFAULT_QMP_2              1
#define RSAVPAVDA_VAL_AVDA_PICTURE_FAILURE_POINT_DEFAULT_MBRAI              2
#define RSAVPAVDA_VAL_AVDA_PICTURE_FAILURE_POINT_DEFAULT_D_BOOK             3

// rsavpavda_rngAVDAInputColorAnalog
#define RSAVPAVDA_VAL_AVDA_COLOR_AUTO                                       0
#define RSAVPAVDA_VAL_AVDA_COLOR_RGB                                        1
#define RSAVPAVDA_VAL_AVDA_COLOR_YPBPR                                      2

// rsavpavda_rngAVDAInputColorDigital
#define RSAVPAVDA_VAL_AVDA_COLOR_YCBCR                                      2

// rsavpavda_rngAVDAInputAspectRatio
#define RSAVPAVDA_VAL_AVDA_INPUT_F43                                        0
#define RSAVPAVDA_VAL_AVDA_INPUT_F169                                       1
#define RSAVPAVDA_VAL_AVDA_INPUT_NAV                                        -1

// rsavpavda_rngAVDAInputColorDepth
#define RSAVPAVDA_VAL_AVDA_INPUT_COLOR_DEPTH_420                            0
#define RSAVPAVDA_VAL_AVDA_INPUT_COLOR_DEPTH_422                            1
#define RSAVPAVDA_VAL_AVDA_INPUT_COLOR_DEPTH_444                            2
#define RSAVPAVDA_VAL_AVDA_INPUT_COLOR_DEPTH_NAV                            -1

// rsavpavda_rngAVDAInputColorCoding
#define RSAVPAVDA_VAL_AVDA_INPUT_COLOR_CODE_RGB                             0
#define RSAVPAVDA_VAL_AVDA_INPUT_COLOR_CODE_YCBC                            1
#define RSAVPAVDA_VAL_AVDA_INPUT_COLOR_CODE_YPBP                            2
#define RSAVPAVDA_VAL_AVDA_INPUT_COLOR_CODE_NAV                             -1

// rsavpavda_rngAVDAInputSignalSynchronisation
#define RSAVPAVDA_VAL_AVDA_INPUT_SYNCHRONISATION_OK                         0
#define RSAVPAVDA_VAL_AVDA_INPUT_SYNCHRONISATION_FAIL                       1
#define RSAVPAVDA_VAL_AVDA_INPUT_SYNCHRONISATION_NAV                        -1

// rsavpavda_rngQueryAVDADetectionStatus
#define RSAVPAVDA_VAL_AVDA_DETECTION_STOPPED                                0
#define RSAVPAVDA_VAL_AVDA_DETECTION_RUNNING                                1
#define RSAVPAVDA_VAL_AVDA_DETECTION_PASSED                                 2
#define RSAVPAVDA_VAL_AVDA_DETECTION_ABORTED                                3
#define RSAVPAVDA_VAL_AVDA_DETECTION_NAV                                    -1

// rsavpavda_rngAVDAVisibleErrorAudioDefault
#define RSAVPAVDA_VAL_AVDA_AUDIO_FAILURE_POINT_DEFAULT_ANALOG               0
#define RSAVPAVDA_VAL_AVDA_AUDIO_FAILURE_POINT_DEFAULT_DIGITAL              1
/* END GENERATE */

/*****************************************************************************
 *- Hidden Attribute Declarations -------------------------------------------*
 *****************************************************************************/

#define RSAVPAVDA_ATTR_OPC_TIMEOUT               (RS_ATTR_OPC_TIMEOUT)           /* ViInt32 */
#define RSAVPAVDA_ATTR_IO_SESSION                (RS_ATTR_IO_SESSION)            /* ViSession */
#define RSAVPAVDA_ATTR_OPC_CALLBACK              (RS_ATTR_OPC_CALLBACK)          /* ViAddr */
#define RSAVPAVDA_ATTR_CHECK_STATUS_CALLBACK     (RS_ATTR_CHECK_STATUS_CALLBACK) /* ViAddr */

/****************************************************************************
 *---------------- Constant definition  ------------------------------------*
 ****************************************************************************/

#define RSAVPAVDA_VAL_AVDA_REF_LOAD  0
#define RSAVPAVDA_VAL_AVDA_REF_SAVE  1
	
#define RSAVPAVDA_VAL_AVDA_INIT   0
#define RSAVPAVDA_VAL_AVDA_ABORT  1
#define RSAVPAVDA_VAL_AVDA_CLEAR  2

#define RSAVPAVDA_VAL_AVDA_VIDEO_Y   0
#define RSAVPAVDA_VAL_AVDA_VIDEO_CB  1
#define RSAVPAVDA_VAL_AVDA_VIDEO_CR  2


#define RSAVPAVDA_VAL_AVDA_LOG_ALL_FRAMES  0
#define RSAVPAVDA_VAL_AVDA_LOG_LIMIT_VIOLATION 1

#define RSAVPAVDA_VAL_AVDA_LOG_COMMA  0
#define RSAVPAVDA_VAL_AVDA_LOG_DOT    1

#define RSAVPAVDA_VAL_RST_BASE 1
#define RSAVPAVDA_VAL_RST_ALL  2

/****************************************************************************
*---------------- Instrument Driver Function Declarations -----------------*
 ****************************************************************************/

ViStatus _VI_FUNC rsavpavda_init (ViRsrc resourceName, ViBoolean IDQuery,
                                  ViBoolean resetDevice,
                                  ViSession* instrumentHandle);
ViStatus _VI_FUNC rsavpavda_InitWithOptions (ViRsrc resourceName, ViBoolean IDQuery,
                                             ViBoolean resetDevice,
                                             ViChar _VI_FAR optionString[],
                                             ViSession* instrumentHandle);
ViStatus _VI_FUNC rsavpavda_AVDAOperation (ViSession instrumentHandle,
                                           ViInt32 board, ViInt32 operation);
ViStatus _VI_FUNC rsavpavda_QueryAVDAReferenceState (ViSession instrumentHandle,
                                                     ViInt32 board, ViInt32* state);
ViStatus _VI_FUNC rsavpavda_ConfigureAVDASynchronization (ViSession instrumentHandle,
                                                          ViInt32 board,
                                                          ViInt32 synchronization);
ViStatus _VI_FUNC rsavpavda_ConfigureAVDALoopAPL (ViSession instrumentHandle,
                                                  ViInt32 board,
                                                  ViReal64 loopDetectionTime,
                                                  ViReal64 sectionLeft,
                                                  ViReal64 sectionRight,
                                                  ViReal64 sectionTop,
                                                  ViReal64 sectionBottom);
ViStatus _VI_FUNC rsavpavda_ConfigureAVDALoopTimeCode (ViSession instrumentHandle,
                                                       ViInt32 board,
                                                       ViInt32 allowedMissingFrames,
                                                       ViReal64 TCWindowLeft,
                                                       ViReal64 TCWindowRight,
                                                       ViReal64 TCWindowTop,
                                                       ViReal64 TCWindowBottom);
ViStatus _VI_FUNC rsavpavda_QueryAVDAReferencePlaybackProgress
             (ViSession instrumentHandle, ViInt32 board, ViReal64* progress);
ViStatus _VI_FUNC rsavpavda_AVDAReferenceOperation (ViSession instrumentHandle,
                                                    ViInt32 board,
                                                    ViInt32 operation);
ViStatus _VI_FUNC rsavpavda_AVDAReferenceSequenceLoadSave
             (ViSession instrumentHandle, ViInt32 board, ViInt32 operation,
              ViString fileName);
ViStatus _VI_FUNC rsavpavda_ConfigureAVDA (ViSession instrumentHandle,
                                           ViInt32 board, ViBoolean signal,
                                           ViBoolean loopDetection,
                                           ViBoolean loopDetectionTime,
                                           ViReal64 loopDetectionTimeLimit,
                                           ViBoolean frameNumber,
                                           ViBoolean cycleNumber);
ViStatus _VI_FUNC rsavpavda_ConfigureAVDAFailurePointVisibleError
             (ViSession instrumentHandle, ViInt32 board, ViBoolean enabled,
              ViBoolean statusTrace, ViReal64 thresholdSSIM,
              ViReal64 thresholdPSNRY, ViInt32 duration);
ViStatus _VI_FUNC rsavpavda_ConfigureAVDAPictureFailurePoint
             (ViSession instrumentHandle, ViInt32 board, ViBoolean enabled,
              ViBoolean statusTrace, ViInt32 length_ofPeriod,
              ViInt32 totalNumber_ofPeriods, ViInt32 gapBetweenPeriods,
              ViInt32 allowedPeriods);
ViStatus _VI_FUNC rsavpavda_ConfigureAVDAAudioFailurePoint
             (ViSession instrumentHandle, ViInt32 board, ViBoolean enabled,
              ViBoolean statusTrace, ViInt32 duration);
ViStatus _VI_FUNC rsavpavda_ConfigureAVDAVideoFreeze (ViSession instrumentHandle,
                                                      ViInt32 board,
                                                      ViBoolean enabled,
                                                      ViBoolean statusTrace);
ViStatus _VI_FUNC rsavpavda_ConfigureAVDAVideoDroppedFrames
             (ViSession instrumentHandle, ViInt32 board, ViBoolean enabled,
              ViBoolean resultTrace, ViInt32 limit);
ViStatus _VI_FUNC rsavpavda_ConfigureAVDAVideoblackFrame (ViSession instrumentHandle,
                                                          ViInt32 board,
                                                          ViBoolean enabled,
                                                          ViBoolean statusTrace);
ViStatus _VI_FUNC rsavpavda_ConfigureAVDAVideoAveragePictureLevel
             (ViSession instrumentHandle, ViInt32 board, ViBoolean enabled,
              ViBoolean resultTrace);
ViStatus _VI_FUNC rsavpavda_ConfigureAVDAVideoPixelError (ViSession instrumentHandle,
                                                          ViInt32 board,
                                                          ViInt32 parameter,
                                                          ViBoolean enabled,
                                                          ViBoolean resultTrace,
                                                          ViInt32 limit);
ViStatus _VI_FUNC rsavpavda_ConfigureAVDAVideoPSNR (ViSession instrumentHandle,
                                                    ViInt32 board,
                                                    ViInt32 parameter,
                                                    ViBoolean enabled,
                                                    ViBoolean resultTrace,
                                                    ViReal64 limit);
ViStatus _VI_FUNC rsavpavda_ConfigureAVDAVideoSSIM (ViSession instrumentHandle,
                                                    ViInt32 board,
                                                    ViBoolean enabled,
                                                    ViBoolean resultTrace,
                                                    ViReal64 limit);
ViStatus _VI_FUNC rsavpavda_ConfigureAVDAVideoMOSV (ViSession instrumentHandle,
                                                    ViInt32 board,
                                                    ViBoolean enabled,
                                                    ViBoolean resultTrace,
                                                    ViReal64 limit);
ViStatus _VI_FUNC rsavpavda_ConfigureAVDAVideoHorizontalShift
             (ViSession instrumentHandle, ViInt32 board, ViBoolean enabled,
              ViReal64 lowerLimit, ViReal64 upperLimit);
ViStatus _VI_FUNC rsavpavda_ConfigureAVDAVideoVerticalShift
             (ViSession instrumentHandle, ViInt32 board, ViBoolean enabled,
              ViReal64 lowerLimit, ViReal64 upperLimit);
ViStatus _VI_FUNC rsavpavda_ConfigureAVDAAudioLoss (ViSession instrumentHandle,
                                                    ViInt32 board,
                                                    ViBoolean _VI_FAR enabled[],
                                                    ViBoolean _VI_FAR statusTrace[],
                                                    ViInt32 _VI_FAR limit[]);
ViStatus _VI_FUNC rsavpavda_ConfigureAVDAAudioLevel (ViSession instrumentHandle,
                                                     ViInt32 board,
                                                     ViBoolean _VI_FAR enabled[],
                                                     ViBoolean _VI_FAR resultTrace[],
                                                     ViReal64 _VI_FAR lowerLimit[],
                                                     ViReal64 _VI_FAR upperLimit[]);
ViStatus _VI_FUNC rsavpavda_ConfigureAVDAAVDelay (ViSession instrumentHandle,
                                                  ViInt32 board,
                                                  ViBoolean _VI_FAR enabled[],
                                                  ViBoolean _VI_FAR resultTrace[],
                                                  ViReal64 _VI_FAR lowerLimit[],
                                                  ViReal64 _VI_FAR upperLimit[]);
ViStatus _VI_FUNC rsavpavda_ConfigureAVDAInput (ViSession instrumentHandle,
                                                ViInt32 board, ViString input,
                                                ViBoolean externalSync,
                                                ViInt32 colorAnalog,
                                                ViInt32 colorDigital);
ViStatus _VI_FUNC rsavpavda_QueryAVDAModuleCatalog (ViSession instrumentHandle,
                                                    ViInt32 board,
                                                    ViInt32 bufferSize,
                                                    ViChar _VI_FAR catalog[]);
ViStatus _VI_FUNC rsavpavda_QueryAVDADetectionStatus (ViSession instrumentHandle,
                                                      ViInt32 board,
                                                      ViInt32* detectionState);
ViStatus _VI_FUNC rsavpavda_QueryAVDADetectionTime (ViSession instrumentHandle,
                                                    ViInt32 board,
                                                    ViInt32* detectionTime);
ViStatus _VI_FUNC rsavpavda_QueryAVDAList (ViSession instrumentHandle,
                                           ViInt32 board, ViInt32* signal,
                                           ViInt32* frameNumberCurrent,
                                           ViInt32* frameNumberWorst,
                                           ViInt32* cycleNumberCurrent,
                                           ViInt32* cycleNumberWorst);
ViStatus _VI_FUNC rsavpavda_QueryAVDAListFailurePointVisibleError
             (ViSession instrumentHandle, ViInt32 board, ViInt32* current,
              ViInt32* worst, ViInt32* faultFrames);
ViStatus _VI_FUNC rsavpavda_QueryAVDAListPictureFailurePoint
             (ViSession instrumentHandle, ViInt32 board, ViInt32* current,
              ViInt32* worst, ViInt32* faultFrames);
ViStatus _VI_FUNC rsavpavda_QueryAVDAListAudioFailurePoint
             (ViSession instrumentHandle, ViInt32 board, ViInt32* current,
              ViInt32* worst, ViInt32* faultFrames);
ViStatus _VI_FUNC rsavpavda_QueryAVDAListVideoFreeze (ViSession instrumentHandle,
                                                      ViInt32 board,
                                                      ViInt32* current,
                                                      ViInt32* worst,
                                                      ViInt32* faultFrames);
ViStatus _VI_FUNC rsavpavda_QueryAVDAListVideoDroppedFrames
             (ViSession instrumentHandle, ViInt32 board, ViInt32* current,
              ViInt32* worst, ViInt32* faultFrames);
ViStatus _VI_FUNC rsavpavda_QueryAVDAListVideoBlackFrames
             (ViSession instrumentHandle, ViInt32 board, ViInt32* current,
              ViInt32* worst, ViInt32* faultFrames);
ViStatus _VI_FUNC rsavpavda_QueryAVDAListVideoAveragePictureLevel
             (ViSession instrumentHandle, ViInt32 board, ViReal64* current);
ViStatus _VI_FUNC rsavpavda_QueryAVDAListVideoPixelError (ViSession instrumentHandle,
                                                          ViInt32 board,
                                                          ViInt32 parameter,
                                                          ViInt32* current,
                                                          ViInt32* worst,
                                                          ViInt32* faultFrames);
ViStatus _VI_FUNC rsavpavda_QueryAVDAListVideoPSNR (ViSession instrumentHandle,
                                                    ViInt32 board,
                                                    ViInt32 parameter,
                                                    ViReal64* current,
                                                    ViReal64* worst,
                                                    ViInt32* faultFrames);
ViStatus _VI_FUNC rsavpavda_QueryAVDAListVideoSSIM (ViSession instrumentHandle,
                                                    ViInt32 board,
                                                    ViReal64* current,
                                                    ViReal64* worst,
                                                    ViInt32* faultFrames);
ViStatus _VI_FUNC rsavpavda_QueryAVDAListVideoMOSV (ViSession instrumentHandle,
                                                    ViInt32 board,
                                                    ViReal64* current,
                                                    ViReal64* worst,
                                                    ViInt32* faultFrames);
ViStatus _VI_FUNC rsavpavda_QueryAVDAListAudioLoss (ViSession instrumentHandle,
                                                    ViInt32 board,
                                                    ViInt32 arraySize,
                                                    ViInt32 _VI_FAR current[],
                                                    ViInt32 _VI_FAR worst[]);
ViStatus _VI_FUNC rsavpavda_QueryAVDAListAudioLevel (ViSession instrumentHandle,
                                                     ViInt32 board,
                                                     ViInt32 arraySize,
                                                     ViReal64 _VI_FAR current[],
                                                     ViReal64 _VI_FAR worst[]);
ViStatus _VI_FUNC rsavpavda_QueryAVDAListAVDelay (ViSession instrumentHandle,
                                                  ViInt32 board, ViInt32 arraySize,
                                                  ViReal64 _VI_FAR current[],
                                                  ViReal64 _VI_FAR worst[]);
ViStatus _VI_FUNC rsavpavda_ConfigureAVDALog (ViSession instrumentHandle,
                                              ViInt32 board, ViInt32 logData,
                                              ViInt32 decimalPoint);
ViStatus _VI_FUNC rsavpavda_SaveAVDALog (ViSession instrumentHandle, ViInt32 board,
                                         ViString fileName);
ViStatus _VI_FUNC rsavpavda_reset (ViSession instrumentHandle);

ViStatus _VI_FUNC rsavpavda_ResetSystem (ViSession instrumentHandle,
                                         ViInt32 reference);

ViStatus _VI_FUNC rsavpavda_Preset (ViSession instrumentHandle, ViInt32 reference);
ViStatus _VI_FUNC rsavpavda_self_test (ViSession instrumentHandle,
                                       ViInt16* selfTestResult,
                                       ViChar _VI_FAR selfTestMessage[]);
ViStatus _VI_FUNC rsavpavda_revision_query (ViSession instrumentHandle,
                                            ViChar _VI_FAR instrumentDriverRevision[],
                                            ViChar _VI_FAR firmwareRevision[]);
ViStatus _VI_FUNC rsavpavda_error_query (ViSession instrumentHandle,
                                         ViInt32* errorCode,
                                         ViChar _VI_FAR errorMessage[]);
ViStatus _VI_FUNC rsavpavda_error_message (ViSession instrumentHandle,
                                           ViStatus errorCode,
                                           ViChar _VI_FAR errorMessage[]);
ViStatus _VI_FUNC rsavpavda_SystemVersion (ViSession instrumentHandle,
                                           ViInt32 length,
                                           ViChar _VI_FAR systemVersion[]);
ViStatus _VI_FUNC rsavpavda_setCheckStatus (ViSession instrumentHandle,
                                            ViBoolean state);
ViStatus _VI_FUNC rsavpavda_ConfigureDisplayUpdate (ViSession instrumentHandle,
                                                    ViBoolean displayUpdate);
ViStatus _VI_FUNC rsavpavda_GetError (ViSession instrumentHandle, ViStatus* code,
                                      ViInt32 bufferSize,
                                      ViChar _VI_FAR description[]);
ViStatus _VI_FUNC rsavpavda_ClearError (ViSession instrumentHandle);
ViStatus _VI_FUNC rsavpavda_WriteInstrData (ViSession instrumentHandle,
                                            ViChar _VI_FAR writeBuffer[]);
ViStatus _VI_FUNC rsavpavda_ReadInstrData (ViSession instrumentHandle,
                                           ViInt32 number_ofBytesToRead,
                                           ViChar _VI_FAR readBuffer[],
                                           ViUInt32* numBytesRead);
ViStatus _VI_FUNC rsavpavda_QueryViBoolean (ViSession instrumentHandle,
                                            ViString command, ViBoolean* value);
ViStatus _VI_FUNC rsavpavda_QueryViInt32 (ViSession instrumentHandle,
                                          ViString command, ViInt32* value);
ViStatus _VI_FUNC rsavpavda_QueryViReal64 (ViSession instrumentHandle,
                                           ViString command, ViReal64* value);
ViStatus _VI_FUNC rsavpavda_QueryViString (ViSession instrumentHandle,
                                           ViString command, ViInt32 bufferSize,
                                           ViChar _VI_FAR value[]);
ViStatus _VI_FUNC rsavpavda_SetAttributeViInt32 (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViInt32 attributeValue);
ViStatus _VI_FUNC rsavpavda_SetAttributeViInt64 (ViSession instrumentHandle,
                                                 ViChar _VI_FAR repeatedCapabilityName[],
                                                 ViAttr attributeID,
                                                 ViInt64 attributeValue);
ViStatus _VI_FUNC rsavpavda_SetAttributeViReal64 (ViSession instrumentHandle,
                                                  ViChar _VI_FAR channelName[],
                                                  ViAttr attributeID,
                                                  ViReal64 attributeValue);
ViStatus _VI_FUNC rsavpavda_SetAttributeViString (ViSession instrumentHandle,
                                                  ViChar _VI_FAR channelName[],
                                                  ViAttr attributeID,
                                                  ViChar _VI_FAR attributeValue[]);
ViStatus _VI_FUNC rsavpavda_SetAttributeViBoolean (ViSession instrumentHandle,
                                                   ViChar _VI_FAR channelName[],
                                                   ViAttr attributeID,
                                                   ViBoolean attributeValue);
ViStatus _VI_FUNC rsavpavda_SetAttributeViSession (ViSession instrumentHandle,
                                                   ViChar _VI_FAR channelName[],
                                                   ViAttr attributeID,
                                                   ViSession attributeValue);
ViStatus _VI_FUNC rsavpavda_GetAttributeViInt32 (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViInt32* attributeValue);
ViStatus _VI_FUNC rsavpavda_GetAttributeViInt64 (ViSession instrumentHandle,
                                                 ViChar _VI_FAR repeatedCapabilityName[],
                                                 ViAttr attributeID,
                                                 ViInt64* attributeValue);
ViStatus _VI_FUNC rsavpavda_GetAttributeViReal64 (ViSession instrumentHandle,
                                                  ViChar _VI_FAR channelName[],
                                                  ViAttr attributeID,
                                                  ViReal64* attributeValue);
ViStatus _VI_FUNC rsavpavda_GetAttributeViString (ViSession instrumentHandle,
                                                  ViChar _VI_FAR channelName[],
                                                  ViAttr attributeID,
                                                  ViInt32 bufferSize,
                                                  ViChar _VI_FAR attributeValue[]);
ViStatus _VI_FUNC rsavpavda_GetAttributeViBoolean (ViSession instrumentHandle,
                                                   ViChar _VI_FAR channelName[],
                                                   ViAttr attributeID,
                                                   ViBoolean* attributeValue);
ViStatus _VI_FUNC rsavpavda_GetAttributeViSession (ViSession instrumentHandle,
                                                   ViChar _VI_FAR channelName[],
                                                   ViAttr attributeID,
                                                   ViSession* attributeValue);
ViStatus _VI_FUNC rsavpavda_CheckAttributeViInt32 (ViSession instrumentHandle,
                                                   ViChar _VI_FAR channelName[],
                                                   ViAttr attributeID,
                                                   ViInt32 attributeValue);
ViStatus _VI_FUNC rsavpavda_CheckAttributeViInt64 (ViSession instrumentHandle,
                                                   ViChar _VI_FAR channelName[],
                                                   ViAttr attributeID,
                                                   ViInt64 attributeValue);
ViStatus _VI_FUNC rsavpavda_CheckAttributeViReal64 (ViSession instrumentHandle,
                                                    ViChar _VI_FAR channelName[],
                                                    ViAttr attributeID,
                                                    ViReal64 attributeValue);
ViStatus _VI_FUNC rsavpavda_CheckAttributeViString (ViSession instrumentHandle,
                                                    ViChar _VI_FAR channelName[],
                                                    ViAttr attributeID,
                                                    ViChar _VI_FAR attributeValue[]);
ViStatus _VI_FUNC rsavpavda_CheckAttributeViBoolean (ViSession instrumentHandle,
                                                     ViChar _VI_FAR channelName[],
                                                     ViAttr attributeID,
                                                     ViBoolean attributeValue);
ViStatus _VI_FUNC rsavpavda_CheckAttributeViSession (ViSession instrumentHandle,
                                                     ViChar _VI_FAR channelName[],
                                                     ViAttr attributeID,
                                                     ViSession attributeValue);
ViStatus _VI_FUNC rsavpavda_GetAttributeRepeatedCapabilityIds
             (ViSession instrumentHandle, ViUInt32 attributeID, ViInt32 bufferSize,
              ViChar _VI_FAR repeatedCapabilityId_s[]);
ViStatus _VI_FUNC rsavpavda_GetAttributeRepeatedCapabilityIdNames
             (ViSession instrumentHandle, ViUInt32 attributeID,
              ViString repeatedCapabilityId, ViInt32 bufferSize,
              ViChar _VI_FAR repeatedCapabilityIdName_s[]);
ViStatus _VI_FUNC rsavpavda_close (ViSession instrumentHandle);



/****************************************************************************
 *------------------------ Error And Completion Codes ----------------------*
 ****************************************************************************/

#define RSAVPAVDA_WARN_MEASURE_UNCALIBRATED    (RS_CLASS_WARN_BASE + 0x0001L)
#define RSAVPAVDA_WARN_OVER_RANGE              (RS_CLASS_WARN_BASE + 0x0002L)

#define RSAVPAVDA_ERROR_MARKER_NOT_ENABLED     (RS_CLASS_ERROR_BASE + 0x0003L)
#define RSAVPAVDA_ERROR_NOT_DELTA_MARKER       (RS_CLASS_ERROR_BASE + 0x0002L)
#define RSAVPAVDA_ERROR_MAX_TIME_EXCEEDED      (RS_CLASS_ERROR_BASE + 0x0001L)

#define RSAVPAVDA_WARNMSG_MEASURE_UNCALIBRATED "Uncalibrated measurement"
#define RSAVPAVDA_WARNMSG_OVER_RANGE           "Measurement overrange"

#define RSAVPAVDA_ERRMSG_MARKER_NOT_ENABLED    "Marker Not Enabled"
#define RSAVPAVDA_ERRMSG_NOT_DELTA_MARKER      "Not Delta Marker"
#define RSAVPAVDA_ERRMSG_MAX_TIME_EXCEEDED     "Max Time Exceeded"

#define RSAVPAVDA_ERROR_CODES_AND_MSGS \
    {RSAVPAVDA_WARN_MEASURE_UNCALIBRATED,  RSAVPAVDA_WARNMSG_MEASURE_UNCALIBRATED},\
    {RSAVPAVDA_WARN_OVER_RANGE,            RSAVPAVDA_WARNMSG_OVER_RANGE},\
    {RSAVPAVDA_ERROR_MARKER_NOT_ENABLED,   RSAVPAVDA_ERRMSG_MARKER_NOT_ENABLED},\
    {RSAVPAVDA_ERROR_NOT_DELTA_MARKER,     RSAVPAVDA_ERRMSG_NOT_DELTA_MARKER},\
    {RSAVPAVDA_ERROR_MAX_TIME_EXCEEDED,    RSAVPAVDA_ERRMSG_MAX_TIME_EXCEEDED}

/****************************************************************************
 *---------------------------- End Include File ----------------------------*
 ****************************************************************************/

#if defined(__cplusplus) || defined(__cplusplus__)
}
#endif
#endif /* __RSAVPAVDA_HEADER */



