/*****************************************************************************
 *  Rohde&Schwarz Video Tester Instrument Driver                               
 *  LabWindows/CVI Instrument Driver                                     
 *  Original Release: 2013-09-19                                 
 *  By: Nabil Ouddane
        Martin Krcmar
 *                                                                           
 *  Modification History:                                                    
 *                                                                           
 *       See ChangeLog
 *                                                                           
 *****************************************************************************/
                         
#include <string.h>
#include <stdio.h>  
#include <stdlib.h>
#include <math.h>
#include "rsavpha.h"

ViStatus _VI_FUNC  rsavpha_RsClose (ViSession vi);

/*****************************************************************************
 *- Value Definition and Range Tables ---------------------------------------*
 *****************************************************************************/
static ViString SwitchArr[] = {"OFF","ON",VI_NULL};
static ViString HDMIAudioParametersArr[] = {"APPL","SAMP","COPY","PRE","MODE","CAT","NSO","NCH","FS","CACC","LENGth:MAX" ,"LENG", VI_NULL};
static ViString HDMIAnalyzerTestCaseArr[] = {"M001","M002","M003","M004","M005","M006","M007","M008","M009","M010","M011","M012","M013","M014","M015","M016","M017","M018","M019","M020","M021","M022","M023","CONC",VI_NULL};
static ViString TestLengthArr[] = {"SEC2","FRAM",VI_NULL};
static ViString PixelEncodingArr[] = {"RGB","Y422","Y444",VI_NULL};
static ViString HDMITestSetupArr[] = {"AI0","AI1",VI_NULL};
static ViString HDMIPixelEncodingTestSetupArr[] = {"PENC","YQ",VI_NULL};
static ViString HDMICDFSetupArr[] = {"Y","N",VI_NULL};
static ViString HDMIVideoFormatArr[] = {"PROG","INT",VI_NULL};
static ViString HDMIContentTypeArr[] = {"PHOT","CIN","GAME", VI_NULL};
static ViString HDCPStateArr[] = {"INV","ACT","INAC", "UNKN", VI_NULL};
static ViString SyncPolarityArr[] = {"NEG", "POS", VI_NULL};
/*****************************************************************************
 *- Callback Declarations ---------------------------------------------------*
 *****************************************************************************/

/*****************************************************************************
 *- User-Callable Functions (Exportable Functions) --------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: rsavpha_init
 * Purpose:  VXIplug&play required function. Calls the
 *           rsavpha_InitWithOptions function.
 *****************************************************************************/
ViStatus _VI_FUNC rsavpha_init (ViRsrc resourceName, ViBoolean IDQuery,
                                 ViBoolean resetDevice, ViSession *newInstrSession)
{
    ViStatus    error = VI_SUCCESS;

    if (newInstrSession == VI_NULL)
        {
        (void) Rs_SetErrorInfo (VI_NULL, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                          VI_ERROR_PARAMETER4, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
        }

    checkErr( rsavpha_InitWithOptions (resourceName, IDQuery, resetDevice,
                                      "QueryInstrStatus=1", newInstrSession));

Error:
    return error;
}

/*****************************************************************************
 * Function: rsavpha_InitWithOptions
 * Purpose:  This function creates a new RS session and calls the
 *           RsInit function.
 *****************************************************************************/
ViStatus _VI_FUNC rsavpha_InitWithOptions(
    ViRsrc          resourceName,
    ViBoolean       IDQuery,
    ViBoolean       resetDevice,
    ViString        optionString,
    ViSession      *newInstrSession
)
{
    ViStatus                error = VI_SUCCESS;
    ViChar                  newResourceName[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar                  newOptionString[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViBoolean               isLogicalName = 0;
    ViSession               instrSession = 0, rmSession = 0;
    RsAttrPropertiesPtr     *attrList=NULL;
    RsRepCapPtr             repCapTable = NULL;
    
    if (newInstrSession == VI_NULL)
        {
        (void) Rs_SetErrorInfo (VI_NULL, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                         VI_ERROR_PARAMETER5, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
        }

    /* Prepare resource manager session handle */
    checkErr (viOpenDefaultRM (&rmSession));

    /* Establish communication session with the instrument */

    viCheckErr( viOpen (rmSession, resourceName, VI_NULL, VI_NULL, &instrSession));
    
    /* Configure VISA Formatted I/O */
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, 5000 ));
    viCheckErr( viSetBuf (instrSession, (ViUInt16) (VI_READ_BUF | VI_WRITE_BUF), 4000));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_WR_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_RD_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));

    checkErr( Rs_GetInfoFromResourceName (resourceName,
                                          (ViString)optionString,
                                          newResourceName,
                                          newOptionString,
                                          &isLogicalName));
    
    ATTRLIST(attrList,rsavpha);  
    checkErr( Rs_SpecificDriverNew (instrSession, "rsavpha", newOptionString, attrList));
    
    /* Build repeated capability table */
    REPCAPTABLE(repCapTable,rsavpha);
    checkErr( Rs_BuildRepCapTable (instrSession, repCapTable));

    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_VISA_RM_SESSION, 0, &rmSession));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_IO_SESSION, 0, &instrSession));

    /* With remote control via Ethernet, remote control is not automatically
       set by means of a command. The instrument must be explicitly set to
       the REMOTE state, e.g. by sending the intf command &GTR
    */
    /*- Reset instrument ----------------------------------------------------*/
    if (resetDevice == VI_TRUE)
    {
        checkErr( rsavpha_reset (instrSession));
    }
    else  /*- Send Default Instrument Setup ---------------------------------*/
        checkErr( rsavpha_DefaultInstrSetup (instrSession));

    /*- Identification Query ------------------------------------------------*/
    if (IDQuery == VI_TRUE)
        {
        ViChar rdBuffer[RSAVPHA_IO_BUFFER_SIZE] = "";

        checkErr( Rs_GetAttribute (instrSession, VI_NULL, (ViAttr) RSAVPHA_ATTR_ID_QUERY_RESPONSE,
                                   0, RSAVPHA_IO_BUFFER_SIZE, rdBuffer));

        if (strncmp (rdBuffer, RSAVPHA_VALID_ID_RESPONSE_STRING, strlen(RSAVPHA_VALID_ID_RESPONSE_STRING)) != 0)
            {
            viCheckErr( VI_ERROR_FAIL_ID_QUERY);
            }
        }

    if (isLogicalName == VI_FALSE)
        {
        ViInt32 oldFlag = 0;

        checkErr (Rs_GetAttributeFlags (instrSession, RS_ATTR_IO_RESOURCE_DESCRIPTOR, &oldFlag));
        checkErr (Rs_SetAttributeFlags (instrSession, RS_ATTR_IO_RESOURCE_DESCRIPTOR, oldFlag & 0xfffb | 0x0010));
        checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_IO_RESOURCE_DESCRIPTOR, 0, newResourceName));
        }
    
    if (isLogicalName == VI_TRUE)
        checkErr( Rs_ApplyDefaultSetup (instrSession));

    /* Instrument session accessible by the end user is native VISA session,
       or when simulated virtual instrument (instrSession) session. */
    
Error:

    if (error < VI_SUCCESS)
    {
        ViSession   rmSess = VI_NULL;
        
        (void) Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_VISA_RM_SESSION, 0, (ViInt32) sizeof (rmSess), &rmSess);
        (void) Rs_Dispose (instrSession); 
        if (instrSession != 0)
            (void) viClose (instrSession);
        (void) viClose (rmSess);
    }
    else
    {
        *newInstrSession = instrSession;
    }
    
    return error;
    
}

/// HIFN  This function sets the value of a ViInt32 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpha_SetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function sets the value of a ViInt64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpha_SetAttributeViInt64 (ViSession instrSession,
                                                 ViChar channelName[],
                                                 ViAttr attributeId,
                                                 ViInt64 value)
{
	return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function sets the value of a ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpha_SetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 value)
{
     return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function sets the value of a ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpha_SetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViString value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/// HIFN  This function sets the value of a ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpha_SetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function sets the value of a ViSession attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpha_SetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function queries the value of a ViInt32 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViInt32 variable.
ViStatus _VI_FUNC rsavpha_GetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViInt32), (ViInt32 *)value);
}

/// HIFN  This function queries the value of a ViInt64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViInt64 variable.
ViStatus _VI_FUNC rsavpha_GetAttributeViInt64 (ViSession instrSession,
                                                 ViChar channelName[],
                                                 ViAttr attributeId,
                                                 ViInt64 *value)
{
	return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViInt64), (ViInt64 *)value);
}

/// HIFN  This function queries the value of a ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViReal64 variable.
ViStatus _VI_FUNC rsavpha_GetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViReal64), (ViReal64 *)value);
}

/// HIFN  This function queries the value of a ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR bufSize/Pass the number of bytes in the ViChar array you specify for the 
/// HIPAR bufSize/Attribute Value parameter.  
/// HIPAR value/The buffer in which the function returns the current value of the 
/// HIPAR value/attribute.  The buffer must be of type ViChar and have at least as many 
/// HIPAR value/bytes as indicated in the Buffer Size parameter.
ViStatus _VI_FUNC rsavpha_GetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViInt32 bufSize,
                                                 ViChar value[])
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            bufSize, value);
}

/// HIFN  This function queries the value of a ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViBoolean variable.
ViStatus _VI_FUNC rsavpha_GetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViBoolean), (ViBoolean *)value);
}

/// HIFN  This function queries the value of a ViSession attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViSession variable.
ViStatus _VI_FUNC rsavpha_GetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViSession), (ViSession *)value);
}

/// HIFN  This function checks the validity of a value you specify for a ViInt32 
/// HIFN  attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpha_CheckAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViInt32 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function checks the validity of a value you specify for a ViInt64 
/// HIFN  attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpha_CheckAttributeViInt64 (ViSession instrSession,
                                                   ViChar channelName[],
                                                   ViAttr attributeId,
                                                   ViInt64 value)
{
	return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpha_CheckAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViReal64 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpha_CheckAttributeViString (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViString value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpha_CheckAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViBoolean value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViSession attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpha_CheckAttributeViSession (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViSession value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN This function returns the repeated capability id(s) that belongs
/// HIFN to the attribute you specify. More then one id is listed in
/// HIFN comma separated string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Repeated Capability Id(s) parameter.
/// HIPAR repeatedCapabilityIds/The buffer in which the function returns the repeated capability id(s) belonging to the attribute.
ViStatus _VI_FUNC rsavpha_GetAttributeRepeatedCapabilityIds (ViSession instrSession,
                                                            ViUInt32 attributeID,
                                                            ViInt32 bufferSize,
                                                            ViChar _VI_FAR repeatedCapabilityIds[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      repCapNameId[RS_MAX_MESSAGE_BUF_SIZE] = "";

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 3, "Buffer Size");

    viCheckErr (Rs_GetAttributeRepCapNameId (instrSession, attributeID, repCapNameId));

    strncpy (repeatedCapabilityIds, repCapNameId, RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIds[(bufferSize > RS_MAX_MESSAGE_BUF_SIZE) ?
        RS_MAX_MESSAGE_BUF_SIZE : bufferSize] = '\0';

Error:
    return error;
}

/// HIFN This function returns the repeated capability name(s) that
/// HIFN belongs to the attribute and single repeated capability id you
/// HIFN may specify. More then one name is listed in comma separated
/// HIFN string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR repeatedCapabilityId/If the attribute applies to a repeated capability such as a trace, this parameter defines the Id of the repeated capability on which to query repeated capability id name(s).
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Repeated Capability Id Name(s) parameter.
/// HIPAR repeatedCapabilityIdNames/The buffer in which the function returns the repeated capability id name(s) belonging to the attribute.
ViStatus _VI_FUNC rsavpha_GetAttributeRepeatedCapabilityIdNames (ViSession instrSession,
                                                                ViUInt32 attributeID,
                                                                ViString repeatedCapabilityId,
                                                                ViInt32 bufferSize,
                                                                ViChar _VI_FAR repeatedCapabilityIdNames[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      repCapName[RS_MAX_MESSAGE_BUF_SIZE] = "";

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 4, "Buffer Size");

    viCheckErr (Rs_GetAttributeRepCapName (instrSession,
                                           attributeID,
                                           repeatedCapabilityId,
                                           repCapName));

    strncpy (repeatedCapabilityIdNames, repCapName, RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIdNames[(bufferSize >= RS_MAX_MESSAGE_BUF_SIZE) ?
        RS_MAX_MESSAGE_BUF_SIZE - 1 : bufferSize - 1] = '\0';

Error:
    return error;
}

/// HIFN  This function configures the HDMI analyzer input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR input/This control selects the input signal for the measurement.
/// HIPAR HPDSignal/Enables or disables the hotplug detection (HPD) signal.
/// HIPAR device/This control sets the input device.
ViStatus _VI_FUNC rsavpha_ConfigureHDMIAnalyzerInput (ViSession instrSession,
                                                    ViInt32 board, ViString input,
                                                    ViBoolean HPDSignal,
                                                    ViInt32 device)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpha_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpha_SetAttributeViString(instrSession, rep_cap,
        RSAVPHA_ATTR_HDMI_ANALYZER_INPUT, input), 3, "Input");
	
	viCheckParm(rsavpha_SetAttributeViBoolean(instrSession, rep_cap,
        RSAVPHA_ATTR_HDMI_ANALYZER_HPD_SIGNAL_STATE, HPDSignal), 4, "HPD Signal");
	
	viCheckParm(rsavpha_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPHA_ATTR_HDMI_ANALYZER_INPUT_DEVICE, device), 5, "Device");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN This function returns the HDMI analyzer input parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR bufferSize/This control sets the buffer size.
/// HIPAR pixelClockLocked/This control returns if the R&S VTE has detected a valid HDMI input clock.
/// HIPAR HDCP/This control returns if an HDCP encryption is currently active.
/// HIPAR pixelClock/This control returns the pixel clock.
/// HIPAR videoCode/This control returns the video code derived from the AVI (auxiliary video information) InfoFrame.
/// HIPAR audioCoding/This control returns the audio coding type derived from audio InfoFrame.
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerInput (ViSession instrSession,
                                                ViInt32 board,
                                                ViInt32 bufferSize,
                                                ViInt32 *pixelClockLocked,
                                                ViInt32 *HDCP,
                                                ViReal64 *pixelClock,
                                                ViChar videoCode[],
                                                ViChar audioCoding[])
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
	ViChar      rd_buffer[RSAVPHA_IO_BUFFER_SIZE] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpha_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpha_GetAttributeViInt32 (instrSession, rep_cap, 
                                                RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INPUT_PIXEL_CLOCK_LOCKED, 
												pixelClockLocked),4, "Pixel Clock Locked");
	
	viCheckParm(rsavpha_GetAttributeViInt32 (instrSession, rep_cap, 
                                                RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_HDCP_ACTIVE,
                                                HDCP),5, "HDCP");
	
	viCheckParm(rsavpha_GetAttributeViReal64 (instrSession, rep_cap, 
                                                RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INPUT_PIXEL_CLOCK,
                                                pixelClock),6, "Pixel Clock");
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                             RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INPUT_VIDEO_CODE, bufferSize, rd_buffer), 7, "Video Code");
	
	rsavpha_ParseStatusString (rd_buffer, videoCode);
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                             RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INPUT_AUDIO_CODING, bufferSize, rd_buffer), 8, "Audio Coding");

	rsavpha_ParseStatusString (rd_buffer, audioCoding);
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Queries a list of available HDMI modules.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR bufferSize/This control sets the buffer size.
/// HIPAR catalog/Returns the catalog of the HDMI analyzer.
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerCatalog (ViSession instrSession,
                                                  ViInt32 board,
                                                  ViInt32 bufferSize,
                                                  ViChar catalog[])
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpha_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_CATALOG, bufferSize,
                                                catalog),4, "Catalog");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN This function returns the HDMI analyzer video parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR bufferSize/This control sets the buffer size.
/// HIPAR videoCode/Returns the video code of the Video Measurement.
/// HIPAR pixelClock/Returns the measured pixel clock of the HDMI input signal.
/// HIPAR videoFormat/Returns the video format.
/// HIPAR verticalFrequency/Returns measured vertical frequency of the HDMI video content.
/// HIPAR horizontalFrequency/Returns measured horizontal frequency of the HDMI video content.
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerVideo (ViSession instrSession,
                                                ViInt32 board,
                                                ViInt32 bufferSize,
                                                ViChar videoCode[],
                                                ViReal64 *pixelClock,
                                                ViInt32 *videoFormat,
                                                ViReal64 *verticalFrequency,
                                                ViReal64 *horizontalFrequency)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
	ViChar      rd_buffer[RSAVPHA_IO_BUFFER_SIZE] = "";
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpha_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                             RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_VIDEO_CODE, bufferSize, rd_buffer), 4, "Video Code");
	
	rsavpha_ParseStatusString (rd_buffer, videoCode);
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_VIDEO_PIXEL_CLOCK, bufferSize,
                                                rd_buffer),5, "Pixel Clock");
	
	rsavpha_ParseStatusReal (rd_buffer, pixelClock);
	
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_VIDEO_FORMAT, bufferSize,
                                                rd_buffer),6, "Video Format");
	
	rsavpha_ParseStatusEnum (rd_buffer, videoFormat, HDMIVideoFormatArr);
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_VIDEO_VERTICAL_FREQUENCY, bufferSize,
                                                rd_buffer),7, "Vertical Frequency");

	rsavpha_ParseStatusReal (rd_buffer, verticalFrequency);
	
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_VIDEO_HORIZONTAL_FREQUENCY, bufferSize, 
                                                rd_buffer),8, "Horizontal Frequency");
	
	rsavpha_ParseStatusReal (rd_buffer, horizontalFrequency);

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;	
}

/// HIFN This function returns the HDMI analyzer video pixel parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR videoParameter/This control selects the direction.
/// HIPAR parameter/Selects the parameter to be queried.
/// HIPAR pixelValue/Returns pixel parameter.
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerVideoPixels (ViSession instrSession,
                                                      ViInt32 board,
                                                      ViInt32 videoParameter,
                                                      ViInt32 parameter,
                                                      ViInt32 *pixelValue)
{
 	ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
	ViChar      rd_buffer[RSAVPHA_IO_BUFFER_SIZE] = "";
	ViAttr      attr[2][6] = {{RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_VIDEO_HORIZONTAL_PIXEL_TOTAL,
                             RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_VIDEO_HORIZONTAL_PIXEL_ACTIVE,
                             RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_VIDEO_HORIZONTAL_PIXEL_FRONT_PORCH,
                             RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_VIDEO_HORIZONTAL_PIXEL_SYNCHRONIZATION,
                             RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_VIDEO_HORIZONTAL_PIXEL_BACK_PORCH,
                             RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_VIDEO_HORIZONTAL_PIXEL_SYNCHRONIZATION_POLARITY
                            },
							{RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_VIDEO_VERTICAL_LINE_TOTAL,
							 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_VIDEO_VERTICAL_LINE_ACTIVE,
							 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_VIDEO_VERTICAL_LINE_FRONT_PORCH,
							 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_VIDEO_VERTICAL_LINE_SYNCHRONIZATION,
							 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_VIDEO_VERTICAL_LINE_BACK_PORCH,
							 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_VIDEO_VERTICAL_LINE_SYNCHRONIZATION_POLARITY}};
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsavpha_invalidViInt32Range (parameter, RSAVPHA_VAL_HDMI_PIXEL_TOTAL, RSAVPHA_VAL_HDMI_PIXEL_SYNC_POLARITY), 4, "Parameter");
	
    checkErr (rsavpha_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
	                                      attr[videoParameter][parameter], RSAVPHA_IO_BUFFER_SIZE,
	                                      rd_buffer), 5, "Pixel Value");

	if (parameter == RSAVPHA_VAL_HDMI_PIXEL_SYNC_POLARITY)
		rsavpha_ParseStatusEnum (rd_buffer, pixelValue, SyncPolarityArr);
	else
		rsavpha_ParseStatusInt (rd_buffer, pixelValue);
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;	
}

/// HIFN This function returns the HDMI analyzer audio parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR bufferSize/This control sets the buffer size.
/// HIPAR n/Returns the N value derived from the audio clock regeneration packet.
/// HIPAR CTS/Returns the CTS (Cycle Time Stamp) value derived from the audio clock regeneration packet.
/// HIPAR audioCoding/Returns the audio coding type derived from the audio InfoFrame.
/// HIPAR layout/Returns the layout value derived from the audio sample packet.
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerAudio (ViSession instrSession,
                                                ViInt32 board,
                                                ViInt32 bufferSize, ViInt32 *n,
                                                ViInt32 *CTS,
                                                ViChar audioCoding[],
                                                ViChar layout[])
{   
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
	ViChar      rd_buffer[RSAVPHA_IO_BUFFER_SIZE] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpha_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_AUDIO_N, bufferSize,
                                                rd_buffer),4, "N");
	
	rsavpha_ParseStatusInt (rd_buffer, n);
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_AUDIO_CYCLE_TIME_STAMP, bufferSize,
                                                rd_buffer),5, "CTS");
	
	rsavpha_ParseStatusInt (rd_buffer, CTS);
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                             RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_AUDIO_CODING, bufferSize, rd_buffer), 6, "Audio Coding");
	
	rsavpha_ParseStatusString (rd_buffer, audioCoding);

	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                             RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_AUDIO_LAYOUT, bufferSize, rd_buffer), 7, "Layout");
	
	rsavpha_ParseStatusString (rd_buffer, layout);

	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;	
}

/// HIFN This function returns the HDMI analyzer audio channel status bit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR parameter/This control selects the parameter to be queried.
/// HIPAR bufferSize/This control sets the buffer size.
/// HIPAR result/This control returns the result.
/// HIPAR channelStatusBit/This control returns audio parameter in integer or binary format.
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerAudioChannelStatusBits (ViSession instrSession,
                                                                 ViInt32 board,
                                                                 ViInt32 parameter,
                                                                 ViInt32 bufferSize,
                                                                 ViChar result[],
                                                                 ViChar channelStatusBit[])
{
    ViStatus    error  = VI_SUCCESS;
    ViChar     *p2buf=VI_NULL; 
    ViChar      wr_buffer[RSAVPHA_IO_BUFFER_SIZE] = "";
	ViChar      rd_buffer[RSAVPHA_IO_BUFFER_SIZE] = "";
    
		
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsavpha_invalidViInt32Range (board, 1, 8), 2, "Board");
	viCheckParm (rsavpha_invalidViInt32Range (parameter, RSAVPHA_VAL_HDMI_AUDIO_APPLICATION, RSAVPHA_VAL_HDMI_AUDIO_LENGTH), 3, "Parameter");
    if (result == NULL)
	{
		viCheckParm( RS_ERROR_INVALID_PARAMETER, 5, "Result");
	}
    if (channelStatusBit == NULL)
	{
		viCheckParm( RS_ERROR_INVALID_PARAMETER, 6, "Channel Status Bit");
	}

    sprintf (wr_buffer, "READ%ld:HDMI:AUD:%s?" , board, HDMIAudioParametersArr[parameter]);
	
    viCheckErr (rsavpha_QueryViString (instrSession, wr_buffer, bufferSize, rd_buffer));
	
	p2buf = strtok (rd_buffer, ",");

	if (strcmp (p2buf , "OK") == 0)
	{
		p2buf = strtok (NULL, ",");
		strcpy (result, p2buf);
		p2buf = strtok (NULL, ",");
		strcpy (channelStatusBit, p2buf);
	}
	else
	{
		strcpy (result, p2buf);
	}
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;    
}

/// HIFN This function returns the HDMI analyzer HDCP parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR bufferSize/This control sets the buffer size.
/// HIPAR authentication/
/// HIPAR HDCP/Returns if an HDCP content protection is currently active.
/// HIPAR AKSV/Returns the source public key to contribute in R0 calcualtion.
/// HIPAR BKSV/Returns the sink public key to contribute in R0 calcualtion.
/// HIPAR ri/Returns the current link synchronization verification values.
/// HIPAR an/Returns the session pseudo random values sent to HDCP receiver/repeater by transmitter.
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerHDCP (ViSession instrSession,
                                               ViInt32 board, ViInt32 bufferSize,
                                               ViChar authentication[],
                                               ViInt32 *HDCP, ViInt64 *AKSV,
                                               ViInt64 *BKSV, ViInt32 *ri,
                                               ViInt32 an[])
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";
    ViChar *pbuffer;
    ViInt32  i = 0;
	ViChar      rd_buffer[RSAVPHA_IO_BUFFER_SIZE] = ""; 
	ViChar      wr_buffer[RSAVPHA_IO_BUFFER_SIZE] = "";
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsavpha_invalidViInt32Range (board, 1, 8), 2, "Board");
    
    checkErr (rsavpha_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                             RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_HDCP_AUTHENTICATION, bufferSize, rd_buffer), 4, "Authentication");
	
	rsavpha_ParseStatusString (rd_buffer, authentication);
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_HDCP_STATE, bufferSize, 
                                                rd_buffer),5, "HDCP");
	rsavpha_ParseStatusEnum (rd_buffer, HDCP, HDCPStateArr);
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_HDCP_SOURCE_PUBLIC_KEY, bufferSize,
                                                rd_buffer),6, "AKSV");
	rsavpha_ParseStatusInt64 (rd_buffer, AKSV);

	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_HDCP_SINK_PUBLIC_KEY, bufferSize,
                                                rd_buffer),7, "BKSV");
	rsavpha_ParseStatusInt64 (rd_buffer, BKSV);
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_HDCP_LINK_SYNCHRONIZATION_VALUE, bufferSize,
                                                rd_buffer),8, "Ri");
	rsavpha_ParseStatusInt (rd_buffer, ri);
	
	sprintf (wr_buffer, "READ%ld:HDMI:HDCP:AN?", board);

	viCheckErr (rsavpha_QueryViString (instrSession, wr_buffer, bufferSize, rd_buffer));
	
	pbuffer = strtok(rd_buffer, ",");
    
	if (strcmp (pbuffer , "OK") == 0)
	{
		pbuffer = strtok(NULL, ",");
		
		while (pbuffer && i < bufferSize) 
	    {
	        an[i++] = rsavpha_atol (pbuffer);
	        pbuffer = strtok(NULL, ",");
	    }
	}
	else
	{
		an[i] = RSAVPHA_NAN_VI_INT32;
	}
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
	return error;
}

/// HIFN  This function returns the HDMI analyzer auxiliary video information (AVI) parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR board/This control selects the board.
/// HIPAR version/Returns the version of the AVI Info Frame Measurement.
/// HIPAR length/Returns the length of the AVI Info Frame Measurement.
/// HIPAR checksum/Returns the checksum of the AVI Info Frame Measurement.
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerInfoFrameAVI (ViSession instrSession,
                                                       ViInt32 board,
                                                       ViInt32 *version,
                                                       ViInt32 *length,
                                                       ViInt32 *checksum)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";
	ViChar      rd_buffer[RSAVPHA_IO_BUFFER_SIZE] = ""; 
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpha_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_VERSION, 1024,
                                                rd_buffer),3, "Version");
	
	rsavpha_ParseStatusInt (rd_buffer, version);
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_LENGTH, 1024,
                                                rd_buffer),4, "Length");

	rsavpha_ParseStatusInt (rd_buffer, length);
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_CHECKSUM, 1024,
                                                rd_buffer),5, "Checksum");
	
	rsavpha_ParseStatusInt (rd_buffer, checksum);

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function returns the HDMI analyzer Info Frame parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR board/This control selects the board.
/// HIPAR parameter/This control selects the info frame parameter to be queried.
/// HIPAR bufferSize/This control sets the buffer size.
/// HIPAR result/This control returns the info frame parameter result.
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerInfoFrameAVISpecific (ViSession instrSession,
                                                               ViInt32 board,
                                                               ViInt32 parameter,
                                                               ViInt32 bufferSize,
                                                               ViChar result[])
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";
	ViChar      rd_buffer[RSAVPHA_IO_BUFFER_SIZE] = ""; 
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsavpha_invalidViInt32Range (parameter, RSAVPHA_VAL_HDMI_INFO_FRAME_RGB, RSAVPHA_VAL_HDMI_INFO_FRAME_PREP), 3, "Parameter");
    
    checkErr (rsavpha_hardwareRepCap (instrSession, board, rep_cap));
	
	switch (parameter)
    {
        case RSAVPHA_VAL_HDMI_INFO_FRAME_RGB:
	
		viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_COLOR_SPACE, bufferSize, 
												 rd_buffer), 5, "RGB or YCbCr");
		
		rsavpha_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPHA_VAL_HDMI_INFO_FRAME_AINFO:
	
		viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_ACTIVE_FORMAT, bufferSize, 
												 rd_buffer), 5, "Active Format Information Present");
		
		rsavpha_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPHA_VAL_HDMI_INFO_FRAME_BINFO:
	
		viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_BAR_INFORMATION, bufferSize, 
												 rd_buffer), 5, "Bar Information");
		
		rsavpha_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPHA_VAL_HDMI_INFO_FRAME_SINFO:
	
		viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_SCAN_INFORMATION, bufferSize, 
												 rd_buffer), 5, "Scan Information");
		
		rsavpha_ParseStatusString (rd_buffer, result);
		
		break;	
		
		case RSAVPHA_VAL_HDMI_INFO_FRAME_COL:
	
		viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_COLORIMETRY, bufferSize, 
												 rd_buffer), 5, "Colorimetry");
		
		rsavpha_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPHA_VAL_HDMI_INFO_FRAME_ARAT:
	
		viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_PICTURE_ASPECT_RATIO, bufferSize, 
												 rd_buffer), 5, "Picture Aspect Ratio");
		
		rsavpha_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPHA_VAL_HDMI_INFO_FRAME_AFAR:
	
		viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_ACTIVE_FORMAT_ASPECT_RATIO, bufferSize, 
												 rd_buffer), 5, "Active Format Aspect Ratio");
		
		rsavpha_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPHA_VAL_HDMI_INFO_FRAME_ITC:
	
		viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_IT_CONTENT, bufferSize, 
												 rd_buffer), 5, "IT Content");
		
		rsavpha_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPHA_VAL_HDMI_INFO_FRAME_ECOL:
	
		viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_EXTENDED_COLORIMETRY, bufferSize, 
												 rd_buffer), 5, "Extended Colorimetry");
		
		rsavpha_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPHA_VAL_HDMI_INFO_FRAME_RGBQ:
	
		viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_RGB_QUANTIZATION_RANGE, bufferSize, 
												 rd_buffer), 5, "RGB Quantization Range");
		
		rsavpha_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPHA_VAL_HDMI_INFO_FRAME_NUPS:
	
		viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_NONUNIFORM_PICTURE_SCALING, bufferSize, 
												 rd_buffer), 5, "Nonuniform Picture Scaling");
		
		rsavpha_ParseStatusString (rd_buffer, result); 
		
		break;
		
		case RSAVPHA_VAL_HDMI_INFO_FRAME_VIDC:
	
		viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_VIDEO_IDENTIFICATION_CODE, bufferSize, 
												 rd_buffer), 5, "Video Identification Code");
		
		rsavpha_ParseStatusString (rd_buffer, result); 
		
		break;
		
		case RSAVPHA_VAL_HDMI_INFO_FRAME_YCCQ:
	
		viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_YCC_QUANTIZATION, bufferSize, 
												 rd_buffer), 5, "YCC Quantization");
		
		rsavpha_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPHA_VAL_HDMI_INFO_FRAME_CTYP:
	
		viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_CONTENT_TYPE, bufferSize, 
												 rd_buffer), 5, "Content Type");
		
		rsavpha_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPHA_VAL_HDMI_INFO_FRAME_PREP:
	
		viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_PIXEL_REPETITION, bufferSize, 
												 rd_buffer), 5, "Pixel Repetition");
		
		rsavpha_ParseStatusString (rd_buffer, result);
		
		break;
		
		default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER,3,"Parameter");
            break;
			
    }			
	
	checkErr (rsavpha_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the HDMI analyzer Info Frame Bar parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR board/This control selects the board.
/// HIPAR lineNumber_ofTopBarEnd/Returns the Line Number of Top Bar End.
/// HIPAR lineNumber_ofBottomBarStart/Returns the Line Number of Bottom Bar Start.
/// HIPAR pixelNumber_ofLeftBarEnd/Returns the Pixel Number of Left Bar End.
/// HIPAR pixelNumber_ofRightBarStart/Returns the pixel number of right bar start.
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerInfoFrameAVIBar (ViSession instrSession,
                                                          ViInt32 board,
                                                          ViInt32 *lineNumber_ofTopBarEnd,
                                                          ViInt32 *lineNumber_ofBottomBarStart,
                                                          ViInt32 *pixelNumber_ofLeftBarEnd,
                                                          ViInt32 *pixelNumber_ofRightBarStart)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";
	ViChar      rd_buffer[RSAVPHA_IO_BUFFER_SIZE] = ""; 
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpha_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_TOP_BAR_END, 1024,
                                                rd_buffer),3, "Line Number of Top Bar End");
	
	rsavpha_ParseStatusInt (rd_buffer, lineNumber_ofTopBarEnd);
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_BOTTOM_BAR_START,  1024,
                                                rd_buffer),4, "Line Number of Bottom Bar Start");
	
	rsavpha_ParseStatusInt (rd_buffer, lineNumber_ofBottomBarStart);

	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_LEFT_BAR_END, 1024,
                                                rd_buffer),5, "Pixel Number of Left Bar End");
	
	rsavpha_ParseStatusInt (rd_buffer, pixelNumber_ofLeftBarEnd);

	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_RIGHT_BAR_START, 1024,
                                                rd_buffer),6, "Pixel Number of Right Bar Start");
	
	rsavpha_ParseStatusInt (rd_buffer, pixelNumber_ofRightBarStart);


Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function returns the HDMI analyzer Info Frame header and packet bytes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR board/This control selects the board.
/// HIPAR arraySize/Sets the array size for Packet Bytes control.
/// HIPAR headerBytes/Returns header bytes 0 to 2. 
/// HIPAR packetBytes/Returns packet bytes 0 to 13.
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerInfoFrameAVIHeaderPacketData (ViSession instrSession,
                                                                       ViInt32 board,
                                                                       ViInt32 arraySize,
                                                                       ViInt32 headerBytes[],
                                                                       ViInt32 packetBytes[])
{
    
	ViStatus error = VI_SUCCESS;
    ViChar   *p2buf  = NULL;
	ViInt32	 i = 0;
	ViInt32  x = 0;
	ViChar      rd_buffer[RSAVPHA_IO_BUFFER_SIZE] = ""; 
	ViChar      wr_buffer[RSAVPHA_IO_BUFFER_SIZE] = ""; 

    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	viCheckParm (rsavpha_invalidViInt32Range (board, 1, 8), 3, "Board");
    
	sprintf (wr_buffer, "READ%ld:HDMI:INFO:AVI:PDAT?" , board);
	
    viCheckErr (rsavpha_QueryViString (instrSession, wr_buffer, RSAVPHA_IO_BUFFER_SIZE, rd_buffer));
	
	p2buf = strtok (rd_buffer, ",");
	
	if (strcmp (p2buf , "OK") != 0)
	{
		headerBytes[0] = RSAVPHA_NAN_VI_INT32;
		packetBytes[0] = RSAVPHA_NAN_VI_INT32;
	}
	else
	{
		p2buf = strtok (NULL, ",");
	    
		for (x = 0; x < 3; x++)
	    {
		    headerBytes[x] = atol (p2buf);
    	    p2buf = strtok (NULL, ",");
		}
		arraySize -= x;
		
		while (p2buf && (i<arraySize))
		{
	        packetBytes[i++] = atol (p2buf);
	        p2buf = strtok (NULL, ",");
		}
	}
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the HDMI analyzer Info Frame parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR board/This control selects the board.
/// HIPAR version/Returns the version of the audio info frame Measurement.
/// HIPAR length/Returns the length of the audio info frame Measurement.
/// HIPAR checksum/Returns the checksum of the audio info frame Measurement.
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerInfoFrameAudio (ViSession instrSession,
                                                         ViInt32 board,
                                                         ViInt32 *version,
                                                         ViInt32 *length,
                                                         ViInt32 *checksum)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";
	ViChar      rd_buffer[RSAVPHA_IO_BUFFER_SIZE] = ""; 
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpha_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AUDIO_VERSION, 1024,
                                                rd_buffer),3, "Version");
	
	rsavpha_ParseStatusInt (rd_buffer, version);
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AUDIO_LENGTH, 1024,
                                                rd_buffer),4, "Length");
	
	rsavpha_ParseStatusInt (rd_buffer, length);

	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AUDIO_CHECKSUM, 1024,
                                                rd_buffer),5, "Checksum");

	rsavpha_ParseStatusInt (rd_buffer, checksum);
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function returns the HDMI analyzer Info Frame parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR board/This control selects the board.
/// HIPAR parameter/Selects the info frame parameter to be queried.
/// HIPAR bufferSize/Sets the buffer size.
/// HIPAR result/Returns the audio info frame parameter result.
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerInfoFrameAudioSpecific (ViSession instrSession,
                                                                 ViInt32 board,
                                                                 ViInt32 parameter,
                                                                 ViInt32 bufferSize,
                                                                 ViChar result[])
{
   
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";
	ViChar      rd_buffer[RSAVPHA_IO_BUFFER_SIZE] = ""; 
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsavpha_invalidViInt32Range (parameter, RSAVPHA_VAL_HDMI_INFO_FRAME_CODING_TYPE, RSAVPHA_VAL_HDMI_INFO_FRAME_LFE), 3, "Parameter");
    
    checkErr (rsavpha_hardwareRepCap (instrSession, board, rep_cap));
	
	switch (parameter)
    {
        case RSAVPHA_VAL_HDMI_INFO_FRAME_CODING_TYPE:
	
		viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AUDIO_CODING_TYPE, bufferSize, 
												 rd_buffer), 5, "Coding Type");
		
		rsavpha_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPHA_VAL_HDMI_INFO_FRAME_CHAN_COUNT:
	
		viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AUDIO_CHANNEL_COUNT, bufferSize, 
												 rd_buffer), 5, "Channel Count");
		
		rsavpha_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPHA_VAL_HDMI_INFO_FRAME_SAMP_FREQ:
	
		viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AUDIO_SAMPLING_FREQUENCY, bufferSize, 
												 rd_buffer), 5, "Sampling Frequency");
		
		rsavpha_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPHA_VAL_HDMI_INFO_FRAME_SAMP_SIZE:
	
		viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AUDIO_SAMPLING_SIZE, bufferSize, 
												 rd_buffer), 5, "Sampling Size");
		
		rsavpha_ParseStatusString (rd_buffer, result);
		
		break;	
		
		case RSAVPHA_VAL_HDMI_INFO_FRAME_CODE_EXT:
	
		viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AUDIO_CODE_EXTENSION, bufferSize, 
												 rd_buffer), 5, "Code Extension");
		
		rsavpha_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPHA_VAL_HDMI_INFO_FRAME_CHAN_ALLOC:
	
		viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AUDIO_CHANNEL_ALLOCATION, bufferSize, 
												 rd_buffer), 5, "Channel Allocation");
		
		rsavpha_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPHA_VAL_HDMI_INFO_FRAME_DMIX_INHIB:
	
		viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AUDIO_DOWNMIX_INHIBIT, bufferSize, 
												 rd_buffer), 5, "Downmix Inhibit");
		
		rsavpha_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPHA_VAL_HDMI_INFO_FRAME_LEVEL_SHIFT:
	
		viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AUDIO_LEVEL_SHIFT_VALUE, bufferSize, 
												 rd_buffer), 5, "Level Shift Value");
		
		rsavpha_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPHA_VAL_HDMI_INFO_FRAME_LFE:
	
		viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AUDIO_LFE_PLAYBACK_LEVEL, bufferSize, 
												 rd_buffer), 5, "LFE Playback Level");
		
		rsavpha_ParseStatusString (rd_buffer, result);
		
		break;
		
		default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER,3,"Parameter");
            break;
			
    }			
	
	checkErr (rsavpha_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the HDMI analyzer Info Frame header and packet bytes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR board/This control selects the board.
/// HIPAR arraySize/Sets the array size for Packet Bytes control.
/// HIPAR headerBytes/Returns header bytes 0 to 2.
/// HIPAR packetBytes/Returns packet bytes 0 to 10.
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerInfoFrameAudioHeaderPacketData (ViSession instrSession,
                                                                         ViInt32 board,
                                                                         ViInt32 arraySize,
                                                                         ViInt32 headerBytes[],
                                                                         ViInt32 packetBytes[])
{
 
	ViStatus error = VI_SUCCESS;
    ViChar   *p2buf  = NULL;
	ViInt32	 i = 0;
	ViInt32  x = 0;
	ViChar      wr_buffer[RSAVPHA_IO_BUFFER_SIZE] = ""; 
	ViChar      rd_buffer[RSAVPHA_IO_BUFFER_SIZE] = ""; 
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));    
   
	viCheckParm (rsavpha_invalidViInt32Range (board, 1,8), 2, "Board");
    
	sprintf (wr_buffer, "READ%ld:HDMI:INFO:AUD:PDAT?", board);
	
    viCheckErr (rsavpha_QueryViString (instrSession, wr_buffer, RSAVPHA_IO_BUFFER_SIZE, rd_buffer));
	
	p2buf = strtok (rd_buffer, ",");
	
	if (strcmp (p2buf , "OK") != 0)
	{
		headerBytes[0] = RSAVPHA_NAN_VI_INT32;
		packetBytes[0] = RSAVPHA_NAN_VI_INT32;
	}
	else
	{
		p2buf = strtok (NULL, ",");
	    
		for (x = 0; x < 3; x++)
	    {
		    headerBytes[x] = atol (p2buf);
    	    p2buf = strtok (NULL, ",");
		}
		arraySize -= x;
		
		while (p2buf && (i<arraySize))
		{
	        packetBytes[i++] = atol (p2buf);
	        p2buf = strtok (NULL, ",");
		}
	}
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the HDMI analyzer Info Frame parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR board/This control selects the board.
/// HIPAR version/Returns the version.
/// HIPAR length/Returns the length.
/// HIPAR checksum/Returns the checksum.
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerInfoFrameSPD (ViSession instrSession,
                                                       ViInt32 board,
                                                       ViInt32 *version,
                                                       ViInt32 *length,
                                                       ViInt32 *checksum)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";
	ViChar      rd_buffer[RSAVPHA_IO_BUFFER_SIZE] = ""; 
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpha_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_SPD_VERSION, 1024,
                                                rd_buffer),3, "Version");
	
	rsavpha_ParseStatusInt (rd_buffer, version);
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_SPD_LENGTH, 1024,
                                                rd_buffer),4, "Length");
	
	rsavpha_ParseStatusInt (rd_buffer, length);

	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_SPD_CHECKSUM, 1024,
                                                rd_buffer),5, "Checksum");
	rsavpha_ParseStatusInt (rd_buffer, checksum);

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function returns the HDMI analyzer Info Frame parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR board/This control selects the board.
/// HIPAR bufferSize/sets the buffer size.
/// HIPAR parameter/selects the info frame parameter to be queried.
/// HIPAR result/Returns the info frame parameter result.
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerInfoFrameSPDSpecific (ViSession instrSession,
                                                               ViInt32 board,
															   ViInt32 parameter,
                                                               ViInt32 bufferSize,
                                                               ViChar result[])
{   
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";
	ViChar      rd_buffer[RSAVPHA_IO_BUFFER_SIZE] = ""; 
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsavpha_invalidViInt32Range (parameter, RSAVPHA_VAL_HDMI_INFO_FRAME_VENDOR, RSAVPHA_VAL_HDMI_INFO_FRAME_PRODUCT), 3, "Parameter");
    	
    checkErr (rsavpha_hardwareRepCap (instrSession, board, rep_cap));
	
	switch (parameter)
    {
        case RSAVPHA_VAL_HDMI_INFO_FRAME_VENDOR:
	
		viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_SPD_VENDOR, bufferSize, 
												 rd_buffer), 5, "Vendor Name");
		
		rsavpha_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPHA_VAL_HDMI_INFO_FRAME_PRODUCT:
	
		viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_SPD_PRODUCT_DESCRIPTION, bufferSize, 
												 rd_buffer), 5, "Product Description");
		
		rsavpha_ParseStatusString (rd_buffer, result);
		
		
		break;
	
	 default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER,3,"Parameter");
            break;
			
    }			
	
	checkErr (rsavpha_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the HDMI analyzer Info Frame header and packet bytes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR board/This control selects the board.
/// HIPAR arraySize/sets the array size for Packet Bytes control.
/// HIPAR headerBytes/returns header bytes 0 to 2.
/// HIPAR packetBytes/returns packet bytes 0 to 25.
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerInfoFrameSPDHeaderPacketData (ViSession instrSession,
                                                                       ViInt32 board,
                                                                       ViInt32 arraySize,
                                                                       ViInt32 headerBytes[],
                                                                       ViInt32 packetBytes[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   *p2buf  = NULL;
	ViInt32	 i = 0;
	ViInt32  x = 0;
	ViChar      wr_buffer[RSAVPHA_IO_BUFFER_SIZE] = ""; 
	ViChar      rd_buffer[RSAVPHA_IO_BUFFER_SIZE] = ""; 

    checkErr (Rs_LockSession (instrSession, VI_NULL));    
	
	viCheckParm (rsavpha_invalidViInt32Range (board, 1, 8), 2, "Board");
    
	sprintf (wr_buffer, "READ%ld:HDMI:INFO:SPD:PDAT?" , board);
	
    viCheckErr (rsavpha_QueryViString (instrSession, wr_buffer, RSAVPHA_IO_BUFFER_SIZE, rd_buffer));
	
	p2buf = strtok (rd_buffer, ",");
	
	if (strcmp (p2buf , "OK") != 0)
	{
		headerBytes[0] = RSAVPHA_NAN_VI_INT32;
		packetBytes[0] = RSAVPHA_NAN_VI_INT32;
	}
	else
	{
		p2buf = strtok (NULL, ",");
	    
		for (x = 0; x < 3; x++)
	    {
		    headerBytes[x] = atol (p2buf);
    	    p2buf = strtok (NULL, ",");
		}
		arraySize -= x;
		
		while (p2buf && (i<arraySize))
		{
	        packetBytes[i++] = atol (p2buf);
	        p2buf = strtok (NULL, ",");
		}
	}
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the HDMI analyzer Info Frame header and packet bytes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR board/This control selects the board.
/// HIPAR version/Returns the version.
/// HIPAR length/Returns the length.
/// HIPAR checksum/Returns the checksum.
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerInfoFrameMPEGSource (ViSession instrSession,
                                                              ViInt32 board,
                                                              ViInt32 *version,
                                                              ViInt32 *length,
                                                              ViInt32 *checksum)
{
 
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";
	ViChar      rd_buffer[RSAVPHA_IO_BUFFER_SIZE] = ""; 

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpha_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_MPEG_VERSION, 1024,
                                                rd_buffer),3, "Version");
	rsavpha_ParseStatusInt (rd_buffer, version);
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_MPEG_LENGTH, 1024,
                                                rd_buffer),4, "Length");
	
	rsavpha_ParseStatusInt (rd_buffer, length);

	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_MPEG_CHECKSUM, 1024,
                                                rd_buffer),5, "Checksum");
	
	rsavpha_ParseStatusInt (rd_buffer, checksum);
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function returns the HDMI analyzer Info Frame parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR board/This control selects the board.
/// HIPAR bufferSize/This control sets the buffer size.
/// HIPAR MPEGBitRate/Returns the MPEG Bit Rate.
/// HIPAR fieldRepeat/Returns the Field Repeat.
/// HIPAR MPEGFrame/Returns the MPEG Frame.
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerInfoFrameMPEGSourceSpecific (ViSession instrSession,
                                                                      ViInt32 board,
                                                                      ViInt32 bufferSize,
                                                                      ViInt32 *MPEGBitRate,
                                                                      ViChar fieldRepeat[],
                                                                      ViChar MPEGFrame[])
{
 	ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";
	ViChar      rd_buffer[RSAVPHA_IO_BUFFER_SIZE] = ""; 

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpha_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_MPEG_BIT_RATE, bufferSize,
                                                rd_buffer),4, "MPEG Bit Rate");
	
	rsavpha_ParseStatusInt (rd_buffer, MPEGBitRate);
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_MPEG_FIELD_REPEAT, bufferSize, 
												 rd_buffer), 5, "Field Repeat");
	
	rsavpha_ParseStatusString (rd_buffer, fieldRepeat);
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_MPEG_FRAME, bufferSize, 
												 rd_buffer), 5, "MPEG Frame");
	
	rsavpha_ParseStatusString (rd_buffer, MPEGFrame);
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function returns the HB0PB10 values of the MPEG info frame section.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR board/This control selects the board.
/// HIPAR arraySize/sets the array size for Packet Bytes control.
/// HIPAR headerBytes/returns header bytes 0 to 2.
/// HIPAR packetBytes/returns packet bytes 0 to 10.
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerInfoFrameMPEGSourceHeaderPacketData (ViSession instrSession,
                                                                              ViInt32 board,
                                                                              ViInt32 arraySize,
                                                                              ViInt32 headerBytes[],
                                                                              ViInt32 packetBytes[])
{
	ViStatus error = VI_SUCCESS;
    ViChar  wr_buffer[RSAVPHA_IO_BUFFER_SIZE] = ""; 
	ViChar  rd_buffer[RSAVPHA_IO_BUFFER_SIZE] = ""; 
    ViChar  *p2buf  = NULL;
	ViInt32	 i = 0;
	ViInt32  x = 0;

    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	viCheckParm (rsavpha_invalidViInt32Range (board, 1, 8), 2, "Board");
    
	sprintf (wr_buffer, "READ%ld:HDMI:INFO:MPEG:PDAT?" , board);
	
    viCheckErr (rsavpha_QueryViString (instrSession, wr_buffer, RSAVPHA_IO_BUFFER_SIZE, rd_buffer));
	
	p2buf = strtok (rd_buffer, ",");
	
	if (strcmp (p2buf , "OK") != 0)
	{
		headerBytes[0] = RSAVPHA_NAN_VI_INT32;
		packetBytes[0] = RSAVPHA_NAN_VI_INT32;
	}
	else
	{
		p2buf = strtok (NULL, ",");
	    
		for (x = 0; x < 3; x++)
	    {
		    headerBytes[x] = atol (p2buf);
    	    p2buf = strtok (NULL, ",");
		}
		arraySize -= x;
		
		while (p2buf && (i<arraySize))
		{
	        packetBytes[i++] = atol (p2buf);
	        p2buf = strtok (NULL, ",");
		}
	}
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the HDMI analyzer vendor specific Info Frame parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR board/This control selects the board.
/// HIPAR version/Returns the version.
/// HIPAR length/Returns the length.
/// HIPAR checksum/Returns the checksum.
/// HIPAR IEEERegistrationID/Returns the IEEE registration ID.
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerInfoFrameVendor (ViSession instrSession,
                                                          ViInt32 board,
                                                          ViInt32 *version,
                                                          ViInt32 *length,
                                                          ViInt32 *checksum,
                                                          ViInt32 *IEEERegistrationID)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";
	ViChar  rd_buffer[RSAVPHA_IO_BUFFER_SIZE] = ""; 
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpha_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_VENDOR_VERSION, 1024,
                                                rd_buffer),3, "Version");
	
	rsavpha_ParseStatusInt (rd_buffer, version);
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_VENDOR_LENGTH,  1024,
                                                rd_buffer),4, "Length");
	
	rsavpha_ParseStatusInt (rd_buffer, length);
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_VENDOR_CHECKSUM, 1024,
                                                rd_buffer),5, "checksum");
	
	rsavpha_ParseStatusInt (rd_buffer, checksum);
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_VENDOR_IEEE_ID, 1024,
                                                rd_buffer),6, "IEEE Registration ID");
	
	rsavpha_ParseStatusInt (rd_buffer, IEEERegistrationID);

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Returns the video parameters for the HDMI vendor info frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR board/This control selects the board.
/// HIPAR bufferSize/This control sets the buffer size.
/// HIPAR HDMIVideoFormat/Returns the HDMI video format.
/// HIPAR HDMIPixelEncoding/Returns the HDMI pixel encoding.
/// HIPAR HDMIVIC/Returns the video format identification code (VIC).
/// HIPAR _3DStructure/Returns the HDMI three dimensional structure.
/// HIPAR _3DExtData/Returns the HDMI three dimensional external data.
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerInfoFrameVendorSpecific (ViSession instrSession,
                                                                  ViInt32 board,
                                                                  ViInt32 bufferSize,
                                                                  ViChar HDMIVideoFormat[],
                                                                  ViChar HDMIPixelEncoding[],
                                                                  ViChar HDMIVIC[],
                                                                  ViChar _3DStructure[],
                                                                  ViChar _3DExtData[])
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";
	ViChar      rd_buffer[RSAVPHA_IO_BUFFER_SIZE] = ""; 

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpha_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_VENDOR_VIDEO_FORMAT, bufferSize, 
												 rd_buffer), 4, "HDMI Video Format");
	
	rsavpha_ParseStatusString (rd_buffer, HDMIVideoFormat);
	
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_VENDOR_PIXEL_ENCODING, bufferSize, 
												 rd_buffer), 5, "HDMI Pixel Encoding");
	
	rsavpha_ParseStatusString (rd_buffer, HDMIPixelEncoding);
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_VENDOR_VIDEO_FORMAT_IDENTIFICATION_CODE, bufferSize, 
												 rd_buffer), 6, "HDMI VIC");
	
	rsavpha_ParseStatusString (rd_buffer, HDMIVIC);
	
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_VENDOR_3D_STRUCTURE, bufferSize, 
												 rd_buffer), 7, "3D Structure");
	
	rsavpha_ParseStatusString (rd_buffer, _3DStructure);
	
	
	viCheckParm(rsavpha_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_VENDOR_3D_EXTERNAL_DATA, bufferSize, 
												 rd_buffer), 7, "3D External Data");
	
	rsavpha_ParseStatusString (rd_buffer, _3DExtData);

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Returns the HDMI info frame header packet data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR board/This control selects the board.
/// HIPAR arraySize/This control sets the array size for Packet Bytes control.
/// HIPAR headerBytes/This control returns header bytes 0 to 2.
/// HIPAR packetBytes/This control returns packet bytes 0 to 10.
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerInfoFrameVendorHeaderPacketData (ViSession instrSession,
                                                                          ViInt32 board,
                                                                          ViInt32 arraySize,
                                                                          ViInt32 headerBytes[],
                                                                          ViInt32 packetBytes[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   *p2buf  = NULL;
	ViInt32	 i = 0;
	ViInt32  x = 0;
	ViChar  wr_buffer[RSAVPHA_IO_BUFFER_SIZE] = ""; 
	ViChar  rd_buffer[RSAVPHA_IO_BUFFER_SIZE] = ""; 

    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	viCheckParm (rsavpha_invalidViInt32Range (board, 1, 8), 2, "Board");
    
	sprintf (wr_buffer, "READ%ld:HDMI:INFO:VEND:PDAT?", board);
	
    viCheckErr (rsavpha_QueryViString (instrSession, wr_buffer, RSAVPHA_IO_BUFFER_SIZE, rd_buffer));
	
	p2buf = strtok (rd_buffer, ",");
	
	if (strcmp (p2buf , "OK") != 0)
	{
		headerBytes[0] = RSAVPHA_NAN_VI_INT32;
		packetBytes[0] = RSAVPHA_NAN_VI_INT32;
	}
	else
	{
		p2buf = strtok (NULL, ",");
	    
		for (x = 0; x < 3; x++)
	    {
		    headerBytes[x] = atol (p2buf);
    	    p2buf = strtok (NULL, ",");
		}
		arraySize -= x;
		
		while (p2buf && (i<arraySize))
		{
	        packetBytes[i++] = atol (p2buf);
	        p2buf = strtok (NULL, ",");
		}
	}
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
	return error;
}

/// HIFN  This function returns the HDMI analyzer CTS state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR state/Returns the HDMI analyzer CTS state.
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerCTSState (ViSession instrSession,
                                                   ViInt32 board, ViInt32 *state)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpha_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpha_GetAttributeViInt32(instrSession, rep_cap,
        RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_CTS_STATE, state), 3, "State");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function selects the operation to be performed on the HDMI analyzer CTS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR operation/Selects the operation.
ViStatus _VI_FUNC rsavpha_HDMIAnalyzerCTSOperation (ViSession instrSession,
                                                  ViInt32 board,
                                                  ViInt32 operation)
{
   ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsavpha_invalidViInt32Range (operation, RSAVPHA_VAL_HDMI_STOP, RSAVPHA_VAL_HDMI_START), 3, "Operation");
    
    checkErr (rsavpha_hardwareRepCap (instrSession, board, rep_cap));
	
	switch (operation)
    {
        case RSAVPHA_VAL_HDMI_START:
	
		viCheckErr(rsavpha_SetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_HDMI_ANALYZER_CTS_INIT, NULL));
												 
		
		break;
		
		case RSAVPHA_VAL_HDMI_STOP:
	
		viCheckErr(rsavpha_SetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_HDMI_ANALYZER_CTS_STOP, NULL));
												 
		
		
		break;
	
	 default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER,3,"Operation");
            break;
			
    }			
	
	checkErr (rsavpha_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;  
}

/// HIFN  This function returns the HDMI analyzer CTS measurement progress.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR progress/Returns the HDMI analyzer CTS measurement progress.
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerCTSProgress (ViSession instrSession,
                                                      ViInt32 board,
                                                      ViInt32 *progress)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpha_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpha_GetAttributeViInt32(instrSession, rep_cap,
        RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_CTS_PROGRESS, progress), 3, "Progress");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error; 
}

/// HIFN  This function returns the HDMI analyzer CTS measurement progress.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR testCase/Selects the test case.
/// HIPAR state/Selected test case on or off.
ViStatus _VI_FUNC rsavpha_SelectHDMIAnalyzerCTSTestCase (ViSession instrSession,
                                                       ViInt32 board,
                                                       ViInt32 testCase,
                                                       ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[150] = "";
	
    checkErr (Rs_LockSession (instrSession, NULL));

    viCheckParm (rsavpha_invalidViInt32Range (board, 1, 8), 2, "Board");
    viCheckParm (rsavpha_invalidViInt32Range (testCase, RSAVPHA_VAL_HDMI_TEST_CASE_EDID, RSAVPHA_VAL_HDMI_TEST_CASE_CONC), 3, "Test Case");
    
    sprintf (command, "CONF%ld:HDMI:CTS:SEL %s,%s", board, HDMIAnalyzerTestCaseArr[testCase], SwitchArr[state]);
		
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    
    checkErr( rsavpha_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  This function configures the HDMI analyzer CTS protocol.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR testCase/Selects the protocol test case.
/// HIPAR testSetup/Selects the packet type test setup.
/// HIPAR testLength/Selects the test length.
/// HIPAR numberOfFrames/Sets the number of frames.
ViStatus _VI_FUNC rsavpha_ConfigureHDMIAnalyzerCTSProtocol (ViSession instrSession,
                                                          ViInt32 board,
                                                          ViInt32 testCase,
                                                          ViInt32 testSetup,
                                                          ViInt32 testLength,
                                                          ViInt32 numberOfFrames)
{
    ViStatus    error = VI_SUCCESS;
   
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    viCheckParm (rsavpha_invalidViInt32Range (board, 1, 8), 2, "Board");
    viCheckParm (rsavpha_invalidViInt32Range (testCase, RSAVPHA_VAL_HDMI_TEST_CASE_LCODES, RSAVPHA_VAL_HDMI_TEST_CASE_PACKET), 3, "Test Case");
    viCheckParm (rsavpha_invalidViInt32Range (testSetup, RSAVPHA_VAL_TEST_SETUP_AI0, RSAVPHA_VAL_TEST_SETUP_AI1), 4, "Test Setup");
    viCheckParm (rsavpha_invalidViInt32Range (testLength, RSAVPHA_VAL_HDMI_TEST_LENGTH_2_SECONDS, RSAVPHA_VAL_HDMI_TEST_LENGTH_FRAMES), 5, "Test Length");
    viCheckParm (rsavpha_invalidViInt32Range (numberOfFrames, 1, 120), 6, "Number Of Frames");
    
	switch (testCase)
	{
		case RSAVPHA_VAL_HDMI_TEST_CASE_LCODES:
		case RSAVPHA_VAL_HDMI_TEST_CASE_BASPROT:
		case RSAVPHA_VAL_HDMI_TEST_CASE_CPERIOD:
			  
			viCheckErr (viPrintf (instrSession, "CONF%ld:HDMI:CTS:TEST:MODE %s,%s", board, HDMIAnalyzerTestCaseArr[testCase], TestLengthArr[testLength]));
			
			checkErr(rsavpha_CheckStatus (instrSession));
	
			if (testLength == RSAVPHA_VAL_HDMI_TEST_LENGTH_FRAMES)
			{
				viCheckErr (viPrintf (instrSession, "CONF%ld:HDMI:CTS:TEST:FRAM %s,%ld", board, HDMIAnalyzerTestCaseArr[testCase], numberOfFrames));   
			
				checkErr(rsavpha_CheckStatus (instrSession));		
			}
	    break;
	   
		case RSAVPHA_VAL_HDMI_TEST_CASE_PACKET: 
			   
			viCheckErr (viPrintf (instrSession, "CONF%ld:HDMI:CTS:PTYP:SET %s", board, HDMITestSetupArr[testSetup]));
    
		    checkErr(rsavpha_CheckStatus (instrSession));	
		
			viCheckErr (viPrintf (instrSession, "CONF%ld:HDMI:CTS:TEST:MODE %s,%s", board, HDMIAnalyzerTestCaseArr[testCase], TestLengthArr[testLength]));
			
	     	checkErr(rsavpha_CheckStatus (instrSession));
	
	     	if (testLength == RSAVPHA_VAL_HDMI_TEST_LENGTH_FRAMES)
			{
				viCheckErr (viPrintf (instrSession, "CONF%ld:HDMI:CTS:TEST:FRAM %s,%ld", board, HDMIAnalyzerTestCaseArr[testCase], numberOfFrames));   

				checkErr(rsavpha_CheckStatus (instrSession));		
			}
	
		break;
	}
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function returns the HDMI analyzer CTS test output report data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR testCase/Selects the test case.
/// HIPAR testSetup/Selects the test setup.
/// HIPAR CDFSetup/Selects the pixel encoding.
/// HIPAR pixelEncoding/Selects the video configuration pixel encoding.
/// HIPAR videoFormat/Selects the video configuration video format.
/// HIPAR contentType/Selects the video configuration content type.
/// HIPAR testLength/Selects the test length.
/// HIPAR numberOfFrames/sets the number of frames.
ViStatus _VI_FUNC rsavpha_ConfigureHDMIAnalyzerCTSVideoModes (ViSession instrSession,
                                                            ViInt32 board,
                                                            ViInt32 testCase,
                                                            ViInt32 testSetup,
                                                            ViInt32 CDFSetup,
                                                            ViInt32 pixelEncoding,
                                                            ViInt32 videoFormat,
                                                            ViInt32 contentType,
                                                            ViInt32 testLength,
                                                            ViInt32 numberOfFrames)
{
    ViStatus    error = VI_SUCCESS;
   
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    viCheckParm (rsavpha_invalidViInt32Range (board, 1, 8), 2, "Board");
    viCheckParm (rsavpha_invalidViInt32Range (testCase, RSAVPHA_VAL_HDMI_TEST_CASE_PIXELR, RSAVPHA_VAL_HDMI_TEST_CASE_AVI), 3, "Test Case");
    viCheckParm (rsavpha_invalidViInt32Range (testSetup, RSAVPHA_VAL_TEST_SETUP_PIXEL_ENCONDING, RSAVPHA_VAL_TEST_SETUP_YQ), 4, "Test Setup");
    viCheckParm (rsavpha_invalidViInt32Range (CDFSetup, RSAVPHA_VAL_CDF_TEST_Y, RSAVPHA_VAL_CDF_TEST_N), 5, "CDF Setup");
    viCheckParm (rsavpha_invalidViInt32Range (pixelEncoding, RSAVPHA_VAL_PIXEL_ENCODING_Y422, RSAVPHA_VAL_PIXEL_ENCODING_Y444), 6, "Pixel Encoding");
    viCheckParm (rsavpha_invalidViInt32Range (videoFormat, RSAVPHA_VAL_VIDEO_FORMAT_PROGRESSSIVE, RSAVPHA_VAL_VIDEO_FORMAT_INTERLACED), 7, "Video Format");
    viCheckParm (rsavpha_invalidViInt32Range (contentType, RSAVPHA_VAL_CONTENT_PHOTO, RSAVPHA_VAL_CONTENT_GAME), 8, "Content Type");
    viCheckParm (rsavpha_invalidViInt32Range (testLength, RSAVPHA_VAL_HDMI_TEST_LENGTH_2_SECONDS, RSAVPHA_VAL_HDMI_TEST_LENGTH_FRAMES), 9, "Test Length");
    viCheckParm (rsavpha_invalidViInt32Range (numberOfFrames, 1, 120), 10, "Number Of Frames");
    
	switch (testCase)
		
	  {	
		case RSAVPHA_VAL_HDMI_TEST_CASE_PIXELR:
			  
     	viCheckErr (viPrintf (instrSession, "CONF%ld:HDMI:CTS:PENC:RGB:SET %s", board, HDMIPixelEncodingTestSetupArr[testSetup]));
			
	    checkErr(rsavpha_CheckStatus (instrSession));
		
		viCheckErr (viPrintf (instrSession, "CONF%ld:HDMI:CTS:PENC:RGB:CDF %s", board, HDMICDFSetupArr[CDFSetup]));
			
	    checkErr(rsavpha_CheckStatus (instrSession));
		
		viCheckErr (viPrintf (instrSession, "CONF%ld:HDMI:CTS:TEST:MODE %s,%s", board, HDMIAnalyzerTestCaseArr[testCase], TestLengthArr[testLength]));
			
     	checkErr(rsavpha_CheckStatus (instrSession));
		
	    if (testLength == RSAVPHA_VAL_HDMI_TEST_LENGTH_FRAMES)
	    {
		viCheckErr (viPrintf (instrSession, "CONF%ld:HDMI:CTS:TEST:FRAM %s,%ld", board, HDMIAnalyzerTestCaseArr[testCase], numberOfFrames));   
			
		checkErr(rsavpha_CheckStatus (instrSession));		
	    }

	    break;
	   
		case RSAVPHA_VAL_HDMI_TEST_CASE_PIXELY:
			
		viCheckErr (viPrintf (instrSession, "CONF%ld:HDMI:CTS:PENC:YCBCr:SET %s", board, HDMIPixelEncodingTestSetupArr[testSetup]));
			
	    checkErr(rsavpha_CheckStatus (instrSession));
		
		viCheckErr (viPrintf (instrSession, "CONF%ld:HDMI:CTS:PENC:YCBCr:CDF %s", board, HDMICDFSetupArr[CDFSetup]));
			
	    checkErr(rsavpha_CheckStatus (instrSession));
		
		viCheckErr (viPrintf (instrSession, "CONF%ld:HDMI:CTS:PENC:YCBC:PENC %s", board, PixelEncodingArr[pixelEncoding]));
			
	    checkErr(rsavpha_CheckStatus (instrSession));
		
		viCheckErr (viPrintf (instrSession, "CONF%ld:HDMI:CTS:TEST:MODE %s,%s", board, HDMIAnalyzerTestCaseArr[testCase], TestLengthArr[testLength]));
			
     	checkErr(rsavpha_CheckStatus (instrSession));
		
	    if (testLength == RSAVPHA_VAL_HDMI_TEST_LENGTH_FRAMES)
	    {
		viCheckErr (viPrintf (instrSession, "CONF%ld:HDMI:CTS:TEST:FRAM %s,%ld", board, HDMIAnalyzerTestCaseArr[testCase], numberOfFrames));   
			
		checkErr(rsavpha_CheckStatus (instrSession));		
	    }
		
	    break;
		
		case RSAVPHA_VAL_HDMI_TEST_CASE_VFORMAT:  
		case RSAVPHA_VAL_HDMI_TEST_CASE_PREP:
			
		viCheckErr (viPrintf (instrSession, "CONF%ld:HDMI:CTS:VFOR %s, %s", board, HDMIAnalyzerTestCaseArr[testCase], HDMIVideoFormatArr[videoFormat]));
			
	    checkErr(rsavpha_CheckStatus (instrSession));
		
		viCheckErr (viPrintf (instrSession, "CONF%ld:HDMI:CTS:TEST:MODE %s,%s", board, HDMIAnalyzerTestCaseArr[testCase], TestLengthArr[testLength]));
			
     	checkErr(rsavpha_CheckStatus (instrSession));
		
	    if (testLength == RSAVPHA_VAL_HDMI_TEST_LENGTH_FRAMES)
	    {
		viCheckErr (viPrintf (instrSession, "CONF%ld:HDMI:CTS:TEST:FRAM %s,%ld", board, HDMIAnalyzerTestCaseArr[testCase], numberOfFrames));   
			
		checkErr(rsavpha_CheckStatus (instrSession));		
	    }
		
		break;
		
		case RSAVPHA_VAL_HDMI_TEST_CASE_AVI:  
			
		viCheckErr (viPrintf (instrSession, "CONF%ld:HDMI:CTS:VFOR %s,%s", board, HDMIAnalyzerTestCaseArr[testCase], HDMIVideoFormatArr[videoFormat]));
			
	    checkErr(rsavpha_CheckStatus (instrSession));
		
		viCheckErr (viPrintf (instrSession, "CONF%ld:HDMI:CTS:AVI:CTYP %s", board, HDMIContentTypeArr[contentType]));
			
	    checkErr(rsavpha_CheckStatus (instrSession));
		
		viCheckErr (viPrintf (instrSession, "CONF%ld:HDMI:CTS:TEST:MODE %s,%s", board, HDMIAnalyzerTestCaseArr[testCase], TestLengthArr[testLength]));
			
     	checkErr(rsavpha_CheckStatus (instrSession));
		
	    if (testLength == RSAVPHA_VAL_HDMI_TEST_LENGTH_FRAMES)
	    {
		viCheckErr (viPrintf (instrSession, "CONF%ld:HDMI:CTS:TEST:FRAM %s,%ld", board, HDMIAnalyzerTestCaseArr[testCase], numberOfFrames));   
			
		checkErr(rsavpha_CheckStatus (instrSession));
		
	    }
		
		break;
		
		default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER,3,"Test Case");
        break;
			
	   }
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the HDMI analyzer CTS data source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR testCase/selects the audio test cases.
/// HIPAR testLength/Selects the test length.
/// HIPAR numberOfFrames/Sets the number of frames.
ViStatus _VI_FUNC rsavpha_ConfigureHDMIAnalyzerCTSAudioTests (ViSession instrSession,
                                                            ViInt32 board,
                                                            ViInt32 testCase,
                                                            ViInt32 testLength,
                                                            ViInt32 numberOfFrames)
{
	ViStatus    error = VI_SUCCESS;
   
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    viCheckParm (rsavpha_invalidViInt32Range (board, 1, 8), 2, "Board");
    viCheckParm (rsavpha_invalidViInt32Range (testCase, RSAVPHA_VAL_HDMI_TEST_CASE_IEC6, RSAVPHA_VAL_HDMI_TEST_CASE_ASPLAY), 3, "Test Case");
    viCheckParm (rsavpha_invalidViInt32Range (testLength, RSAVPHA_VAL_HDMI_TEST_LENGTH_2_SECONDS, RSAVPHA_VAL_HDMI_TEST_LENGTH_FRAMES), 4, "Test Length");
    viCheckParm (rsavpha_invalidViInt32Range (numberOfFrames, 1, 120), 5, "Number Of Frames");
    
	switch (testCase)
		
	  {	
		case RSAVPHA_VAL_HDMI_TEST_CASE_IEC6:
		case RSAVPHA_VAL_HDMI_TEST_CASE_AUDIOI:
		case RSAVPHA_VAL_HDMI_TEST_CASE_ASPLAY:
			  
		viCheckErr (viPrintf (instrSession, "CONF%ld:HDMI:CTS:TEST:MODE %s,%s", board, HDMIAnalyzerTestCaseArr[testCase], TestLengthArr[testLength]));
			
     	checkErr(rsavpha_CheckStatus (instrSession));
		
	    if (testLength == RSAVPHA_VAL_HDMI_TEST_LENGTH_FRAMES)
	    {
		viCheckErr (viPrintf (instrSession, "CONF%ld:HDMI:CTS:TEST:FRAM %s,%ld", board, HDMIAnalyzerTestCaseArr[testCase], numberOfFrames));   
			
		checkErr(rsavpha_CheckStatus (instrSession));		
	    }

	    break;
		
		case RSAVPHA_VAL_HDMI_TEST_CASE_ACR:
		case RSAVPHA_VAL_HDMI_TEST_CASE_ASPJITT:
	
		viCheckErr (viPrintf (instrSession, "CONF%ld:HDMI:CTS:TEST:MODE %s,%s", board, HDMIAnalyzerTestCaseArr[testCase], TestLengthArr[testLength]));
			
     	checkErr(rsavpha_CheckStatus (instrSession));
		
		break;
		
		default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER,3,"Test Case");
        break;
		
	  }
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the HDMI analyzer CTS interoperability with DVI.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR testCase/Selects the interoperability with DVI test case.
/// HIPAR HDMIVSDB/Selects the HDMI VSDB.
/// HIPAR testLength/Selects the test length.
/// HIPAR numberOfFrames/Sets the number of frames.
ViStatus _VI_FUNC rsavpha_ConfigureHDMIAnalyzerCTSInteroperabilityWithDVI (ViSession instrSession,
                                                                         ViInt32 board,
                                                                         ViInt32 testCase,
                                                                         ViInt32 HDMIVSDB,
                                                                         ViInt32 testLength,
                                                                         ViInt32 numberOfFrames)
{
	ViStatus    error = VI_SUCCESS;
   
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    viCheckParm (rsavpha_invalidViInt32Range (board, 1, 8), 2, "Board");
    viCheckParm (rsavpha_invalidViInt32Range (HDMIVSDB, RSAVPHA_VAL_HDMI_VSBD_DVI, RSAVPHA_VAL_HDMI_VSBD_GT5), 4, "HDMI VSDB");
    viCheckParm (rsavpha_invalidViInt32Range (testLength, RSAVPHA_VAL_HDMI_TEST_LENGTH_2_SECONDS, RSAVPHA_VAL_HDMI_TEST_LENGTH_FRAMES), 5, "Test Length");
    viCheckParm (rsavpha_invalidViInt32Range (numberOfFrames, 1, 120), 6, "Number Of Frames");
    
	viCheckErr (viPrintf (instrSession, "CONF%ld:HDMI:CTS:TEST:MODE %s,%s", board, HDMIAnalyzerTestCaseArr[testCase], TestLengthArr[testLength]));
			
    checkErr(rsavpha_CheckStatus (instrSession));
		
	if (testLength == RSAVPHA_VAL_HDMI_TEST_LENGTH_FRAMES)
	    {
		viCheckErr (viPrintf (instrSession, "CONF%ld:HDMI:CTS:TEST:FRAM %s,%ld", board, HDMIAnalyzerTestCaseArr[testCase], numberOfFrames));   
			
		checkErr(rsavpha_CheckStatus (instrSession));		
	    }
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the HDMI analyzer CTS Advanced options.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR testCase/Selects the interoperability with DVI test case.
/// HIPAR videoFormat/Selects the video configuration pixel encoding.
/// HIPAR HDMIVSDB/Selects the HDMI VSDB.
/// HIPAR testLength/Selects the test length.
/// HIPAR numberOfFrames/Sets the number of frames.
ViStatus _VI_FUNC rsavpha_ConfigureHDMIAnalyzerCTSAdvanced (ViSession instrSession,
                                                          ViInt32 board,
                                                          ViInt32 testCase,
                                                          ViInt32 videoFormat,
                                                          ViInt32 HDMIVSDB,
                                                          ViInt32 testLength,
                                                          ViInt32 numberOfFrames)
{
	ViStatus    error = VI_SUCCESS;
   
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    viCheckParm (rsavpha_invalidViInt32Range (board, 1, 8), 2, "Board");
    viCheckParm (rsavpha_invalidViInt32Range (testCase, RSAVPHA_VAL_HDMI_TEST_CASE_DEEPC, RSAVPHA_VAL_HDMI_TEST_CASE_ECTRANS), 3, "Test Case");
    viCheckParm (rsavpha_invalidViInt32Range (videoFormat, RSAVPHA_VAL_VIDEO_FORMAT_PROGRESSSIVE, RSAVPHA_VAL_VIDEO_FORMAT_INTERLACED), 4, "Video Format");
    viCheckParm (rsavpha_invalidViInt32Range (HDMIVSDB, RSAVPHA_VAL_HDMI_VSBD_EQ5, RSAVPHA_VAL_HDMI_VSBD_GT5), 5, "HDMI VSDB");
    viCheckParm (rsavpha_invalidViInt32Range (testLength, RSAVPHA_VAL_HDMI_TEST_LENGTH_2_SECONDS, RSAVPHA_VAL_HDMI_TEST_LENGTH_FRAMES), 6, "Test Length");
    viCheckParm (rsavpha_invalidViInt32Range (numberOfFrames, 1, 120), 7, "Number Of Frames");
    
	switch (testCase)
	{	
		case RSAVPHA_VAL_HDMI_TEST_CASE_DEEPC:
		case RSAVPHA_VAL_HDMI_TEST_CASE_4K2KVFT:
			  
	        viCheckErr (viPrintf (instrSession, "CONF%ld:HDMI:CTS:VFOR %s,%s", board, HDMIAnalyzerTestCaseArr[testCase], HDMIVideoFormatArr[videoFormat]));
			
		    checkErr(rsavpha_CheckStatus (instrSession));
		
			viCheckErr (viPrintf (instrSession, "CONF%ld:HDMI:CTS:TEST:MODE %s,%s", board, HDMIAnalyzerTestCaseArr[testCase], TestLengthArr[testLength]));
			
	     	checkErr(rsavpha_CheckStatus (instrSession));
		
		    if (testLength == RSAVPHA_VAL_HDMI_TEST_LENGTH_FRAMES)
		    {
				viCheckErr (viPrintf (instrSession, "CONF%ld:HDMI:CTS:TEST:FRAM %s,%ld", board, HDMIAnalyzerTestCaseArr[testCase], numberOfFrames));   
			
				checkErr(rsavpha_CheckStatus (instrSession));		
		    }

	    break;
	   
		case RSAVPHA_VAL_HDMI_TEST_CASE_GMTRANS:
		case RSAVPHA_VAL_HDMI_TEST_CASE_ECTRANS:
			
			viCheckErr (viPrintf (instrSession, "CONF%ld:HDMI:CTS:TEST:MODE %s,%s", board, HDMIAnalyzerTestCaseArr[testCase], TestLengthArr[testLength]));
			
	     	checkErr(rsavpha_CheckStatus (instrSession));
		
		    if (testLength == RSAVPHA_VAL_HDMI_TEST_LENGTH_FRAMES)
		    {
				viCheckErr (viPrintf (instrSession, "CONF%ld:HDMI:CTS:TEST:FRAM %s,%ld", board, HDMIAnalyzerTestCaseArr[testCase], numberOfFrames));   
			
				checkErr(rsavpha_CheckStatus (instrSession));	
			}
		
	    break;
		
		case RSAVPHA_VAL_HDMI_TEST_CASE_HBAUDIO:
		case RSAVPHA_VAL_HDMI_TEST_CASE_LBAUDIO:
		
			viCheckErr (viPrintf (instrSession, "CONF%ld:HDMI:CTS:TEST:MODE %s,%s", board, HDMIAnalyzerTestCaseArr[testCase], TestLengthArr[testLength]));
			
	     	checkErr(rsavpha_CheckStatus (instrSession));
		
		break;
		
		case RSAVPHA_VAL_HDMI_TEST_CASE_3DVFMT: 
			
			viCheckErr (viPrintf (instrSession, "CONF%ld:HDMI:CTS:VFOR %s,%s", board, HDMIAnalyzerTestCaseArr[testCase], HDMIVideoFormatArr[videoFormat]));
			
		    checkErr(rsavpha_CheckStatus (instrSession));
		
			/*HDMI VSDB command is missed*/   
		
			viCheckErr (viPrintf (instrSession, "CONF%ld:HDMI:CTS:TEST:MODE %s,%s", board, HDMIAnalyzerTestCaseArr[testCase], TestLengthArr[testLength]));
			
	     	checkErr(rsavpha_CheckStatus (instrSession));
		
		    if (testLength == RSAVPHA_VAL_HDMI_TEST_LENGTH_FRAMES)
			{
				viCheckErr (viPrintf (instrSession, "CONF%ld:HDMI:CTS:TEST:FRAM %s,%ld", board, HDMIAnalyzerTestCaseArr[testCase], numberOfFrames));   

				checkErr(rsavpha_CheckStatus (instrSession));		
			}
		
		break;
		
		default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER,3,"Test Case");
        break;
		
	}
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
	
/// HIFN  This function configures the HDMI analyzer CTS data source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR dataSource/Selects the data source.
/// HIPAR mode/Selects the mode.
/// HIPAR numberOfFrames/Sets the number of frames.
ViStatus _VI_FUNC rsavpha_ConfigureHDMIAnalyzerCTSDataSource (ViSession instrSession,
                                                            ViInt32 board,
                                                            ViInt32 dataSource,
                                                            ViInt32 mode,
                                                            ViInt32 numberOfFrames)
															
{
	
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

	checkErr (rsavpha_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpha_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPHA_ATTR_HDMI_ANALYZER_CTS_DATA_SOURCE, dataSource), 3, "Data Source");
	
	viCheckParm(rsavpha_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPHA_ATTR_HDMI_ANALYZER_CTS_MODE, mode), 4, "Mode");
	
	viCheckParm(rsavpha_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPHA_ATTR_HDMI_ANALYZER_CTS_NUMBER_OF_FRAMES, numberOfFrames), 5, "Number Of Frames");
	
	checkErr (rsavpha_CheckStatus (instrSession));
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function saves or loads the HDMI analyzer CTS data source to/from a file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR operation/Selects the data source.
/// HIPAR fileName/Selects the mode.
ViStatus _VI_FUNC rsavpha_HDMIAnalyzerCTSDataSourceLoadSave (ViSession instrSession,
                                                           ViInt32 board,
                                                           ViInt32 operation,
                                                           ViString fileName)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    viCheckParm (rsavpha_invalidViInt32Range (operation, RSAVPHA_VAL_HDMI_DATA_LOAD, RSAVPHA_VAL_HDMI_DATA_SAVE), 3, "Operation");
	checkErr (rsavpha_hardwareRepCap (instrSession, board, rep_cap));
    
	switch (operation)
		
	   {	
		  case RSAVPHA_VAL_HDMI_DATA_LOAD:
			 
			   viCheckParm (rsavpha_SetAttributeViString (instrSession, rep_cap,
                                               RSAVPHA_ATTR_HDMI_ANALYZER_CTS_DATA_LOAD,
                                               fileName), 4, "File Name");
		  break;
		  
		  case RSAVPHA_VAL_HDMI_DATA_SAVE:
			  
			   viCheckParm (rsavpha_SetAttributeViString (instrSession, rep_cap,
                                               RSAVPHA_ATTR_HDMI_ANALYZER_CTS_DATA_STORE,
                                               fileName), 4, "File Name");
			  
		  break;
		  
		  default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER,3,"Operation");
            break;
	   }
	
	checkErr (rsavpha_CheckStatus (instrSession));
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function returns the HDMI analyzer CTS test output report count.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR count/Returns the MHL analyzer CTS test output report count.
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerCTSTestOutputReportCount (ViSession instrSession,
                                                                   ViInt32 board,
                                                                   ViInt32 *count)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpha_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpha_GetAttributeViInt32(instrSession, rep_cap,
        RSAVPHA_ATTR_HDMI_ANALYZER_CTS_TEST_OUTPUT_REPORT_COUNT, count), 3, "Count");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error; 
}

/// HIFN  This function returns the HDMI analyzer CTS test output report data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR index/Sets the index.
/// HIPAR lineNumber/returns line number referring to TestOutput scroll view. First line has #0
/// HIPAR referenceNumber/Returns reference number of text message.
/// HIPAR textMessage/Returns text message.
/// HIPAR optionalParameters/Returns the optional parameter.
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerCTSTestOutputReport (ViSession instrSession,
                                                              ViInt32 board,
                                                              ViInt32 index,
															  ViInt32 arraySize,
                                                              ViInt32 *lineNumber,
                                                              ViInt32 *referenceNumber,
                                                              ViChar textMessage[],
                                                              ViInt32 optionalParameters[])
                                                              
                                                              
                                                              
{
    ViStatus error = VI_SUCCESS;
	ViChar   command[RSAVPHA_IO_BUFFER_SIZE] = "";
    ViUInt32 count = 0;
	ViChar   *p2buf  = NULL;
	ViChar   *p2data  = NULL;
	ViInt32 i = 0;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	viCheckParm (rsavpha_invalidViInt32Range (board, 1, 8), 2, "Board");
	
    sprintf (command, "READ%ld:HDMI:CST:REP? %ld\n", board, index);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsavpha_CheckStatus (instrSession));
    
    p2buf = strtok (p2data, ",");
    if (lineNumber != NULL) 
	*lineNumber = atoi (p2buf);

    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        return error;
    if (referenceNumber != NULL) 
		sscanf (p2buf, "%x", &referenceNumber);
	
    p2buf = strtok (NULL, ",");
	if (strspn (p2buf, "\"") != 0)
	{
		sscanf (p2buf, "\"%[^\"]", textMessage);
		p2buf = strtok (NULL, ",");
	}
	
	for (i=0;i<arraySize;i++)
    {
	   optionalParameters[i] = atol (p2buf);
	   p2buf = strtok (NULL, ",");
    }

Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function sets the logical address of the DUT that should receive 
/// HIFN  the CEC messages sent by the R&S VTC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR board/This control selects the board.
/// HIPAR destination/Sets the logical address of the DUT that should receive the CEC 
/// HIPAR destination/messages sent by the R&S VTC.
ViStatus _VI_FUNC rsavpha_ConfigureHDMIAnalyzerCECDestination(
	ViSession	instrSession,
	ViInt32	board,
	ViInt32	destination
)
{
	ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpha_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm (rsavpha_SetAttributeViInt32 (instrSession, rep_cap,
        RSAVPHA_ATTR_HDMI_ANALYZER_CEC_DESTINATION, destination), 3, "Destination");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;     
}


/// HIFN  Sets the selected operation code (opcode) of the CEC command that is 
/// HIFN  sent by the R&S VTC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR board/This control selects the board.
/// HIPAR operationCode/Sets the selected operation code (opcode) of the CEC command that is 
/// HIPAR operationCode/sent by the R&S VTC.
ViStatus _VI_FUNC rsavpha_ConfigureHDMIAnalyzerCECOperationCode(
	ViSession	instrSession,
	ViInt32	board,
	ViInt32	operationCode
)
{
	ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpha_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm (rsavpha_SetAttributeViInt32 (instrSession, rep_cap,
        RSAVPHA_ATTR_HDMI_ANALYZER_CEC_OPERATION_CODE, operationCode), 3, "Operation Code");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;      
}


/// HIFN  This function sets operation code parameter length.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR board/This control selects the board.
/// HIPAR operationCode/Sets the selected operation code (opcode) of the CEC command that is 
/// HIPAR operationCode/sent by the R&S VTC.
/// HIPAR parameter/This control selects parameter 1.
ViStatus _VI_FUNC rsavpha_ConfigureHDMIAnalyzerCECOperationCodeParameterLength(
	ViSession	instrSession,
	ViInt32	board,
	ViInt32	operationCode,
	ViInt32	length
)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[150] = "";
	ViReal64 	discrete;
    ViChar   	*p2Code = NULL;
	int 		i = 0;
	
    checkErr (Rs_LockSession (instrSession, NULL));
	
	do
    {
        discrete = rsavpha_rngHDMIAnalyzerCECOperationCode.rangeValues[i].discreteOrMinValue;
        p2Code = rsavpha_rngHDMIAnalyzerCECOperationCode.rangeValues[i].cmdString;

        if (p2Code == RS_RANGE_TABLE_END_STRING)
            viCheckParm(RS_ERROR_INVALID_PARAMETER, 3, "Operation Code");

        if (fabs(discrete - operationCode) <= 0.01) break;

        i++;

    } while (i != 0);

    viCheckParm (rsavpha_invalidViInt32Range (board, 1, 8), 2, "Board");
    viCheckParm (rsavpha_invalidViInt32Range (length, 0, 14), 4, "length");
    
    sprintf (command, "CONF%ld:HDMI:CEC:OPC:PAR:LENG %s,%ld", board, p2Code, length);
		
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    
    checkErr( rsavpha_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;    
}


/// HIFN  This function sets operation code parameter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR board/This control selects the board.
/// HIPAR operationCode/Sets the selected operation code (opcode) of the CEC command that is 
/// HIPAR operationCode/sent by the R&S VTC.
/// HIPAR parameter1/This control selects parameter 1.
/// HIPAR parameter2/This control selects parameter 2.
ViStatus _VI_FUNC rsavpha_ConfigureHDMIAnalyzerCECOperationCodeParameter (ViSession instrSession,
                                                                          ViInt32 board,
                                                                          ViInt32 operationCode,
                                                                          ViInt32 index,
                                                                          ViInt32 parameter)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[150] = "";
	ViReal64 	discrete;
    ViChar   	*p2Code = NULL;
	int 		i = 0;
	
    checkErr (Rs_LockSession (instrSession, NULL));
	
	do
    {
        discrete = rsavpha_rngHDMIAnalyzerCECOperationCode.rangeValues[i].discreteOrMinValue;
        p2Code = rsavpha_rngHDMIAnalyzerCECOperationCode.rangeValues[i].cmdString;

        if (p2Code == RS_RANGE_TABLE_END_STRING)
            viCheckParm(RS_ERROR_INVALID_PARAMETER, 3, "Operation Code");

        if (fabs(discrete - operationCode) <= 0.01) break;

        i++;

    } while (i != 0);

    viCheckParm (rsavpha_invalidViInt32Range (board, 1, 8), 2, "Board");
    viCheckParm (rsavpha_invalidViInt32Range (index, 0, 14), 4, "Index");
    
    sprintf (command, "CONF%ld:HDMI:CEC:OPC:PAR %s,%ld,#H%x", board, p2Code, index, parameter);
		
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    
    checkErr (rsavpha_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}


/// HIFN  This function selects the operation to be performed on the HDMI 
/// HIFN  analyzer CTS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR board/This control selects the board.
/// HIPAR operation/This control selects the operation.
ViStatus _VI_FUNC rsavpha_HDMIAnalyzerCECOperation(
	ViSession	instrSession,
	ViInt32	board,
	ViInt32	operation
)
{
	ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsavpha_invalidViInt32Range (operation, RSAVPHA_VAL_HDMI_STOP, RSAVPHA_VAL_HDMI_START), 3, "Operation");
    
    checkErr (rsavpha_hardwareRepCap (instrSession, board, rep_cap));
	
	switch (operation)
    {
        case RSAVPHA_VAL_HDMI_START:
			viCheckErr(rsavpha_SetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_HDMI_ANALYZER_CEC_INIT, NULL));
			break;
		
		case RSAVPHA_VAL_HDMI_STOP:
			viCheckErr(rsavpha_SetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_HDMI_ANALYZER_CEC_STOP, NULL));
		break;
	 default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER,3,"Operation");
            break;
			
    }			
	
	checkErr (rsavpha_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;     
}


/// HIFN  This function sends a command with the selected opcode and parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR board/This control selects the board.
ViStatus _VI_FUNC rsavpha_HDMIAnalyzerCECSend(
	ViSession	instrSession,
	ViInt32	board
)
{
	ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
	
	checkErr (Rs_LockSession (instrSession, VI_NULL)); 
    
    checkErr (rsavpha_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckErr(rsavpha_SetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHA_ATTR_HDMI_ANALYZER_CEC_SEND, NULL));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;     
}


/// HIFN  This function saves the protocol trace into a file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR board/This control selects the board.
/// HIPAR fileName/Selects the file name for saving captured data.
ViStatus _VI_FUNC rsavpha_HDMIAnalyzerCECDataSave(
	ViSession	instrSession,
	ViInt32	board,
	ViString	fileName
)
{
	ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpha_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm (rsavpha_SetAttributeViString (instrSession, rep_cap,
        RSAVPHA_ATTR_HDMI_ANALYZER_CEC_DATA_STORE, fileName), 3, "File Name");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;     
}


/// HIFN  This function returns the HDMI analyzer CEC test output report count.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR board/This control selects the board.
/// HIPAR count/This control returns the HDMI analyzer CEC test output report count.
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerCECTestOutputReportCount(
	ViSession	instrSession,
	ViInt32	board,
	ViInt32*	count
)
{
	ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpha_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm (rsavpha_GetAttributeViInt32 (instrSession, rep_cap,
        RSAVPHA_ATTR_HDMI_ANALYZER_CEC_TEST_OUTPUT_REPORT_COUNT, count), 3, "Count");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;    
}


/// HIFN  This function returns the HDMI analyzer CEC test output report data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR board/This control selects the board.
/// HIPAR index/This control sets the index.
/// HIPAR arraySize/This control sets the array size.
/// HIPAR lineNumber/This control returns line number referring to TestOutput scroll view. 
/// HIPAR lineNumber/First line has #0
/// HIPAR referenceNumber/This control returns reference number of text message
/// HIPAR textMessage/This control returns text message.
/// HIPAR optionalParameters/This control returns the optional parameters.
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerCECTestOutputReport (ViSession instrSession,
                                                                ViInt32 board,
                                                                ViInt32 index,
                                                                ViInt32 textMessageBufferSize,
                                                                ViInt32 arraySize,
                                                                ViInt32 *lineNumber,
                                                                ViInt32 *referenceNumber,
                                                                ViChar textMessage[],
                                                                ViInt32 optionalParameters[],
                                                                ViInt32 *returnedValues)
{
	ViStatus error = VI_SUCCESS;
	ViChar   command[RSAVPHA_IO_BUFFER_SIZE] = "";
	ViChar   buffer[RSAVPHA_IO_BUFFER_SIZE] = "";
    ViUInt32 count = 0;
	ViChar   *p2buf  = NULL;
	ViChar   *p2data  = NULL;
	ViInt32 i = 0;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	viCheckParm (rsavpha_invalidViInt32Range (board, 1, 8), 2, "Board");
	
    sprintf (command, "READ%ld:HDMI:CEC:REP? %ld\n", board, index);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsavpha_CheckStatus (instrSession));
    
    p2buf = strtok (p2data, ",");
    if (lineNumber != NULL) 
	*lineNumber = atoi (p2buf);

    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        return error;
    if (referenceNumber != NULL) 
		sscanf (p2buf, "%x", &referenceNumber);
	
    p2buf = strtok (NULL, ",");
	if (strspn (p2buf, "\"") != 0)
	{
		sscanf (p2buf, "\"%[^\"]", buffer);
		strncpy (textMessage, buffer, textMessageBufferSize);
		p2buf = strtok (NULL, ",");
	}
	
	while (p2buf && i<arraySize)
    {
	   optionalParameters[i++] = atol (p2buf);
	   p2buf = strtok (NULL, ",");
    }
	
	*returnedValues = i;
Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function resets the instrument to a known state and sends initialization commands to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
ViStatus _VI_FUNC rsavpha_reset (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (Rs_Simulating(instrSession) == 0)
        {
        viCheckErr( viPrintf (instrSession, "*RST\n"));
        }

    checkErr( rsavpha_DefaultInstrSetup (instrSession));
    checkErr( rsavpha_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;
}

/// HIFN This function resets the instrument to a known state and sends initialization commands to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR reference/Selects the reference of preset. 
ViStatus _VI_FUNC rsavpha_ResetSystem (ViSession instrSession,
                                       ViInt32 reference)
{
    ViStatus	error = VI_SUCCESS;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	switch (reference)
	{
		case RSAVPHA_VAL_RST_BASE:
			viCheckErr (rsavpha_SetAttributeViString(instrSession, "",
        		RSAVPHA_ATTR_SYSTEM_RESET_BASE, NULL));
			break;
		case RSAVPHA_VAL_RST_ALL:
			viCheckErr (rsavpha_SetAttributeViString(instrSession, "",
        		RSAVPHA_ATTR_SYSTEM_RESET_ALL, NULL));
			break;
		default:
			viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Reference");
			break;
	}

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN The preset sets the parameters to values suitable for local/manual interaction, a reset to values suitable for remote operation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR reference/Selects the reference of preset.
ViStatus _VI_FUNC rsavpha_Preset (ViSession instrSession, ViInt32 reference)
{
    ViStatus	error = VI_SUCCESS;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	switch (reference)
	{
		case RSAVPHA_VAL_RST_BASE:
			viCheckErr (rsavpha_SetAttributeViString(instrSession, "",
        		RSAVPHA_ATTR_SYSTEM_PRESET_BASE, NULL));
			break;
		case RSAVPHA_VAL_RST_ALL:
			viCheckErr (rsavpha_SetAttributeViString(instrSession, "",
        		RSAVPHA_ATTR_SYSTEM_PRESET_ALL, NULL));
			break;
		default:
			viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Reference");
			break;
	}

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error; 
}

/// HIFN This function runs the instrument's self test routine and returns the test result(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR testResult/Contains the value returned from the instrument self test.
/// HIPAR testMessage/Returns the self-test response string from the instrument.
ViStatus _VI_FUNC rsavpha_self_test (ViSession instrSession, ViInt16 *testResult, 
                                      ViChar testMessage[])
{
    ViStatus    error   = VI_SUCCESS;
    ViInt32     result = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (testResult == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Test Result");
    if (testMessage == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Test Message");

    viCheckErr (rsavpha_GetAttributeViInt32 (instrSession, NULL, RSAVPHA_ATTR_SELF_TEST_QUERY, &result));
    if (*testResult == 0)
        strcpy (testMessage, "Self-Test Passed.");
    else
        strcpy (testMessage, "Self-Test Failed.");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function returns the driver and instrument revisions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR driverRev/Returns the instrument driver software revision numbers in the form of a string.
/// HIPAR instrRev/Returns the instrument firmware revision numbers in the form of a string.
ViStatus _VI_FUNC rsavpha_revision_query (ViSession instrSession, ViChar driverRev[], 
                                           ViChar instrRev[])
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (driverRev == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Driver Revision");
    if (instrRev == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Instrument Revision");

    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RSAVPHA_ATTR_SPECIFIC_DRIVER_REVISION,
                                        0, 256, driverRev));
    checkErr( Rs_GetAttribute (instrSession, "", RSAVPHA_ATTR_INSTRUMENT_FIRMWARE_REVISION,
                                        0, 256, instrRev));
    checkErr( rsavpha_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function reads an error code and a message from the
/// HIFN instrument's error queue.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR errCode/Returns the error code read from the instrument's error queue.
/// HIPAR errMessage/Returns the error message string read from the instrument's error message queue.
ViStatus _VI_FUNC rsavpha_error_query (ViSession instrSession, ViInt32 *errCode, 
                                        ViChar errMessage[])
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSAVPHA_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (errCode == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error Code");
    if (errMessage == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
        {
        viCheckErr (rsavpha_GetAttributeViString( instrSession, "", RSAVPHA_ATTR_SYSTEM_ERROR, RSAVPHA_IO_BUFFER_SIZE, buffer));
        if  (sscanf (buffer, "%ld,\"%256[^\"]", errCode, errMessage) != 2)
            viCheckErr(RS_ERROR_UNEXPECTED_RESPONSE);

        }
    else
        {
            /* Simulate Error Query */
        *errCode = 0;
        strcpy (errMessage, "No error.");
        }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function converts a status code returned by an instrument driver function into a user-readable string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR errorCode/Pass the Status parameter that is returned from any of the instrument driver functions.
/// HIPAR errorMessage/Returns the user-readable message string that corresponds to the status code you specify.
ViStatus _VI_FUNC rsavpha_error_message (ViSession instrSession, ViStatus errorCode,
                                          ViChar errorMessage[256])
{
    ViStatus    error   = VI_SUCCESS;

    static      RsStringValueTable errorTable =
        {
            RSAVPHA_ERROR_CODES_AND_MSGS,
            {VI_NULL, VI_NULL}
        };

    (void) Rs_LockSession(instrSession, VI_NULL);

        /* all VISA and RS error codes are handled as well as codes in the table */
    if (errorMessage == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    checkErr( Rs_GetSpecificDriverStatusDesc(instrSession, errorCode, errorMessage, errorTable));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function queries the number of the SCPI version, which is 
/// HIFN  relevant for the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR length/This control specifies the size of the buffer.
/// HIPAR systemVersion/Returns the number of the SCPI version, which is relevant for the 
/// HIPAR systemVersion/instrument.
ViStatus _VI_FUNC rsavpha_SystemVersion(
	ViSession	instrSession,
	ViInt32		length,
	ViChar	  	systemVersion[]
)
{
    ViStatus error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsavpha_GetAttributeViString (instrSession, "",
                                                RSAVPHA_ATTR_SYST_VERSION,
												length,
												systemVersion
                                                ), 2, "System Version");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function switches ON/OFF state checking of the instrument 
/// HIFN  (reading of the Standard Event Register and checking it for error). All 
/// HIFN  driver functions are using state checking. Switch this function to ON 
/// HIFN  when debug your application. For better bus throughput and instrument 
/// HIFN  performance switch it OFF. As default is state checking switched to ON!
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/This control selects ON or OFF instrument state checking.
ViStatus _VI_FUNC rsavpha_setCheckStatus(
	ViSession	instrSession,
	ViBoolean	state
)
{
    ViStatus error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsavpha_SetAttributeViBoolean (instrSession, "",
                                                RS_ATTR_QUERY_INSTRUMENT_STATUS,
												state
                                                ), 2, "State");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function configures the local oscillator.
/// HIFN for the session or the current execution thread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR displayUpdate/Selects the local oscillator mode.
ViStatus _VI_FUNC rsavpha_ConfigureDisplayUpdate (ViSession instrSession,
                                                ViBoolean displayUpdate)
{
    ViStatus	error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm(rsavpha_SetAttributeViBoolean (instrSession, "",
        RSAVPHA_ATTR_DISPLAY_UPDATE, displayUpdate), 2, "Display Update");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN This function retrieves and then clears the error information
/// HIFN for the session or the current execution thread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR errorCode/Returns the error code for the session or execution thread.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Description parameter.
/// HIPAR description/Returns the error description for the session or execution thread. If there is no description, the function returns an empty string.
ViStatus _VI_FUNC rsavpha_GetError (ViSession instrSession, 
                                     ViStatus *errorCode, 
                                     ViInt32 bufferSize,
                                     ViChar description[])  
{
    ViStatus    error       = VI_SUCCESS;
    ViStatus    primary     = VI_SUCCESS,
                secondary   = VI_SUCCESS;
    ViChar      elaboration[RS_MAX_MESSAGE_BUF_SIZE]    = "";
    ViChar      errorMessage[RS_MAX_MESSAGE_BUF_SIZE]  = "";
    ViChar     *appendPoint = errorMessage;
    ViInt32     actualSize  = 0;
    ViBoolean   locked      = VI_FALSE;
    RsSessionPropertiesPtr  sessionProperties = Rs_ViSession (instrSession);
 
    /* Lock Session */
    checkErr( Rs_LockSession(instrSession, &locked));
 
    /* Test for nulls and acquire error data */
    if (bufferSize != 0)
    {
        if (errorCode == VI_NULL)
        {
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error");
        }
        if (sessionProperties == NULL)
        {
            primary = VI_ERROR_RSRC_NFOUND;
            secondary = 0;
        }
        else
        {
            checkErr( Rs_GetErrorInfo (instrSession, &primary, &secondary, elaboration));
        }
    }
 
    else
        {
            if (sessionProperties == NULL)
            {
                (void) Rs_UnlockSession (instrSession, &locked);
                return 1024;
            }
            else
            {
                checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_ERROR_ELABORATION, 0, 256, elaboration));
                checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_SECONDARY_ERROR, 0, (ViInt32) sizeof (secondary), &secondary));
                checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_PRIMARY_ERROR, 0, (ViInt32) sizeof (primary), &primary));
            }
        }
 
    /* Format data */
    if (primary != VI_SUCCESS)
        {
        ViChar msg[256] = "";
        checkErr( rsavpha_error_message (instrSession, primary, msg));
        appendPoint += sprintf(appendPoint, "Primary Error: (Hex 0x%08X) %s\n", (unsigned int) primary, msg);
        }
 
    if (secondary != VI_SUCCESS)
        {
        ViChar msg[256] = "";
        checkErr( rsavpha_error_message (instrSession, secondary, msg));
        appendPoint += sprintf(appendPoint, "Secondary Error: (Hex 0x%08X) %s\n", (unsigned int) secondary, msg);
        }
 
    if (strlen (elaboration) > 0)
        {
        sprintf(appendPoint, "Elaboration: %s", elaboration);
        }
 
    actualSize = (ViInt32) (strlen(errorMessage) + 1);
 
    /* Prepare return values */
    if (bufferSize == 0)
        {
        error = (ViStatus) actualSize;
        }
    else
        {
        if (bufferSize > 0)
            {
            if (actualSize > bufferSize)
                {
                error = actualSize;
                actualSize = bufferSize;
                }
            }
        if (description == VI_NULL)
            {
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Null address for Description");
            }
        memcpy(description, errorMessage, (size_t)(actualSize-1));
        description[actualSize-1] = '\0';
        }
 
    if (errorCode)
        {
        *errorCode = primary;
        }
 
Error:
    /* Unlock Session */
    (void) Rs_UnlockSession (instrSession, &locked);
    return error;
}

/// HIFN This function clears the error code and error description for
/// HIFN the session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
ViStatus _VI_FUNC rsavpha_ClearError (ViSession instrSession)                                                        
{                                                                                                           
    return Rs_ClearErrorInfo (instrSession);                                                                             
}

/// HIFN This function writes a user-specified string to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
ViStatus _VI_FUNC rsavpha_WriteInstrData (ViSession instrSession, ViString writeBuffer)
{
    ViStatus error = VI_SUCCESS;

    checkErr (Rs_WriteInstrData (instrSession, writeBuffer));
    
    checkErr (rsavpha_CheckStatus (instrSession));
 
Error:   
    return error;
}

/// HIFN Document your function here.
/// HIFN You may use multiple lines for documentation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR numBytes/Pass the maximum number of bytes to read from the instruments.
/// HIPAR rdBuf/After this function executes, this parameter contains the data that was read from the instrument.
/// HIPAR bytesRead/Returns the number of bytes actually read from the instrument and stored in the Read Buffer.
ViStatus _VI_FUNC rsavpha_ReadInstrData (ViSession instrSession, ViInt32 numBytes,
                                          ViChar rdBuf[], ViUInt32 *bytesRead)
{
    return Rs_ReadInstrData (instrSession, numBytes, rdBuf, bytesRead);
}

/// HIFN This function queries the ViBoolean value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViBoolean value.
ViStatus _VI_FUNC rsavpha_QueryViBoolean (ViSession instrSession,
                                           ViString command, 
                                           ViBoolean *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%d", command, value));
    checkErr( rsavpha_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function queries the ViInt32 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViInt32 value.
ViStatus _VI_FUNC rsavpha_QueryViInt32 (ViSession instrSession,
                                         ViString command, 
                                         ViInt32 *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%ld", command, value));
    checkErr( rsavpha_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function queries the ViReal64 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViReal64 value.
ViStatus _VI_FUNC rsavpha_QueryViReal64 (ViSession instrSession,
                                          ViString command, 
                                          ViReal64 *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%le", command, value));
    checkErr( rsavpha_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function queries the ViString value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Value parameter.
/// HIPAR value/Returns the string value.
ViStatus _VI_FUNC rsavpha_QueryViString (ViSession instrSession,
                                          ViString command, 
                                          ViInt32 bufferSize,
                                          ViChar _VI_FAR value[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      *pbuffer = NULL;
    ViUInt32        bytesRead;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viPrintf (instrSession, "%s\n", command));
    viCheckErr(Rs_ReadDataUnknownLength (instrSession, &pbuffer, &bytesRead));  
    if (bytesRead>=(ViUInt32)bufferSize)
    {
        strncpy(value, pbuffer, (size_t) (bufferSize-1));
        value[bufferSize-1]='\0';
        error = (ViStatus) (bytesRead+1);
    } 
    else
    {
        strncpy(value, pbuffer, (size_t) bytesRead); 
        value[bytesRead]='\0';   
    }
    checkErr( rsavpha_CheckStatus (instrSession)); 
Error:
    if (pbuffer) free (pbuffer);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function closes the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
ViStatus _VI_FUNC rsavpha_close (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    checkErr( rsavpha_RsClose (instrSession));

Error:

    (void) Rs_UnlockSession (instrSession, VI_NULL);
    (void) Rs_Dispose (instrSession);

    return error;
}

/// HIFN This function performs all of the drivers clean-up operations
/// HIFN except for closing the RS session.  This function is called by
/// HIFN rsavpha_close or by an RS class driver.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpha_init or 
/// HIPAR instrSession/rsavpha_InitWithOptions function.
ViStatus _VI_FUNC rsavpha_RsClose (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;
    ViSession   io      = VI_NULL;

    /* Do not lock here. The caller manages the lock. */

    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_IO_SESSION, 0, (ViInt32) sizeof (io), &io));

Error:

    (void) Rs_SetAttribute (instrSession, VI_NULL, RS_ATTR_IO_SESSION, 0, VI_NULL);
    
    if(io != 0)
        {
        ViSession   rmSession = VI_NULL;

        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_VISA_RM_SESSION, 0, (ViInt32) sizeof (rmSession), &rmSession));
        
        (void) viClose (io);
        (void) viClose (rmSession);
        }
    
    return error;
}

/*****************************************************************************
 *------------------- End Instrument Driver Source Code ---------------------*
 *****************************************************************************/
