/****************************************************************************
 *
 *  Rohde & Schwarz Vector Audio Video Platform HDMI Analyzer  instrument driver include file
 *
 ****************************************************************************/

#ifndef __RSAVPHA_HEADER
#define __RSAVPHA_HEADER

#include "rsidr_core.h"
#include "rsavpha_attributes.h"
#include "rsavpha_utility.h"

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#if defined(__cplusplus) || defined(__cplusplus__)
extern "C" {
#endif

/*****************************************************************************
 *- Useful Macros & Constants -----------------------------------------------*
 *****************************************************************************/
#define RSAVPHA_SIMULATION_ID_QUERY      "Rohde&Schwarz,R&S VTE,2115.7300k02/900000,1.45.0.0"  /* Default definition of ID Query Response for simulation */
#define RSAVPHA_VALID_ID_RESPONSE_STRING "Rohde&Schwarz"      /* Valid response for identification query */
#define RSAVPHA_SIMULATION_OPT_QUERY     ""      /* Simulated response for *OPT? command */

#define RSAVPHA_IO_BUFFER_SIZE           1024L   /* I/O buffer size */
#define RSAVPHA_OPC_TIMEOUT              5000L   /* Maximum time to wait for OPC in milliseconds */
#define IEEE_488_2_ERROR_MASK             0x4    /* 488.2 STB Bits */   
#define IEEE_488_2_ESE_MASK               0x3D    /* 488.2 Event Status Register (ESE) Bits */
#define RSAVPHA_NAN                 9.91e+37
#define RSAVPHA_NAN_VI_INT32             0x80000000
#define RSAVPHA_NAN_VI_INT64             0x8000000000000000     
/****************************************************************************
 *----------------- Instrument Driver Revision Information -----------------*
 ****************************************************************************/

/* Class Driver Identification */

#define RSAVPHA_CLASS_DRIVER_DESCRIPTION           ""
#define RSAVPHA_CLASS_DRIVER_PREFIX                ""
#define RSAVPHA_CLASS_DRIVER_VENDOR                ""
#define RSAVPHA_CLASS_DRIVER_REVISION              ""
#define RSAVPHA_CLASS_SPEC_MAJOR_VERSION           1L    /* Class specification major version */
#define RSAVPHA_CLASS_SPEC_MINOR_VERSION           0L    /* Class specification minor version */

/* Driver Identification */

#define RSAVPHA_SPECIFIC_DRIVER_DESCRIPTION        ""
#define RSAVPHA_SPECIFIC_DRIVER_PREFIX             "RSAVPHA"
#define RSAVPHA_SPECIFIC_DRIVER_LOCATOR            ""
#define RSAVPHA_SPECIFIC_DRIVER_VENDOR             ""
#define RSAVPHA_MAJOR_VERSION                      1L    /* Instrument driver major version          */
#define RSAVPHA_MINOR_VERSION                      1L    /* Instrument driver minor version          */
#define RSAVPHA_MINOR_MINOR_VERSION                0L    /* Instrument driver minor minor version    */

/* Driver Capabilities */          

#define RSAVPHA_SUPPORTED_INSTRUMENT_MODELS        ""    /* Instrument driver supported model(s)     */
#define RSAVPHA_GROUP_CAPABILITIES                 ""
#define RSAVPHA_FUNCTION_CAPABILITIES              ""

/****************************************************************************
 *---------------------------- Attribute Defines ---------------------------*
 ****************************************************************************/

    /*- Inherent Instrument Specific Attributes ----------------------------*/

        /* User Options */

#define RSAVPHA_ATTR_RANGE_CHECK                                           RS_ATTR_RANGE_CHECK                                     /* ViBoolean */
#define RSAVPHA_ATTR_QUERY_INSTRUMENT_STATUS                               RS_ATTR_QUERY_INSTRUMENT_STATUS                         /* ViBoolean */
#define RSAVPHA_ATTR_CACHE                                                 RS_ATTR_CACHE                                           /* ViBoolean */
#define RSAVPHA_ATTR_SIMULATE                                              RS_ATTR_SIMULATE                                        /* ViBoolean */
#define RSAVPHA_ATTR_RECORD_COERCIONS                                      RS_ATTR_RECORD_COERCIONS                                /* ViBoolean */
#define RSAVPHA_ATTR_INTERCHANGE_CHECK                                     RS_ATTR_INTERCHANGE_CHECK                               /* ViBoolean */
#define RSAVPHA_ATTR_SPY                                                   RS_ATTR_SPY                                             /* ViBoolean */
#define RSAVPHA_ATTR_USE_SPECIFIC_SIMULATION                               RS_ATTR_USE_SPECIFIC_SIMULATION                         /* ViBoolean */

        /* Class Driver Identification */

#define RSAVPHA_ATTR_CLASS_DRIVER_DESCRIPTION                              RS_ATTR_CLASS_DRIVER_DESCRIPTION                        /* ViString, read-only */
#define RSAVPHA_ATTR_CLASS_DRIVER_PREFIX                                   RS_ATTR_CLASS_DRIVER_PREFIX                             /* ViString, read-only */
#define RSAVPHA_ATTR_CLASS_DRIVER_VENDOR                                   RS_ATTR_CLASS_DRIVER_VENDOR                             /* ViString, read-only */
#define RSAVPHA_ATTR_CLASS_DRIVER_REVISION                                 RS_ATTR_CLASS_DRIVER_REVISION                           /* ViString, read-only */
#define RSAVPHA_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION           /* ViInt32,  read-only */
#define RSAVPHA_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION           /* ViInt32,  read-only */

        /* Driver Identification */

#define RSAVPHA_ATTR_SPECIFIC_DRIVER_DESCRIPTION                           RS_ATTR_SPECIFIC_DRIVER_DESCRIPTION                     /* ViString, read-only */
#define RSAVPHA_ATTR_SPECIFIC_DRIVER_PREFIX                                RS_ATTR_SPECIFIC_DRIVER_PREFIX                          /* ViString, read-only */
#define RSAVPHA_ATTR_SPECIFIC_DRIVER_LOCATOR                               RS_ATTR_SPECIFIC_DRIVER_LOCATOR                         /* ViString, read-only */
#define RSAVPHA_ATTR_SPECIFIC_DRIVER_VENDOR                                RS_ATTR_SPECIFIC_DRIVER_VENDOR                          /* ViString, read-only */
#define RSAVPHA_ATTR_SPECIFIC_DRIVER_REVISION                              RS_ATTR_SPECIFIC_DRIVER_REVISION                        /* ViString, read-only */
#define RSAVPHA_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION        /* ViInt32,  read-only */
#define RSAVPHA_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION        /* ViInt32,  read-only */

        /* Driver Capabilities */

#define RSAVPHA_ATTR_SUPPORTED_INSTRUMENT_MODELS                           RS_ATTR_SUPPORTED_INSTRUMENT_MODELS                     /* ViString, read-only */
#define RSAVPHA_ATTR_GROUP_CAPABILITIES                                    RS_ATTR_GROUP_CAPABILITIES                              /* ViString, read-only */
#define RSAVPHA_ATTR_FUNCTION_CAPABILITIES                                 RS_ATTR_FUNCTION_CAPABILITIES                           /* ViString, read-only */
#define RSAVPHA_ATTR_CHANNEL_COUNT                                         RS_ATTR_CHANNEL_COUNT                                   /* ViInt32,  read-only */

        /* Driver Setup */

#define RSAVPHA_ATTR_DRIVER_SETUP                                          RS_ATTR_DRIVER_SETUP                                    /* ViString            */

        /* Instrument Identification */

#define RSAVPHA_ATTR_INSTRUMENT_MANUFACTURER                               RS_ATTR_INSTRUMENT_MANUFACTURER                         /* ViString, read-only */
#define RSAVPHA_ATTR_INSTRUMENT_MODEL                                      RS_ATTR_INSTRUMENT_MODEL                                /* ViString, read-only */
#define RSAVPHA_ATTR_INSTRUMENT_FIRMWARE_REVISION                          RS_ATTR_INSTRUMENT_FIRMWARE_REVISION                    /* ViString, read-only */

        /* Advanced Session Information */

#define RSAVPHA_ATTR_IO_RESOURCE_DESCRIPTOR                                RS_ATTR_IO_RESOURCE_DESCRIPTOR                          /* ViString, read-only */
#define RSAVPHA_ATTR_LOGICAL_NAME                                          RS_ATTR_LOGICAL_NAME

/* ViString, read-only */

    /*- Instrument-Specific Attributes -------------------------------------*/

//#define RSAVPHA_ATTR_ID_QUERY_RESPONSE                                     (RS_SPECIFIC_PUBLIC_ATTR_BASE + 1L)                     /* ViString (Read Only)              */

/* BEGIN GENERATE */
#define RSAVPHA_ATTR_HDMI_ANALYZER_INPUT                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1
#define RSAVPHA_ATTR_HDMI_ANALYZER_HPD_SIGNAL_STATE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2
#define RSAVPHA_ATTR_HDMI_ANALYZER_INPUT_DEVICE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_CATALOG                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INPUT_PIXEL_CLOCK_LOCKED           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_HDCP_ACTIVE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INPUT_PIXEL_CLOCK                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INPUT_VIDEO_CODE                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INPUT_AUDIO_CODING                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 9
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_VIDEO_CODE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_VIDEO_PIXEL_CLOCK                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 11
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_VIDEO_FORMAT                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 12
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_VIDEO_VERTICAL_FREQUENCY           RS_SPECIFIC_PUBLIC_ATTR_BASE + 13
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_VIDEO_HORIZONTAL_FREQUENCY         RS_SPECIFIC_PUBLIC_ATTR_BASE + 14
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_VIDEO_HORIZONTAL_PIXEL_TOTAL       RS_SPECIFIC_PUBLIC_ATTR_BASE + 15
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_VIDEO_HORIZONTAL_PIXEL_ACTIVE      RS_SPECIFIC_PUBLIC_ATTR_BASE + 16
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_VIDEO_HORIZONTAL_PIXEL_FRONT_PORCH RS_SPECIFIC_PUBLIC_ATTR_BASE + 17
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_VIDEO_HORIZONTAL_PIXEL_SYNCHRONIZATION RS_SPECIFIC_PUBLIC_ATTR_BASE + 18
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_VIDEO_HORIZONTAL_PIXEL_BACK_PORCH  RS_SPECIFIC_PUBLIC_ATTR_BASE + 19
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_VIDEO_HORIZONTAL_PIXEL_SYNCHRONIZATION_POLARITY RS_SPECIFIC_PUBLIC_ATTR_BASE + 20
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_VIDEO_VERTICAL_LINE_TOTAL          RS_SPECIFIC_PUBLIC_ATTR_BASE + 101
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_VIDEO_VERTICAL_LINE_ACTIVE         RS_SPECIFIC_PUBLIC_ATTR_BASE + 102
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_VIDEO_VERTICAL_LINE_FRONT_PORCH    RS_SPECIFIC_PUBLIC_ATTR_BASE + 103
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_VIDEO_VERTICAL_LINE_SYNCHRONIZATION RS_SPECIFIC_PUBLIC_ATTR_BASE + 104
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_VIDEO_VERTICAL_LINE_BACK_PORCH     RS_SPECIFIC_PUBLIC_ATTR_BASE + 105
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_VIDEO_VERTICAL_LINE_SYNCHRONIZATION_POLARITY RS_SPECIFIC_PUBLIC_ATTR_BASE + 106
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_AUDIO_CYCLE_TIME_STAMP             RS_SPECIFIC_PUBLIC_ATTR_BASE + 21
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_AUDIO_N                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 22
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_AUDIO_CODING                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 23
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_AUDIO_LAYOUT                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 24
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_HDCP_AUTHENTICATION                RS_SPECIFIC_PUBLIC_ATTR_BASE + 25
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_HDCP_STATE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 26
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_HDCP_SOURCE_PUBLIC_KEY             RS_SPECIFIC_PUBLIC_ATTR_BASE + 27
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_HDCP_SINK_PUBLIC_KEY               RS_SPECIFIC_PUBLIC_ATTR_BASE + 28
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_HDCP_LINK_SYNCHRONIZATION_VALUE    RS_SPECIFIC_PUBLIC_ATTR_BASE + 29
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_VERSION             RS_SPECIFIC_PUBLIC_ATTR_BASE + 30
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_LENGTH              RS_SPECIFIC_PUBLIC_ATTR_BASE + 31
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_CHECKSUM            RS_SPECIFIC_PUBLIC_ATTR_BASE + 32
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_COLOR_SPACE         RS_SPECIFIC_PUBLIC_ATTR_BASE + 33
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_ACTIVE_FORMAT       RS_SPECIFIC_PUBLIC_ATTR_BASE + 34
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_BAR_INFORMATION     RS_SPECIFIC_PUBLIC_ATTR_BASE + 35
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_SCAN_INFORMATION    RS_SPECIFIC_PUBLIC_ATTR_BASE + 36
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_COLORIMETRY         RS_SPECIFIC_PUBLIC_ATTR_BASE + 37
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_PICTURE_ASPECT_RATIO RS_SPECIFIC_PUBLIC_ATTR_BASE + 38
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_ACTIVE_FORMAT_ASPECT_RATIO RS_SPECIFIC_PUBLIC_ATTR_BASE + 39
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_IT_CONTENT          RS_SPECIFIC_PUBLIC_ATTR_BASE + 40
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_EXTENDED_COLORIMETRY RS_SPECIFIC_PUBLIC_ATTR_BASE + 41
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_RGB_QUANTIZATION_RANGE RS_SPECIFIC_PUBLIC_ATTR_BASE + 42
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_NONUNIFORM_PICTURE_SCALING RS_SPECIFIC_PUBLIC_ATTR_BASE + 43
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_VIDEO_IDENTIFICATION_CODE RS_SPECIFIC_PUBLIC_ATTR_BASE + 44
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_YCC_QUANTIZATION    RS_SPECIFIC_PUBLIC_ATTR_BASE + 45
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_CONTENT_TYPE        RS_SPECIFIC_PUBLIC_ATTR_BASE + 46
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_PIXEL_REPETITION    RS_SPECIFIC_PUBLIC_ATTR_BASE + 47
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_TOP_BAR_END         RS_SPECIFIC_PUBLIC_ATTR_BASE + 48
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_BOTTOM_BAR_START    RS_SPECIFIC_PUBLIC_ATTR_BASE + 49
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_LEFT_BAR_END        RS_SPECIFIC_PUBLIC_ATTR_BASE + 50
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AVI_RIGHT_BAR_START     RS_SPECIFIC_PUBLIC_ATTR_BASE + 51
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AUDIO_VERSION           RS_SPECIFIC_PUBLIC_ATTR_BASE + 52
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AUDIO_LENGTH            RS_SPECIFIC_PUBLIC_ATTR_BASE + 53
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AUDIO_CHECKSUM          RS_SPECIFIC_PUBLIC_ATTR_BASE + 54
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AUDIO_CODING_TYPE       RS_SPECIFIC_PUBLIC_ATTR_BASE + 55
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AUDIO_CHANNEL_COUNT     RS_SPECIFIC_PUBLIC_ATTR_BASE + 56
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AUDIO_SAMPLING_FREQUENCY RS_SPECIFIC_PUBLIC_ATTR_BASE + 57
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AUDIO_SAMPLING_SIZE     RS_SPECIFIC_PUBLIC_ATTR_BASE + 58
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AUDIO_CODE_EXTENSION    RS_SPECIFIC_PUBLIC_ATTR_BASE + 59
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AUDIO_CHANNEL_ALLOCATION RS_SPECIFIC_PUBLIC_ATTR_BASE + 60
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AUDIO_DOWNMIX_INHIBIT   RS_SPECIFIC_PUBLIC_ATTR_BASE + 61
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AUDIO_LEVEL_SHIFT_VALUE RS_SPECIFIC_PUBLIC_ATTR_BASE + 62
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_AUDIO_LFE_PLAYBACK_LEVEL RS_SPECIFIC_PUBLIC_ATTR_BASE + 63
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_SPD_VERSION             RS_SPECIFIC_PUBLIC_ATTR_BASE + 64
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_SPD_LENGTH              RS_SPECIFIC_PUBLIC_ATTR_BASE + 65
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_SPD_CHECKSUM            RS_SPECIFIC_PUBLIC_ATTR_BASE + 66
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_SPD_VENDOR              RS_SPECIFIC_PUBLIC_ATTR_BASE + 67
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_SPD_PRODUCT_DESCRIPTION RS_SPECIFIC_PUBLIC_ATTR_BASE + 68
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_MPEG_VERSION            RS_SPECIFIC_PUBLIC_ATTR_BASE + 69
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_MPEG_LENGTH             RS_SPECIFIC_PUBLIC_ATTR_BASE + 70
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_MPEG_CHECKSUM           RS_SPECIFIC_PUBLIC_ATTR_BASE + 71
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_MPEG_BIT_RATE           RS_SPECIFIC_PUBLIC_ATTR_BASE + 72
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_MPEG_FIELD_REPEAT       RS_SPECIFIC_PUBLIC_ATTR_BASE + 73
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_MPEG_FRAME              RS_SPECIFIC_PUBLIC_ATTR_BASE + 74
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_VENDOR_VERSION          RS_SPECIFIC_PUBLIC_ATTR_BASE + 75
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_VENDOR_LENGTH           RS_SPECIFIC_PUBLIC_ATTR_BASE + 76
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_VENDOR_CHECKSUM         RS_SPECIFIC_PUBLIC_ATTR_BASE + 77
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_VENDOR_IEEE_ID          RS_SPECIFIC_PUBLIC_ATTR_BASE + 78
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_VENDOR_VIDEO_FORMAT     RS_SPECIFIC_PUBLIC_ATTR_BASE + 79
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_VENDOR_PIXEL_ENCODING   RS_SPECIFIC_PUBLIC_ATTR_BASE + 80
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_VENDOR_VIDEO_FORMAT_IDENTIFICATION_CODE RS_SPECIFIC_PUBLIC_ATTR_BASE + 81
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_VENDOR_3D_STRUCTURE     RS_SPECIFIC_PUBLIC_ATTR_BASE + 82
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_INFO_FRAME_VENDOR_3D_EXTERNAL_DATA RS_SPECIFIC_PUBLIC_ATTR_BASE + 83
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_CTS_STATE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 84
#define RSAVPHA_ATTR_HDMI_ANALYZER_CTS_INIT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 85
#define RSAVPHA_ATTR_HDMI_ANALYZER_CTS_STOP                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 86
#define RSAVPHA_ATTR_QUERY_HDMI_ANALYZER_CTS_PROGRESS                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 87
#define RSAVPHA_ATTR_HDMI_ANALYZER_CTS_DATA_SOURCE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 88
#define RSAVPHA_ATTR_HDMI_ANALYZER_CTS_MODE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 89
#define RSAVPHA_ATTR_HDMI_ANALYZER_CTS_NUMBER_OF_FRAMES                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 90
#define RSAVPHA_ATTR_HDMI_ANALYZER_CTS_DATA_STORE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 91
#define RSAVPHA_ATTR_HDMI_ANALYZER_CTS_DATA_LOAD                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 92
#define RSAVPHA_ATTR_HDMI_ANALYZER_CTS_TEST_OUTPUT_REPORT_COUNT             RS_SPECIFIC_PUBLIC_ATTR_BASE + 93
#define RSAVPHA_ATTR_HDMI_ANALYZER_CEC_DESTINATION                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 107
#define RSAVPHA_ATTR_HDMI_ANALYZER_CEC_OPERATION_CODE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 108
#define RSAVPHA_ATTR_HDMI_ANALYZER_CEC_INIT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 111
#define RSAVPHA_ATTR_HDMI_ANALYZER_CEC_STOP                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 112
#define RSAVPHA_ATTR_HDMI_ANALYZER_CEC_SEND                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 113
#define RSAVPHA_ATTR_HDMI_ANALYZER_CEC_DATA_STORE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 114
#define RSAVPHA_ATTR_HDMI_ANALYZER_CEC_TEST_OUTPUT_REPORT_COUNT             RS_SPECIFIC_PUBLIC_ATTR_BASE + 115
#define RSAVPHA_ATTR_SELF_TEST_QUERY                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 94
#define RSAVPHA_ATTR_SYST_VERSION                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 95
#define RSAVPHA_ATTR_SYSTEM_RESET_ALL                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 96
#define RSAVPHA_ATTR_SYSTEM_RESET_BASE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 109
#define RSAVPHA_ATTR_SYSTEM_PRESET_ALL                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 97
#define RSAVPHA_ATTR_SYSTEM_PRESET_BASE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 110
#define RSAVPHA_ATTR_ID_QUERY_RESPONSE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 98
#define RSAVPHA_ATTR_SYSTEM_ERROR                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 99
#define RSAVPHA_ATTR_DISPLAY_UPDATE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 100

// rsavpha_rngHDMIAnalyzerInputDevice
#define RSAVPHA_VAL_HDMI_INPUT_DEVICE_SINK                                  0
#define RSAVPHA_VAL_HDMI_INPUT_DEVICE_REPEATER                              1

// rsavpha_rngQueryHDMIAnalyzerHDCPActive
#define RSAVPHA_VAL_HDMI_HDCP_INVALID                                       0
#define RSAVPHA_VAL_HDMI_HDCP_INACTIVE                                      1
#define RSAVPHA_VAL_HDMI_HDCP_ACTIVE                                        2

// rsavpha_rngQueryHDMIAnalyzerCTSState
#define RSAVPHA_VAL_HDMI_STATE_IDLE                                         0
#define RSAVPHA_VAL_HDMI_STATE_CAPTURING                                    1
#define RSAVPHA_VAL_HDMI_STATE_RUNNING                                      2
#define RSAVPHA_VAL_HDMI_STATE_FINISHED                                     3

// rsavpha_rngHDMIAnalyzerCTSDataSource
#define RSAVPHA_VAL_HDMI_DATA_SOURCE_AUTO                                   0
#define RSAVPHA_VAL_HDMI_DATA_SOURCE_MANUAL                                 1
#define RSAVPHA_VAL_HDMI_DATA_SOURCE_FILE                                   2

// rsavpha_rngHDMIAnalyzerCTSMode
#define RSAVPHA_VAL_HDMI_DATA_SIZE_MODE_2_SECONDS                           0
#define RSAVPHA_VAL_HDMI_DATA_SIZE_MODE_FRAMES                              1

// rsavpha_rngQueryHDMIAnalyzerInputPixelClockLocked
#define RSAVPHA_VAL_HDMI_PIXEL_CLOCK_UNLOCKED                               0
#define RSAVPHA_VAL_HDMI_PIXEL_CLOCK_LOCKED                                 1

// rsavpha_rngHDMIAnalyzerCECDestination
#define RSAVPHA_VAL_HDMI_CEC_DESTINATION_RD1                                0
#define RSAVPHA_VAL_HDMI_CEC_DESTINATION_RD2                                1
#define RSAVPHA_VAL_HDMI_CEC_DESTINATION_TUN1                               3
#define RSAVPHA_VAL_HDMI_CEC_DESTINATION_TUN2                               4
#define RSAVPHA_VAL_HDMI_CEC_DESTINATION_TUN3                               5
#define RSAVPHA_VAL_HDMI_CEC_DESTINATION_TUN4                               6
#define RSAVPHA_VAL_HDMI_CEC_DESTINATION_PD1                                7
#define RSAVPHA_VAL_HDMI_CEC_DESTINATION_PD2                                8
#define RSAVPHA_VAL_HDMI_CEC_DESTINATION_PD3                                9
#define RSAVPHA_VAL_HDMI_CEC_DESTINATION_RD3                                2
#define RSAVPHA_VAL_HDMI_CEC_DESTINATION_AUD                                10
#define RSAVPHA_VAL_HDMI_CEC_DESTINATION_SPEC                               11
#define RSAVPHA_VAL_HDMI_CEC_DESTINATION_BRO                                12
#define RSAVPHA_VAL_HDMI_CEC_DESTINATION_R12                                13
#define RSAVPHA_VAL_HDMI_CEC_DESTINATION_R13                                14

// rsavpha_rngHDMIAnalyzerCECOperationCode
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_OP04                                    0
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_OP0D                                    1
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_OP82                                    2
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_RC36                                    3
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_RC80                                    4
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_RC81                                    5
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_RC82                                    6
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_RC85                                    7
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_RC86                                    8
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_RC9D                                    9
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_OR09                                    10
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_OR0A                                    11
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_OR0B                                    12
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_OR0F                                    13
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_TP33                                    14
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_TP34                                    15
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_TP35                                    16
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_TP43                                    17
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_TP67                                    18
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_TP97                                    19
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_TP99                                    20
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_TPA1                                    21
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_TPA2                                    22
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_SIPM                                    23
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_SI32                                    24
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_SI83                                    25
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_SI84                                    26
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_SI91                                    27
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_SI9E                                    28
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_SI9F                                    29
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_DC1A                                    30
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_DC1B                                    31
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_DC41                                    32
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_DC42                                    33
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_TC05                                    34
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_TC06                                    35
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_TC07                                    36
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_TC08                                    37
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_TC92                                    39
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_TC93                                    40
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_VC87                                    41
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_VC89                                    42
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_VC8A                                    43
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_VC8B                                    44
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_VC8C                                    45
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_VC9E                                    46
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_VC9F                                    47
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_VCA0                                    48
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_OD64                                    49
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_DT46                                    50
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_DT47                                    51
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_DM44                                    52
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_DM45                                    53
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_DM8D                                    54
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_DM8E                                    55
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_RC44                                    56
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_RC45                                    57
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_PS8F                                    58
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_PS90                                    59
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_GP00                                    60
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_GPFF                                    61
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_SA44                                    62
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_SA45                                    63
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_SA70                                    64
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_SA71                                    65
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_SA72                                    66
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_SA7A                                    67
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_SA7D                                    68
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_SA7E                                    69
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_SAA3                                    70
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_SAA4                                    71
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_AR9A                                    72
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_ARC0                                    73
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_ARC1                                    74
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_ARC2                                    75
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_ARC3                                    76
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_ARC4                                    77
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_ARC5                                    78
#define RSAVPHA_VAL_HDMI_CEC_OPCODE_CDF8                                    79
/* END GENERATE */

/*****************************************************************************
 *- Hidden Attribute Declarations -------------------------------------------*
 *****************************************************************************/

#define RSAVPHA_ATTR_OPC_TIMEOUT               (RS_ATTR_OPC_TIMEOUT)           /* ViInt32 */
#define RSAVPHA_ATTR_IO_SESSION                (RS_ATTR_IO_SESSION)            /* ViSession */
#define RSAVPHA_ATTR_OPC_CALLBACK              (RS_ATTR_OPC_CALLBACK)          /* ViAddr */
#define RSAVPHA_ATTR_CHECK_STATUS_CALLBACK     (RS_ATTR_CHECK_STATUS_CALLBACK) /* ViAddr */

/****************************************************************************
 *---------------- Constant definition  ------------------------------------*
 ****************************************************************************/

#define RSAVPHA_VAL_HDMI_HORIZONTAL            0
#define RSAVPHA_VAL_HDMI_VERTICAL              1

#define	RSAVPHA_VAL_HDMI_PIXEL_TOTAL           0
#define RSAVPHA_VAL_HDMI_PIXEL_ACTIVE          1
#define RSAVPHA_VAL_HDMI_PIXEL_FRONT_PORCH     2
#define RSAVPHA_VAL_HDMI_PIXEL_BACK_PORCH      3
#define RSAVPHA_VAL_HDMI_PIXEL_SYNC            4
#define RSAVPHA_VAL_HDMI_PIXEL_SYNC_POLARITY   5
	
#define RSAVPHA_VAL_HDMI_AUDIO_APPLICATION 0
#define RSAVPHA_VAL_HDMI_AUDIO_SAMPLE          1
#define RSAVPHA_VAL_HDMI_AUDIO_COPYRIGHT       2
#define RSAVPHA_VAL_HDMI_AUDIO_PREEMPHASIS     3
#define RSAVPHA_VAL_HDMI_AUDIO_MODE            4
#define RSAVPHA_VAL_HDMI_AUDIO_CATEGORY_CODE   5
#define RSAVPHA_VAL_HDMI_AUDIO_SOURCE_NUMBER   6
#define RSAVPHA_VAL_HDMI_AUDIO_CHANNEL_NUMBER  7
#define RSAVPHA_VAL_HDMI_AUDIO_FS              8
#define RSAVPHA_VAL_HDMI_AUDIO_CLOCK_ACCURACY  9
#define RSAVPHA_VAL_HDMI_AUDIO_MAX_LENGTH      10
#define RSAVPHA_VAL_HDMI_AUDIO_LENGTH          11
	
#define RSAVPHA_VAL_HDMI_INFO_FRAME_RGB    0
#define RSAVPHA_VAL_HDMI_INFO_FRAME_AINFO  1
#define RSAVPHA_VAL_HDMI_INFO_FRAME_BINFO  2
#define RSAVPHA_VAL_HDMI_INFO_FRAME_SINFO  3
#define RSAVPHA_VAL_HDMI_INFO_FRAME_COL    4
#define RSAVPHA_VAL_HDMI_INFO_FRAME_ARAT   5
#define RSAVPHA_VAL_HDMI_INFO_FRAME_AFAR   6
#define RSAVPHA_VAL_HDMI_INFO_FRAME_ITC    7
#define RSAVPHA_VAL_HDMI_INFO_FRAME_ECOL   8
#define RSAVPHA_VAL_HDMI_INFO_FRAME_RGBQ   9
#define RSAVPHA_VAL_HDMI_INFO_FRAME_NUPS   10
#define RSAVPHA_VAL_HDMI_INFO_FRAME_VIDC   11
#define RSAVPHA_VAL_HDMI_INFO_FRAME_YCCQ   12
#define RSAVPHA_VAL_HDMI_INFO_FRAME_CTYP   13
#define RSAVPHA_VAL_HDMI_INFO_FRAME_PREP   14

#define RSAVPHA_VAL_HDMI_INFO_FRAME_CODING_TYPE  0
#define RSAVPHA_VAL_HDMI_INFO_FRAME_CHAN_COUNT   1
#define RSAVPHA_VAL_HDMI_INFO_FRAME_SAMP_FREQ    2
#define RSAVPHA_VAL_HDMI_INFO_FRAME_SAMP_SIZE    3
#define RSAVPHA_VAL_HDMI_INFO_FRAME_CODE_EXT     4
#define RSAVPHA_VAL_HDMI_INFO_FRAME_CHAN_ALLOC   5
#define RSAVPHA_VAL_HDMI_INFO_FRAME_DMIX_INHIB   6
#define RSAVPHA_VAL_HDMI_INFO_FRAME_LEVEL_SHIFT  7
#define RSAVPHA_VAL_HDMI_INFO_FRAME_LFE          8
	
#define	RSAVPHA_VAL_HDMI_INFO_FRAME_VENDOR   0
#define RSAVPHA_VAL_HDMI_INFO_FRAME_PRODUCT  1

#define RSAVPHA_VAL_HDMI_TEST_CASE_EDID      0
#define RSAVPHA_VAL_HDMI_TEST_CASE_LCODES    1
#define RSAVPHA_VAL_HDMI_TEST_CASE_BASPROT   2
#define RSAVPHA_VAL_HDMI_TEST_CASE_CPERIOD   3
#define RSAVPHA_VAL_HDMI_TEST_CASE_PACKET    4
#define RSAVPHA_VAL_HDMI_TEST_CASE_PIXELR    5
#define RSAVPHA_VAL_HDMI_TEST_CASE_PIXELY    6
#define RSAVPHA_VAL_HDMI_TEST_CASE_VFORMAT   7
#define RSAVPHA_VAL_HDMI_TEST_CASE_PREP      8
#define RSAVPHA_VAL_HDMI_TEST_CASE_AVI       9
#define RSAVPHA_VAL_HDMI_TEST_CASE_IEC6     10
#define RSAVPHA_VAL_HDMI_TEST_CASE_ACR      11
#define RSAVPHA_VAL_HDMI_TEST_CASE_ASPJITT  12
#define RSAVPHA_VAL_HDMI_TEST_CASE_AUDIOI   13
#define RSAVPHA_VAL_HDMI_TEST_CASE_ASPLAY   14
#define RSAVPHA_VAL_HDMI_TEST_CASE_DVI      15
#define RSAVPHA_VAL_HDMI_TEST_CASE_DEEPC    16
#define RSAVPHA_VAL_HDMI_TEST_CASE_GMTRANS  17
#define RSAVPHA_VAL_HDMI_TEST_CASE_HBAUDIO  18
#define RSAVPHA_VAL_HDMI_TEST_CASE_LBAUDIO  19
#define RSAVPHA_VAL_HDMI_TEST_CASE_3DVFMT   20
#define RSAVPHA_VAL_HDMI_TEST_CASE_4K2KVFT  21
#define RSAVPHA_VAL_HDMI_TEST_CASE_ECTRANS  22
#define RSAVPHA_VAL_HDMI_TEST_CASE_CONC	  23

#define RSAVPHA_VAL_HDMI_STOP    0
#define RSAVPHA_VAL_HDMI_START   1

#define RSAVPHA_VAL_HDMI_DATA_LOAD  0
#define RSAVPHA_VAL_HDMI_DATA_SAVE  1

#define RSAVPHA_VAL_TEST_SETUP_AI0  0
#define RSAVPHA_VAL_TEST_SETUP_AI1  1

#define RSAVPHA_VAL_HDMI_TEST_LENGTH_2_SECONDS  0
#define RSAVPHA_VAL_HDMI_TEST_LENGTH_FRAMES     1

#define RSAVPHA_VAL_CDF_TEST_Y  0
#define RSAVPHA_VAL_CDF_TEST_N  1

#define RSAVPHA_VAL_TEST_SETUP_PIXEL_ENCONDING  0
#define RSAVPHA_VAL_TEST_SETUP_YQ               1

#define RSAVPHA_VAL_VIDEO_FORMAT_PROGRESSSIVE  0
#define RSAVPHA_VAL_VIDEO_FORMAT_INTERLACED    1

#define RSAVPHA_VAL_CONTENT_PHOTO    0
#define RSAVPHA_VAL_CONTENT_CINEMA   1
#define RSAVPHA_VAL_CONTENT_GAME     2

#define RSAVPHA_VAL_HDMI_VSBD_DVI   0
#define RSAVPHA_VAL_HDMI_VSBD_EQ5   1
#define RSAVPHA_VAL_HDMI_VSBD_GT5   2

#define RSAVPHA_VAL_RST_BASE  0
#define RSAVPHA_VAL_RST_ALL   1

#define RSAVPHA_VAL_PIXEL_ENCODING_Y422  1
#define RSAVPHA_VAL_PIXEL_ENCODING_Y444  2
	
#define RSAVPHA_VAL_HDMI_HDCP_STATE_INVALID                                 0
#define RSAVPHA_VAL_HDMI_HDCP_STATE_ACTIVE                                  1
#define RSAVPHA_VAL_HDMI_HDCP_STATE_INACTIVE                                2
#define RSAVPHA_VAL_HDMI_HDCP_STATE_UNKNOWN                                 3
	
/****************************************************************************
*---------------- Instrument Driver Function Declarations -----------------*
 ****************************************************************************/

ViStatus _VI_FUNC rsavpha_init (ViRsrc resourceName, ViBoolean IDQuery,
                                ViBoolean resetDevice, ViSession* instrumentHandle);
ViStatus _VI_FUNC rsavpha_InitWithOptions (ViRsrc resourceName, ViBoolean IDQuery,
                                           ViBoolean resetDevice,
                                           ViChar _VI_FAR optionString[],
                                           ViSession* instrumentHandle);
ViStatus _VI_FUNC rsavpha_ConfigureHDMIAnalyzerInput (ViSession instrumentHandle,
                                                      ViInt32 board, ViString input,
                                                      ViBoolean HPDSignal,
                                                      ViInt32 device);
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerInput (ViSession instrumentHandle,
                                                  ViInt32 board, ViInt32 bufferSize,
                                                  ViInt32* pixelClockLocked,
                                                  ViInt32* HDCP,
                                                  ViReal64* pixelClock,
                                                  ViChar _VI_FAR videoCode[],
                                                  ViChar _VI_FAR audioCoding[]);
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerCatalog (ViSession instrumentHandle,
                                                    ViInt32 board,
                                                    ViInt32 bufferSize,
                                                    ViChar _VI_FAR catalog[]);
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerVideo (ViSession instrumentHandle,
                                                  ViInt32 board, ViInt32 bufferSize,
                                                  ViChar _VI_FAR videoCode[],
                                                  ViReal64* pixelClock,
                                                  ViInt32* videoFormat,
                                                  ViReal64* verticalFrequency,
                                                  ViReal64* horizontalFrequency);
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerVideoPixels (ViSession instrumentHandle,
                                                        ViInt32 board,
                                                        ViInt32 videoParameter,
                                                        ViInt32 parameter,
                                                        ViInt32* pixelValue);
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerAudio (ViSession instrumentHandle,
                                                  ViInt32 board, ViInt32 bufferSize,
                                                  ViInt32* n, ViInt32* CTS,
                                                  ViChar _VI_FAR audioCoding[],
                                                  ViChar _VI_FAR layout[]);
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerAudioChannelStatusBits
             (ViSession instrumentHandle, ViInt32 board, ViInt32 parameter,
              ViInt32 bufferSize, ViChar _VI_FAR result[],
              ViChar _VI_FAR channelStatusBit[]);
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerHDCP (ViSession instrumentHandle,
                                                 ViInt32 board, ViInt32 bufferSize,
                                                 ViChar _VI_FAR authentication[],
                                                 ViInt32* HDCP, ViInt64* AKSV,
                                                 ViInt64* BKSV, ViInt32* ri,
                                                 ViInt32 _VI_FAR an[]);
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerInfoFrameAVI (ViSession instrumentHandle,
                                                         ViInt32 board,
                                                         ViInt32* version,
                                                         ViInt32* length,
                                                         ViInt32* checksum);
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerInfoFrameAVISpecific
             (ViSession instrumentHandle, ViInt32 board, ViInt32 parameter,
              ViInt32 bufferSize, ViChar _VI_FAR result[]);
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerInfoFrameAVIBar
             (ViSession instrumentHandle, ViInt32 board,
              ViInt32* lineNumberOfTopBarEnd, ViInt32* lineNumberOfBottomBarStart,
              ViInt32* pixelNumberOfLeftBarEnd,
              ViInt32* pixelNumberOfRightBarStart);
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerInfoFrameAVIHeaderPacketData
             (ViSession instrumentHandle, ViInt32 board, ViInt32 arraySize,
              ViInt32 _VI_FAR headerBytes[], ViInt32 _VI_FAR packetBytes[]);
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerInfoFrameAudio
             (ViSession instrumentHandle, ViInt32 board, ViInt32* version,
              ViInt32* length, ViInt32* checksum);
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerInfoFrameAudioSpecific
             (ViSession instrumentHandle, ViInt32 board, ViInt32 parameter,
              ViInt32 bufferSize, ViChar _VI_FAR result[]);
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerInfoFrameAudioHeaderPacketData
             (ViSession instrumentHandle, ViInt32 board, ViInt32 arraySize,
              ViInt32 _VI_FAR headerBytes[], ViInt32 _VI_FAR packetBytes[]);
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerInfoFrameSPD (ViSession instrumentHandle,
                                                         ViInt32 board,
                                                         ViInt32* version,
                                                         ViInt32* length,
                                                         ViInt32* checksum);
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerInfoFrameSPDSpecific
             (ViSession instrumentHandle, ViInt32 board, ViInt32 parameter,
              ViInt32 bufferSize, ViChar _VI_FAR result[]);
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerInfoFrameSPDHeaderPacketData
             (ViSession instrumentHandle, ViInt32 board, ViInt32 arraySize,
              ViInt32 _VI_FAR headerBytes[], ViInt32 _VI_FAR packetBytes[]);
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerInfoFrameMPEGSource
             (ViSession instrumentHandle, ViInt32 board, ViInt32* version,
              ViInt32* length, ViInt32* checksum);
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerInfoFrameMPEGSourceSpecific
             (ViSession instrumentHandle, ViInt32 board, ViInt32 bufferSize,
              ViInt32* MPEGBitRate, ViChar _VI_FAR fieldRepeat[],
              ViChar _VI_FAR MPEGFrame[]);
ViStatus 
    _VI_FUNC rsavpha_QueryHDMIAnalyzerInfoFrameMPEGSourceHeaderPacketData
        (ViSession instrumentHandle, ViInt32 board, ViInt32 arraySize,
         ViInt32 _VI_FAR headerBytes[], ViInt32 _VI_FAR packetBytes[]);
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerInfoFrameVendor
             (ViSession instrumentHandle, ViInt32 board, ViInt32* version,
              ViInt32* length, ViInt32* checksum, ViInt32* IEEERegistrationID);
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerInfoFrameVendorSpecific
             (ViSession instrumentHandle, ViInt32 board, ViInt32 bufferSize,
              ViChar _VI_FAR HDMIVideoFormat[], ViChar _VI_FAR HDMIPixelEncoding[],
              ViChar _VI_FAR HDMIVIC[], ViChar _VI_FAR _3DStructure[],
              ViChar _VI_FAR _3DExtData[]);
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerInfoFrameVendorHeaderPacketData
             (ViSession instrumentHandle, ViInt32 board, ViInt32 arraySize,
              ViInt32 _VI_FAR headerBytes[], ViInt32 _VI_FAR packetBytes[]);
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerCTSState (ViSession instrumentHandle,
                                                     ViInt32 board, ViInt32* state);
ViStatus _VI_FUNC rsavpha_HDMIAnalyzerCTSOperation (ViSession instrumentHandle,
                                                    ViInt32 board,
                                                    ViInt32 operation);
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerCTSProgress (ViSession instrumentHandle,
                                                        ViInt32 board,
                                                        ViInt32* progress);
ViStatus _VI_FUNC rsavpha_SelectHDMIAnalyzerCTSTestCase (ViSession instrumentHandle,
                                                         ViInt32 board,
                                                         ViInt32 testCase,
                                                         ViBoolean state);
ViStatus _VI_FUNC rsavpha_ConfigureHDMIAnalyzerCTSProtocol
             (ViSession instrumentHandle, ViInt32 board, ViInt32 testCase,
              ViInt32 testSetup, ViInt32 testLength, ViInt32 numberOfFrames);
ViStatus _VI_FUNC rsavpha_ConfigureHDMIAnalyzerCTSVideoModes
             (ViSession instrumentHandle, ViInt32 board, ViInt32 testCase,
              ViInt32 testSetup, ViInt32 CDFSetup, ViInt32 pixelEncoding,
              ViInt32 videoFormat, ViInt32 contentType, ViInt32 testLength,
              ViInt32 numberOfFrames);
ViStatus _VI_FUNC rsavpha_ConfigureHDMIAnalyzerCTSAudioTests
             (ViSession instrumentHandle, ViInt32 board, ViInt32 testCase,
              ViInt32 testLength, ViInt32 numberOfFrames);
ViStatus _VI_FUNC rsavpha_ConfigureHDMIAnalyzerCTSInteroperabilityWithDVI
             (ViSession instrumentHandle, ViInt32 board, ViInt32 testCase,
              ViInt32 HDMIVSDB, ViInt32 testLength, ViInt32 numberOfFrames);
ViStatus _VI_FUNC rsavpha_ConfigureHDMIAnalyzerCTSAdvanced
             (ViSession instrumentHandle, ViInt32 board, ViInt32 testCase,
              ViInt32 videoFormat, ViInt32 HDMIVSDB, ViInt32 testLength,
              ViInt32 numberOfFrames);
ViStatus _VI_FUNC rsavpha_ConfigureHDMIAnalyzerCTSDataSource
             (ViSession instrumentHandle, ViInt32 board, ViInt32 dataSource,
              ViInt32 mode, ViInt32 numberOfFrames);
ViStatus _VI_FUNC rsavpha_HDMIAnalyzerCTSDataSourceLoadSave
             (ViSession instrumentHandle, ViInt32 board, ViInt32 operation,
              ViString fileName);
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerCTSTestOutputReportCount
             (ViSession instrumentHandle, ViInt32 board, ViInt32* count);
ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerCTSTestOutputReport
             (ViSession instrumentHandle, ViInt32 board, ViInt32 index,
              ViInt32 arraySize, ViInt32* lineNumber, ViInt32* referenceNumber,
              ViChar _VI_FAR textMessage[], ViInt32 _VI_FAR optionalParameters[]);

ViStatus _VI_FUNC rsavpha_ConfigureHDMIAnalyzerCECDestination (ViSession instrumentHandle,
                                                               ViInt32 board,
                                                               ViInt32 destination);

ViStatus _VI_FUNC rsavpha_ConfigureHDMIAnalyzerCECOperationCode (ViSession instrumentHandle,
                                                                 ViInt32 board,
                                                                 ViInt32 operationCode);

ViStatus _VI_FUNC rsavpha_ConfigureHDMIAnalyzerCECOperationCodeParameterLength (ViSession instrumentHandle,
                                                                                ViInt32 board,
                                                                                ViInt32 operationCode,
                                                                                ViInt32 parameter);

ViStatus _VI_FUNC rsavpha_ConfigureHDMIAnalyzerCECOperationCodeParameter (ViSession instrumentHandle,
                                                                          ViInt32 board,
                                                                          ViInt32 operationCode,
                                                                          ViInt32 index,
                                                                          ViInt32 parameter);

ViStatus _VI_FUNC rsavpha_HDMIAnalyzerCECOperation (ViSession instrumentHandle,
                                                    ViInt32 board,
                                                    ViInt32 operation);

ViStatus _VI_FUNC rsavpha_HDMIAnalyzerCECSend (ViSession instrumentHandle,
                                               ViInt32 board);

ViStatus _VI_FUNC rsavpha_HDMIAnalyzerCECDataSave (ViSession instrumentHandle,
                                                   ViInt32 board,
                                                   ViString fileName);

ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerCECTestOutputReportCount (ViSession instrumentHandle,
                                                                     ViInt32 board,
                                                                     ViInt32 *count);

ViStatus _VI_FUNC rsavpha_QueryHDMIAnalyzerCECTestOutputReport (ViSession instrumentHandle,
                                                                ViInt32 board,
                                                                ViInt32 index,
                                                                ViInt32 textMessageBufferSize,
                                                                ViInt32 arraySize,
                                                                ViInt32 *lineNumber,
                                                                ViInt32 *referenceNumber,
                                                                ViChar textMessage[],
                                                                ViInt32 optionalParameters[],
                                                                ViInt32 *returnedValues);

ViStatus _VI_FUNC rsavpha_reset (ViSession instrumentHandle);
ViStatus _VI_FUNC rsavpha_ResetSystem (ViSession instrumentHandle,
                                       ViInt32 reference);
ViStatus _VI_FUNC rsavpha_Preset (ViSession instrumentHandle, ViInt32 reference);
ViStatus _VI_FUNC rsavpha_self_test (ViSession instrumentHandle,
                                     ViInt16* selfTestResult,
                                     ViChar _VI_FAR selfTestMessage[]);
ViStatus _VI_FUNC rsavpha_revision_query (ViSession instrumentHandle,
                                          ViChar _VI_FAR instrumentDriverRevision[],
                                          ViChar _VI_FAR firmwareRevision[]);
ViStatus _VI_FUNC rsavpha_error_query (ViSession instrumentHandle,
                                       ViInt32* errorCode,
                                       ViChar _VI_FAR errorMessage[]);
ViStatus _VI_FUNC rsavpha_error_message (ViSession instrumentHandle,
                                         ViStatus errorCode,
                                         ViChar _VI_FAR errorMessage[]);
ViStatus _VI_FUNC rsavpha_SystemVersion (ViSession instrumentHandle, ViInt32 length,
                                         ViChar _VI_FAR systemVersion[]);
ViStatus _VI_FUNC rsavpha_setCheckStatus (ViSession instrumentHandle,
                                          ViBoolean state);
ViStatus _VI_FUNC rsavpha_ConfigureDisplayUpdate (ViSession instrumentHandle,
                                                  ViBoolean displayUpdate);
ViStatus _VI_FUNC rsavpha_GetError (ViSession instrumentHandle, ViStatus* code,
                                    ViInt32 bufferSize,
                                    ViChar _VI_FAR description[]);
ViStatus _VI_FUNC rsavpha_ClearError (ViSession instrumentHandle);
ViStatus _VI_FUNC rsavpha_WriteInstrData (ViSession instrumentHandle,
                                          ViChar _VI_FAR writeBuffer[]);
ViStatus _VI_FUNC rsavpha_ReadInstrData (ViSession instrumentHandle,
                                         ViInt32 numberOfBytesToRead,
                                         ViChar _VI_FAR readBuffer[],
                                         ViUInt32* numBytesRead);
ViStatus _VI_FUNC rsavpha_QueryViBoolean (ViSession instrumentHandle,
                                          ViString command, ViBoolean* value);
ViStatus _VI_FUNC rsavpha_QueryViInt32 (ViSession instrumentHandle,
                                        ViString command, ViInt32* value);
ViStatus _VI_FUNC rsavpha_QueryViReal64 (ViSession instrumentHandle,
                                         ViString command, ViReal64* value);
ViStatus _VI_FUNC rsavpha_QueryViString (ViSession instrumentHandle,
                                         ViString command, ViInt32 bufferSize,
                                         ViChar _VI_FAR value[]);
ViStatus _VI_FUNC rsavpha_SetAttributeViInt32 (ViSession instrumentHandle,
                                               ViChar _VI_FAR channelName[],
                                               ViAttr attributeID,
                                               ViInt32 attributeValue);
ViStatus _VI_FUNC rsavpha_SetAttributeViInt64 (ViSession instrumentHandle,
                                               ViChar _VI_FAR repeatedCapabilityName[],
                                               ViAttr attributeID,
                                               ViInt64 attributeValue);
ViStatus _VI_FUNC rsavpha_SetAttributeViReal64 (ViSession instrumentHandle,
                                                ViChar _VI_FAR channelName[],
                                                ViAttr attributeID,
                                                ViReal64 attributeValue);
ViStatus _VI_FUNC rsavpha_SetAttributeViString (ViSession instrumentHandle,
                                                ViChar _VI_FAR channelName[],
                                                ViAttr attributeID,
                                                ViChar _VI_FAR attributeValue[]);
ViStatus _VI_FUNC rsavpha_SetAttributeViBoolean (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViBoolean attributeValue);
ViStatus _VI_FUNC rsavpha_SetAttributeViSession (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViSession attributeValue);
ViStatus _VI_FUNC rsavpha_GetAttributeViInt32 (ViSession instrumentHandle,
                                               ViChar _VI_FAR channelName[],
                                               ViAttr attributeID,
                                               ViInt32* attributeValue);
ViStatus _VI_FUNC rsavpha_GetAttributeViInt64 (ViSession instrumentHandle,
                                               ViChar _VI_FAR repeatedCapabilityName[],
                                               ViAttr attributeID,
                                               ViInt64* attributeValue);
ViStatus _VI_FUNC rsavpha_GetAttributeViReal64 (ViSession instrumentHandle,
                                                ViChar _VI_FAR channelName[],
                                                ViAttr attributeID,
                                                ViReal64* attributeValue);
ViStatus _VI_FUNC rsavpha_GetAttributeViString (ViSession instrumentHandle,
                                                ViChar _VI_FAR channelName[],
                                                ViAttr attributeID,
                                                ViInt32 bufferSize,
                                                ViChar _VI_FAR attributeValue[]);
ViStatus _VI_FUNC rsavpha_GetAttributeViBoolean (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViBoolean* attributeValue);
ViStatus _VI_FUNC rsavpha_GetAttributeViSession (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViSession* attributeValue);
ViStatus _VI_FUNC rsavpha_CheckAttributeViInt32 (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViInt32 attributeValue);
ViStatus _VI_FUNC rsavpha_CheckAttributeViInt64 (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViInt64 attributeValue);
ViStatus _VI_FUNC rsavpha_CheckAttributeViReal64 (ViSession instrumentHandle,
                                                  ViChar _VI_FAR channelName[],
                                                  ViAttr attributeID,
                                                  ViReal64 attributeValue);
ViStatus _VI_FUNC rsavpha_CheckAttributeViString (ViSession instrumentHandle,
                                                  ViChar _VI_FAR channelName[],
                                                  ViAttr attributeID,
                                                  ViChar _VI_FAR attributeValue[]);
ViStatus _VI_FUNC rsavpha_CheckAttributeViBoolean (ViSession instrumentHandle,
                                                   ViChar _VI_FAR channelName[],
                                                   ViAttr attributeID,
                                                   ViBoolean attributeValue);
ViStatus _VI_FUNC rsavpha_CheckAttributeViSession (ViSession instrumentHandle,
                                                   ViChar _VI_FAR channelName[],
                                                   ViAttr attributeID,
                                                   ViSession attributeValue);
ViStatus _VI_FUNC rsavpha_GetAttributeRepeatedCapabilityIds
             (ViSession instrumentHandle, ViUInt32 attributeID, ViInt32 bufferSize,
              ViChar _VI_FAR repeatedCapabilityId_s[]);
ViStatus _VI_FUNC rsavpha_GetAttributeRepeatedCapabilityIdNames
             (ViSession instrumentHandle, ViUInt32 attributeID,
              ViString repeatedCapabilityId, ViInt32 bufferSize,
              ViChar _VI_FAR repeatedCapabilityIdName_s[]);
ViStatus _VI_FUNC rsavpha_close (ViSession instrumentHandle);

/****************************************************************************
 *------------------------ Error And Completion Codes ----------------------*
 ****************************************************************************/

#define RSAVPHA_WARN_MEASURE_UNCALIBRATED    (RS_CLASS_WARN_BASE + 0x0001L)
#define RSAVPHA_WARN_OVER_RANGE              (RS_CLASS_WARN_BASE + 0x0002L)

#define RSAVPHA_ERROR_MARKER_NOT_ENABLED     (RS_CLASS_ERROR_BASE + 0x0003L)
#define RSAVPHA_ERROR_NOT_DELTA_MARKER       (RS_CLASS_ERROR_BASE + 0x0002L)
#define RSAVPHA_ERROR_MAX_TIME_EXCEEDED      (RS_CLASS_ERROR_BASE + 0x0001L)

#define RSAVPHA_WARNMSG_MEASURE_UNCALIBRATED "Uncalibrated measurement"
#define RSAVPHA_WARNMSG_OVER_RANGE           "Measurement overrange"

#define RSAVPHA_ERRMSG_MARKER_NOT_ENABLED    "Marker Not Enabled"
#define RSAVPHA_ERRMSG_NOT_DELTA_MARKER      "Not Delta Marker"
#define RSAVPHA_ERRMSG_MAX_TIME_EXCEEDED     "Max Time Exceeded"

#define RSAVPHA_ERROR_CODES_AND_MSGS \
    {RSAVPHA_WARN_MEASURE_UNCALIBRATED,  RSAVPHA_WARNMSG_MEASURE_UNCALIBRATED},\
    {RSAVPHA_WARN_OVER_RANGE,            RSAVPHA_WARNMSG_OVER_RANGE},\
    {RSAVPHA_ERROR_MARKER_NOT_ENABLED,   RSAVPHA_ERRMSG_MARKER_NOT_ENABLED},\
    {RSAVPHA_ERROR_NOT_DELTA_MARKER,     RSAVPHA_ERRMSG_NOT_DELTA_MARKER},\
    {RSAVPHA_ERROR_MAX_TIME_EXCEEDED,    RSAVPHA_ERRMSG_MAX_TIME_EXCEEDED}

/****************************************************************************
 *---------------------------- End Include File ----------------------------*
 ****************************************************************************/

#if defined(__cplusplus) || defined(__cplusplus__)
}
#endif
#endif /* __RSAVPHA_HEADER */



