/****************************************************************************
 *
 *  Rohde & Schwarz Audio Video Platform HDMI Generator instrument driver include file
 *
 ****************************************************************************/

#ifndef __RSAVPHG_HEADER
#define __RSAVPHG_HEADER

#include "rsidr_core.h"
#include "rsavphg_attributes.h"
#include "rsavphg_utility.h"

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#if defined(__cplusplus) || defined(__cplusplus__)
extern "C" {
#endif

/*****************************************************************************
 *- Useful Macros & Constants -----------------------------------------------*
 *****************************************************************************/
#define RSAVPHG_SIMULATION_ID_QUERY      "Rohde&Schwarz,R&S VTE,2115.7300k02/900000,1.45.0.0"  /* Default definition of ID Query Response for simulation */
#define RSAVPHG_VALID_ID_RESPONSE_STRING "Rohde&Schwarz"      /* Valid response for identification query */
#define RSAVPHG_SIMULATION_OPT_QUERY     ""      /* Simulated response for *OPT? command */

#define RSAVPHG_IO_BUFFER_SIZE           1024L   /* I/O buffer size */
#define RSAVPHG_OPC_TIMEOUT              5000L   /* Maximum time to wait for OPC in milliseconds */
#define IEEE_488_2_ERROR_MASK             0x4    /* 488.2 STB Bits */   
#define IEEE_488_2_ESE_MASK               0x3D    /* 488.2 Event Status Register (ESE) Bits */
#define RSAVPHG_NAN                 9.91e+37
#define RSAVPHG_NAN_VI_INT32             0x80000000     
/****************************************************************************
 *----------------- Instrument Driver Revision Information -----------------*
 ****************************************************************************/

/* Class Driver Identification */

#define RSAVPHG_CLASS_DRIVER_DESCRIPTION           ""
#define RSAVPHG_CLASS_DRIVER_PREFIX                ""
#define RSAVPHG_CLASS_DRIVER_VENDOR                ""
#define RSAVPHG_CLASS_DRIVER_REVISION              ""
#define RSAVPHG_CLASS_SPEC_MAJOR_VERSION           1L    /* Class specification major version */
#define RSAVPHG_CLASS_SPEC_MINOR_VERSION           0L    /* Class specification minor version */

/* Driver Identification */

#define RSAVPHG_SPECIFIC_DRIVER_DESCRIPTION        ""
#define RSAVPHG_SPECIFIC_DRIVER_PREFIX             "RSAVPHG"
#define RSAVPHG_SPECIFIC_DRIVER_LOCATOR            ""
#define RSAVPHG_SPECIFIC_DRIVER_VENDOR             ""
#define RSAVPHG_MAJOR_VERSION                      1L    /* Instrument driver major version          */
#define RSAVPHG_MINOR_VERSION                      1L    /* Instrument driver minor version          */
#define RSAVPHG_MINOR_MINOR_VERSION                0L    /* Instrument driver minor minor version    */

/* Driver Capabilities */          

#define RSAVPHG_SUPPORTED_INSTRUMENT_MODELS        ""    /* Instrument driver supported model(s)     */
#define RSAVPHG_GROUP_CAPABILITIES                 ""
#define RSAVPHG_FUNCTION_CAPABILITIES              ""

/****************************************************************************
 *---------------------------- Attribute Defines ---------------------------*
 ****************************************************************************/

    /*- Inherent Instrument Specific Attributes ----------------------------*/

        /* User Options */

#define RSAVPHG_ATTR_RANGE_CHECK                                           RS_ATTR_RANGE_CHECK                                     /* ViBoolean */
#define RSAVPHG_ATTR_QUERY_INSTRUMENT_STATUS                               RS_ATTR_QUERY_INSTRUMENT_STATUS                         /* ViBoolean */
#define RSAVPHG_ATTR_CACHE                                                 RS_ATTR_CACHE                                           /* ViBoolean */
#define RSAVPHG_ATTR_SIMULATE                                              RS_ATTR_SIMULATE                                        /* ViBoolean */
#define RSAVPHG_ATTR_RECORD_COERCIONS                                      RS_ATTR_RECORD_COERCIONS                                /* ViBoolean */
#define RSAVPHG_ATTR_INTERCHANGE_CHECK                                     RS_ATTR_INTERCHANGE_CHECK                               /* ViBoolean */
#define RSAVPHG_ATTR_SPY                                                   RS_ATTR_SPY                                             /* ViBoolean */
#define RSAVPHG_ATTR_USE_SPECIFIC_SIMULATION                               RS_ATTR_USE_SPECIFIC_SIMULATION                         /* ViBoolean */

        /* Class Driver Identification */

#define RSAVPHG_ATTR_CLASS_DRIVER_DESCRIPTION                              RS_ATTR_CLASS_DRIVER_DESCRIPTION                        /* ViString, read-only */
#define RSAVPHG_ATTR_CLASS_DRIVER_PREFIX                                   RS_ATTR_CLASS_DRIVER_PREFIX                             /* ViString, read-only */
#define RSAVPHG_ATTR_CLASS_DRIVER_VENDOR                                   RS_ATTR_CLASS_DRIVER_VENDOR                             /* ViString, read-only */
#define RSAVPHG_ATTR_CLASS_DRIVER_REVISION                                 RS_ATTR_CLASS_DRIVER_REVISION                           /* ViString, read-only */
#define RSAVPHG_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION           /* ViInt32,  read-only */
#define RSAVPHG_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION           /* ViInt32,  read-only */

        /* Driver Identification */

#define RSAVPHG_ATTR_SPECIFIC_DRIVER_DESCRIPTION                           RS_ATTR_SPECIFIC_DRIVER_DESCRIPTION                     /* ViString, read-only */
#define RSAVPHG_ATTR_SPECIFIC_DRIVER_PREFIX                                RS_ATTR_SPECIFIC_DRIVER_PREFIX                          /* ViString, read-only */
#define RSAVPHG_ATTR_SPECIFIC_DRIVER_LOCATOR                               RS_ATTR_SPECIFIC_DRIVER_LOCATOR                         /* ViString, read-only */
#define RSAVPHG_ATTR_SPECIFIC_DRIVER_VENDOR                                RS_ATTR_SPECIFIC_DRIVER_VENDOR                          /* ViString, read-only */
#define RSAVPHG_ATTR_SPECIFIC_DRIVER_REVISION                              RS_ATTR_SPECIFIC_DRIVER_REVISION                        /* ViString, read-only */
#define RSAVPHG_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION        /* ViInt32,  read-only */
#define RSAVPHG_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION        /* ViInt32,  read-only */

        /* Driver Capabilities */

#define RSAVPHG_ATTR_SUPPORTED_INSTRUMENT_MODELS                           RS_ATTR_SUPPORTED_INSTRUMENT_MODELS                     /* ViString, read-only */
#define RSAVPHG_ATTR_GROUP_CAPABILITIES                                    RS_ATTR_GROUP_CAPABILITIES                              /* ViString, read-only */
#define RSAVPHG_ATTR_FUNCTION_CAPABILITIES                                 RS_ATTR_FUNCTION_CAPABILITIES                           /* ViString, read-only */
#define RSAVPHG_ATTR_CHANNEL_COUNT                                         RS_ATTR_CHANNEL_COUNT                                   /* ViInt32,  read-only */

        /* Driver Setup */

#define RSAVPHG_ATTR_DRIVER_SETUP                                          RS_ATTR_DRIVER_SETUP                                    /* ViString            */

        /* Instrument Identification */

#define RSAVPHG_ATTR_INSTRUMENT_MANUFACTURER                               RS_ATTR_INSTRUMENT_MANUFACTURER                         /* ViString, read-only */
#define RSAVPHG_ATTR_INSTRUMENT_MODEL                                      RS_ATTR_INSTRUMENT_MODEL                                /* ViString, read-only */
#define RSAVPHG_ATTR_INSTRUMENT_FIRMWARE_REVISION                          RS_ATTR_INSTRUMENT_FIRMWARE_REVISION                    /* ViString, read-only */

        /* Advanced Session Information */

#define RSAVPHG_ATTR_IO_RESOURCE_DESCRIPTOR                                RS_ATTR_IO_RESOURCE_DESCRIPTOR                          /* ViString, read-only */
#define RSAVPHG_ATTR_LOGICAL_NAME                                          RS_ATTR_LOGICAL_NAME

/* ViString, read-only */

    /*- Instrument-Specific Attributes -------------------------------------*/

//#define RSAVPHG_ATTR_ID_QUERY_RESPONSE                                     (RS_SPECIFIC_PUBLIC_ATTR_BASE + 1L)                     /* ViString (Read Only)              */

/* BEGIN GENERATE */
#define RSAVPHG_ATTR_HDMI_GENERATOR_HDMI_OUTPUT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 44
#define RSAVPHG_ATTR_HDMI_GENERATOR_HDCP                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 45
#define RSAVPHG_ATTR_HDMI_GENERATOR_HOT_PLUG_DETECT_STATE                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 48
#define RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_PATTERN                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1
#define RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_COMMAND                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2
#define RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_CODE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3
#define RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_CODE_HDMI                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5
#define RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_CODE_VESA                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6
#define RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_STATUS                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 4
#define RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_FRAME_RATE                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7
#define RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_ASPECT_RATIO               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8
#define RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_PIXEL_ENCODING             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9
#define RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_SIGNALING_MODE             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10
#define RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_DEEP_COLOR                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 11
#define RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_ADJUSTMENT                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 13
#define RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_RESOLUTION_CLIPPING        RS_SPECIFIC_PUBLIC_ATTR_BASE + 14
#define RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_OFFSET                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 15
#define RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_RGB_OVERALL_AMPLIFICATION  RS_SPECIFIC_PUBLIC_ATTR_BASE + 16
#define RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_YCBCR_OVERALL_AMPLIFICATION RS_SPECIFIC_PUBLIC_ATTR_BASE + 17
#define RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_CHROMA_AMPLIFICATION       RS_SPECIFIC_PUBLIC_ATTR_BASE + 18
#define RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_PIXEL_SHIFT_VERTICAL_STATE RS_SPECIFIC_PUBLIC_ATTR_BASE + 19
#define RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_PIXEL_SHIFT_VERTICAL_VALUE RS_SPECIFIC_PUBLIC_ATTR_BASE + 20
#define RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_RED_LEVEL                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 21
#define RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_GREEN_LEVEL                RS_SPECIFIC_PUBLIC_ATTR_BASE + 22
#define RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_BLUE_LEVEL                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 23
#define RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_LUMA_Y_LEVEL               RS_SPECIFIC_PUBLIC_ATTR_BASE + 24
#define RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_CB_PB_LEVEL                RS_SPECIFIC_PUBLIC_ATTR_BASE + 25
#define RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_CR_PR_LEVEL                RS_SPECIFIC_PUBLIC_ATTR_BASE + 26
#define RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_DEFAULT_SETTINGS           RS_SPECIFIC_PUBLIC_ATTR_BASE + 27
#define RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_AUDIO_SELECTION                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 28
#define RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_AUDIO_SAMPLING_RATE              RS_SPECIFIC_PUBLIC_ATTR_BASE + 29
#define RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_AUDIO_LEVEL                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 30
#define RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_AUDIO_RESOLUTION                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 31
#define RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_AUDIO_CHANNEL_ALLOCATION         RS_SPECIFIC_PUBLIC_ATTR_BASE + 32
#define RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_AUDIO_CHANNEL_STATE              RS_SPECIFIC_PUBLIC_ATTR_BASE + 46
#define RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_AUDIO_CHANNEL_FREQUENCY          RS_SPECIFIC_PUBLIC_ATTR_BASE + 47
#define RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_AUDIO_DEFAULT_SETTINGS           RS_SPECIFIC_PUBLIC_ATTR_BASE + 33
#define RSAVPHG_ATTR_HDMI_GENERATOR_EDID_OUTPUT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 34
#define RSAVPHG_ATTR_SAVE_HDMI_GENERATOR_EDID_OUTPUT                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 35
#define RSAVPHG_ATTR_UPDATE_HDMI_GENERATOR_EDID_OUTPUT                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 36
#define RSAVPHG_ATTR_HDMI_GENERATOR_CEC_OUTPUT                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 49
#define RSAVPHG_ATTR_HDMI_GENERATOR_CEC_LOGICAL_ADDRESS                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 50
#define RSAVPHG_ATTR_HDMI_GENERATOR_CEC_DESTINATION                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 59
#define RSAVPHG_ATTR_HDMI_GENERATOR_CEC_OPERATION_CODE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 51
#define RSAVPHG_ATTR_HDMI_GENERATOR_CEC_INIT                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 52
#define RSAVPHG_ATTR_HDMI_GENERATOR_CEC_STOP                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 53
#define RSAVPHG_ATTR_HDMI_GENERATOR_CEC_SEND                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 54
#define RSAVPHG_ATTR_HDMI_GENERATOR_CEC_DATA_STORE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 55
#define RSAVPHG_ATTR_HDMI_GENERATOR_CEC_TEST_OUTPUT_REPORT_COUNT            RS_SPECIFIC_PUBLIC_ATTR_BASE + 56
#define RSAVPHG_ATTR_SELF_TEST_QUERY                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 37
#define RSAVPHG_ATTR_SYST_VERSION                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 38
#define RSAVPHG_ATTR_SYSTEM_RESET_ALL                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 39
#define RSAVPHG_ATTR_SYSTEM_RESET_BASE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 57
#define RSAVPHG_ATTR_SYSTEM_PRESET_ALL                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 40
#define RSAVPHG_ATTR_SYSTEM_PRESET_BASE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 58
#define RSAVPHG_ATTR_ID_QUERY_RESPONSE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 41
#define RSAVPHG_ATTR_SYSTEM_ERROR                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 42
#define RSAVPHG_ATTR_DISPLAY_UPDATE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 43

// rsavphg_rngHDMIGeneratorSignalCommand
#define RSAVPHG_VAL_HDMI_SIGNAL_STOP                                        0
#define RSAVPHG_VAL_HDMI_SIGNAL_PAUSE                                       1
#define RSAVPHG_VAL_HDMI_SIGNAL_PLAY                                        2

// rsavphg_rngHDMIGeneratorSignalStatus
#define RSAVPHG_VAL_HDMI_SIGNAL_STOPPED                                     0
#define RSAVPHG_VAL_HDMI_SIGNAL_PAUSED                                      1
#define RSAVPHG_VAL_HDMI_SIGNAL_RUNNING                                     2

// rsavphg_rngHDMIGeneratorSignalVideoPixelEncoding
#define RSAVPHG_VAL_PIXEL_ENCODING_RGB                                      0
#define RSAVPHG_VAL_PIXEL_ENCODING_Y444                                     1
#define RSAVPHG_VAL_PIXEL_ENCODING_Y422                                     2
#define RSAVPHG_VAL_PIXEL_ENCODING_Y420                                     3

// rsavphg_rngHDMIGeneratorSignalVideoSignalingMode
#define RSAVPHG_VAL_SIGNALING_MODE_HDMI                                     0
#define RSAVPHG_VAL_SIGNALING_MODE_DVI                                      1

// rsavphg_rngHDMIGeneratorSignalVideoDeepColor
#define RSAVPHG_VAL_DEEP_COLOR_OFF                                          0
#define RSAVPHG_VAL_DEEP_COLOR_B10                                          1
#define RSAVPHG_VAL_DEEP_COLOR_B12                                          2

// rsavphg_rngHDMIGeneratorSignalVideo3D
#define RSAVPHG_VAL_3D_OFF                                                  0
#define RSAVPHG_VAL_3D_FRAME_PACKING                                        1
#define RSAVPHG_VAL_3D_TOP_AND_BOTTOM                                       2
#define RSAVPHG_VAL_3D_SIDE_BY_SIDE                                         3

// rsavphg_rngHDMIGeneratorSignalVideoAdjustment
#define RSAVPHG_VAL_ADJUSTMENT_CALIBRATED                                   0
#define RSAVPHG_VAL_ADJUSTMENT_VARIABLE                                     1

// rsavphg_rngHDMIGeneratorSignalVideoResolutionClipping
#define RSAVPHG_VAL_RESOLUTION_CLIPPING_OFF                                 0
#define RSAVPHG_VAL_RESOLUTION_CLIPPING_B11                                 1
#define RSAVPHG_VAL_RESOLUTION_CLIPPING_B10                                 2
#define RSAVPHG_VAL_RESOLUTION_CLIPPING_B9                                  3
#define RSAVPHG_VAL_RESOLUTION_CLIPPING_B8                                  4
#define RSAVPHG_VAL_RESOLUTION_CLIPPING_B7                                  5
#define RSAVPHG_VAL_RESOLUTION_CLIPPING_B6                                  6
#define RSAVPHG_VAL_RESOLUTION_CLIPPING_B5                                  7

// rsavphg_rngHDMIGeneratorSignalAudioSelection
#define RSAVPHG_VAL_AUDIO_MUTE                                              0
#define RSAVPHG_VAL_AUDIO_VIDEO_FILE                                        1
#define RSAVPHG_VAL_AUDIO_GENERATOR                                         2

// rsavphg_rngHDMIGeneratorSignalAudioSamplingRate
#define RSAVPHG_VAL_AUDIO_SAMPLING_RATE_32000HZ                             0
#define RSAVPHG_VAL_AUDIO_SAMPLING_RATE_44100HZ                             1
#define RSAVPHG_VAL_AUDIO_SAMPLING_RATE_48000HZ                             2
#define RSAVPHG_VAL_AUDIO_SAMPLING_RATE_88200HZ                             3
#define RSAVPHG_VAL_AUDIO_SAMPLING_RATE_96000HZ                             4
#define RSAVPHG_VAL_AUDIO_SAMPLING_RATE_176400HZ                            5
#define RSAVPHG_VAL_AUDIO_SAMPLING_RATE_192000HZ                            6

// rsavphg_rngHDMIGeneratorSignalAudioResolution
#define RSAVPHG_VAL_AUDIO_RESOLUTION_B16                                    0
#define RSAVPHG_VAL_AUDIO_RESOLUTION_B20                                    1
#define RSAVPHG_VAL_AUDIO_RESOLUTION_B24                                    2

// rsavphg_rngHDMIGeneratorSignalPattern
#define RSAVPHG_VAL_COLOR_BAR_75                                            0
#define RSAVPHG_VAL_COLOR_BAR_100                                           1
#define RSAVPHG_VAL_PLAIN_RED                                               2
#define RSAVPHG_VAL_PLAIN_GREEN                                             3
#define RSAVPHG_VAL_PLAIN_BLUE                                              4
#define RSAVPHG_VAL_PLAIN_MAGENTA                                           5
#define RSAVPHG_VAL_PLAIN_YELLOW                                            6
#define RSAVPHG_VAL_PLAIN_CYAN                                              7
#define RSAVPHG_VAL_WHITE                                                   8
#define RSAVPHG_VAL_BLACK                                                   9
#define RSAVPHG_VAL_GRAY75                                                  10
#define RSAVPHG_VAL_GRAY50                                                  11
#define RSAVPHG_VAL_GRAY25                                                  12
#define RSAVPHG_VAL_GRAY10                                                  13
#define RSAVPHG_VAL_GRAY5                                                   14
#define RSAVPHG_VAL_WHITE_BLUE_GREEN_RED                                    15
#define RSAVPHG_VAL_WHITE_NOISE                                             16

// rsavphg_rngHDMIGeneratorSignalVideoAspectRatio
#define RSAVPHG_VAL_HDMI_GENERATOR_ASPECT_RATIO_4_3                         0
#define RSAVPHG_VAL_HDMI_GENERATOR_ASPECT_RATIO_16_9                        1
#define RSAVPHG_VAL_HDMI_GENERATOR_ASPECT_RATIO_21_9                        2
#define RSAVPHG_VAL_HDMI_GENERATOR_ASPECT_RATIO_64_27                       3

// rsavphg_rngHDMIGeneratorEDIDOutput
#define RSAVPHG_VAL_HDMI_EDID_OUTPUT_1                                      1
#define RSAVPHG_VAL_HDMI_EDID_OUTPUT_2                                      2
#define RSAVPHG_VAL_HDMI_EDID_OUTPUT_3                                      3
#define RSAVPHG_VAL_HDMI_EDID_OUTPUT_4                                      4

// rsavphg_rngHDMIGeneratorHotPlugDetectState
#define RSAVPHG_VAL_HDMI_HPD_NOT_ASSERTED                                   0
#define RSAVPHG_VAL_HDMI_HPD_ASSERTED                                       1

// rsavphg_rngHDMIAnalyzerCECAddressDestination
#define RSAVPHG_VAL_HDMI_CEC_DESTINATION_RD1                                0
#define RSAVPHG_VAL_HDMI_CEC_DESTINATION_RD2                                1
#define RSAVPHG_VAL_HDMI_CEC_DESTINATION_TUN1                               3
#define RSAVPHG_VAL_HDMI_CEC_DESTINATION_TUN2                               4
#define RSAVPHG_VAL_HDMI_CEC_DESTINATION_TUN3                               5
#define RSAVPHG_VAL_HDMI_CEC_DESTINATION_TUN4                               6
#define RSAVPHG_VAL_HDMI_CEC_DESTINATION_PD1                                7
#define RSAVPHG_VAL_HDMI_CEC_DESTINATION_PD2                                8
#define RSAVPHG_VAL_HDMI_CEC_DESTINATION_PD3                                9
#define RSAVPHG_VAL_HDMI_CEC_DESTINATION_RD3                                2
#define RSAVPHG_VAL_HDMI_CEC_DESTINATION_AUD                                10
#define RSAVPHG_VAL_HDMI_CEC_DESTINATION_SPEC                               11
#define RSAVPHG_VAL_HDMI_CEC_DESTINATION_BRO                                12
#define RSAVPHG_VAL_HDMI_CEC_DESTINATION_R12                                13
#define RSAVPHG_VAL_HDMI_CEC_DESTINATION_R13                                14
#define RSAVPHG_VAL_HDMI_CEC_DESTINATION_TV0                                15

// rsavphg_rngHDMIAnalyzerCECOperationCode
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_OP04                                    0
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_OP0D                                    1
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_OP82                                    2
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_RC36                                    3
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_RC80                                    4
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_RC81                                    5
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_RC82                                    6
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_RC85                                    7
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_RC86                                    8
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_RC9D                                    9
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_OR09                                    10
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_OR0A                                    11
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_OR0B                                    12
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_OR0F                                    13
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_TP33                                    14
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_TP34                                    15
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_TP35                                    16
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_TP43                                    17
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_TP67                                    18
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_TP97                                    19
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_TP99                                    20
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_TPA1                                    21
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_TPA2                                    22
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_SIPM                                    23
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_SI32                                    24
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_SI83                                    25
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_SI84                                    26
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_SI91                                    27
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_SI9E                                    28
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_SI9F                                    29
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_DC1A                                    30
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_DC1B                                    31
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_DC41                                    32
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_DC42                                    33
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_TC05                                    34
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_TC06                                    35
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_TC07                                    36
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_TC08                                    37
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_TC92                                    39
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_TC93                                    40
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_VC87                                    41
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_VC89                                    42
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_VC8A                                    43
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_VC8B                                    44
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_VC8C                                    45
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_VC9E                                    46
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_VC9F                                    47
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_VCA0                                    48
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_OD64                                    49
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_DT46                                    50
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_DT47                                    51
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_DM44                                    52
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_DM45                                    53
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_DM8D                                    54
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_DM8E                                    55
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_RC44                                    56
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_RC45                                    57
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_PS8F                                    58
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_PS90                                    59
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_GP00                                    60
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_GPFF                                    61
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_SA44                                    62
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_SA45                                    63
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_SA70                                    64
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_SA71                                    65
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_SA72                                    66
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_SA7A                                    67
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_SA7D                                    68
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_SA7E                                    69
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_SAA3                                    70
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_SAA4                                    71
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_AR9A                                    72
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_ARC0                                    73
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_ARC1                                    74
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_ARC2                                    75
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_ARC3                                    76
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_ARC4                                    77
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_ARC5                                    78
#define RSAVPHG_VAL_HDMI_CEC_OPCODE_CDF8                                    79

// rsavphg_rngHDMIGeneratorCECOutput
#define RSAVPHG_VAL_HDMI_CEC_OUTPUT_1                                       1
#define RSAVPHG_VAL_HDMI_CEC_OUTPUT_2                                       2
#define RSAVPHG_VAL_HDMI_CEC_OUTPUT_3                                       3
#define RSAVPHG_VAL_HDMI_CEC_OUTPUT_4                                       4
/* END GENERATE */

/*****************************************************************************
 *- Hidden Attribute Declarations -------------------------------------------*
 *****************************************************************************/

#define RSAVPHG_ATTR_OPC_TIMEOUT               (RS_ATTR_OPC_TIMEOUT)           /* ViInt32 */
#define RSAVPHG_ATTR_IO_SESSION                (RS_ATTR_IO_SESSION)            /* ViSession */
#define RSAVPHG_ATTR_OPC_CALLBACK              (RS_ATTR_OPC_CALLBACK)          /* ViAddr */
#define RSAVPHG_ATTR_CHECK_STATUS_CALLBACK     (RS_ATTR_CHECK_STATUS_CALLBACK) /* ViAddr */

/****************************************************************************
 *---------------- Constant definition  ------------------------------------*
 ****************************************************************************/

#define	RSAVPHG_VAL_RST_BASE  0
#define RSAVPHG_VAL_RST_ALL   1
	
#define RSAVPHG_VAL_CEA  (0)
#define RSAVPHG_VAL_HDMI (1)
#define RSAVPHG_VAL_VESA (2)

#define RSAVPHG_VAL_HDMI_STOP  0
#define RSAVPHG_VAL_HDMI_START 1
	
/****************************************************************************
*---------------- Instrument Driver Function Declarations -----------------*
 ****************************************************************************/

ViStatus _VI_FUNC rsavphg_init (ViRsrc resourceName, ViBoolean IDQuery,
                                ViBoolean resetDevice, ViSession* instrumentHandle);
ViStatus _VI_FUNC rsavphg_InitWithOptions (ViRsrc resourceName, ViBoolean IDQuery,
                                           ViBoolean resetDevice,
                                           ViChar _VI_FAR optionString[],
                                           ViSession* instrumentHandle);
ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorOutputState
             (ViSession instrumentHandle, ViInt32 board, 
              ViBoolean HDMIOut1, ViBoolean HDMIOut2, ViBoolean HDMIOut3,
              ViBoolean HDMIOut4, ViBoolean HDCPOut1, ViBoolean HDCPOut2,
              ViBoolean HDCPOut3, ViBoolean HDCPOut4);
ViStatus _VI_FUNC rsavphg_QueryHDMIGeneratorOutputHPDState
             (ViSession instrumentHandle, ViInt32 board, ViInt32* hotPlugDetect1,
              ViInt32* hotPlugDetect2, ViInt32* hotPlugDetect3,
              ViInt32* hotPlugDetect4);
ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorSignalPattern
             (ViSession instrumentHandle, ViInt32 board, ViInt32 pattern);
ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorSignalCommand
             (ViSession instrumentHandle, ViInt32 board, ViInt32 signalCommand);
ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorSignalVideoCode
             (ViSession instrumentHandle, ViInt32 board, ViInt32 type,
              ViInt32 videoCodeIndex);
ViStatus _VI_FUNC rsavphg_ReadHDMIGeneratorSignalStatus (ViSession instrumentHandle,
                                                         ViInt32 board,
                                                         ViInt32* signalStatus);
ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorSignalVideoFrameRate
             (ViSession instrumentHandle, ViInt32 board, ViReal64 frameRate);
ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorSignalVideoAspectRatio
             (ViSession instrumentHandle, ViInt32 board, ViInt32 aspectRatio);
ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorSignalVideoSettings
             (ViSession instrumentHandle, ViInt32 board, ViInt32 pixelEncoding,
              ViInt32 signalingMode, ViInt32 deepColor, ViInt32 _3D,
              ViInt32 adjustment, ViInt32 resolutionClipping, ViReal64 offset,
			  ViReal64 overallAmplification, ViReal64 chromaAmplification);
ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorSignalVideoVertical
             (ViSession instrumentHandle, ViInt32 board,
              ViBoolean pixelShiftVertical, ViReal64 vertical);
ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorSignalVideoRGBLevel
             (ViSession instrumentHandle, ViInt32 board, ViReal64 rLevel,
              ViReal64 gLevel, ViReal64 bLevel);
ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorSignalVideoYCbCrLevel
             (ViSession instrumentHandle, ViInt32 board, ViReal64 yLevel,
              ViReal64 cbPbLevel, ViReal64 crPrLevel);
ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorSignalVideoDefaultSettings
             (ViSession instrumentHandle, ViInt32 board);
ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorSignalAudioSettings
             (ViSession instrumentHandle, ViInt32 board, ViInt32 audio,
              ViInt32 samplingRate, ViReal64 audioLevel, ViInt32 resolution,
              ViInt32 channelAllocation_hex);
ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorSignalAudioChannel
             (ViSession instrumentHandle, ViInt32 board, ViInt32 port,
              ViBoolean channelState, ViInt32 channelFrequency);
ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorSignalAudioDefaultSettings
             (ViSession instrumentHandle, ViInt32 board);
ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorEDIDOutput
             (ViSession instrumentHandle, ViInt32 board, ViInt32 output);
ViStatus _VI_FUNC rsavphg_SaveHDMIGeneratorEDIDOutput (ViSession instrumentHandle,
                                                       ViInt32 board,
                                                       ViString fileName);
ViStatus _VI_FUNC rsavphg_UpdateHDMGeneratorEDIDOutput (ViSession instrumentHandle,
                                                        ViInt32 board);

ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorCECOutput (ViSession instrumentHandle,
                                                           ViInt32 board,
                                                           ViInt32 output);

ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorCECLogicalAddress (ViSession instrumentHandle,
                                                                   ViInt32 board,
                                                                   ViInt32 logicalAddress,
                                                                   ViInt32 destination);

ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorCECOperationCode (ViSession instrumentHandle,
                                                                  ViInt32 board,
                                                                  ViInt32 operationCode);

ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorCECOperationCodeParameterLength (ViSession instrumentHandle,
                                                                                 ViInt32 board,
                                                                                 ViInt32 operationCode,
                                                                                 ViInt32 parameter);

ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorCECOperationCodeParameter (ViSession instrumentHandle,
                                                                           ViInt32 board,
                                                                           ViInt32 operationCode,
                                                                           ViInt32 index,
                                                                           ViInt32 parameter);

ViStatus _VI_FUNC rsavphg_HDMIGeneratorCECOperation (ViSession instrumentHandle,
                                                     ViInt32 board,
                                                     ViInt32 operation);

ViStatus _VI_FUNC rsavphg_HDMIGeneratorCECSend (ViSession instrumentHandle,
                                                ViInt32 board);

ViStatus _VI_FUNC rsavphg_HDMIGeneratorCECDataSave (ViSession instrumentHandle,
                                                    ViInt32 board,
                                                    ViString fileName);

ViStatus _VI_FUNC rsavphg_QueryHDMIGeneratorCECTestOutputReportCount (ViSession instrumentHandle,
                                                                      ViInt32 board,
                                                                      ViInt32 *count);

ViStatus _VI_FUNC rsavphg_QueryHDMIGeneratorCECTestOutputReport (ViSession instrumentHandle,
                                                                 ViInt32 board,
                                                                 ViInt32 index,
                                                                 ViInt32 textMessageBufferSize,
                                                                 ViInt32 arraySize,
                                                                 ViInt32 *lineNumber,
                                                                 ViInt32 *referenceNumber,
                                                                 ViChar textMessage[],
                                                                 ViInt32 optionalParameters[],
                                                                 ViInt32 *returnedValues);

ViStatus _VI_FUNC rsavphg_reset (ViSession instrumentHandle);
ViStatus _VI_FUNC rsavphg_ResetSystem (ViSession instrumentHandle, ViInt32 reference);
ViStatus _VI_FUNC rsavphg_Preset (ViSession instrumentHandle, ViInt32 reference);
ViStatus _VI_FUNC rsavphg_self_test (ViSession instrumentHandle,
                                     ViInt16* selfTestResult,
                                     ViChar _VI_FAR selfTestMessage[]);
ViStatus _VI_FUNC rsavphg_revision_query (ViSession instrumentHandle,
                                          ViChar _VI_FAR instrumentDriverRevision[],
                                          ViChar _VI_FAR firmwareRevision[]);
ViStatus _VI_FUNC rsavphg_error_query (ViSession instrumentHandle,
                                       ViInt32* errorCode,
                                       ViChar _VI_FAR errorMessage[]);
ViStatus _VI_FUNC rsavphg_error_message (ViSession instrumentHandle,
                                         ViStatus errorCode,
                                         ViChar _VI_FAR errorMessage[]);
ViStatus _VI_FUNC rsavphg_SystemVersion (ViSession instrumentHandle, ViInt32 length,
                                         ViChar _VI_FAR systemVersion[]);
ViStatus _VI_FUNC rsavphg_setCheckStatus (ViSession instrumentHandle,
                                          ViBoolean state);
ViStatus _VI_FUNC rsavphg_ConfigureDisplayUpdate (ViSession instrumentHandle,
                                                  ViBoolean displayUpdate);
ViStatus _VI_FUNC rsavphg_GetError (ViSession instrumentHandle, ViStatus* code,
                                    ViInt32 bufferSize,
                                    ViChar _VI_FAR description[]);
ViStatus _VI_FUNC rsavphg_ClearError (ViSession instrumentHandle);
ViStatus _VI_FUNC rsavphg_WriteInstrData (ViSession instrumentHandle,
                                          ViChar _VI_FAR writeBuffer[]);
ViStatus _VI_FUNC rsavphg_ReadInstrData (ViSession instrumentHandle,
                                         ViInt32 numberOfBytesToRead,
                                         ViChar _VI_FAR readBuffer[],
                                         ViUInt32* numBytesRead);
ViStatus _VI_FUNC rsavphg_QueryViBoolean (ViSession instrumentHandle,
                                          ViString command, ViBoolean* value);
ViStatus _VI_FUNC rsavphg_QueryViInt32 (ViSession instrumentHandle,
                                        ViString command, ViInt32* value);
ViStatus _VI_FUNC rsavphg_QueryViReal64 (ViSession instrumentHandle,
                                         ViString command, ViReal64* value);
ViStatus _VI_FUNC rsavphg_QueryViString (ViSession instrumentHandle,
                                         ViString command, ViInt32 bufferSize,
                                         ViChar _VI_FAR value[]);
ViStatus _VI_FUNC rsavphg_SetAttributeViInt32 (ViSession instrumentHandle,
                                               ViChar _VI_FAR channelName[],
                                               ViAttr attributeID,
                                               ViInt32 attributeValue);
ViStatus _VI_FUNC rsavphg_SetAttributeViInt64 (ViSession instrumentHandle,
                                               ViChar _VI_FAR repeatedCapabilityName[],
                                               ViAttr attributeID,
                                               ViInt64 attributeValue);
ViStatus _VI_FUNC rsavphg_SetAttributeViReal64 (ViSession instrumentHandle,
                                                ViChar _VI_FAR channelName[],
                                                ViAttr attributeID,
                                                ViReal64 attributeValue);
ViStatus _VI_FUNC rsavphg_SetAttributeViString (ViSession instrumentHandle,
                                                ViChar _VI_FAR channelName[],
                                                ViAttr attributeID,
                                                ViChar _VI_FAR attributeValue[]);
ViStatus _VI_FUNC rsavphg_SetAttributeViBoolean (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViBoolean attributeValue);
ViStatus _VI_FUNC rsavphg_SetAttributeViSession (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViSession attributeValue);
ViStatus _VI_FUNC rsavphg_GetAttributeViInt32 (ViSession instrumentHandle,
                                               ViChar _VI_FAR channelName[],
                                               ViAttr attributeID,
                                               ViInt32* attributeValue);
ViStatus _VI_FUNC rsavphg_GetAttributeViInt64 (ViSession instrumentHandle,
                                               ViChar _VI_FAR repeatedCapabilityName[],
                                               ViAttr attributeID,
                                               ViInt64* attributeValue);
ViStatus _VI_FUNC rsavphg_GetAttributeViReal64 (ViSession instrumentHandle,
                                                ViChar _VI_FAR channelName[],
                                                ViAttr attributeID,
                                                ViReal64* attributeValue);
ViStatus _VI_FUNC rsavphg_GetAttributeViString (ViSession instrumentHandle,
                                                ViChar _VI_FAR channelName[],
                                                ViAttr attributeID,
                                                ViInt32 bufferSize,
                                                ViChar _VI_FAR attributeValue[]);
ViStatus _VI_FUNC rsavphg_GetAttributeViBoolean (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViBoolean* attributeValue);
ViStatus _VI_FUNC rsavphg_GetAttributeViSession (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViSession* attributeValue);
ViStatus _VI_FUNC rsavphg_CheckAttributeViInt32 (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViInt32 attributeValue);
ViStatus _VI_FUNC rsavphg_CheckAttributeViInt64 (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViInt64 attributeValue);
ViStatus _VI_FUNC rsavphg_CheckAttributeViReal64 (ViSession instrumentHandle,
                                                  ViChar _VI_FAR channelName[],
                                                  ViAttr attributeID,
                                                  ViReal64 attributeValue);
ViStatus _VI_FUNC rsavphg_CheckAttributeViString (ViSession instrumentHandle,
                                                  ViChar _VI_FAR channelName[],
                                                  ViAttr attributeID,
                                                  ViChar _VI_FAR attributeValue[]);
ViStatus _VI_FUNC rsavphg_CheckAttributeViBoolean (ViSession instrumentHandle,
                                                   ViChar _VI_FAR channelName[],
                                                   ViAttr attributeID,
                                                   ViBoolean attributeValue);
ViStatus _VI_FUNC rsavphg_CheckAttributeViSession (ViSession instrumentHandle,
                                                   ViChar _VI_FAR channelName[],
                                                   ViAttr attributeID,
                                                   ViSession attributeValue);
ViStatus _VI_FUNC rsavphg_GetAttributeRepeatedCapabilityIds
             (ViSession instrumentHandle, ViUInt32 attributeID, ViInt32 bufferSize,
              ViChar _VI_FAR repeatedCapabilityId_s[]);
ViStatus _VI_FUNC rsavphg_GetAttributeRepeatedCapabilityIdNames
             (ViSession instrumentHandle, ViUInt32 attributeID,
              ViString repeatedCapabilityId, ViInt32 bufferSize,
              ViChar _VI_FAR repeatedCapabilityIdName_s[]);
ViStatus _VI_FUNC rsavphg_close (ViSession instrumentHandle);

/****************************************************************************
 *------------------------ Error And Completion Codes ----------------------*
 ****************************************************************************/

#define RSAVPHG_WARN_MEASURE_UNCALIBRATED    (RS_CLASS_WARN_BASE + 0x0001L)
#define RSAVPHG_WARN_OVER_RANGE              (RS_CLASS_WARN_BASE + 0x0002L)

#define RSAVPHG_ERROR_MARKER_NOT_ENABLED     (RS_CLASS_ERROR_BASE + 0x0003L)
#define RSAVPHG_ERROR_NOT_DELTA_MARKER       (RS_CLASS_ERROR_BASE + 0x0002L)
#define RSAVPHG_ERROR_MAX_TIME_EXCEEDED      (RS_CLASS_ERROR_BASE + 0x0001L)

#define RSAVPHG_WARNMSG_MEASURE_UNCALIBRATED "Uncalibrated measurement"
#define RSAVPHG_WARNMSG_OVER_RANGE           "Measurement overrange"

#define RSAVPHG_ERRMSG_MARKER_NOT_ENABLED    "Marker Not Enabled"
#define RSAVPHG_ERRMSG_NOT_DELTA_MARKER      "Not Delta Marker"
#define RSAVPHG_ERRMSG_MAX_TIME_EXCEEDED     "Max Time Exceeded"

#define RSAVPHG_ERROR_CODES_AND_MSGS \
    {RSAVPHG_WARN_MEASURE_UNCALIBRATED,  RSAVPHG_WARNMSG_MEASURE_UNCALIBRATED},\
    {RSAVPHG_WARN_OVER_RANGE,            RSAVPHG_WARNMSG_OVER_RANGE},\
    {RSAVPHG_ERROR_MARKER_NOT_ENABLED,   RSAVPHG_ERRMSG_MARKER_NOT_ENABLED},\
    {RSAVPHG_ERROR_NOT_DELTA_MARKER,     RSAVPHG_ERRMSG_NOT_DELTA_MARKER},\
    {RSAVPHG_ERROR_MAX_TIME_EXCEEDED,    RSAVPHG_ERRMSG_MAX_TIME_EXCEEDED}

/****************************************************************************
 *---------------------------- End Include File ----------------------------*
 ****************************************************************************/

#if defined(__cplusplus) || defined(__cplusplus__)
}
#endif
#endif /* __RSAVPHG_HEADER */



