/*****************************************************************************
 *  Rohde&Schwarz Audio Video Platform MHL Analyzer
 *  LabWindows/CVI Instrument Driver                                     
 *  Original Release: 2013-09-18                                 
 *  By: Nabil Ouddane 
        Martin Krcmar
 *                                                                           
 *  Modification History:                                                    
 *                                                                           
 *       See ChangeLog
 *                                                                           
 *****************************************************************************/
                         
#include <string.h>
#include <stdio.h>  
#include <stdlib.h>
#include <math.h>
#include "rsavpma.h"

ViStatus _VI_FUNC  rsavpma_RsClose (ViSession vi);

/*****************************************************************************
 *- Value Definition and Range Tables ---------------------------------------*
 *****************************************************************************/
static ViString SwitchArr[] = {"OFF","ON",VI_NULL};
static ViString MHLAudioParametersArr[] = {"APPL","SAMP","COPY","PRE","MODE","CAT","NSO","NCH","FS","CACC","LENGth:MAX" ,"LENG", VI_NULL};
static ViString AnalyzerTestCaseArr[] = {"M000","M001","M002","M003","M004","M005","M006","M007","M008","M009","M010","M011","M012","M013","M014","M015","M016","M017","M018","M019","M020","CONC",VI_NULL};
static ViString VideoFormatArr[] = {"AUTO","C1","C2","C17","C4","C19","C5","C20","C32","C34","C6","C21","C16","C31",VI_NULL};;
static ViString TestLengthArr[] = {"SEC2","FRAM",VI_NULL};
static ViString PixelEncodingArr[] = {"RGB","Y422","Y444",VI_NULL};
static ViString TestPatternArr[] = {"FULL","VAN",VI_NULL};
static ViString TestSetupArr[] = {"YQY0","YQY1","RGB",VI_NULL};
static ViString MHLVideoFormatArr[] = {"PROG","INT",VI_NULL};
static ViString SyncPolarityArr[] = {"NEG","POS",VI_NULL};
static ViString HDCPStateArr[] = {"INV","ACT","INAC",VI_NULL};
static ViString AudioSamplingSizeArr[] ={"B16","B20","B24", VI_NULL};
static ViString EDIDTestArr[] = {"BLK2","BLK4",VI_NULL};
static ViString DSRTModeArr[] = {"NORM","PACK",VI_NULL};
static ViString TDFormatArr[] = {"AUTO","TBOT","LRIG","FPAC", VI_NULL};
/*****************************************************************************
 *- Callback Declarations ---------------------------------------------------*
 *****************************************************************************/

/*****************************************************************************
 *- User-Callable Functions (Exportable Functions) --------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: rsavpma_init
 * Purpose:  VXIplug&play required function. Calls the
 *           rsavpma_InitWithOptions function.
 *****************************************************************************/
ViStatus _VI_FUNC rsavpma_init (ViRsrc resourceName, ViBoolean IDQuery,
                                 ViBoolean resetDevice, ViSession *newInstrSession)
{
    ViStatus    error = VI_SUCCESS;

    if (newInstrSession == VI_NULL)
        {
        (void) Rs_SetErrorInfo (VI_NULL, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                          VI_ERROR_PARAMETER4, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
        }

    checkErr( rsavpma_InitWithOptions (resourceName, IDQuery, resetDevice,
                                      "QueryInstrStatus=1", newInstrSession));

Error:
    return error;
}

/*****************************************************************************
 * Function: rsavpma_InitWithOptions
 * Purpose:  This function creates a new RS session and calls the
 *           RsInit function.
 *****************************************************************************/
ViStatus _VI_FUNC rsavpma_InitWithOptions(
    ViRsrc          resourceName,
    ViBoolean       IDQuery,
    ViBoolean       resetDevice,
    ViString        optionString,
    ViSession      *newInstrSession
)
{
    ViStatus                error = VI_SUCCESS;
    ViChar                  newResourceName[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar                  newOptionString[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViBoolean               isLogicalName = 0;
    ViSession               instrSession = 0, rmSession = 0;
    RsAttrPropertiesPtr     *attrList=NULL;
    RsRepCapPtr             repCapTable = NULL;
    
    if (newInstrSession == VI_NULL)
        {
        (void) Rs_SetErrorInfo (VI_NULL, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                         VI_ERROR_PARAMETER5, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
        }

    /* Prepare resource manager session handle */
    checkErr (viOpenDefaultRM (&rmSession));

    /* Establish communication session with the instrument */

    viCheckErr( viOpen (rmSession, resourceName, VI_NULL, VI_NULL, &instrSession));
    
    /* Configure VISA Formatted I/O */
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, 5000 ));
    viCheckErr( viSetBuf (instrSession, (ViUInt16) (VI_READ_BUF | VI_WRITE_BUF), 4000));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_WR_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_RD_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));

    checkErr( Rs_GetInfoFromResourceName (resourceName,
                                          (ViString)optionString,
                                          newResourceName,
                                          newOptionString,
                                          &isLogicalName));
    
    ATTRLIST(attrList,rsavpma);  
    checkErr( Rs_SpecificDriverNew (instrSession, "rsavpma", newOptionString, attrList));
    
    /* Build repeated capability table */
    REPCAPTABLE(repCapTable,rsavpma);
    checkErr( Rs_BuildRepCapTable (instrSession, repCapTable));

    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_VISA_RM_SESSION, 0, &rmSession));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_IO_SESSION, 0, &instrSession));

    /* With remote control via Ethernet, remote control is not automatically
       set by means of a command. The instrument must be explicitly set to
       the REMOTE state, e.g. by sending the intf command &GTR
    */
    /*- Reset instrument ----------------------------------------------------*/
    if (resetDevice == VI_TRUE)
    {
        checkErr( rsavpma_reset (instrSession));
    }
    else  /*- Send Default Instrument Setup ---------------------------------*/
        checkErr( rsavpma_DefaultInstrSetup (instrSession));

    /*- Identification Query ------------------------------------------------*/
    if (IDQuery == VI_TRUE)
        {
        ViChar rdBuffer[RSAVPMA_IO_BUFFER_SIZE] = "";

        checkErr( Rs_GetAttribute (instrSession, VI_NULL, (ViAttr) RSAVPMA_ATTR_ID_QUERY_RESPONSE,
                                   0, RSAVPMA_IO_BUFFER_SIZE, rdBuffer));

        if (strncmp (rdBuffer, RSAVPMA_VALID_ID_RESPONSE_STRING, strlen(RSAVPMA_VALID_ID_RESPONSE_STRING)) != 0)
            {
            viCheckErr( VI_ERROR_FAIL_ID_QUERY);
            }
        }

    if (isLogicalName == VI_FALSE)
        {
        ViInt32 oldFlag = 0;

        checkErr (Rs_GetAttributeFlags (instrSession, RS_ATTR_IO_RESOURCE_DESCRIPTOR, &oldFlag));
        checkErr (Rs_SetAttributeFlags (instrSession, RS_ATTR_IO_RESOURCE_DESCRIPTOR, oldFlag & 0xfffb | 0x0010));
        checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_IO_RESOURCE_DESCRIPTOR, 0, newResourceName));
        }
    
    if (isLogicalName == VI_TRUE)
        checkErr( Rs_ApplyDefaultSetup (instrSession));

    /* Instrument session accessible by the end user is native VISA session,
       or when simulated virtual instrument (instrSession) session. */
    
Error:

    if (error < VI_SUCCESS)
    {
        ViSession   rmSess = VI_NULL;
        
        (void) Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_VISA_RM_SESSION, 0, (ViInt32) sizeof (rmSess), &rmSess);
        (void) Rs_Dispose (instrSession); 
        if (instrSession != 0)
            (void) viClose (instrSession);
        (void) viClose (rmSess);
    }
    else
    {
        *newInstrSession = instrSession;
    }
    
    return error;
    
}

/// HIFN  This function sets the value of a ViInt32 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpma_SetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function sets the value of a ViInt64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpma_SetAttributeViInt64 (ViSession instrSession,
                                                 ViChar channelName[],
                                                 ViAttr attributeId,
                                                 ViInt64 value)
{
	return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function sets the value of a ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpma_SetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 value)
{
     return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function sets the value of a ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpma_SetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViString value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/// HIFN  This function sets the value of a ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpma_SetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function sets the value of a ViSession attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpma_SetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function queries the value of a ViInt32 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViInt32 variable.
ViStatus _VI_FUNC rsavpma_GetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViInt32), (ViInt32 *)value);
}

/// HIFN  This function queries the value of a ViInt64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViInt64 variable.
ViStatus _VI_FUNC rsavpma_GetAttributeViInt64 (ViSession instrSession,
                                                 ViChar channelName[],
                                                 ViAttr attributeId,
                                                 ViInt64 *value)
{
	return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViInt64), (ViInt64 *)value);
}

/// HIFN  This function queries the value of a ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViReal64 variable.
ViStatus _VI_FUNC rsavpma_GetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViReal64), (ViReal64 *)value);
}

/// HIFN  This function queries the value of a ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR bufSize/Pass the number of bytes in the ViChar array you specify for the 
/// HIPAR bufSize/Attribute Value parameter.  
/// HIPAR value/The buffer in which the function returns the current value of the 
/// HIPAR value/attribute.  The buffer must be of type ViChar and have at least as many 
/// HIPAR value/bytes as indicated in the Buffer Size parameter.
ViStatus _VI_FUNC rsavpma_GetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViInt32 bufSize,
                                                 ViChar value[])
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            bufSize, value);
}

/// HIFN  This function queries the value of a ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViBoolean variable.
ViStatus _VI_FUNC rsavpma_GetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViBoolean), (ViBoolean *)value);
}

/// HIFN  This function queries the value of a ViSession attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViSession variable.
ViStatus _VI_FUNC rsavpma_GetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViSession), (ViSession *)value);
}

/// HIFN  This function checks the validity of a value you specify for a ViInt32 
/// HIFN  attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpma_CheckAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViInt32 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function checks the validity of a value you specify for a ViInt64 
/// HIFN  attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpma_CheckAttributeViInt64 (ViSession instrSession,
                                                   ViChar channelName[],
                                                   ViAttr attributeId,
                                                   ViInt64 value)
{
	return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpma_CheckAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViReal64 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpma_CheckAttributeViString (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViString value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpma_CheckAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViBoolean value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViSession attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpma_CheckAttributeViSession (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViSession value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN This function returns the repeated capability id(s) that belongs
/// HIFN to the attribute you specify. More then one id is listed in
/// HIFN comma separated string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Repeated Capability Id(s) parameter.
/// HIPAR repeatedCapabilityIds/The buffer in which the function returns the repeated capability id(s) belonging to the attribute.
ViStatus _VI_FUNC rsavpma_GetAttributeRepeatedCapabilityIds (ViSession instrSession,
                                                            ViUInt32 attributeID,
                                                            ViInt32 bufferSize,
                                                            ViChar _VI_FAR repeatedCapabilityIds[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      repCapNameId[RS_MAX_MESSAGE_BUF_SIZE] = "";

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 3, "Buffer Size");

    viCheckErr (Rs_GetAttributeRepCapNameId (instrSession, attributeID, repCapNameId));

    strncpy (repeatedCapabilityIds, repCapNameId, RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIds[(bufferSize > RS_MAX_MESSAGE_BUF_SIZE) ?
        RS_MAX_MESSAGE_BUF_SIZE : bufferSize] = '\0';

Error:
    return error;
}

/// HIFN This function returns the repeated capability name(s) that
/// HIFN belongs to the attribute and single repeated capability id you
/// HIFN may specify. More then one name is listed in comma separated
/// HIFN string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR repeatedCapabilityId/If the attribute applies to a repeated capability such as a trace, this parameter defines the Id of the repeated capability on which to query repeated capability id name(s).
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Repeated Capability Id Name(s) parameter.
/// HIPAR repeatedCapabilityIdNames/The buffer in which the function returns the repeated capability id name(s) belonging to the attribute.
ViStatus _VI_FUNC rsavpma_GetAttributeRepeatedCapabilityIdNames (ViSession instrSession,
                                                                ViUInt32 attributeID,
                                                                ViString repeatedCapabilityId,
                                                                ViInt32 bufferSize,
                                                                ViChar _VI_FAR repeatedCapabilityIdNames[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      repCapName[RS_MAX_MESSAGE_BUF_SIZE] = "";

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 4, "Buffer Size");

    viCheckErr (Rs_GetAttributeRepCapName (instrSession,
                                           attributeID,
                                           repeatedCapabilityId,
                                           repCapName));

    strncpy (repeatedCapabilityIdNames, repCapName, RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIdNames[(bufferSize >= RS_MAX_MESSAGE_BUF_SIZE) ?
        RS_MAX_MESSAGE_BUF_SIZE - 1 : bufferSize - 1] = '\0';

Error:
    return error;
}

/// HIFN This function configures the MHL analyzer input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR input/This control selects the input signal for the measurement.
/// HIPAR HPDSignal/Sets the hotplug detect (HPD) signal.
/// HIPAR device/This control sets input device.
/// HIPAR module/This control selects the module.
ViStatus _VI_FUNC rsavpma_ConfigureMHLAnalyzerInput (ViSession instrSession,
                                                   ViInt32 board, ViInt32 input,
                                                   ViBoolean HPDSignal,
                                                   ViInt32 device,
                                                   ViInt32 module)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpma_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpma_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPMA_ATTR_MHL_ANALYZER_INPUT, input), 3, "Input");
	
	viCheckParm(rsavpma_SetAttributeViBoolean(instrSession, rep_cap,
        RSAVPMA_ATTR_MHL_ANALYZER_HPD_STATE, HPDSignal), 4, "HPD Signal");
	
	viCheckParm(rsavpma_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPMA_ATTR_MHL_ANALYZER_INPUT_DEVICE, device), 5, "Device");
	
	viCheckParm(rsavpma_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPMA_ATTR_MHL_ANALYZER_MODULE, module), 5, "Device");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function returns the MHL analyzer input parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR bufferSize/Sets the buffer size.
/// HIPAR CBUS/Returns the CBUS connection.
/// HIPAR pixelClockLocked/Returns if the R&S VTE has detected a valid MHL input clock.
/// HIPAR HDCP/Returns if an HDCP encryption is currently active.
/// HIPAR pixelClock/Returns the pixel clock.
/// HIPAR videoCode/returns the video code derived from the AVI (auxiliary video information) InfoFrame.
/// HIPAR audioCoding/returns the audio coding type derived from audio InfoFrame.
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerInput (ViSession instrSession,
                                               ViInt32 board, ViInt32 bufferSize,
                                               ViInt32 *CBUS,
                                               ViInt32 *pixelClockLocked,
                                               ViInt32 *HDCP,
                                               ViReal64 *pixelClock,
                                               ViChar videoCode[],
                                               ViChar audioCoding[])
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
	ViChar      rd_buffer[RSAVPMA_IO_BUFFER_SIZE] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpma_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpma_GetAttributeViInt32 (instrSession, rep_cap, 
                                                RSAVPMA_ATTR_QUERY_MHL_ANALYZER_CBUS_CONNECTION,
                                                CBUS),4, "CBUS");
	

	viCheckParm(rsavpma_GetAttributeViInt32 (instrSession, rep_cap, 
                                                RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INPUT_PIXEL_CLOCK_LOCKED,
                                                pixelClockLocked),5, "Pixel Clock Locked");
	

	viCheckParm(rsavpma_GetAttributeViInt32 (instrSession, rep_cap, 
                                                RSAVPMA_ATTR_QUERY_MHL_ANALYZER_HDCP_ACTIVE,
                                                HDCP),6, "HDCP");
	
	viCheckParm(rsavpma_GetAttributeViReal64 (instrSession, rep_cap, 
                             RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INPUT_PIXEL_CLOCK, pixelClock), 7, "pixel Clock");
	
	
	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                             RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INPUT_VIDEO_CODE, bufferSize, rd_buffer), 8, "Video Code");

	rsavpma_ParseStatusString (rd_buffer, videoCode);
	
	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                             RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INPUT_AUDIO_CODING, bufferSize, rd_buffer), 9, "Audio Coding");

	rsavpma_ParseStatusString (rd_buffer, audioCoding);
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function returns the MHL analyzer input parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR bufferSize/Sets the buffer size.
/// HIPAR catalog/Returns the MHL analyzer module catalog.
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerModuleCatalog (ViSession instrSession,
                                                       ViInt32 board,
                                                       ViInt32 bufferSize,
                                                       ViChar catalog[])
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpma_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPMA_ATTR_QUERY_MHL_ANALYZER_MODULE_CATALOG, bufferSize,
                                                catalog),4, "Catalog");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function returns the MHL analyzer video parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR bufferSize/Sets the buffer size.
/// HIPAR videoCode/Returns the video code derived from the AVI InfoFrame.
/// HIPAR pixelClock/Returns the measured pixel clock of the MHL input signal.
/// HIPAR videoFormat/Returns the video format.
/// HIPAR verticalFrequency/Returns measured vertical frequency of the MHL video content.
/// HIPAR horizontalFrequency/Returns measured horizontal frequency of the MHL video content.
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerVideo (ViSession instrSession,
                                               ViInt32 board, ViInt32 bufferSize,
                                               ViChar videoCode[],
                                               ViReal64 *pixelClock,
                                               ViInt32 *videoFormat,
                                               ViReal64 *verticalFrequency,
                                               ViReal64 *horizontalFrequency)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
	ViChar      rd_buffer[RSAVPMA_IO_BUFFER_SIZE] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpma_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPMA_ATTR_QUERY_MHL_ANALYZER_VIDEO_CODE, bufferSize,
                                                rd_buffer),4, "Video Code");
	
	rsavpma_ParseStatusString (rd_buffer, videoCode);
	
	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPMA_ATTR_QUERY_MHL_ANALYZER_PIXEL_CLOCK, bufferSize,
                                                rd_buffer),5, "Pixel Clock");
	
	rsavpma_ParseStatusReal (rd_buffer, pixelClock);
	
	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPMA_ATTR_QUERY_MHL_ANALYZER_VIDEO_FORMAT, bufferSize,
                                                rd_buffer),6, "Video Format");
	
	rsavpma_ParseStatusEnum (rd_buffer, videoFormat, MHLVideoFormatArr);
	
	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                             RSAVPMA_ATTR_QUERY_MHL_ANALYZER_VERTICAL_FREQUENCY, bufferSize, rd_buffer), 7, "Vertical Frequency");
	
	rsavpma_ParseStatusReal (rd_buffer, verticalFrequency);
	
	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                             RSAVPMA_ATTR_QUERY_MHL_ANALYZER_HORIZONTAL_FREQUENCY, bufferSize, rd_buffer), 8, "Horizontal Frequency");

	rsavpma_ParseStatusReal (rd_buffer, horizontalFrequency);
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function returns the MHL analyzer video pixel parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init.
/// HIPAR board/This control selects the board.
/// HIPAR videoParameter/Selects the direction.
/// HIPAR parameter/Selects the parameter to be queried.
/// HIPAR pixelValue/Returns pixel parameter.
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerVideoPixels (ViSession instrSession,
                                                     ViInt32 board,
                                                     ViInt32 videoParameter,
                                                     ViInt32 parameter,
                                                     ViInt32 *pixelValue)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
	ViChar      rd_buffer[RSAVPMA_IO_BUFFER_SIZE] = "";
	ViAttr      attr[2][6] = {{RSAVPMA_ATTR_QUERY_MHL_ANALYZER_VIDEO_HORIZONTAL_TOTAL,
                             RSAVPMA_ATTR_QUERY_MHL_ANALYZER_VIDEO_HORIZONTAL_ACTIVE,
                             RSAVPMA_ATTR_QUERY_MHL_ANALYZER_VIDEO_HORIZONTAL_FRONT_PORCH,
                             RSAVPMA_ATTR_QUERY_MHL_ANALYZER_VIDEO_HORIZONTAL_SYNCHRONIZATION,
                             RSAVPMA_ATTR_QUERY_MHL_ANALYZER_VIDEO_HORIZONTAL_BACK_PORCH,
                             RSAVPMA_ATTR_QUERY_MHL_ANALYZER_VIDEO_HORIZONTAL_SYNCHRONIZATION_POLARITY
                            },
							{RSAVPMA_ATTR_QUERY_MHL_ANALYZER_VIDEO_VERTICAL_TOTAL,
							 RSAVPMA_ATTR_QUERY_MHL_ANALYZER_VIDEO_VERTICAL_ACTIVE,
							 RSAVPMA_ATTR_QUERY_MHL_ANALYZER_VIDEO_VERTICAL_FRONT_PORCH,
							 RSAVPMA_ATTR_QUERY_MHL_ANALYZER_VIDEO_VERTICAL_SYNCHRONIZATION,
							 RSAVPMA_ATTR_QUERY_MHL_ANALYZER_VIDEO_VERTICAL_BACK_PORCH,
							 RSAVPMA_ATTR_QUERY_MHL_ANALYZER_VIDEO_VERTICAL_SYNCHRONIZATION_POLARITY}};
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsavpma_invalidViInt32Range (parameter, RSAVPMA_VAL_MHL_PIXEL_TOTAL, RSAVPMA_VAL_MHL_PIXEL_SYNC_POLARITY), 4, "Parameter");
	
    checkErr (rsavpma_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
	                                      attr[videoParameter][parameter], RSAVPMA_IO_BUFFER_SIZE,
	                                      rd_buffer), 5, "Pixel Value");

	if (parameter == RSAVPMA_VAL_MHL_PIXEL_SYNC_POLARITY)
		rsavpma_ParseStatusEnum (rd_buffer, pixelValue, SyncPolarityArr);
	else
		rsavpma_ParseStatusInt (rd_buffer, pixelValue);
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;	
}

/// HIFN  This function returns the MHL analyzer audio parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR bufferSize/Sets the buffer size.
/// HIPAR n/Returns the N value derived from the audio clock regeneration packet.
/// HIPAR CTS/Returns the CTS (cycle timestamp) value derived from the audio clock regeneration packet.
/// HIPAR audioCoding/Returns the audio coding type derived from the audio InfoFrame.
/// HIPAR layout/Returns the layout value derived from the audio sample packet.
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerAudio (ViSession instrSession,
                                               ViInt32 board, ViInt32 bufferSize,
                                               ViInt32 *n, ViInt32 *CTS,
                                               ViChar audioCoding[],
                                               ViChar layout[])
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
	ViChar      rd_buffer[RSAVPMA_IO_BUFFER_SIZE] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpma_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPMA_ATTR_QUERY_MHL_ANALYZER_AUDIO_N, bufferSize,
                                                rd_buffer),4, "N");
	
	rsavpma_ParseStatusInt (rd_buffer, n);
	
	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPMA_ATTR_QUERY_MHL_ANALYZER_AUDIO_CYCLE_TIMESTAMP, bufferSize,
                                                rd_buffer),5, "CTS");
	
	rsavpma_ParseStatusInt (rd_buffer, CTS);
	
	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                             RSAVPMA_ATTR_QUERY_MHL_ANALYZER_AUDIO_CODING, bufferSize, rd_buffer), 6, "Audio Coding");
	
	rsavpma_ParseStatusString (rd_buffer, audioCoding);

	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                             RSAVPMA_ATTR_QUERY_MHL_ANALYZER_AUDIO_LAYOUT, bufferSize, rd_buffer), 7, "Layout");
	
	rsavpma_ParseStatusString (rd_buffer, layout);

	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;	 
}

/// HIFN  This function returns the MHL analyzer audio channel status bit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR parameter/Selects the parameter to be queried.
/// HIPAR bufferSize/Sets the buffer size.
/// HIPAR result/Returns the result.
/// HIPAR channelStatusBit/Returns audio parameter in integer or binary format.
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerAudioChannelStatusBits (ViSession instrSession,
                                                                ViInt32 board,
                                                                ViInt32 parameter,
                                                                ViInt32 bufferSize,
                                                                ViChar result[],
                                                                ViChar channelStatusBit[])
{
    ViStatus    error  = VI_SUCCESS;
    ViChar     *p2buf=VI_NULL; 
    ViChar      wr_buffer[RSAVPMA_IO_BUFFER_SIZE] = "";
	ViChar      rd_buffer[RSAVPMA_IO_BUFFER_SIZE] = "";
    
		
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsavpma_invalidViInt32Range (board, 1, 4), 2, "Board");
	viCheckParm (rsavpma_invalidViInt32Range (parameter, RSAVPMA_VAL_MHL_AUDIO_APPLICATION, RSAVPMA_VAL_MHL_AUDIO_LENGTH), 3, "Parameter");
    viCheckParm ((result == NULL), 5, "Result");
    viCheckParm ((channelStatusBit == NULL), 6, "Channel Status Bit");

    sprintf (wr_buffer, "READ%ld:MHL:AUD:%s?" , board, MHLAudioParametersArr[parameter]);
	
    viCheckErr (rsavpma_QueryViString (instrSession, wr_buffer, bufferSize, rd_buffer));
	
	p2buf = strtok (rd_buffer, ",");

	if (strcmp (p2buf , "OK"))
	{
		p2buf = strtok (p2buf, ",");
		strcpy (result, p2buf);
		p2buf = strtok (NULL, ",");
		strcpy (channelStatusBit, p2buf);
	}
	else
	{
		strcpy (result, p2buf);
	}
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function returns the MHL analyzer HDCP parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR bufferSize/Sets the buffer size.
/// HIPAR authentication/Returns if an HDCP authentication was attempted by the MHL source.
/// HIPAR HDCP/Returns if an HDCP content protection is currently active.
/// HIPAR AKSV/Returns the transmitter's key selection vector.
/// HIPAR BKSV/Returns the receiver's key selection vector.
/// HIPAR ri/Returns the current link synchronization verification values.
/// HIPAR an/returns the session random number.
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerHDCP (ViSession instrSession,
                                              ViInt32 board, ViInt32 bufferSize,
                                              ViChar authentication[],
                                              ViInt32 *HDCP, ViInt32 *AKSV,
                                              ViInt32 *BKSV, ViInt32 *ri,
                                              ViInt32 an[])
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";
    ViChar *pbuffer;
    ViInt32  i = 0;
	ViChar      rd_buffer[RSAVPMA_IO_BUFFER_SIZE] = ""; 
	ViChar      wr_buffer[RSAVPMA_IO_BUFFER_SIZE] = "";
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpma_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                             RSAVPMA_ATTR_QUERY_MHL_ANALYZER_HDCP_AUTHENTICATION, bufferSize, rd_buffer), 4, "Authentication");
	
	rsavpma_ParseStatusString (rd_buffer, authentication);
	
	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPMA_ATTR_QUERY_MHL_ANALYZER_HDCP_STATE, bufferSize, 
                                                rd_buffer),5, "HDCP");
	
	rsavpma_ParseStatusEnum (rd_buffer, HDCP, HDCPStateArr);
	
	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPMA_ATTR_QUERY_MHL_ANALYZER_HDCP_SOURCE_PUBLIC_KEY, bufferSize,
                                                rd_buffer),6, "AKSV");
	rsavpma_ParseStatusInt (rd_buffer, AKSV);

	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPMA_ATTR_QUERY_MHL_ANALYZER_HDCP_SINK_PUBLIC_KEY, bufferSize,
                                                rd_buffer),7, "BKSV");
	rsavpma_ParseStatusInt (rd_buffer, BKSV);
	
	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPMA_ATTR_QUERY_MHL_ANALYZER_HDCP_LINK_SYNCHRONIZATION_VALUE, bufferSize,
                                                rd_buffer),8, "Ri");
	rsavpma_ParseStatusInt (rd_buffer, ri);
	
	sprintf (wr_buffer, "READ%ld:MHL:HDCP:AN?", board);

	viCheckErr (rsavpma_QueryViString (instrSession, wr_buffer, bufferSize, rd_buffer));
	
	pbuffer = strtok(rd_buffer, ",");
    i=0;
    while (pbuffer) 
    {
        if (i < bufferSize)
        {
            an[i++] = atol (pbuffer);
        }   
        pbuffer = strtok(NULL, ",");        
    }
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
	return error;
}

/// HIFN  This function returns the MHL analyzer Info Frame parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR version/Returns the version.
/// HIPAR length/Returns the length.
/// HIPAR checksum/Returns the checksum.
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerInfoFrameAVI (ViSession instrSession,
                                                      ViInt32 board,
                                                      ViInt32 *version,
                                                      ViInt32 *length,
                                                      ViInt32 *checksum)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";
	ViChar      rd_buffer[RSAVPMA_IO_BUFFER_SIZE] = ""; 
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpma_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_VERSION, 1024,
                                                rd_buffer),3, "Version");
	
	rsavpma_ParseStatusInt (rd_buffer, version);
	
	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_LENGTH, 1024,
                                                rd_buffer),4, "Length");

	rsavpma_ParseStatusInt (rd_buffer, length);
	
	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_CHECKSUM, 1024,
                                                rd_buffer),5, "Checksum");
	
	rsavpma_ParseStatusInt (rd_buffer, checksum);

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function returns the MHL analyzer Info Frame parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR parameter/Selects the info frame parameter to be queried.
/// HIPAR bufferSize/Sets the buffer size.
/// HIPAR result/Returns the info frame parameter result.
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerInfoFrameAVISpecific (ViSession instrSession,
                                                              ViInt32 board,
                                                              ViInt32 parameter,
                                                              ViInt32 bufferSize,
                                                              ViChar result[])
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";
	ViChar      rd_buffer[RSAVPMA_IO_BUFFER_SIZE] = ""; 
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsavpma_invalidViInt32Range (parameter, RSAVPMA_VAL_MHL_INFO_FRAME_RGB, RSAVPMA_VAL_MHL_INFO_FRAME_PREP), 3, "Parameter");
	
    checkErr (rsavpma_hardwareRepCap (instrSession, board, rep_cap));
	
	switch (parameter)
    {
        case RSAVPMA_VAL_MHL_INFO_FRAME_RGB:
	
		viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_COLOR_SPACE, bufferSize, 
												 rd_buffer), 5, "RGB or YCbCr");
		
		rsavpma_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPMA_VAL_MHL_INFO_FRAME_AINFO:
	
		viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_ACTIVE_FORMAT, bufferSize, 
												 rd_buffer), 5, "Active Format Information Present");
		
		rsavpma_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPMA_VAL_MHL_INFO_FRAME_BINFO:
	
		viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_BAR_INFORMATION, bufferSize, 
												 rd_buffer), 5, "Bar Information");
		
		rsavpma_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPMA_VAL_MHL_INFO_FRAME_SINFO:
	
		viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_SCAN_INFORMATION, bufferSize, 
												 rd_buffer), 5, "Scan Information");
		
		rsavpma_ParseStatusString (rd_buffer, result);
		
		break;	
		
		case RSAVPMA_VAL_MHL_INFO_FRAME_COL:
	
		viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_COLORIMETRY, bufferSize, 
												 rd_buffer), 5, "Colorimetry");
		
		rsavpma_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPMA_VAL_MHL_INFO_FRAME_ARAT:
	
		viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_PICTURE_ASPECT_RATIO, bufferSize, 
												 rd_buffer), 5, "Picture Aspect Ratio");
		
		rsavpma_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPMA_VAL_MHL_INFO_FRAME_AFAR:
	
		viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_ACTIVE_FORMAT_ASPECT_RATIO, bufferSize, 
												 rd_buffer), 5, "Active Format Aspect Ratio");
		
		rsavpma_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPMA_VAL_MHL_INFO_FRAME_ITC:
	
		viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_IT_CONTENT, bufferSize, 
												 rd_buffer), 5, "IT Content");
		
		rsavpma_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPMA_VAL_MHL_INFO_FRAME_ECOL:
	
		viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_EXTENDED_COLORIMETRY, bufferSize, 
												 rd_buffer), 5, "Extended Colorimetry");
		
		rsavpma_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPMA_VAL_MHL_INFO_FRAME_RGBQ:
	
		viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_RGB_QUANTIZATION_RANGE, bufferSize, 
												 rd_buffer), 5, "RGB Quantization Range");
		
		rsavpma_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPMA_VAL_MHL_INFO_FRAME_NUPS:
	
		viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_NONUNIFORM_PICTURE_SCALING, bufferSize, 
												 rd_buffer), 5, "Nonuniform Picture Scaling");
		
		rsavpma_ParseStatusString (rd_buffer, result); 
		
		break;
		
		case RSAVPMA_VAL_MHL_INFO_FRAME_VIDC:
	
		viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_VIDEO_IDENTIFICATION_CODE, bufferSize, 
												 rd_buffer), 5, "Video Identification Code");
		
		rsavpma_ParseStatusString (rd_buffer, result); 
		
		break;
		
		case RSAVPMA_VAL_MHL_INFO_FRAME_YCCQ:
	
		viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_YCC_QUANTIZATION, bufferSize, 
												 rd_buffer), 5, "YCC Quantization");
		
		rsavpma_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPMA_VAL_MHL_INFO_FRAME_CTYP:
	
		viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_CONTENT_TYPE, bufferSize, 
												 rd_buffer), 5, "Content Type");
		
		rsavpma_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPMA_VAL_MHL_INFO_FRAME_PREP:
	
		viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_PIXEL_REPETITION, bufferSize, 
												 rd_buffer), 5, "Pixel Repetition");
		
		rsavpma_ParseStatusString (rd_buffer, result);
		
		break;
		
		default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER,3,"Parameter");
            break;
			
    }			
	
	checkErr (rsavpma_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the MHL analyzer Info Frame Bar parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR lineNumber_ofTopBarEnd/Returns Line Number of Top Bar End.
/// HIPAR lineNumber_ofBottomBarStart/Returns Line Number of Bottom Bar Start.
/// HIPAR pixelNumber_ofLeftBarEnd/returns Pixel Number of Left Bar End.
/// HIPAR pixelNumber_ofRightBarStart/returns Pixel Number of Right Bar Start.
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerInfoFrameAVIBar (ViSession instrSession,
                                                         ViInt32 board,
                                                         ViInt32 *lineNumber_ofTopBarEnd,
                                                         ViInt32 *lineNumber_ofBottomBarStart,
                                                         ViInt32 *pixelNumber_ofLeftBarEnd,
                                                         ViInt32 *pixelNumber_ofRightBarStart)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";
	ViChar      rd_buffer[RSAVPMA_IO_BUFFER_SIZE] = ""; 
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpma_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_TOP_BAR_END, 1024,
                                                rd_buffer),3, "Line Number of Top Bar End");
	
	rsavpma_ParseStatusInt (rd_buffer, lineNumber_ofTopBarEnd);
	
	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_BOTTOM_BAR_START,  1024,
                                                rd_buffer),4, "Line Number of Bottom Bar Start");
	
	rsavpma_ParseStatusInt (rd_buffer, lineNumber_ofBottomBarStart);

	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_LEFT_BAR_END, 1024,
                                                rd_buffer),5, "Pixel Number of Left Bar End");
	
	rsavpma_ParseStatusInt (rd_buffer, pixelNumber_ofLeftBarEnd);

	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_RIGHT_BAR_START, 1024,
                                                rd_buffer),6, "Pixel Number of Right Bar Start");
	
	rsavpma_ParseStatusInt (rd_buffer, pixelNumber_ofRightBarStart);


Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error; 
}

/// HIFN  This function returns the MHL analyzer Info Frame header and packet bytes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR arraySize/Sets the array size for Packet Bytes control.
/// HIPAR headerBytes/returns header bytes 0 to 2.
/// HIPAR packetBytes/This control returns packet bytes 0 to 13.
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerInfoFrameAVIHeaderPacketData (ViSession instrSession,
                                                                      ViInt32 board,
                                                                      ViInt32 arraySize,
                                                                      ViInt32 headerBytes[],
                                                                      ViInt32 packetBytes[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   *p2buf  = NULL;
	ViInt32	 i = 0;
	ViInt32	 x = 0;
	ViChar      rd_buffer[RSAVPMA_IO_BUFFER_SIZE] = ""; 
	ViChar      wr_buffer[RSAVPMA_IO_BUFFER_SIZE] = ""; 

    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	viCheckParm (rsavpma_invalidViInt32Range (board, 1, 8), 2, "Board");
	
	sprintf (wr_buffer, "READ%ld:MHL:INFO:AVI:PDAT?", board);
	
    viCheckErr (rsavpma_QueryViString (instrSession, wr_buffer, arraySize, rd_buffer));
	
	p2buf = strtok (rd_buffer, ",");

    if (strcmp (p2buf , "OK") != 0)
	{
		headerBytes[0] = RSAVPMA_NAN_VI_INT32;
		packetBytes[0] = RSAVPMA_NAN_VI_INT32;
	}
	else
	{
		p2buf = strtok (NULL, ",");
	    
		for (x = 0; x < 3; x++)
	    {
		    headerBytes[x] = atol (p2buf);
    	    p2buf = strtok (NULL, ",");
		}
		arraySize -= x;
		
		while (p2buf && (i<arraySize))
		{
	        packetBytes[i] = atol (p2buf);
	        p2buf = strtok (NULL, ",");
		}
	}
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the MHL analyzer Info Frame parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR version/Returns the version.
/// HIPAR length/Returns the length.
/// HIPAR checksum/Returns the checksum.
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerInfoFrameAudio (ViSession instrSession,
                                                        ViInt32 board,
                                                        ViInt32 *version,
                                                        ViInt32 *length,
                                                        ViInt32 *checksum)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";
	ViChar      rd_buffer[RSAVPMA_IO_BUFFER_SIZE] = ""; 
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpma_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AUDIO_VERSION, 1024,
                                                rd_buffer),3, "Version");
	
	rsavpma_ParseStatusInt (rd_buffer, version);
	
	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AUDIO_LENGTH, 1024,
                                                rd_buffer),4, "Length");
	
	rsavpma_ParseStatusInt (rd_buffer, length);

	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AUDIO_CHECKSUM, 1024,
                                                rd_buffer),5, "Checksum");

	rsavpma_ParseStatusInt (rd_buffer, checksum);
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function returns the MHL analyzer Info Frame parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR parameter/Selects the info frame parameter to be queried.
/// HIPAR bufferSize/Sets the buffer size.
/// HIPAR result/Returns the info frame parameter result.
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerInfoFrameAudioSpecific (ViSession instrSession,
                                                                ViInt32 board,
                                                                ViInt32 parameter,
                                                                ViInt32 bufferSize,
                                                                ViChar result[])
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";
	ViChar      rd_buffer[RSAVPMA_IO_BUFFER_SIZE] = ""; 
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpma_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm (rsavpma_invalidViInt32Range (parameter, RSAVPMA_VAL_MHL_INFO_FRAME_CODING_TYPE, RSAVPMA_VAL_MHL_INFO_FRAME_LFE), 3, "Parameter");
	
	switch (parameter)
    {
        case RSAVPMA_VAL_MHL_INFO_FRAME_CODING_TYPE:
	
		viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AUDIO_CODING_TYPE, bufferSize, 
												 rd_buffer), 5, "Coding Type");
		
		rsavpma_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPMA_VAL_MHL_INFO_FRAME_CHAN_COUNT:
	
		viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AUDIO_CHANNEL_COUNT, bufferSize, 
												 rd_buffer), 5, "Channel Count");
		
		rsavpma_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPMA_VAL_MHL_INFO_FRAME_SAMP_FREQ:
	
		viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AUDIO_SAMPLING_FREQUENCY, bufferSize, 
												 rd_buffer), 5, "Sampling Frequency");
		
		rsavpma_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPMA_VAL_MHL_INFO_FRAME_SAMP_SIZE:
	
		viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AUDIO_SAMPLING_SIZE, bufferSize, 
												 rd_buffer), 5, "Sampling Size");
		
		rsavpma_ParseStatusString (rd_buffer, result);
		
		break;	
		
		case RSAVPMA_VAL_MHL_INFO_FRAME_CODE_EXT:
	
		viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AUDIO_CODE_EXTENSION, bufferSize, 
												 rd_buffer), 5, "Code Extension");
		
		rsavpma_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPMA_VAL_MHL_INFO_FRAME_CHAN_ALLOC:
	
		viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AUDIO_CHANNEL_ALLOCATION, bufferSize, 
												 rd_buffer), 5, "Channel Allocation");
		
		rsavpma_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPMA_VAL_MHL_INFO_FRAME_DMIX_INHIB:
	
		viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AUDIO_DOWNMIX_INHIBIT, bufferSize, 
												 rd_buffer), 5, "Downmix Inhibit");
		
		rsavpma_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPMA_VAL_MHL_INFO_FRAME_LEVEL_SHIFT:
	
		viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AUDIO_LEVEL_SHIFT_VALUE, bufferSize, 
												 rd_buffer), 5, "Level Shift Value");
		
		rsavpma_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPMA_VAL_MHL_INFO_FRAME_LFE:
	
		viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AUDIO_LFE_PLAYBACK_LEVEL, bufferSize, 
												 rd_buffer), 5, "LFE Playback Level");
		
		rsavpma_ParseStatusString (rd_buffer, result);
		
		break;
		
		default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER,3,"Parameter");
            break;
			
    }			
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the MHL analyzer Info Frame header and packet bytes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR arraySize/Sets the array size for Packet Bytes control.
/// HIPAR headerBytes/Returns header bytes 0 to 2
/// HIPAR packetBytes/Returns packet bytes 0 to 10.
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerInfoFrameAudioHeaderPacketData (ViSession instrSession,
                                                                        ViInt32 board,
                                                                        ViInt32 arraySize,
                                                                        ViInt32 headerBytes[],
                                                                        ViInt32 packetBytes[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   *p2buf  = NULL;
	ViInt32	 i = 0;
	ViInt32	 x = 0;
	ViChar      wr_buffer[RSAVPMA_IO_BUFFER_SIZE] = ""; 
	ViChar      rd_buffer[RSAVPMA_IO_BUFFER_SIZE] = ""; 
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));    
   
	viCheckParm (rsavpma_invalidViInt32Range (board, 1, 8), 2, "Board");
	
	sprintf (wr_buffer, "READ%ld:MHL:INFO:AUD:PDAT?", board);
	
    viCheckErr (rsavpma_QueryViString (instrSession, wr_buffer, arraySize, rd_buffer));
	
	p2buf = strtok (rd_buffer, ",");
	
	if (strcmp (p2buf , "OK") != 0)
	{
		headerBytes[0] = RSAVPMA_NAN_VI_INT32;
		packetBytes[0] = RSAVPMA_NAN_VI_INT32;
	}
	else
	{
		p2buf = strtok (NULL, ",");
	    
		for (x = 0; x < 3; x++)
	    {
		    headerBytes[x] = atol (p2buf);
    	    p2buf = strtok (NULL, ",");
		}
		arraySize -= x;
		
		while (p2buf && (i<arraySize))
		{
	        packetBytes[i] = atol (p2buf);
	        p2buf = strtok (NULL, ",");
		}
	}
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the MHL analyzer Info Frame parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR version/Returns the version.
/// HIPAR length/Returns the length.
/// HIPAR checksum/Returns the checksum.
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerInfoFrameSPD (ViSession instrSession,
                                                      ViInt32 board,
                                                      ViInt32 *version,
                                                      ViInt32 *length,
                                                      ViInt32 *checksum)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";
	ViChar      rd_buffer[RSAVPMA_IO_BUFFER_SIZE] = ""; 
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpma_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_SPD_VERSION, 1024,
                                                rd_buffer),3, "Version");
	
	rsavpma_ParseStatusInt (rd_buffer, version);
	
	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_SPD_LENGTH, 1024,
                                                rd_buffer),4, "Length");
	
	rsavpma_ParseStatusInt (rd_buffer, length);

	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_SPD_CHECKSUM, 1024,
                                                rd_buffer),5, "Checksum");
	rsavpma_ParseStatusInt (rd_buffer, checksum);

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function returns the MHL analyzer Info Frame parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR parameter/Selects the info frame parameter to be queried.
/// HIPAR bufferSize/Sets the buffer size.
/// HIPAR result/returns the info frame parameter result.
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerInfoFrameSPDSpecific (ViSession instrSession,
                                                              ViInt32 board,
                                                              ViInt32 parameter,
                                                              ViInt32 bufferSize,
                                                              ViChar result[])
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";
	ViChar      rd_buffer[RSAVPMA_IO_BUFFER_SIZE] = ""; 
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsavpma_invalidViInt32Range (parameter, RSAVPMA_VAL_MHL_INFO_FRAME_VENDOR, RSAVPMA_VAL_MHL_INFO_FRAME_PRODUCT), 3, "Parameter");
	
    checkErr (rsavpma_hardwareRepCap (instrSession, board, rep_cap));
	
	switch (parameter)
    {
        case RSAVPMA_VAL_MHL_INFO_FRAME_VENDOR:
	
		viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_SPD_VENDOR, bufferSize, 
												 rd_buffer), 5, "Vendor Name");
		
		rsavpma_ParseStatusString (rd_buffer, result);
		
		break;
		
		case RSAVPMA_VAL_MHL_INFO_FRAME_PRODUCT:
	
		viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_SPD_PRODUCT_DESCRIPTION, bufferSize, 
												 rd_buffer), 5, "Product Description");
		
		rsavpma_ParseStatusString (rd_buffer, result);
		
		
		break;
	
	 default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER,3,"Parameter");
            break;
			
    }			
	
	checkErr (rsavpma_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error; 
}

/// HIFN  This function returns the MHL analyzer Info Frame header and packet bytes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR arraySize/Sets the array size for Packet Bytes control.
/// HIPAR headerBytes/Returns header bytes 0 to 2.
/// HIPAR packetBytes/Returns packet bytes 0 to 25.
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerInfoFrameSPDHeaderPacketData (ViSession instrSession,
                                                                      ViInt32 board,
                                                                      ViInt32 arraySize,
                                                                      ViInt32 headerBytes[],
                                                                      ViInt32 packetBytes[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   *p2buf  = NULL;
	ViInt32	 i = 0;
	ViInt32  x = 0;
	ViChar      wr_buffer[RSAVPMA_IO_BUFFER_SIZE] = ""; 
	ViChar      rd_buffer[RSAVPMA_IO_BUFFER_SIZE] = ""; 

    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	viCheckParm (rsavpma_invalidViInt32Range (board, 1, 8), 2, "Board");
	
	sprintf (wr_buffer, "READ%ld:MHL:INFO:SPD:PDAT?" , board);
	
    viCheckErr (rsavpma_QueryViString (instrSession, wr_buffer, arraySize, rd_buffer));
	
p2buf = strtok (rd_buffer, ",");
	
	if (strcmp (p2buf , "OK") != 0)
	{
		headerBytes[0] = RSAVPMA_NAN_VI_INT32;
		packetBytes[0] = RSAVPMA_NAN_VI_INT32;
	}
	else
	{
		p2buf = strtok (NULL, ",");
	    
		for (x = 0; x < 3; x++)
	    {
		    headerBytes[x] = atol (p2buf);
    	    p2buf = strtok (NULL, ",");
		}
		arraySize -= x;
		
		while (p2buf && (i<arraySize))
		{
	        packetBytes[i] = atol (p2buf);
	        p2buf = strtok (NULL, ",");
		}
	}
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the MHL analyzer Info Frame parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR version/Returns the version.
/// HIPAR length/Returns the length.
/// HIPAR checksum/Returns the checksum.
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerInfoFrameMPEGSource (ViSession instrSession,
                                                             ViInt32 board,
                                                             ViInt32 *version,
                                                             ViInt32 *length,
                                                             ViInt32 *checksum)
{
     ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";
	ViChar      rd_buffer[RSAVPMA_IO_BUFFER_SIZE] = ""; 

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpma_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_MPEG_VERSION, 1024,
                                                rd_buffer),3, "Version");
	rsavpma_ParseStatusInt (rd_buffer, version);
	
	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_MPEG_LENGTH, 1024,
                                                rd_buffer),4, "Length");
	
	rsavpma_ParseStatusInt (rd_buffer, length);

	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_MPEG_CHECKSUM, 1024,
                                                rd_buffer),5, "Checksum");
	
	rsavpma_ParseStatusInt (rd_buffer, checksum);
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function returns the MHL analyzer Info Frame parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR bufferSize/Sets the buffer size.
/// HIPAR MPEGBitRate/Returns the MPEG Bit Rate.
/// HIPAR fieldRepeat/Returns the Field Repeat.
/// HIPAR MPEGFrame/Returns the MPEG Frame.
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerInfoFrameMPEGSourceSpecific (ViSession instrSession,
                                                                     ViInt32 board,
                                                                     ViInt32 bufferSize,
                                                                     ViInt32 *MPEGBitRate,
                                                                     ViChar fieldRepeat[],
                                                                     ViChar MPEGFrame[])
{
 	ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";
	ViChar      rd_buffer[RSAVPMA_IO_BUFFER_SIZE] = ""; 

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpma_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_MPEG_BIT_RATE, bufferSize,
                                                rd_buffer),4, "MPEG Bit Rate");
	
	rsavpma_ParseStatusInt (rd_buffer, MPEGBitRate);
	
	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_MPEG_FIELD_REPEAT, bufferSize, 
												 rd_buffer), 5, "Field Repeat");
	
	rsavpma_ParseStatusString (rd_buffer, fieldRepeat);
	
	viCheckParm(rsavpma_GetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_MPEG_FRAME, bufferSize, 
												 rd_buffer), 6, "MPEG Frame");
	
	rsavpma_ParseStatusString (rd_buffer, MPEGFrame);
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function returns the MHL analyzer Info Frame header and packet bytes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR arraySize/Sets the array size for Packet Bytes control.
/// HIPAR headerBytes/Returns header bytes 0 to 2.
/// HIPAR packetBytes/Returns packet bytes 0 to 10.
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerInfoFrameMPEGSourceHeaderPacketData (ViSession instrSession,
                                                                             ViInt32 board,
                                                                             ViInt32 arraySize,
                                                                             ViInt32 headerBytes[],
                                                                             ViInt32 packetBytes[])
{
    ViStatus error = VI_SUCCESS;
    ViChar  wr_buffer[RSAVPMA_IO_BUFFER_SIZE] = ""; 
	ViChar  rd_buffer[RSAVPMA_IO_BUFFER_SIZE] = ""; 
    ViChar  *p2buf  = NULL;
	ViInt32	 i = 0;
	ViInt32  x = 0;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	viCheckParm (rsavpma_invalidViInt32Range (board, 1, 8), 2, "Board");
	
	sprintf (wr_buffer, "READ%ld:MHL:INFO:MPEG:PDAT?", board);
	
    viCheckErr (rsavpma_QueryViString (instrSession, wr_buffer, arraySize, rd_buffer));
	
	p2buf = strtok (rd_buffer, ",");
	
	if (strcmp (p2buf , "OK") != 0)
	{
		headerBytes[0] = RSAVPMA_NAN_VI_INT32;
		packetBytes[0] = RSAVPMA_NAN_VI_INT32;
	}
	else
	{
		p2buf = strtok (NULL, ",");
	    
		for (x = 0; x < 3; x++)
	    {
		    headerBytes[x] = atol (p2buf);
    	    p2buf = strtok (NULL, ",");
		}
		arraySize -= x;
		
		while (p2buf && (i<arraySize))
		{
	        packetBytes[i] = atol (p2buf);
	        p2buf = strtok (NULL, ",");
		}
	}
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the MHL analyzer CTS state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR state/Returns the MHL analyzer CTS state.
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerCTSState (ViSession instrSession,
                                                  ViInt32 board, ViInt32 *state)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpma_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpma_GetAttributeViInt32(instrSession, rep_cap,
        RSAVPMA_ATTR_QUERY_MHL_ANALYZER_CTS_STATE, state), 3, "State");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function selects the operation to be performed on the MHL analyzer CTS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR operation/Selects the operation.
ViStatus _VI_FUNC rsavpma_MHLAnalyzerCTSOperation (ViSession instrSession,
                                                 ViInt32 board,
                                                 ViInt32 operation)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpma_hardwareRepCap (instrSession, board, rep_cap));
	
	switch (operation)
    {
        case RSAVPMA_VAL_MHL_START:
	
		viCheckErr(rsavpma_SetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPMA_ATTR_MHL_ANALYZER_CTS_INIT, NULL));
												 
		
		break;
		
		case RSAVPMA_VAL_MHL_STOP:
	
		viCheckErr(rsavpma_SetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPMA_ATTR_MHL_ANALYZER_CTS_STOP, NULL));
												 
		
		
		break;
	
	 default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER,3,"Operation");
            break;
			
    }			
	
	checkErr (rsavpma_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error; 
}

/// HIFN  This function returns the MHL analyzer CTS measurement progress.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR progress/Returns the MHL analyzer CTS measurement progress.
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerCTSProgress (ViSession instrSession,
                                                     ViInt32 board,
                                                     ViInt32 *progress)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpma_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpma_GetAttributeViInt32(instrSession, rep_cap,
        RSAVPMA_ATTR_QUERY_MHL_ANALYZER_CTS_PROGRESS, progress), 3, "Progress");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function returns the MHL analyzer CTS measurement progress.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR testCase/Selects the test case.
/// HIPAR state/Selected test case on or off.
ViStatus _VI_FUNC rsavpma_SelectMHLAnalyzerCTSTestCase (ViSession instrSession,
                                                      ViInt32 board,
                                                      ViInt32 testCase,
                                                      ViBoolean state)
{
  
	ViStatus    error = VI_SUCCESS;
    ViChar      command[150] = "";
	
    checkErr (Rs_LockSession (instrSession, NULL));

    viCheckParm (rsavpma_invalidViInt32Range (board, 1, 8), 2, "Board");
    viCheckParm (rsavpma_invalidViInt32Range (testCase, RSAVPMA_VAL_MHL_TEST_CASE_LCODES, RSAVPMA_VAL_MHL_TEST_CASE_CONCURRENT), 3, "Test Case");
    
    sprintf (command, "CONF%ld:MHL:CTS:SEL %s,%s", board, AnalyzerTestCaseArr[testCase], SwitchArr[state]);
		
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    
    checkErr( rsavpma_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  This function configures the MHL analyzer CTS Legal Codes, Basic Protocol and Packet Types settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR testCase/Selects the test case.
/// HIPAR videoFormat/Selects the video format.
/// HIPAR testLength/Selects the test length.
/// HIPAR numberOfFrames/Sets the number of frames.
ViStatus _VI_FUNC rsavpma_ConfigureMHLAnalyzerCTSTMDSCoding (ViSession instrSession,
                                                           ViInt32 board,
                                                           ViInt32 testCase,
                                                           ViInt32 videoFormat,
                                                           ViInt32 testLength,
                                                           ViInt32 numberOfFrames)
{
 	ViStatus    error = VI_SUCCESS;
   
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    viCheckParm (rsavpma_invalidViInt32Range (board, 1, 8), 2, "Board");
    viCheckParm (rsavpma_invalidViInt32Range (testCase, RSAVPMA_VAL_MHL_TEST_CASE_LCODES, RSAVPMA_VAL_MHL_TEST_CASE_PACKET), 3, "Test Case");
	viCheckParm (rsavpma_invalidViInt32Range (videoFormat, RSAVPMA_VAL_MHL_VIDEO_FORMAT_AUTO, RSAVPMA_VAL_MHL_VIDEO_FORMAT_C21), 4, "Video Format");
	viCheckParm (rsavpma_invalidViInt32Range (testLength, RSAVPMA_VAL_MHL_TEST_LENGTH_2_SECONDS, RSAVPMA_VAL_MHL_TEST_LENGTH_FRAMES), 5, "Test Length");
	viCheckParm (rsavpma_invalidViInt32Range (numberOfFrames, 1, 120), 6, "Number Of Frames");
    
	viCheckErr (viPrintf (instrSession, "CONF%ld:MHL:CTS:VFOR %s,%s", board, AnalyzerTestCaseArr[testCase], VideoFormatArr[videoFormat]));
    
	checkErr(rsavpma_CheckStatus (instrSession));
	
	viCheckErr (viPrintf (instrSession, "CONF%ld:MHL:CTS:TEST:MODE %s,%s", board, AnalyzerTestCaseArr[testCase], TestLengthArr[testLength]));
			
	checkErr(rsavpma_CheckStatus (instrSession));
	
	if (testLength == RSAVPMA_VAL_MHL_TEST_LENGTH_FRAMES)
	{
		viCheckErr (viPrintf (instrSession, "CONF%ld:MHL:CTS:TEST:FRAM %s,%ld", board, AnalyzerTestCaseArr[testCase], numberOfFrames));   
			
		checkErr(rsavpma_CheckStatus (instrSession));		
	}

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  configures the MHL analyzer CTS video modes - Video Formats, Pixel Encoding, AVI Info Frame, Video Quantization Ranges.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR testCase/Selects the test case.
/// HIPAR videoFormat/Selects the video format.
/// HIPAR testLength/Selects the test length.
/// HIPAR numberOfFrames/Sets the number of frames.
/// HIPAR pixelEncoding/Selects the pixel encoding.
/// HIPAR testPattern/Selects the test pattern.
/// HIPAR testSetup/Selects the test setup.
ViStatus _VI_FUNC rsavpma_ConfigureMHLAnalyzerCTSVideoModes (ViSession instrSession,
                                                           ViInt32 board,
                                                           ViInt32 testCase,
                                                           ViInt32 videoFormat,
                                                           ViInt32 testLength,
                                                           ViInt32 numberOfFrames,
                                                           ViInt32 pixelEncoding,
                                                           ViInt32 testPattern,
                                                           ViInt32 testSetup)
{
    ViStatus    error = VI_SUCCESS;
    
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    viCheckParm (rsavpma_invalidViInt32Range (board, 1, 8), 2, "Board");
    viCheckParm (rsavpma_invalidViInt32Range (testCase, RSAVPMA_VAL_MHL_TEST_CASE_VFORMAT, RSAVPMA_VAL_MHL_TEST_CASE_VQRANGE), 3, "Test Case");
	viCheckParm (rsavpma_invalidViInt32Range (videoFormat, RSAVPMA_VAL_MHL_VIDEO_FORMAT_AUTO, RSAVPMA_VAL_MHL_VIDEO_FORMAT_C21), 4, "Video Format");
	viCheckParm (rsavpma_invalidViInt32Range (testLength, RSAVPMA_VAL_MHL_TEST_LENGTH_2_SECONDS, RSAVPMA_VAL_MHL_TEST_LENGTH_FRAMES), 5, "Test Length");
	viCheckParm (rsavpma_invalidViInt32Range (numberOfFrames, 1, 120), 6, "Number Of Frames");
	viCheckParm (rsavpma_invalidViInt32Range (pixelEncoding, RSAVPMA_VAL_MHL_PIXEL_ENCODING_RGB, RSAVPMA_VAL_MHL_PIXEL_ENCODING_Y444), 7, "Pixel Encoding");
	viCheckParm (rsavpma_invalidViInt32Range (testPattern, RSAVPMA_VAL_MHL_TEST_PATTERN_FULLY_SATURATED, RSAVPMA_VAL_MHL_TEST_PATTERN_VISUAL_ANALYSIS), 8, "Test Pattern");
	viCheckParm (rsavpma_invalidViInt32Range (testSetup, RSAVPMA_VAL_MHL_TEST_SETUP_YQY0, RSAVPMA_VAL_MHL_TEST_SETUP_RGB), 9, "Test Setup");
	
	switch (testCase)
		
	   {	
		  case RSAVPMA_VAL_MHL_TEST_CASE_VFORMAT:
		  case RSAVPMA_VAL_MHL_TEST_CASE_AVI:
			  
			viCheckErr (viPrintf (instrSession, "CONF%ld:MHL:CTS:VFOR %s,%s", board, AnalyzerTestCaseArr[testCase], VideoFormatArr[videoFormat]));
    		checkErr (rsavpma_CheckStatus (instrSession));
	        viCheckErr (viPrintf (instrSession, "CONF%ld:MHL:CTS:TEST:MODE %s,%s", board, AnalyzerTestCaseArr[testCase], TestLengthArr[testLength]));
			checkErr (rsavpma_CheckStatus (instrSession));
			if (testLength == RSAVPMA_VAL_MHL_TEST_LENGTH_FRAMES)
			{
			viCheckErr (viPrintf (instrSession, "CONF%ld:MHL:CTS:TEST:FRAM %s,%ld", board, AnalyzerTestCaseArr[testCase], numberOfFrames));
			checkErr (rsavpma_CheckStatus (instrSession));
			}
			
		  break;
		  
		  case RSAVPMA_VAL_MHL_TEST_CASE_PIXEL:
			  
		    viCheckErr (viPrintf (instrSession, "CONF%ld:MHL:CTS:VFOR %s,%s", board, AnalyzerTestCaseArr[testCase], VideoFormatArr[videoFormat]));  
			checkErr (rsavpma_CheckStatus (instrSession));
			viCheckErr (viPrintf (instrSession, "CONF%ld:MHL:CTS:PIX %s,%s", board, AnalyzerTestCaseArr[testCase], PixelEncodingArr[pixelEncoding]));
            checkErr (rsavpma_CheckStatus (instrSession));			
			viCheckErr (viPrintf (instrSession, "CONF%ld:MHL:CTS:TPAT %s,%s", board, AnalyzerTestCaseArr[testCase], TestPatternArr[testPattern]));
			checkErr (rsavpma_CheckStatus (instrSession));
			viCheckErr (viPrintf (instrSession, "CONF%ld:MHL:CTS:TEST:MODE %s,%s", board, AnalyzerTestCaseArr[testCase], TestLengthArr[testLength]));
			checkErr (rsavpma_CheckStatus (instrSession));
			
			if (testLength == RSAVPMA_VAL_MHL_TEST_LENGTH_FRAMES)
			{
			viCheckErr (viPrintf (instrSession, "CONF%ld:MHL:CTS:TEST:FRAM %s,%ld", board, AnalyzerTestCaseArr[testCase], numberOfFrames));
			checkErr (rsavpma_CheckStatus (instrSession));
			}
		  break;
		  
		  case RSAVPMA_VAL_MHL_TEST_CASE_VQRANGE:
			  
			viCheckErr (viPrintf (instrSession, "CONF%ld:MHL:CTS:VFOR %s,%s", board, AnalyzerTestCaseArr[testCase], VideoFormatArr[videoFormat]));
			checkErr (rsavpma_CheckStatus (instrSession));
			viCheckErr (viPrintf (instrSession, "CONF%ld:MHL:CTS:SET %s,%s", board, AnalyzerTestCaseArr[testCase], TestSetupArr[testSetup]));
			checkErr (rsavpma_CheckStatus (instrSession));
			viCheckErr (viPrintf (instrSession, "CONF%ld:MHL:CTS:TEST:MODE %s,%s", board, AnalyzerTestCaseArr[testCase], TestLengthArr[testLength]));
			checkErr (rsavpma_CheckStatus (instrSession));
			
			if (testLength == RSAVPMA_VAL_MHL_TEST_LENGTH_FRAMES)
			{
			viCheckErr (viPrintf (instrSession, "CONF%ld:MHL:CTS:TEST:FRAM %s,%ld", board, AnalyzerTestCaseArr[testCase], numberOfFrames));
			checkErr (rsavpma_CheckStatus (instrSession));
			}
	      break;
		  
		  default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER,3,"Test Case");
            break;
	   }
	
	
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;	
}

/// HIFN  This function configures the MHL analyzer CTS audio tests IEC60958/IEC61937, Audio Clock Regeneration, Audio Info Frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR testCase/Selects the test case.
/// HIPAR audioSamplingSize/Selects the audio sampling size for the audio test.
/// HIPAR testLength/selects the test length for the audio test.
/// HIPAR numberOfFrames/Sets the number of frames for the audio test.
ViStatus _VI_FUNC rsavpma_ConfigureMHLAnalyzerCTSAudioTests (ViSession instrSession,
                                                           ViInt32 board,
														   ViInt32 testCase,
                                                           ViInt32 audioSamplingSize,
														   ViInt32 testLength,
                                                           ViInt32 numberOfFrames)
                                                           
{
    ViStatus    error = VI_SUCCESS;
    
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    viCheckParm (rsavpma_invalidViInt32Range (board, 1, 8), 2, "Board");
    viCheckParm (rsavpma_invalidViInt32Range (testCase, RSAVPMA_VAL_MHL_TEST_CASE_IEC6, RSAVPMA_VAL_MHL_TEST_CASE_AUDIOI), 3, "Test Case");
	viCheckParm (rsavpma_invalidViInt32Range (audioSamplingSize, RSAVPMA_VAL_MHL_AUDIO_SAMPLING_SIZE_16_BIT, RSAVPMA_VAL_MHL_AUDIO_SAMPLING_SIZE_24_BIT), 4, "Audio Sampling Size");
	viCheckParm (rsavpma_invalidViInt32Range (testLength, RSAVPMA_VAL_MHL_TEST_LENGTH_2_SECONDS, RSAVPMA_VAL_MHL_TEST_LENGTH_FRAMES), 5, "Test Length");
	viCheckParm (rsavpma_invalidViInt32Range (numberOfFrames, 1, 120), 6, "Number Of Frames");
	
	switch (testCase)
		
	   {	
		  case RSAVPMA_VAL_MHL_TEST_CASE_IEC6:
			  
			viCheckErr (viPrintf (instrSession, "CONF%ld:MHL:CTS:AUD:SAMP:SIZE %s,%s\n", board, AnalyzerTestCaseArr[testCase], AudioSamplingSizeArr[audioSamplingSize]));
    		checkErr (rsavpma_CheckStatus (instrSession));
	        viCheckErr (viPrintf (instrSession, "CONF%ld:MHL:CTS:TEST:MODE %s,%s\n", board, AnalyzerTestCaseArr[testCase], TestLengthArr[testLength]));
            checkErr (rsavpma_CheckStatus (instrSession));
			
			if (testLength == RSAVPMA_VAL_MHL_TEST_LENGTH_FRAMES)
			{
			viCheckErr (viPrintf (instrSession, "CONF%ld:MHL:CTS:TEST:FRAM %s,%ld\n", board, AnalyzerTestCaseArr[testCase], numberOfFrames));   
			checkErr (rsavpma_CheckStatus (instrSession));
			}
		  break;
		  
		  case RSAVPMA_VAL_MHL_TEST_CASE_ACLOCK:
		  case RSAVPMA_VAL_MHL_TEST_CASE_AUDIOI:
			  
		    viCheckErr (viPrintf (instrSession, "CONF%ld:MHL:CTS:TEST:MODE %s,%s\n", board, AnalyzerTestCaseArr[testCase], TestLengthArr[testLength]));
			checkErr (rsavpma_CheckStatus (instrSession));
			if (testLength == RSAVPMA_VAL_MHL_TEST_LENGTH_FRAMES)
			{
			viCheckErr (viPrintf (instrSession, "CONF%ld:MHL:CTS:TEST:FRAM %s,%ld\n", board, AnalyzerTestCaseArr[testCase], numberOfFrames));
			checkErr (rsavpma_CheckStatus (instrSession));
			}
		  break;

		  default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER,3,"Test Case");
            break;
	   }
	
	checkErr (rsavpma_CheckStatus (instrSession));
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the MHL analyzer CTS EDID And Device Register Test.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR testCase/Selects the test case.
/// HIPAR EDIDReadingTest/Selects the EDID Reading Test.
/// HIPAR mode/Selects the mode.
ViStatus _VI_FUNC rsavpma_ConfigureMHLAnalyzerCTSEDIDAndDeviceRegisterTest (ViSession instrSession,
                                                                          ViInt32 board,
                                                                          ViInt32 testCase,
                                                                          ViInt32 EDIDReadingTest,
                                                                          ViInt32 mode)
{
    ViStatus    error = VI_SUCCESS;
    
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    viCheckParm (rsavpma_invalidViInt32Range (board, 1, 8), 2, "Board");
    viCheckParm (rsavpma_invalidViInt32Range (testCase, RSAVPMA_VAL_MHL_TEST_CASE_EDIT, RSAVPMA_VAL_MHL_TEST_CASE_DSRT), 3, "Test Case");
	viCheckParm (rsavpma_invalidViInt32Range (EDIDReadingTest, RSAVPMA_VAL_MHL_EDID_2_BLOCK, RSAVPMA_VAL_MHL_EDID_4_BLOCK), 4, "EDID Reading Test");
	viCheckParm (rsavpma_invalidViInt32Range (mode, RSAVPMA_VAL_MHL_DSRT_MODE_NORMAL, RSAVPMA_VAL_MHL_DSRT_MODE_PACKED_PIXEL), 5, "Mode");
	
	switch (testCase)
		
	   {	
		  case RSAVPMA_VAL_MHL_TEST_CASE_EDIT:
			  
			viCheckErr (viPrintf (instrSession, "CONF%ld:MHL:CTS:EDID %s,%s\n", board, AnalyzerTestCaseArr[10], EDIDTestArr[EDIDReadingTest]));
			checkErr (rsavpma_CheckStatus (instrSession));
		  break;
		  
		  case RSAVPMA_VAL_MHL_TEST_CASE_DSRT:
			  
		    viCheckErr (viPrintf (instrSession, "CONF%ld:MHL:CTS:MODE %s,%s\n", board, AnalyzerTestCaseArr[14], DSRTModeArr[mode]));
            checkErr (rsavpma_CheckStatus (instrSession));			
		  break;

		  default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER,3,"Test Case");
            break;
	   }
	
	  checkErr (rsavpma_CheckStatus (instrSession));
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the MHL analyzer CTS RCP Sub-Command RX or TX Test.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR testCase/Selects the test case.
/// HIPAR keyID/Sets the Key ID.
ViStatus _VI_FUNC rsavpma_ConfigureMHLAnalyzerCTSRCPSubCommandTest (ViSession instrSession,
                                                                  ViInt32 board,
                                                                  ViInt32 testCase,
                                                                  ViInt32 keyID)
{
    ViStatus    error = VI_SUCCESS;
    
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    viCheckParm (rsavpma_invalidViInt32Range (board, 1, 8), 2, "Board");
    viCheckParm (rsavpma_invalidViInt32Range (testCase, RSAVPMA_VAL_MHL_TEST_CASE_RCPRX, RSAVPMA_VAL_MHL_TEST_CASE_RCPTX), 3, "Test Case");
	viCheckParm (rsavpma_invalidViInt32Range (keyID, 0, 127), 4, "Key ID");
	
	switch (testCase)
		
	   {	
		  case RSAVPMA_VAL_MHL_TEST_CASE_RCPRX:
          case RSAVPMA_VAL_MHL_TEST_CASE_RCPTX:
			  
			viCheckErr (viPrintf (instrSession, "CONF%ld:MHL:CTS:KEY %s,%ld", board, AnalyzerTestCaseArr[testCase], keyID));
    		 checkErr (rsavpma_CheckStatus (instrSession));
		  break;
		  
		  default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER,3,"Test Case");
            break;
	   }
	
	checkErr (rsavpma_CheckStatus (instrSession));
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the MHL analyzer CTS 3D Video Test - 3D Video Format Timings, 3D Video Mode Indicator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR testCase/Selects the test case.
/// HIPAR videoFormat/Selects the video format.
/// HIPAR _3DFormat/Selects the 3D format.
/// HIPAR testLength/Selects the test length.
/// HIPAR numberOfFrames/Sets the number of frames.
ViStatus _VI_FUNC rsavpma_ConfigureMHLAnalyzerCTS3DVideoTest (ViSession instrSession,
                                                            ViInt32 board,
                                                            ViInt32 testCase,
                                                            ViInt32 videoFormat,
															ViInt32 _3DFormat,
                                                            ViInt32 testLength,
                                                            ViInt32 numberOfFrames)
{
    ViStatus    error = VI_SUCCESS;
    
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    viCheckParm (rsavpma_invalidViInt32Range (board, 1, 8), 2, "Board");
    viCheckParm (rsavpma_invalidViInt32Range (testCase, RSAVPMA_VAL_MHL_TEST_CASE_3DVFT, RSAVPMA_VAL_MHL_TEST_CASE_3DVMI), 3, "Test Case");
	viCheckParm (rsavpma_invalidViInt32Range (videoFormat, RSAVPMA_VAL_MHL_VIDEO_FORMAT_AUTO, RSAVPMA_VAL_MHL_VIDEO_FORMAT_C32), 4, "Video Format");
	viCheckParm (rsavpma_invalidViInt32Range (_3DFormat, RSAVPMA_VAL_MHL_3D_FORMAT_AUTO, RSAVPMA_VAL_MHL_3D_FORMAT_LEFT_RIGHT), 5, "3D Format");
	viCheckParm (rsavpma_invalidViInt32Range (testLength, RSAVPMA_VAL_MHL_TEST_LENGTH_2_SECONDS, RSAVPMA_VAL_MHL_TEST_LENGTH_FRAMES), 6, "Test Length");
	viCheckParm (rsavpma_invalidViInt32Range (numberOfFrames, 1, 120), 7, "Number Of Frames");
	
	switch (testCase)
		
	   {	
		  case RSAVPMA_VAL_MHL_TEST_CASE_3DVFT:
		  case RSAVPMA_VAL_MHL_TEST_CASE_3DVMI:	  
			  
			viCheckErr (viPrintf (instrSession, "CONF%ld:MHL:CTS:VFOR %s,%s", board, AnalyzerTestCaseArr[testCase], VideoFormatArr[videoFormat]));
			checkErr (rsavpma_CheckStatus (instrSession));
			viCheckErr (viPrintf (instrSession, "CONF%ld:MHL:CTS:VFOR:HDEX %s,%s", board, AnalyzerTestCaseArr[testCase], TDFormatArr[_3DFormat]));
			checkErr (rsavpma_CheckStatus (instrSession));
			viCheckErr (viPrintf (instrSession, "CONF%ld:MHL:CTS:TEST:MODE %s,%s", board, AnalyzerTestCaseArr[testCase], TestLengthArr[testLength]));
			checkErr (rsavpma_CheckStatus (instrSession));
			
			if (testLength == RSAVPMA_VAL_MHL_TEST_LENGTH_FRAMES)
			{
			viCheckErr (viPrintf (instrSession, "CONF%ld:MHL:CTS:TEST:FRAM %s,%ld", board, AnalyzerTestCaseArr[testCase], numberOfFrames));
			checkErr (rsavpma_CheckStatus (instrSession));
			}
		  break;
		  
		  default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER,3,"Test Case");
            break;
	   }
	
	checkErr (rsavpma_CheckStatus (instrSession));
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function selects the MHL analyzer CTS UCP Sub-Command Test - Receiving and Transmitting.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR testCase/Selects the test case.
/// HIPAR characterSequence/Sets the character sequence.
ViStatus _VI_FUNC rsavpma_ConfigureMHLAnalyzerCTSUCPSubCommandTests (ViSession instrSession,
                                                                   ViInt32 board,
                                                                   ViInt32 testCase,
                                                                   ViString characterSequence)
{
    ViStatus    error = VI_SUCCESS;
    
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    viCheckParm (rsavpma_invalidViInt32Range (board, 1, 8), 2, "Board");
    viCheckParm (rsavpma_invalidViInt32Range (testCase, RSAVPMA_VAL_MHL_TEST_CASE_UCPRX, RSAVPMA_VAL_MHL_TEST_CASE_UCPTX), 3, "Test Case");

	switch (testCase)
		
	   {	
		  case RSAVPMA_VAL_MHL_TEST_CASE_UCPRX:
		  case RSAVPMA_VAL_MHL_TEST_CASE_UCPTX:
			  
			viCheckErr (viPrintf (instrSession, "CONF%ld:MHL:CTS:UCP %s, '%s'", board, AnalyzerTestCaseArr[testCase], characterSequence));
			checkErr (rsavpma_CheckStatus (instrSession));
		  break;
		  
		  default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER,3,"Test Case");
            break;
	   }
	
	checkErr (rsavpma_CheckStatus (instrSession));
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the MHL analyzer CTS data source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR dataSource/Selects the data source.
/// HIPAR mode/Selects the mode.
/// HIPAR numberOfFrames/Sets the number of frames.
ViStatus _VI_FUNC rsavpma_ConfigureMHLAnalyzerCTSDataSource (ViSession instrSession,
                                                           ViInt32 board,
                                                           ViInt32 dataSource,
                                                           ViInt32 mode,
                                                           ViInt32 numberOfFrames)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpma_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpma_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPMA_ATTR_MHL_ANALYZER_CTS_DATA_SOURCE, dataSource), 3, "Data Source");
	
	viCheckParm(rsavpma_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPMA_ATTR_MHL_ANALYZER_CTS_MODE, mode), 4, "Mode");
	
	viCheckParm(rsavpma_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPMA_ATTR_MHL_ANALYZER_CTS_NUMBER_OF_FRAMES, numberOfFrames), 5, "Number Of Frames");
	
	checkErr (rsavpma_CheckStatus (instrSession));
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function saves or loads the MHL analyzer CTS data source to/from a file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR operation/Selects the data source.
/// HIPAR fileName/Selects the mode.
ViStatus _VI_FUNC rsavpma_MHLAnalyzerCTSDataSourceLoadSave (ViSession instrSession,
                                                          ViInt32 board,
                                                          ViInt32 operation,
                                                          ViString fileName)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    checkErr (rsavpma_hardwareRepCap (instrSession, board, rep_cap));

    viCheckParm (rsavpma_invalidViInt32Range (operation, RSAVPMA_VAL_MHL_DATA_LOAD, RSAVPMA_VAL_MHL_DATA_SAVE), 3, "Operation");

	switch (operation)
		
	   {	
		  case RSAVPMA_VAL_MHL_DATA_LOAD:
			 
			   viCheckParm (rsavpma_SetAttributeViString (instrSession, rep_cap,
                                               RSAVPMA_ATTR_MHL_ANALYZER_CTS_DATA_LOAD,
                                               fileName), 4, "File Name");
		  break;
		  
		  case RSAVPMA_VAL_MHL_DATA_SAVE:
			  
			   viCheckParm (rsavpma_SetAttributeViString (instrSession, rep_cap,
                                               RSAVPMA_ATTR_MHL_ANALYZER_CTS_DATA_STORE,
                                               fileName), 4, "File Name");
			  
		  break;
		  
		  default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER,3,"Operation");
            break;
	   }
	
	checkErr (rsavpma_CheckStatus (instrSession));
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function returns the MHL analyzer CTS test output report count.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR count/Returns the MHL analyzer CTS test output report count.
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerCTSTestOutputReportCount (ViSession instrSession,
                                                                  ViInt32 board,
                                                                  ViInt32 *count)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpma_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpma_GetAttributeViInt32(instrSession, rep_cap,
        RSAVPMA_ATTR_MHL_ANALYZER_CTS_TEST_OUTPUT_REPORT_COUNT, count), 3, "Count");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function returns the MHL analyzer CTS test output report data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR index/Sets the index.
/// HIPAR lineNumber/returns line number referring to TestOutput scroll view. First line has #0
/// HIPAR referenceNumber/Returns reference number of text message.
/// HIPAR textMessage/Returns text message.
/// HIPAR optionalParameters/Returns the first optional parameter.
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerCTSTestOutputReport (ViSession instrSession,
                                                             ViInt32 board,
                                                             ViInt32 index,
															 ViInt32 arraySize,
                                                             ViInt32 *lineNumber,
                                                             ViInt32 *referenceNumber,
                                                             ViChar textMessage[],
                                                             ViInt32 optionalParameters[])
                                                             
                                                             
                                                             
{
    ViStatus error = VI_SUCCESS;
	ViChar   command[RSAVPMA_IO_BUFFER_SIZE] = "";
    ViUInt32 count = 0;
	ViChar   *p2buf  = NULL;
	ViChar   *p2data  = NULL;
	ViInt32	 i = 0;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	viCheckParm (rsavpma_invalidViInt32Range (board, 1, 8), 2, "Board");
	viCheckParm (rsavpma_invalidViInt32Range (index, 0, 6), 3, "Index");
		
    sprintf (command, "READ%ld:MHL:CTS:REP? %ld\n", board, index);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsavpma_CheckStatus (instrSession));
    
    p2buf = strtok (p2data, ",");
    if (lineNumber != NULL) 
	*lineNumber = atoi (p2buf);

    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        return error;
    if (referenceNumber != NULL) 
		sscanf (p2buf, "%x", &referenceNumber);
	
    p2buf = strtok (NULL, ",");
	if (strspn (p2buf, "\"") != 0)
	{
		sscanf (p2buf, "\"%[^\"]", textMessage);
		p2buf = strtok (NULL, ",");
	}
	
	for (i=0;i<arraySize;i++)
    {
	   optionalParameters[i] = atol (p2buf);
	   p2buf = strtok (NULL, ",");
    }

Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function resets the instrument to a known state and sends initialization commands to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
ViStatus _VI_FUNC rsavpma_reset (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (viWrite (instrSession, (ViBuf) "*RST\n", 5, NULL));
    
    checkErr( rsavpma_DefaultInstrSetup (instrSession));

Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;
}

/// HIFN The preset sets the parameters to values suitable for local/manual interaction, a reset to values suitable for remote operation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR reference/Selects the reference of preset.
ViStatus _VI_FUNC rsavpma_ResetSystem (ViSession instrSession, ViInt32 reference)
{
    ViStatus	error = VI_SUCCESS;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	switch (reference)
	{
		case RSAVPMA_VAL_RST_BASE:
			viCheckErr (rsavpma_SetAttributeViString(instrSession, "",
        		RSAVPMA_ATTR_SYSTEM_RESET_BASE, NULL));
			break;
		case RSAVPMA_VAL_RST_ALL:
			viCheckErr (rsavpma_SetAttributeViString(instrSession, "",
        		RSAVPMA_ATTR_SYSTEM_RESET_ALL, NULL));
			break;
	}

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error; 
}

/// HIFN The preset sets the parameters to values suitable for local/manual interaction, a reset to values suitable for remote operation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR reference/Selects the reference of preset.
ViStatus _VI_FUNC rsavpma_Preset (ViSession instrSession, ViInt32 reference)
{
    ViStatus	error = VI_SUCCESS;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	switch (reference)
	{
		case RSAVPMA_VAL_RST_BASE:
			viCheckErr (rsavpma_SetAttributeViString(instrSession, "",
        		RSAVPMA_ATTR_SYSTEM_PRESET_BASE, NULL));
			break;
		case RSAVPMA_VAL_RST_ALL:
			viCheckErr (rsavpma_SetAttributeViString(instrSession, "",
        		RSAVPMA_ATTR_SYSTEM_PRESET_ALL, NULL));
			break;
	}

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error; 
}

/// HIFN This function runs the instrument's self test routine and returns the test result(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR testResult/Contains the value returned from the instrument self test.
/// HIPAR testMessage/Returns the self-test response string from the instrument.
ViStatus _VI_FUNC rsavpma_self_test (ViSession instrSession, ViInt16 *testResult, 
                                      ViChar testMessage[])
{
    ViStatus    error   = VI_SUCCESS;
    ViInt32     result = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (testResult == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Test Result");
    if (testMessage == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Test Message");

    viCheckErr (rsavpma_GetAttributeViInt32 (instrSession, NULL, RSAVPMA_ATTR_SELF_TEST_QUERY, &result));
    if (*testResult == 0)
        strcpy (testMessage, "Self-Test Passed.");
    else
        strcpy (testMessage, "Self-Test Failed.");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function returns the driver and instrument revisions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR driverRev/Returns the instrument driver software revision numbers in the form of a string.
/// HIPAR instrRev/Returns the instrument firmware revision numbers in the form of a string.
ViStatus _VI_FUNC rsavpma_revision_query (ViSession instrSession, ViChar driverRev[], 
                                           ViChar instrRev[])
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (driverRev == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Driver Revision");
    if (instrRev == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Instrument Revision");

    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RSAVPMA_ATTR_SPECIFIC_DRIVER_REVISION,
                                        0, 256, driverRev));
    checkErr( Rs_GetAttribute (instrSession, "", RSAVPMA_ATTR_INSTRUMENT_FIRMWARE_REVISION,
                                        0, 256, instrRev));
    checkErr( rsavpma_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function reads an error code and a message from the
/// HIFN instrument's error queue.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR errCode/Returns the error code read from the instrument's error queue.
/// HIPAR errMessage/Returns the error message string read from the instrument's error message queue.
ViStatus _VI_FUNC rsavpma_error_query (ViSession instrSession, ViInt32 *errCode, 
                                        ViChar errMessage[])
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSAVPMA_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (errCode == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error Code");
    if (errMessage == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
        {
        viCheckErr (rsavpma_GetAttributeViString( instrSession, "", RSAVPMA_ATTR_SYSTEM_ERROR, RSAVPMA_IO_BUFFER_SIZE, buffer));
        if  (sscanf (buffer, "%ld,\"%256[^\"]", errCode, errMessage) != 2)
            viCheckErr(RS_ERROR_UNEXPECTED_RESPONSE);

        }
    else
        {
            /* Simulate Error Query */
        *errCode = 0;
        strcpy (errMessage, "No error.");
        }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function converts a status code returned by an instrument driver function into a user-readable string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR errorCode/Pass the Status parameter that is returned from any of the instrument driver functions.
/// HIPAR errorMessage/Returns the user-readable message string that corresponds to the status code you specify.
ViStatus _VI_FUNC rsavpma_error_message (ViSession instrSession, ViStatus errorCode,
                                          ViChar errorMessage[256])
{
    ViStatus    error   = VI_SUCCESS;

    static      RsStringValueTable errorTable =
        {
            RSAVPMA_ERROR_CODES_AND_MSGS,
            {VI_NULL, VI_NULL}
        };

    (void) Rs_LockSession(instrSession, VI_NULL);

        /* all VISA and RS error codes are handled as well as codes in the table */
    if (errorMessage == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    checkErr( Rs_GetSpecificDriverStatusDesc(instrSession, errorCode, errorMessage, errorTable));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function queries the number of the SCPI version, which is 
/// HIFN  relevant for the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR length/This control specifies the size of the buffer.
/// HIPAR systemVersion/Returns the number of the SCPI version, which is relevant for the 
/// HIPAR systemVersion/instrument.
ViStatus _VI_FUNC rsavpma_SystemVersion(
	ViSession	instrSession,
	ViInt32		length,
	ViChar	  	systemVersion[]
)
{
    ViStatus error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsavpma_GetAttributeViString (instrSession, "",
                                                RSAVPMA_ATTR_SYST_VERSION,
												length,
												systemVersion
                                                ), 2, "System Version");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function switches ON/OFF state checking of the instrument 
/// HIFN  (reading of the Standard Event Register and checking it for error). All 
/// HIFN  driver functions are using state checking. Switch this function to ON 
/// HIFN  when debug your application. For better bus throughput and instrument 
/// HIFN  performance switch it OFF. As default is state checking switched to ON!
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/This control selects ON or OFF instrument state checking.
ViStatus _VI_FUNC rsavpma_setCheckStatus(
	ViSession	instrSession,
	ViBoolean	state
)
{
    ViStatus error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsavpma_SetAttributeViBoolean (instrSession, "",
                                                RS_ATTR_QUERY_INSTRUMENT_STATUS,
												state
                                                ), 2, "State");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function configures the local oscillator.
/// HIFN for the session or the current execution thread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR displayUpdate/Selects the local oscillator mode.
ViStatus _VI_FUNC rsavpma_ConfigureDisplayUpdate (ViSession instrSession,
                                                ViBoolean displayUpdate)
{
    ViStatus	error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm(rsavpma_SetAttributeViBoolean (instrSession, "",
        RSAVPMA_ATTR_DISPLAY_UPDATE, displayUpdate), 2, "Display Update");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}
/// HIFN This function retrieves and then clears the error information
/// HIFN for the session or the current execution thread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR errorCode/Returns the error code for the session or execution thread.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Description parameter.
/// HIPAR description/Returns the error description for the session or execution thread. If there is no description, the function returns an empty string.
ViStatus _VI_FUNC rsavpma_GetError (ViSession instrSession, 
                                     ViStatus *errorCode, 
                                     ViInt32 bufferSize,
                                     ViChar description[])  
{
    ViStatus    error       = VI_SUCCESS;
    ViStatus    primary     = VI_SUCCESS,
                secondary   = VI_SUCCESS;
    ViChar      elaboration[RS_MAX_MESSAGE_BUF_SIZE]    = "";
    ViChar      errorMessage[RS_MAX_MESSAGE_BUF_SIZE]  = "";
    ViChar     *appendPoint = errorMessage;
    ViInt32     actualSize  = 0;
    ViBoolean   locked      = VI_FALSE;
    RsSessionPropertiesPtr  sessionProperties = Rs_ViSession (instrSession);

    /* Lock Session */
    checkErr( Rs_LockSession(instrSession, &locked));

    /* Test for nulls and acquire error data */
    if (bufferSize != 0)
    {
        if (errorCode == VI_NULL)
        {
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error");
        }
        if (sessionProperties == NULL)
        {
            primary = VI_ERROR_RSRC_NFOUND;
            secondary = 0;
        }
        else
        {
            checkErr( Rs_GetErrorInfo (instrSession, &primary, &secondary, elaboration));
        }
    }

    else
        {
            if (sessionProperties == NULL)
            {
                (void) Rs_UnlockSession (instrSession, &locked);
                return 1024;
            }
            else
            {
                checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_ERROR_ELABORATION, 0, 256, elaboration));
                checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_SECONDARY_ERROR, 0, (ViInt32) sizeof (secondary), &secondary));
                checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_PRIMARY_ERROR, 0, (ViInt32) sizeof (primary), &primary));
            }
        }

    /* Format data */
    if (primary != VI_SUCCESS)
        {
        ViChar msg[256] = "";
        checkErr( rsavpma_error_message (instrSession, primary, msg));
        appendPoint += sprintf(appendPoint, "Primary Error: (Hex 0x%08X) %s\n", (unsigned int) primary, msg);
        }

    if (secondary != VI_SUCCESS)
        {
        ViChar msg[256] = "";
        checkErr( rsavpma_error_message (instrSession, secondary, msg));
        appendPoint += sprintf(appendPoint, "Secondary Error: (Hex 0x%08X) %s\n", (unsigned int) secondary, msg);
        }

    if (strlen (elaboration) > 0)
        {
        sprintf(appendPoint, "Elaboration: %s", elaboration);
        }

    actualSize = (ViInt32) (strlen(errorMessage) + 1);

    /* Prepare return values */
    if (bufferSize == 0)
        {
        error = (ViStatus) actualSize;
        }
    else
        {
        if (bufferSize > 0)
            {
            if (actualSize > bufferSize)
                {
                error = actualSize;
                actualSize = bufferSize;
                }
            }
        if (description == VI_NULL)
            {
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Null address for Description");
            }
        memcpy(description, errorMessage, (size_t)(actualSize-1));
        description[actualSize-1] = '\0';
        }

    if (errorCode)
        {
        *errorCode = primary;
        }

Error:
    /* Unlock Session */
    (void) Rs_UnlockSession (instrSession, &locked);
    return error;
}

/// HIFN This function clears the error code and error description for
/// HIFN the session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
ViStatus _VI_FUNC rsavpma_ClearError (ViSession instrSession)                                                        
{                                                                                                           
    return Rs_ClearErrorInfo (instrSession);                                                                             
}

/// HIFN This function writes a user-specified string to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
ViStatus _VI_FUNC rsavpma_WriteInstrData (ViSession instrSession, ViString writeBuffer)
{
    ViStatus error = VI_SUCCESS;

    checkErr (Rs_WriteInstrData (instrSession, writeBuffer));
    
    checkErr (rsavpma_CheckStatus (instrSession));
 
Error:   
    return error;
}

/// HIFN Document your function here.
/// HIFN You may use multiple lines for documentation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR numBytes/Pass the maximum number of bytes to read from the instruments.
/// HIPAR rdBuf/After this function executes, this parameter contains the data that was read from the instrument.
/// HIPAR bytesRead/Returns the number of bytes actually read from the instrument and stored in the Read Buffer.
ViStatus _VI_FUNC rsavpma_ReadInstrData (ViSession instrSession, ViInt32 numBytes,
                                          ViChar rdBuf[], ViUInt32 *bytesRead)
{
    return Rs_ReadInstrData (instrSession, numBytes, rdBuf, bytesRead);
}

/// HIFN This function queries the ViBoolean value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViBoolean value.
ViStatus _VI_FUNC rsavpma_QueryViBoolean (ViSession instrSession,
                                           ViString command, 
                                           ViBoolean *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%d", command, value));
    checkErr( rsavpma_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function queries the ViInt32 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViInt32 value.
ViStatus _VI_FUNC rsavpma_QueryViInt32 (ViSession instrSession,
                                         ViString command, 
                                         ViInt32 *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%ld", command, value));
    checkErr( rsavpma_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function queries the ViReal64 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViReal64 value.
ViStatus _VI_FUNC rsavpma_QueryViReal64 (ViSession instrSession,
                                          ViString command, 
                                          ViReal64 *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%le", command, value));
    checkErr( rsavpma_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function queries the ViString value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Value parameter.
/// HIPAR value/Returns the string value.
ViStatus _VI_FUNC rsavpma_QueryViString (ViSession instrSession,
                                          ViString command, 
                                          ViInt32 bufferSize,
                                          ViChar _VI_FAR value[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      *pbuffer = NULL;
    ViUInt32        bytesRead;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viPrintf (instrSession, "%s\n", command));
    viCheckErr(Rs_ReadDataUnknownLength (instrSession, &pbuffer, &bytesRead));  
    if (bytesRead>=(ViUInt32)bufferSize)
    {
        strncpy(value, pbuffer, (size_t) (bufferSize-1));
        value[bufferSize-1]='\0';
        error = (ViStatus) (bytesRead+1);
    } 
    else
    {
        strncpy(value, pbuffer, (size_t) bytesRead); 
        value[bytesRead]='\0';   
    }
    checkErr( rsavpma_CheckStatus (instrSession)); 
Error:
    if (pbuffer) free (pbuffer);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function closes the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
ViStatus _VI_FUNC rsavpma_close (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    checkErr( rsavpma_RsClose (instrSession));

Error:

    (void) Rs_UnlockSession (instrSession, VI_NULL);
    (void) Rs_Dispose (instrSession);

    return error;
}

/// HIFN This function performs all of the drivers clean-up operations
/// HIFN except for closing the RS session.  This function is called by
/// HIFN rsavpma_close or by an RS class driver.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpma_init or 
/// HIPAR instrSession/rsavpma_InitWithOptions function.
ViStatus _VI_FUNC rsavpma_RsClose (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;
    ViSession   io      = VI_NULL;

    /* Do not lock here. The caller manages the lock. */

    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_IO_SESSION, 0, (ViInt32) sizeof (io), &io));

Error:

    (void) Rs_SetAttribute (instrSession, VI_NULL, RS_ATTR_IO_SESSION, 0, VI_NULL);
    
    if(io != 0)
        {
        ViSession   rmSession = VI_NULL;

        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_VISA_RM_SESSION, 0, (ViInt32) sizeof (rmSession), &rmSession));
        
        (void) viClose (io);
        (void) viClose (rmSession);
        }
    
    return error;
}

/*****************************************************************************
 *------------------- End Instrument Driver Source Code ---------------------*
 *****************************************************************************/
