/****************************************************************************
 *
 *  Rohde & Schwarz Audio Video Platform MHL Analyzer instrument driver include file
 *
 ****************************************************************************/

#ifndef __RSAVPMA_HEADER
#define __RSAVPMA_HEADER

#include "rsidr_core.h"
#include "rsavpma_attributes.h"
#include "rsavpma_utility.h"

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#if defined(__cplusplus) || defined(__cplusplus__)
extern "C" {
#endif

/*****************************************************************************
 *- Useful Macros & Constants -----------------------------------------------*
 *****************************************************************************/
#define RSAVPMA_SIMULATION_ID_QUERY      "Rohde&Schwarz,R&S VTE,2115.7300k02/900000,1.45.0.0"  /* Default definition of ID Query Response for simulation */
#define RSAVPMA_VALID_ID_RESPONSE_STRING "Rohde&Schwarz"      /* Valid response for identification query */
#define RSAVPMA_SIMULATION_OPT_QUERY     ""      /* Simulated response for *OPT? command */

#define RSAVPMA_IO_BUFFER_SIZE           1024L   /* I/O buffer size */
#define RSAVPMA_OPC_TIMEOUT              5000L   /* Maximum time to wait for OPC in milliseconds */
#define IEEE_488_2_ERROR_MASK             0x4    /* 488.2 STB Bits */   
#define IEEE_488_2_ESE_MASK               0x3D    /* 488.2 Event Status Register (ESE) Bits */
#define RSAVPMA_NAN                 9.91e+37
#define RSAVPMA_NAN_VI_INT32             0x80000000     
/****************************************************************************
 *----------------- Instrument Driver Revision Information -----------------*
 ****************************************************************************/

/* Class Driver Identification */

#define RSAVPMA_CLASS_DRIVER_DESCRIPTION           ""
#define RSAVPMA_CLASS_DRIVER_PREFIX                ""
#define RSAVPMA_CLASS_DRIVER_VENDOR                ""
#define RSAVPMA_CLASS_DRIVER_REVISION              ""
#define RSAVPMA_CLASS_SPEC_MAJOR_VERSION           1L    /* Class specification major version */
#define RSAVPMA_CLASS_SPEC_MINOR_VERSION           0L    /* Class specification minor version */

/* Driver Identification */

#define RSAVPMA_SPECIFIC_DRIVER_DESCRIPTION        ""
#define RSAVPMA_SPECIFIC_DRIVER_PREFIX             "RSAVPMA"
#define RSAVPMA_SPECIFIC_DRIVER_LOCATOR            ""
#define RSAVPMA_SPECIFIC_DRIVER_VENDOR             ""
#define RSAVPMA_MAJOR_VERSION                      1L    /* Instrument driver major version          */
#define RSAVPMA_MINOR_VERSION                      0L    /* Instrument driver minor version          */
#define RSAVPMA_MINOR_MINOR_VERSION                0L    /* Instrument driver minor minor version    */

/* Driver Capabilities */          

#define RSAVPMA_SUPPORTED_INSTRUMENT_MODELS        ""    /* Instrument driver supported model(s)     */
#define RSAVPMA_GROUP_CAPABILITIES                 ""
#define RSAVPMA_FUNCTION_CAPABILITIES              ""

/****************************************************************************
 *---------------------------- Attribute Defines ---------------------------*
 ****************************************************************************/

    /*- Inherent Instrument Specific Attributes ----------------------------*/

        /* User Options */

#define RSAVPMA_ATTR_RANGE_CHECK                                           RS_ATTR_RANGE_CHECK                                     /* ViBoolean */
#define RSAVPMA_ATTR_QUERY_INSTRUMENT_STATUS                               RS_ATTR_QUERY_INSTRUMENT_STATUS                         /* ViBoolean */
#define RSAVPMA_ATTR_CACHE                                                 RS_ATTR_CACHE                                           /* ViBoolean */
#define RSAVPMA_ATTR_SIMULATE                                              RS_ATTR_SIMULATE                                        /* ViBoolean */
#define RSAVPMA_ATTR_RECORD_COERCIONS                                      RS_ATTR_RECORD_COERCIONS                                /* ViBoolean */
#define RSAVPMA_ATTR_INTERCHANGE_CHECK                                     RS_ATTR_INTERCHANGE_CHECK                               /* ViBoolean */
#define RSAVPMA_ATTR_SPY                                                   RS_ATTR_SPY                                             /* ViBoolean */
#define RSAVPMA_ATTR_USE_SPECIFIC_SIMULATION                               RS_ATTR_USE_SPECIFIC_SIMULATION                         /* ViBoolean */

        /* Class Driver Identification */

#define RSAVPMA_ATTR_CLASS_DRIVER_DESCRIPTION                              RS_ATTR_CLASS_DRIVER_DESCRIPTION                        /* ViString, read-only */
#define RSAVPMA_ATTR_CLASS_DRIVER_PREFIX                                   RS_ATTR_CLASS_DRIVER_PREFIX                             /* ViString, read-only */
#define RSAVPMA_ATTR_CLASS_DRIVER_VENDOR                                   RS_ATTR_CLASS_DRIVER_VENDOR                             /* ViString, read-only */
#define RSAVPMA_ATTR_CLASS_DRIVER_REVISION                                 RS_ATTR_CLASS_DRIVER_REVISION                           /* ViString, read-only */
#define RSAVPMA_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION           /* ViInt32,  read-only */
#define RSAVPMA_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION           /* ViInt32,  read-only */

        /* Driver Identification */

#define RSAVPMA_ATTR_SPECIFIC_DRIVER_DESCRIPTION                           RS_ATTR_SPECIFIC_DRIVER_DESCRIPTION                     /* ViString, read-only */
#define RSAVPMA_ATTR_SPECIFIC_DRIVER_PREFIX                                RS_ATTR_SPECIFIC_DRIVER_PREFIX                          /* ViString, read-only */
#define RSAVPMA_ATTR_SPECIFIC_DRIVER_LOCATOR                               RS_ATTR_SPECIFIC_DRIVER_LOCATOR                         /* ViString, read-only */
#define RSAVPMA_ATTR_SPECIFIC_DRIVER_VENDOR                                RS_ATTR_SPECIFIC_DRIVER_VENDOR                          /* ViString, read-only */
#define RSAVPMA_ATTR_SPECIFIC_DRIVER_REVISION                              RS_ATTR_SPECIFIC_DRIVER_REVISION                        /* ViString, read-only */
#define RSAVPMA_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION        /* ViInt32,  read-only */
#define RSAVPMA_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION        /* ViInt32,  read-only */

        /* Driver Capabilities */

#define RSAVPMA_ATTR_SUPPORTED_INSTRUMENT_MODELS                           RS_ATTR_SUPPORTED_INSTRUMENT_MODELS                     /* ViString, read-only */
#define RSAVPMA_ATTR_GROUP_CAPABILITIES                                    RS_ATTR_GROUP_CAPABILITIES                              /* ViString, read-only */
#define RSAVPMA_ATTR_FUNCTION_CAPABILITIES                                 RS_ATTR_FUNCTION_CAPABILITIES                           /* ViString, read-only */
#define RSAVPMA_ATTR_CHANNEL_COUNT                                         RS_ATTR_CHANNEL_COUNT                                   /* ViInt32,  read-only */

        /* Driver Setup */

#define RSAVPMA_ATTR_DRIVER_SETUP                                          RS_ATTR_DRIVER_SETUP                                    /* ViString            */

        /* Instrument Identification */

#define RSAVPMA_ATTR_INSTRUMENT_MANUFACTURER                               RS_ATTR_INSTRUMENT_MANUFACTURER                         /* ViString, read-only */
#define RSAVPMA_ATTR_INSTRUMENT_MODEL                                      RS_ATTR_INSTRUMENT_MODEL                                /* ViString, read-only */
#define RSAVPMA_ATTR_INSTRUMENT_FIRMWARE_REVISION                          RS_ATTR_INSTRUMENT_FIRMWARE_REVISION                    /* ViString, read-only */

        /* Advanced Session Information */

#define RSAVPMA_ATTR_IO_RESOURCE_DESCRIPTOR                                RS_ATTR_IO_RESOURCE_DESCRIPTOR                          /* ViString, read-only */
#define RSAVPMA_ATTR_LOGICAL_NAME                                          RS_ATTR_LOGICAL_NAME

/* ViString, read-only */

    /*- Instrument-Specific Attributes -------------------------------------*/

//#define RSAVPMA_ATTR_ID_QUERY_RESPONSE                                     (RS_SPECIFIC_PUBLIC_ATTR_BASE + 1L)                     /* ViString (Read Only)              */

/* BEGIN GENERATE */
#define RSAVPMA_ATTR_MHL_ANALYZER_INPUT                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1
#define RSAVPMA_ATTR_MHL_ANALYZER_HPD_STATE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2
#define RSAVPMA_ATTR_MHL_ANALYZER_INPUT_DEVICE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3
#define RSAVPMA_ATTR_MHL_ANALYZER_MODULE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 4
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_CBUS_CONNECTION                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INPUT_PIXEL_CLOCK_LOCKED            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_HDCP_ACTIVE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INPUT_PIXEL_CLOCK                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INPUT_VIDEO_CODE                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 9
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INPUT_AUDIO_CODING                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_MODULE_CATALOG                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 11
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_VIDEO_CODE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 12
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_PIXEL_CLOCK                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 13
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_VIDEO_FORMAT                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 14
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_VERTICAL_FREQUENCY                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 15
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_HORIZONTAL_FREQUENCY                RS_SPECIFIC_PUBLIC_ATTR_BASE + 16
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_VIDEO_HORIZONTAL_TOTAL              RS_SPECIFIC_PUBLIC_ATTR_BASE + 17
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_VIDEO_HORIZONTAL_ACTIVE             RS_SPECIFIC_PUBLIC_ATTR_BASE + 18
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_VIDEO_HORIZONTAL_FRONT_PORCH        RS_SPECIFIC_PUBLIC_ATTR_BASE + 19
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_VIDEO_HORIZONTAL_SYNCHRONIZATION    RS_SPECIFIC_PUBLIC_ATTR_BASE + 20
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_VIDEO_HORIZONTAL_BACK_PORCH         RS_SPECIFIC_PUBLIC_ATTR_BASE + 21
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_VIDEO_HORIZONTAL_SYNCHRONIZATION_POLARITY RS_SPECIFIC_PUBLIC_ATTR_BASE + 22
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_VIDEO_VERTICAL_TOTAL                RS_SPECIFIC_PUBLIC_ATTR_BASE + 96
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_VIDEO_VERTICAL_ACTIVE               RS_SPECIFIC_PUBLIC_ATTR_BASE + 97
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_VIDEO_VERTICAL_FRONT_PORCH          RS_SPECIFIC_PUBLIC_ATTR_BASE + 98
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_VIDEO_VERTICAL_SYNCHRONIZATION      RS_SPECIFIC_PUBLIC_ATTR_BASE + 99
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_VIDEO_VERTICAL_BACK_PORCH           RS_SPECIFIC_PUBLIC_ATTR_BASE + 100
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_VIDEO_VERTICAL_SYNCHRONIZATION_POLARITY RS_SPECIFIC_PUBLIC_ATTR_BASE + 101
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_AUDIO_N                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 23
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_AUDIO_CYCLE_TIMESTAMP               RS_SPECIFIC_PUBLIC_ATTR_BASE + 24
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_AUDIO_CODING                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 25
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_AUDIO_LAYOUT                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 26
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_HDCP_AUTHENTICATION                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 27
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_HDCP_STATE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 28
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_HDCP_SOURCE_PUBLIC_KEY              RS_SPECIFIC_PUBLIC_ATTR_BASE + 29
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_HDCP_SINK_PUBLIC_KEY                RS_SPECIFIC_PUBLIC_ATTR_BASE + 30
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_HDCP_LINK_SYNCHRONIZATION_VALUE     RS_SPECIFIC_PUBLIC_ATTR_BASE + 31
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_VERSION              RS_SPECIFIC_PUBLIC_ATTR_BASE + 32
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_LENGTH               RS_SPECIFIC_PUBLIC_ATTR_BASE + 33
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_CHECKSUM             RS_SPECIFIC_PUBLIC_ATTR_BASE + 34
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_COLOR_SPACE          RS_SPECIFIC_PUBLIC_ATTR_BASE + 35
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_ACTIVE_FORMAT        RS_SPECIFIC_PUBLIC_ATTR_BASE + 36
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_BAR_INFORMATION      RS_SPECIFIC_PUBLIC_ATTR_BASE + 37
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_SCAN_INFORMATION     RS_SPECIFIC_PUBLIC_ATTR_BASE + 38
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_COLORIMETRY          RS_SPECIFIC_PUBLIC_ATTR_BASE + 39
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_PICTURE_ASPECT_RATIO RS_SPECIFIC_PUBLIC_ATTR_BASE + 40
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_ACTIVE_FORMAT_ASPECT_RATIO RS_SPECIFIC_PUBLIC_ATTR_BASE + 41
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_IT_CONTENT           RS_SPECIFIC_PUBLIC_ATTR_BASE + 42
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_EXTENDED_COLORIMETRY RS_SPECIFIC_PUBLIC_ATTR_BASE + 43
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_RGB_QUANTIZATION_RANGE RS_SPECIFIC_PUBLIC_ATTR_BASE + 44
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_NONUNIFORM_PICTURE_SCALING RS_SPECIFIC_PUBLIC_ATTR_BASE + 45
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_VIDEO_IDENTIFICATION_CODE RS_SPECIFIC_PUBLIC_ATTR_BASE + 46
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_YCC_QUANTIZATION     RS_SPECIFIC_PUBLIC_ATTR_BASE + 47
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_CONTENT_TYPE         RS_SPECIFIC_PUBLIC_ATTR_BASE + 48
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_PIXEL_REPETITION     RS_SPECIFIC_PUBLIC_ATTR_BASE + 49
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_TOP_BAR_END          RS_SPECIFIC_PUBLIC_ATTR_BASE + 50
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_BOTTOM_BAR_START     RS_SPECIFIC_PUBLIC_ATTR_BASE + 51
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_LEFT_BAR_END         RS_SPECIFIC_PUBLIC_ATTR_BASE + 52
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AVI_RIGHT_BAR_START      RS_SPECIFIC_PUBLIC_ATTR_BASE + 53
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AUDIO_VERSION            RS_SPECIFIC_PUBLIC_ATTR_BASE + 54
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AUDIO_LENGTH             RS_SPECIFIC_PUBLIC_ATTR_BASE + 55
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AUDIO_CHECKSUM           RS_SPECIFIC_PUBLIC_ATTR_BASE + 56
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AUDIO_CODING_TYPE        RS_SPECIFIC_PUBLIC_ATTR_BASE + 57
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AUDIO_CHANNEL_COUNT      RS_SPECIFIC_PUBLIC_ATTR_BASE + 58
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AUDIO_SAMPLING_FREQUENCY RS_SPECIFIC_PUBLIC_ATTR_BASE + 59
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AUDIO_SAMPLING_SIZE      RS_SPECIFIC_PUBLIC_ATTR_BASE + 60
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AUDIO_CODE_EXTENSION     RS_SPECIFIC_PUBLIC_ATTR_BASE + 61
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AUDIO_CHANNEL_ALLOCATION RS_SPECIFIC_PUBLIC_ATTR_BASE + 62
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AUDIO_DOWNMIX_INHIBIT    RS_SPECIFIC_PUBLIC_ATTR_BASE + 63
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AUDIO_LEVEL_SHIFT_VALUE  RS_SPECIFIC_PUBLIC_ATTR_BASE + 64
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_AUDIO_LFE_PLAYBACK_LEVEL RS_SPECIFIC_PUBLIC_ATTR_BASE + 65
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_SPD_VERSION              RS_SPECIFIC_PUBLIC_ATTR_BASE + 66
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_SPD_LENGTH               RS_SPECIFIC_PUBLIC_ATTR_BASE + 67
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_SPD_CHECKSUM             RS_SPECIFIC_PUBLIC_ATTR_BASE + 68
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_SPD_VENDOR               RS_SPECIFIC_PUBLIC_ATTR_BASE + 69
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_SPD_PRODUCT_DESCRIPTION  RS_SPECIFIC_PUBLIC_ATTR_BASE + 70
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_MPEG_VERSION             RS_SPECIFIC_PUBLIC_ATTR_BASE + 71
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_MPEG_LENGTH              RS_SPECIFIC_PUBLIC_ATTR_BASE + 72
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_MPEG_CHECKSUM            RS_SPECIFIC_PUBLIC_ATTR_BASE + 73
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_MPEG_BIT_RATE            RS_SPECIFIC_PUBLIC_ATTR_BASE + 74
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_MPEG_FIELD_REPEAT        RS_SPECIFIC_PUBLIC_ATTR_BASE + 75
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_INFO_FRAME_MPEG_FRAME               RS_SPECIFIC_PUBLIC_ATTR_BASE + 76
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_CTS_STATE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 77
#define RSAVPMA_ATTR_MHL_ANALYZER_CTS_INIT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 78
#define RSAVPMA_ATTR_MHL_ANALYZER_CTS_STOP                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 79
#define RSAVPMA_ATTR_QUERY_MHL_ANALYZER_CTS_PROGRESS                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 80
#define RSAVPMA_ATTR_MHL_ANALYZER_CTS_DATA_SOURCE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 81
#define RSAVPMA_ATTR_MHL_ANALYZER_CTS_MODE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 82
#define RSAVPMA_ATTR_MHL_ANALYZER_CTS_NUMBER_OF_FRAMES                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 83
#define RSAVPMA_ATTR_MHL_ANALYZER_CTS_DATA_STORE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 84
#define RSAVPMA_ATTR_MHL_ANALYZER_CTS_DATA_LOAD                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 85
#define RSAVPMA_ATTR_MHL_ANALYZER_CTS_TEST_OUTPUT_REPORT_COUNT              RS_SPECIFIC_PUBLIC_ATTR_BASE + 86
#define RSAVPMA_ATTR_SELF_TEST_QUERY                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 87
#define RSAVPMA_ATTR_SYST_VERSION                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 88
#define RSAVPMA_ATTR_SYSTEM_RESET_ALL                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 89
#define RSAVPMA_ATTR_SYSTEM_RESET_BASE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 90
#define RSAVPMA_ATTR_SYSTEM_PRESET_ALL                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 91
#define RSAVPMA_ATTR_SYSTEM_PRESET_BASE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 92
#define RSAVPMA_ATTR_ID_QUERY_RESPONSE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 93
#define RSAVPMA_ATTR_SYSTEM_ERROR                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 94
#define RSAVPMA_ATTR_DISPLAY_UPDATE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 95

// rsavpma_rngMHLAnalyzerInput
#define RSAVPMA_VAL_MHL_INPUT_1_3                                           0
#define RSAVPMA_VAL_MHL_INPUT_2_1                                           1

// rsavpma_rngMHLAnalyzerInputDevice
#define RSAVPMA_VAL_MHL_INPUT_DEVICE_SINK                                   0
#define RSAVPMA_VAL_MHL_INPUT_DEVICE_REPEATER                               1

// rsavpma_rngMHLAnalyzerModule
#define RSAVPMA_VAL_MHL_MODULE_MHL_L1                                       0

// rsavpma_rngQueryMHLAnalyzerCBUSConnection
#define RSAVPMA_VAL_MHL_CBUS_WAIT_FOR_CONNECTION                            0
#define RSAVPMA_VAL_MHL_CBUS_CONNECTED                                      1

// rsavpma_rngQueryMHLAnalyzerInputPixelClockLocked
#define RSAVPMA_VAL_MHL_PIXEL_CLOCK_UNLOCKED                                0
#define RSAVPMA_VAL_MHL_PIXEL_CLOCK_LOCKED                                  1

// rsavpma_rngQueryMHLAnalyzerHDCPActive
#define RSAVPMA_VAL_MHL_HDCP_INVALID                                        0
#define RSAVPMA_VAL_MHL_HDCP_INACTIVE                                       1
#define RSAVPMA_VAL_MHL_HDCP_ACTIVE                                         2
#define RSAVPMA_VAL_MHL_HDCP_NAV                                            3

// rsavpma_rngQueryMHLAnalyzerCTSState
#define RSAVPMA_VAL_MHL_STATE_IDLE                                          0
#define RSAVPMA_VAL_MHL_STATE_CAPTURING                                     1
#define RSAVPMA_VAL_MHL_STATE_RUNNING                                       2
#define RSAVPMA_VAL_MHL_STATE_FINISHED                                      3

// rsavpma_rngMHLAnalyzerCTSDataSource
#define RSAVPMA_VAL_MHL_DATA_SOURCE_AUTO                                    0
#define RSAVPMA_VAL_MHL_DATA_SOURCE_MANUAL                                  1
#define RSAVPMA_VAL_MHL_DATA_SOURCE_FILE                                    2

// rsavpma_rngMHLAnalyzerCTSMode
#define RSAVPMA_VAL_MHL_DATA_SIZE_MODE_2_SECONDS                            0
#define RSAVPMA_VAL_MHL_DATA_SIZE_MODE_FRAMES                               1
/* END GENERATE */

/*****************************************************************************
 *- Hidden Attribute Declarations -------------------------------------------*
 *****************************************************************************/

#define RSAVPMA_ATTR_OPC_TIMEOUT               (RS_ATTR_OPC_TIMEOUT)           /* ViInt32 */
#define RSAVPMA_ATTR_IO_SESSION                (RS_ATTR_IO_SESSION)            /* ViSession */
#define RSAVPMA_ATTR_OPC_CALLBACK              (RS_ATTR_OPC_CALLBACK)          /* ViAddr */
#define RSAVPMA_ATTR_CHECK_STATUS_CALLBACK     (RS_ATTR_CHECK_STATUS_CALLBACK) /* ViAddr */

/****************************************************************************
 *---------------- Constant definition  ------------------------------------*
 ****************************************************************************/


#define	RSAVPMA_VAL_VIDEO_SCALE_X              0
#define RSAVPMA_VAL_VIDEO_SCALE_Y_AMPLITUDE    1
#define RSAVPMA_VAL_VIDEO_SCALE_Y_GROUP_DELAY  2

#define RSAVPMA_VAL_TIME  0
#define RSAVPMA_VAL_PIXEL 1

#define RSAVPMA_VAL_VIDEO_SCOPE_Y_GREEN  0
#define RSAVPMA_VAL_VIDEO_SCOPE_BLUE     1
#define RSAVPMA_VAL_VIDEO_SCOPE_RED      2
#define RSAVPMA_VAL_VIDEO_SCOPE_CCVS     3

#define RSAVPMA_VAL_VECTOR_COLOR_BLACK    0
#define RSAVPMA_VAL_VECTOR_COLOR_BLUE     1
#define RSAVPMA_VAL_VECTOR_COLOR_CYAN     2
#define RSAVPMA_VAL_VECTOR_COLOR_GREEN    3
#define RSAVPMA_VAL_VECTOR_COLOR_MAGENTA  4
#define RSAVPMA_VAL_VECTOR_COLOR_RED      5
#define RSAVPMA_VAL_VECTOR_COLOR_WHITE    6
#define RSAVPMA_VAL_VECTOR_COLOR_YELLOW   7

#define RSAVPMA_VAL_ANALOG   0
#define RSAVPMA_VAL_DIGITAL  1

#define RSAVPMA_VAL_MHL_VERTICAL       0
#define RSAVPMA_VAL_MHL_HORIZONTAL     1

#define	RSAVPMA_VAL_MHL_AUDIO_APPLICATION     0
#define RSAVPMA_VAL_MHL_AUDIO_SAMPLE          1
#define RSAVPMA_VAL_MHL_AUDIO_COPYRIGHT       2
#define RSAVPMA_VAL_MHL_AUDIO_PREEMPHASIS     3
#define RSAVPMA_VAL_MHL_AUDIO_MODE            4
#define RSAVPMA_VAL_MHL_AUDIO_CATEGORY_CODE   5
#define RSAVPMA_VAL_MHL_AUDIO_SOURCE_NUMBER   6
#define RSAVPMA_VAL_MHL_AUDIO_CHANNEL_NUMBER  7
#define RSAVPMA_VAL_MHL_AUDIO_FS              8
#define RSAVPMA_VAL_MHL_AUDIO_CLOCK_ACCURACY  9
#define RSAVPMA_VAL_MHL_AUDIO_MAX_LENGTH      10
#define RSAVPMA_VAL_MHL_AUDIO_LENGTH          11

#define	RSAVPMA_VAL_MHL_PIXEL_TOTAL           0
#define RSAVPMA_VAL_MHL_PIXEL_ACTIVE          1
#define RSAVPMA_VAL_MHL_PIXEL_FRONT_PORCH     2
#define RSAVPMA_VAL_MHL_PIXEL_BACK_PORCH      3
#define RSAVPMA_VAL_MHL_PIXEL_SYNC            4
#define RSAVPMA_VAL_MHL_PIXEL_SYNC_POLARITY   5

#define	RSAVPMA_VAL_MHL_INFO_FRAME_RGB    0
#define RSAVPMA_VAL_MHL_INFO_FRAME_AINFO  1
#define RSAVPMA_VAL_MHL_INFO_FRAME_BINFO  2
#define RSAVPMA_VAL_MHL_INFO_FRAME_SINFO  3
#define RSAVPMA_VAL_MHL_INFO_FRAME_COL    4
#define RSAVPMA_VAL_MHL_INFO_FRAME_ARAT   5
#define RSAVPMA_VAL_MHL_INFO_FRAME_AFAR   6
#define RSAVPMA_VAL_MHL_INFO_FRAME_ITC    7
#define RSAVPMA_VAL_MHL_INFO_FRAME_ECOL   8
#define RSAVPMA_VAL_MHL_INFO_FRAME_RGBQ   9
#define RSAVPMA_VAL_MHL_INFO_FRAME_NUPS   10
#define RSAVPMA_VAL_MHL_INFO_FRAME_VIDC   11
#define RSAVPMA_VAL_MHL_INFO_FRAME_YCCQ   12
#define RSAVPMA_VAL_MHL_INFO_FRAME_CTYP   13
#define RSAVPMA_VAL_MHL_INFO_FRAME_PREP   14

#define RSAVPMA_VAL_MHL_INFO_FRAME_CODING_TYPE  0
#define RSAVPMA_VAL_MHL_INFO_FRAME_CHAN_COUNT   1
#define RSAVPMA_VAL_MHL_INFO_FRAME_SAMP_FREQ    2
#define RSAVPMA_VAL_MHL_INFO_FRAME_SAMP_SIZE    3
#define RSAVPMA_VAL_MHL_INFO_FRAME_CODE_EXT     4
#define RSAVPMA_VAL_MHL_INFO_FRAME_CHAN_ALLOC   5
#define RSAVPMA_VAL_MHL_INFO_FRAME_DMIX_INHIB   6
#define RSAVPMA_VAL_MHL_INFO_FRAME_LEVEL_SHIFT  7
#define RSAVPMA_VAL_MHL_INFO_FRAME_LFE          8

#define	RSAVPMA_VAL_MHL_INFO_FRAME_VENDOR   0
#define RSAVPMA_VAL_MHL_INFO_FRAME_PRODUCT  1

#define RSAVPMA_VAL_MHL_STOP   0
#define RSAVPMA_VAL_MHL_START  1

#define RSAVPMA_VAL_MHL_TEST_CASE_LCODES      0
#define RSAVPMA_VAL_MHL_TEST_CASE_BASPROT     1
#define RSAVPMA_VAL_MHL_TEST_CASE_PACKET      2
#define RSAVPMA_VAL_MHL_TEST_CASE_VFORMAT     3
#define RSAVPMA_VAL_MHL_TEST_CASE_PIXEL       4
#define RSAVPMA_VAL_MHL_TEST_CASE_AVI         5
#define RSAVPMA_VAL_MHL_TEST_CASE_VQRANGE     6
#define RSAVPMA_VAL_MHL_TEST_CASE_IEC6        7
#define RSAVPMA_VAL_MHL_TEST_CASE_ACLOCK      8
#define RSAVPMA_VAL_MHL_TEST_CASE_AUDIOI      9
#define RSAVPMA_VAL_MHL_TEST_CASE_EDIT        10
#define RSAVPMA_VAL_MHL_TEST_CASE_DCRT        11
#define RSAVPMA_VAL_MHL_TEST_CASE_RCPRX       12
#define RSAVPMA_VAL_MHL_TEST_CASE_RCPTX       13
#define RSAVPMA_VAL_MHL_TEST_CASE_DSRT        14
#define RSAVPMA_VAL_MHL_TEST_CASE_RAPSC       15
#define RSAVPMA_VAL_MHL_TEST_CASE_3DVMS       16
#define RSAVPMA_VAL_MHL_TEST_CASE_3DVFT       17
#define RSAVPMA_VAL_MHL_TEST_CASE_3DVMI       18
#define RSAVPMA_VAL_MHL_TEST_CASE_UCPRX       19
#define RSAVPMA_VAL_MHL_TEST_CASE_UCPTX       20
#define RSAVPMA_VAL_MHL_TEST_CASE_CONCURRENT  21	

#define	RSAVPMA_VAL_MHL_AUDIO_SAMPLING_SIZE_16_BIT  0
#define RSAVPMA_VAL_MHL_AUDIO_SAMPLING_SIZE_20_BIT  1
#define RSAVPMA_VAL_MHL_AUDIO_SAMPLING_SIZE_24_BIT  2

#define RSAVPMA_VAL_MHL_EDID_2_BLOCK  0
#define RSAVPMA_VAL_MHL_EDID_4_BLOCK  1
	
#define RSAVPMA_VAL_MHL_DSRT_MODE_NORMAL        0
#define RSAVPMA_VAL_MHL_DSRT_MODE_PACKED_PIXEL  1
	
#define	RSAVPMA_VAL_MHL_3D_FORMAT_AUTO          0
#define RSAVPMA_VAL_MHL_3D_FORMAT_TOP_BOTTOM    1
#define RSAVPMA_VAL_MHL_3D_FORMAT_LEFT_RIGHT    2
	
#define RSAVPMA_VAL_MHL_DATA_LOAD   0
#define RSAVPMA_VAL_MHL_DATA_SAVE   1

#define RSAVPMA_VAL_MHL_3D_FORMAT_FRAME_SEQ     1
	
#define RSAVPMA_VAL_MHL_VIDEO_FORMAT_AUTO       0
#define RSAVPMA_VAL_MHL_VIDEO_FORMAT_C1         1
#define RSAVPMA_VAL_MHL_VIDEO_FORMAT_C2         2
#define RSAVPMA_VAL_MHL_VIDEO_FORMAT_C17        3
#define RSAVPMA_VAL_MHL_VIDEO_FORMAT_C4         4
#define RSAVPMA_VAL_MHL_VIDEO_FORMAT_C19        5
#define RSAVPMA_VAL_MHL_VIDEO_FORMAT_C5         6
#define RSAVPMA_VAL_MHL_VIDEO_FORMAT_C20        7
#define RSAVPMA_VAL_MHL_VIDEO_FORMAT_C32        8
#define RSAVPMA_VAL_MHL_VIDEO_FORMAT_C34        9
#define RSAVPMA_VAL_MHL_VIDEO_FORMAT_C6        10
#define RSAVPMA_VAL_MHL_VIDEO_FORMAT_C21       11

#define RSAVPMA_VAL_RST_BASE 0
#define RSAVPMA_VAL_RST_ALL  1

#define RSAVPMA_VAL_MHL_TEST_LENGTH_2_SECONDS 0
#define RSAVPMA_VAL_MHL_TEST_LENGTH_FRAMES    1

#define RSAVPMA_VAL_MHL_PIXEL_ENCODING_RGB  0
#define RSAVPMA_VAL_MHL_PIXEL_ENCODING_Y422 1
#define RSAVPMA_VAL_MHL_PIXEL_ENCODING_Y444 2

#define RSAVPMA_VAL_MHL_TEST_PATTERN_FULLY_SATURATED 0
#define RSAVPMA_VAL_MHL_TEST_PATTERN_VISUAL_ANALYSIS 1

#define RSAVPMA_VAL_MHL_TEST_SETUP_YQY0 0
#define RSAVPMA_VAL_MHL_TEST_SETUP_YQY1 1
#define RSAVPMA_VAL_MHL_TEST_SETUP_RGB  2
/****************************************************************************
*---------------- Instrument Driver Function Declarations -----------------*
 ****************************************************************************/

ViStatus _VI_FUNC rsavpma_init (ViRsrc resourceName, ViBoolean IDQuery,
                                ViBoolean resetDevice, ViSession* instrumentHandle);
ViStatus _VI_FUNC rsavpma_InitWithOptions (ViRsrc resourceName, ViBoolean IDQuery,
                                           ViBoolean resetDevice,
                                           ViChar _VI_FAR optionString[],
                                           ViSession* instrumentHandle);
ViStatus _VI_FUNC rsavpma_ConfigureMHLAnalyzerInput (ViSession instrumentHandle,
                                                     ViInt32 board, ViInt32 input,
                                                     ViBoolean HPDSignal,
                                                     ViInt32 device,
                                                     ViInt32 module);
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerInput (ViSession instrumentHandle,
                                                 ViInt32 board, ViInt32 bufferSize,
                                                 ViInt32* CBUS,
                                                 ViInt32* pixelClockLocked,
                                                 ViInt32* HDCP,
                                                 ViReal64* pixelClock,
                                                 ViChar _VI_FAR videoCode[],
                                                 ViChar _VI_FAR audioCoding[]);
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerModuleCatalog (ViSession instrumentHandle,
                                                         ViInt32 board,
                                                         ViInt32 bufferSize,
                                                         ViChar _VI_FAR catalog[]);
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerVideo (ViSession instrumentHandle,
                                                 ViInt32 board, ViInt32 bufferSize,
                                                 ViChar _VI_FAR videoCode[],
                                                 ViReal64* pixelClock,
                                                 ViInt32* videoFormat,
                                                 ViReal64* verticalFrequency,
                                                 ViReal64* horizontalFrequency);
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerVideoPixels (ViSession instrumentHandle,
                                                       ViInt32 board,
                                                       ViInt32 videoParameter,
                                                       ViInt32 parameter,
                                                       ViInt32* pixelValue);
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerAudio (ViSession instrumentHandle,
                                                 ViInt32 board, ViInt32 bufferSize,
                                                 ViInt32* n, ViInt32* CTS,
                                                 ViChar _VI_FAR audioCoding[],
                                                 ViChar _VI_FAR layout[]);
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerAudioChannelStatusBits
             (ViSession instrumentHandle, ViInt32 board, ViInt32 parameter,
              ViInt32 bufferSize, ViChar _VI_FAR result[],
              ViChar _VI_FAR channelStatusBit[]);
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerHDCP (ViSession instrumentHandle,
                                                ViInt32 board, ViInt32 bufferSize,
                                                ViChar _VI_FAR authentication[],
                                                ViInt32* HDCP, ViInt32* AKSV,
                                                ViInt32* BKSV, ViInt32* ri,
                                                ViInt32 _VI_FAR an[]);
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerInfoFrameAVI (ViSession instrumentHandle,
                                                        ViInt32 board,
                                                        ViInt32* version,
                                                        ViInt32* length,
                                                        ViInt32* checksum);
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerInfoFrameAVISpecific
             (ViSession instrumentHandle, ViInt32 board, ViInt32 parameter,
              ViInt32 bufferSize, ViChar _VI_FAR result[]);
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerInfoFrameAVIBar
             (ViSession instrumentHandle, ViInt32 board,
              ViInt32* lineNumberOfTopBarEnd, ViInt32* lineNumberOfBottomBarStart,
              ViInt32* pixelNumberOfLeftBarEnd,
              ViInt32* pixelNumberOfRightBarStart);
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerInfoFrameAVIHeaderPacketData
             (ViSession instrumentHandle, ViInt32 board, ViInt32 arraySize,
              ViInt32 _VI_FAR headerBytes[], ViInt32 _VI_FAR packetBytes[]);
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerInfoFrameAudio (ViSession instrumentHandle,
                                                          ViInt32 board,
                                                          ViInt32* version,
                                                          ViInt32* length,
                                                          ViInt32* checksum);
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerInfoFrameAudioSpecific
             (ViSession instrumentHandle, ViInt32 board, ViInt32 parameter, ViInt32 bufferSize,
              ViChar _VI_FAR result[]);
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerInfoFrameAudioHeaderPacketData
             (ViSession instrumentHandle, ViInt32 board, ViInt32 arraySize,
              ViInt32 _VI_FAR headerBytes[], ViInt32 _VI_FAR packetBytes[]);
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerInfoFrameSPD (ViSession instrumentHandle,
                                                        ViInt32 board,
                                                        ViInt32* version,
                                                        ViInt32* length,
                                                        ViInt32* checksum);
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerInfoFrameSPDSpecific
             (ViSession instrumentHandle, ViInt32 board, ViInt32 parameter,
              ViInt32 bufferSize, ViChar _VI_FAR result[]);
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerInfoFrameSPDHeaderPacketData
             (ViSession instrumentHandle, ViInt32 board, ViInt32 arraySize,
              ViInt32 _VI_FAR headerBytes[], ViInt32 _VI_FAR packetBytes[]);
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerInfoFrameMPEGSource
             (ViSession instrumentHandle, ViInt32 board, ViInt32* version,
              ViInt32* length, ViInt32* checksum);
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerInfoFrameMPEGSourceSpecific
             (ViSession instrumentHandle, ViInt32 board, ViInt32 bufferSize,
              ViInt32* MPEGBitRate, ViChar _VI_FAR fieldRepeat[],
              ViChar _VI_FAR MPEGFrame[]);
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerInfoFrameMPEGSourceHeaderPacketData
             (ViSession instrumentHandle, ViInt32 board, ViInt32 arraySize,
              ViInt32 _VI_FAR headerBytes[], ViInt32 _VI_FAR packetBytes[]);
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerCTSState (ViSession instrumentHandle,
                                                    ViInt32 board, ViInt32* state);
ViStatus _VI_FUNC rsavpma_MHLAnalyzerCTSOperation (ViSession instrumentHandle,
                                                   ViInt32 board,
                                                   ViInt32 operation);
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerCTSProgress (ViSession instrumentHandle,
                                                       ViInt32 board,
                                                       ViInt32* progress);
ViStatus _VI_FUNC rsavpma_SelectMHLAnalyzerCTSTestCase (ViSession instrumentHandle,
                                                        ViInt32 board,
                                                        ViInt32 testCase,
                                                        ViBoolean state);
ViStatus _VI_FUNC rsavpma_ConfigureMHLAnalyzerCTSTMDSCoding
             (ViSession instrumentHandle, ViInt32 board, ViInt32 testCase,
              ViInt32 videoFormat, ViInt32 testLength, ViInt32 numberOfFrames);
ViStatus _VI_FUNC rsavpma_ConfigureMHLAnalyzerCTSVideoModes
             (ViSession instrumentHandle, ViInt32 board, ViInt32 testCase,
              ViInt32 videoFormat, ViInt32 testLength, ViInt32 numberOfFrames,
              ViInt32 pixelEncoding, ViInt32 testPattern, ViInt32 testSetup);
ViStatus _VI_FUNC rsavpma_ConfigureMHLAnalyzerCTSAudioTests
             (ViSession instrumentHandle, ViInt32 board, ViInt32 testCase,
              ViInt32 audioSamplingSize, ViInt32 testLength,
              ViInt32 numberOfFrames);
ViStatus _VI_FUNC rsavpma_ConfigureMHLAnalyzerCTSEDIDAndDeviceRegisterTest
             (ViSession instrumentHandle, ViInt32 board, ViInt32 testCase,
              ViInt32 EDIDReadingTest, ViInt32 mode);
ViStatus _VI_FUNC rsavpma_ConfigureMHLAnalyzerCTSRCPSubCommandTest
             (ViSession instrumentHandle, ViInt32 board, ViInt32 testCase,
              ViInt32 keyID);
ViStatus _VI_FUNC rsavpma_ConfigureMHLAnalyzerCTS3DVideoTest
             (ViSession instrumentHandle, ViInt32 board, ViInt32 testCase,
              ViInt32 videoFormat, ViInt32 _3DFormat, ViInt32 testLength,
              ViInt32 numberOfFrames);
ViStatus _VI_FUNC rsavpma_ConfigureMHLAnalyzerCTSUCPSubCommandTests
             (ViSession instrumentHandle, ViInt32 board, ViInt32 testCase,
              ViString characterSequence);
ViStatus _VI_FUNC rsavpma_ConfigureMHLAnalyzerCTSDataSource
             (ViSession instrumentHandle, ViInt32 board, ViInt32 dataSource,
              ViInt32 mode, ViInt32 numberOfFrames);
ViStatus _VI_FUNC rsavpma_MHLAnalyzerCTSDataSourceLoadSave
             (ViSession instrumentHandle, ViInt32 board, ViInt32 operation,
              ViString fileName);
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerCTSTestOutputReportCount
             (ViSession instrumentHandle, ViInt32 board, ViInt32* count);
ViStatus _VI_FUNC rsavpma_QueryMHLAnalyzerCTSTestOutputReport
             (ViSession instrumentHandle, ViInt32 board, ViInt32 index,
              ViInt32 arraySize, ViInt32* lineNumber, ViInt32* referenceNumber,
              ViChar _VI_FAR textMessage[], ViInt32 _VI_FAR optionalParameters[]);
ViStatus _VI_FUNC rsavpma_reset (ViSession instrumentHandle);
ViStatus _VI_FUNC rsavpma_ResetSystem (ViSession instrumentHandle, ViInt32 reference);
ViStatus _VI_FUNC rsavpma_Preset (ViSession instrumentHandle, ViInt32 reference);
ViStatus _VI_FUNC rsavpma_self_test (ViSession instrumentHandle,
                                     ViInt16* selfTestResult,
                                     ViChar _VI_FAR selfTestMessage[]);
ViStatus _VI_FUNC rsavpma_revision_query (ViSession instrumentHandle,
                                          ViChar _VI_FAR instrumentDriverRevision[],
                                          ViChar _VI_FAR firmwareRevision[]);
ViStatus _VI_FUNC rsavpma_error_query (ViSession instrumentHandle,
                                       ViInt32* errorCode,
                                       ViChar _VI_FAR errorMessage[]);
ViStatus _VI_FUNC rsavpma_error_message (ViSession instrumentHandle,
                                         ViStatus errorCode,
                                         ViChar _VI_FAR errorMessage[]);
ViStatus _VI_FUNC rsavpma_SystemVersion (ViSession instrumentHandle, ViInt32 length,
                                         ViChar _VI_FAR systemVersion[]);
ViStatus _VI_FUNC rsavpma_setCheckStatus (ViSession instrumentHandle,
                                          ViBoolean state);
ViStatus _VI_FUNC rsavpma_ConfigureDisplayUpdate (ViSession instrumentHandle,
                                                  ViBoolean displayUpdate);
ViStatus _VI_FUNC rsavpma_GetError (ViSession instrumentHandle, ViStatus* code,
                                    ViInt32 bufferSize,
                                    ViChar _VI_FAR description[]);
ViStatus _VI_FUNC rsavpma_ClearError (ViSession instrumentHandle);
ViStatus _VI_FUNC rsavpma_WriteInstrData (ViSession instrumentHandle,
                                          ViChar _VI_FAR writeBuffer[]);
ViStatus _VI_FUNC rsavpma_ReadInstrData (ViSession instrumentHandle,
                                         ViInt32 numberOfBytesToRead,
                                         ViChar _VI_FAR readBuffer[],
                                         ViUInt32* numBytesRead);
ViStatus _VI_FUNC rsavpma_QueryViBoolean (ViSession instrumentHandle,
                                          ViString command, ViBoolean* value);
ViStatus _VI_FUNC rsavpma_QueryViInt32 (ViSession instrumentHandle,
                                        ViString command, ViInt32* value);
ViStatus _VI_FUNC rsavpma_QueryViReal64 (ViSession instrumentHandle,
                                         ViString command, ViReal64* value);
ViStatus _VI_FUNC rsavpma_QueryViString (ViSession instrumentHandle,
                                         ViString command, ViInt32 bufferSize,
                                         ViChar _VI_FAR value[]);
ViStatus _VI_FUNC rsavpma_SetAttributeViInt32 (ViSession instrumentHandle,
                                               ViChar _VI_FAR channelName[],
                                               ViAttr attributeID,
                                               ViInt32 attributeValue);
ViStatus _VI_FUNC rsavpma_SetAttributeViInt64 (ViSession instrumentHandle,
                                               ViChar _VI_FAR repeatedCapabilityName[],
                                               ViAttr attributeID,
                                               ViInt64 attributeValue);
ViStatus _VI_FUNC rsavpma_SetAttributeViReal64 (ViSession instrumentHandle,
                                                ViChar _VI_FAR channelName[],
                                                ViAttr attributeID,
                                                ViReal64 attributeValue);
ViStatus _VI_FUNC rsavpma_SetAttributeViString (ViSession instrumentHandle,
                                                ViChar _VI_FAR channelName[],
                                                ViAttr attributeID,
                                                ViChar _VI_FAR attributeValue[]);
ViStatus _VI_FUNC rsavpma_SetAttributeViBoolean (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViBoolean attributeValue);
ViStatus _VI_FUNC rsavpma_SetAttributeViSession (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViSession attributeValue);
ViStatus _VI_FUNC rsavpma_GetAttributeViInt32 (ViSession instrumentHandle,
                                               ViChar _VI_FAR channelName[],
                                               ViAttr attributeID,
                                               ViInt32* attributeValue);
ViStatus _VI_FUNC rsavpma_GetAttributeViInt64 (ViSession instrumentHandle,
                                               ViChar _VI_FAR repeatedCapabilityName[],
                                               ViAttr attributeID,
                                               ViInt64* attributeValue);
ViStatus _VI_FUNC rsavpma_GetAttributeViReal64 (ViSession instrumentHandle,
                                                ViChar _VI_FAR channelName[],
                                                ViAttr attributeID,
                                                ViReal64* attributeValue);
ViStatus _VI_FUNC rsavpma_GetAttributeViString (ViSession instrumentHandle,
                                                ViChar _VI_FAR channelName[],
                                                ViAttr attributeID,
                                                ViInt32 bufferSize,
                                                ViChar _VI_FAR attributeValue[]);
ViStatus _VI_FUNC rsavpma_GetAttributeViBoolean (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViBoolean* attributeValue);
ViStatus _VI_FUNC rsavpma_GetAttributeViSession (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViSession* attributeValue);
ViStatus _VI_FUNC rsavpma_CheckAttributeViInt32 (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViInt32 attributeValue);
ViStatus _VI_FUNC rsavpma_CheckAttributeViInt64 (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViInt64 attributeValue);
ViStatus _VI_FUNC rsavpma_CheckAttributeViReal64 (ViSession instrumentHandle,
                                                  ViChar _VI_FAR channelName[],
                                                  ViAttr attributeID,
                                                  ViReal64 attributeValue);
ViStatus _VI_FUNC rsavpma_CheckAttributeViString (ViSession instrumentHandle,
                                                  ViChar _VI_FAR channelName[],
                                                  ViAttr attributeID,
                                                  ViChar _VI_FAR attributeValue[]);
ViStatus _VI_FUNC rsavpma_CheckAttributeViBoolean (ViSession instrumentHandle,
                                                   ViChar _VI_FAR channelName[],
                                                   ViAttr attributeID,
                                                   ViBoolean attributeValue);
ViStatus _VI_FUNC rsavpma_CheckAttributeViSession (ViSession instrumentHandle,
                                                   ViChar _VI_FAR channelName[],
                                                   ViAttr attributeID,
                                                   ViSession attributeValue);
ViStatus _VI_FUNC rsavpma_GetAttributeRepeatedCapabilityIds
             (ViSession instrumentHandle, ViUInt32 attributeID, ViInt32 bufferSize,
              ViChar _VI_FAR repeatedCapabilityId_s[]);
ViStatus _VI_FUNC rsavpma_GetAttributeRepeatedCapabilityIdNames
             (ViSession instrumentHandle, ViUInt32 attributeID,
              ViString repeatedCapabilityId, ViInt32 bufferSize,
              ViChar _VI_FAR repeatedCapabilityIdName_s[]);
ViStatus _VI_FUNC rsavpma_close (ViSession instrumentHandle);

/****************************************************************************
 *------------------------ Error And Completion Codes ----------------------*
 ****************************************************************************/

#define RSAVPMA_WARN_MEASURE_UNCALIBRATED    (RS_CLASS_WARN_BASE + 0x0001L)
#define RSAVPMA_WARN_OVER_RANGE              (RS_CLASS_WARN_BASE + 0x0002L)

#define RSAVPMA_ERROR_MARKER_NOT_ENABLED     (RS_CLASS_ERROR_BASE + 0x0003L)
#define RSAVPMA_ERROR_NOT_DELTA_MARKER       (RS_CLASS_ERROR_BASE + 0x0002L)
#define RSAVPMA_ERROR_MAX_TIME_EXCEEDED      (RS_CLASS_ERROR_BASE + 0x0001L)

#define RSAVPMA_WARNMSG_MEASURE_UNCALIBRATED "Uncalibrated measurement"
#define RSAVPMA_WARNMSG_OVER_RANGE           "Measurement overrange"

#define RSAVPMA_ERRMSG_MARKER_NOT_ENABLED    "Marker Not Enabled"
#define RSAVPMA_ERRMSG_NOT_DELTA_MARKER      "Not Delta Marker"
#define RSAVPMA_ERRMSG_MAX_TIME_EXCEEDED     "Max Time Exceeded"

#define RSAVPMA_ERROR_CODES_AND_MSGS \
    {RSAVPMA_WARN_MEASURE_UNCALIBRATED,  RSAVPMA_WARNMSG_MEASURE_UNCALIBRATED},\
    {RSAVPMA_WARN_OVER_RANGE,            RSAVPMA_WARNMSG_OVER_RANGE},\
    {RSAVPMA_ERROR_MARKER_NOT_ENABLED,   RSAVPMA_ERRMSG_MARKER_NOT_ENABLED},\
    {RSAVPMA_ERROR_NOT_DELTA_MARKER,     RSAVPMA_ERRMSG_NOT_DELTA_MARKER},\
    {RSAVPMA_ERROR_MAX_TIME_EXCEEDED,    RSAVPMA_ERRMSG_MAX_TIME_EXCEEDED}

/****************************************************************************
 *---------------------------- End Include File ----------------------------*
 ****************************************************************************/

#if defined(__cplusplus) || defined(__cplusplus__)
}
#endif
#endif /* __RSAVPMA_HEADER */



