/*****************************************************************************
 *  Rohde&Schwarz Audio Video Platform MHL Generator                           
 *  LabWindows/CVI Instrument Driver                                     
 *  Original Release: 2013-09-17
 *  By: Nabil Ouddane
 *      Martin Krcmar                                                                      
 *  Modification History:                                                    
 *                                                                           
 *       See ChangeLog
 *                                                                           
 *****************************************************************************/
                         
#include <string.h>
#include <stdio.h>  
#include <stdlib.h>
#include <math.h>
#include "rsavpmg.h"

ViStatus _VI_FUNC  rsavpmg_RsClose (ViSession vi);

/*****************************************************************************
 *- Value Definition and Range Tables ---------------------------------------*
 *****************************************************************************/
static ViString GenTestCaseArr[] = {"M000","M001","M002","M003","M004","M005","M006","M007","M008","M009","M010","M030","M031","M032","M033","M035","M036","M011","M012","M013","M014","M015","M016","M017","M018","M019","M020","M021","M022","M023","M024","M025","M026","M027",VI_NULL};
static ViString VideoFormatArr[] = {"AUTO","C1","C2","C17","C4","C19","C5","C20","C32","C34","C6","C21","C16","C31",VI_NULL};;
static ViString PixelEncodingArr[] = {"RGB","Y422","Y444",VI_NULL};
static ViString PatternArr[] = {"PATT","RAMP",VI_NULL};
static ViString AudioFormatArr[] = {"P32","P441","P48",VI_NULL};
static ViString NValueArr[] = {"NMIN","NMAX", VI_NULL};
static ViString OPcodeArr[] = {"POLL","CON","COFF",VI_NULL};
static ViString GenTDFormatArr[] = {"TBOT","FSEQ",VI_NULL};
/*****************************************************************************
 *- Callback Declarations ---------------------------------------------------*
 *****************************************************************************/

/*****************************************************************************
 *- User-Callable Functions (Exportable Functions) --------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: rsavpmg_init
 * Purpose:  VXIplug&play required function. Calls the
 *           rsavpmg_InitWithOptions function.
 *****************************************************************************/
ViStatus _VI_FUNC rsavpmg_init (ViRsrc resourceName, ViBoolean IDQuery,
                                 ViBoolean resetDevice, ViSession *newInstrSession)
{
    ViStatus    error = VI_SUCCESS;

    if (newInstrSession == VI_NULL)
        {
        (void) Rs_SetErrorInfo (VI_NULL, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                          VI_ERROR_PARAMETER4, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
        }

    checkErr( rsavpmg_InitWithOptions (resourceName, IDQuery, resetDevice,
                                      "QueryInstrStatus=1", newInstrSession));

Error:
    return error;
}

/*****************************************************************************
 * Function: rsavpmg_InitWithOptions
 * Purpose:  This function creates a new RS session and calls the
 *           RsInit function.
 *****************************************************************************/
ViStatus _VI_FUNC rsavpmg_InitWithOptions(
    ViRsrc          resourceName,
    ViBoolean       IDQuery,
    ViBoolean       resetDevice,
    ViString        optionString,
    ViSession      *newInstrSession
)
{
    ViStatus                error = VI_SUCCESS;
    ViChar                  newResourceName[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar                  newOptionString[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViBoolean               isLogicalName = 0;
    ViSession               instrSession = 0, rmSession = 0;
    RsAttrPropertiesPtr     *attrList=NULL;
    RsRepCapPtr             repCapTable = NULL;
    
    if (newInstrSession == VI_NULL)
        {
        (void) Rs_SetErrorInfo (VI_NULL, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                         VI_ERROR_PARAMETER5, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
        }

    /* Prepare resource manager session handle */
    checkErr (viOpenDefaultRM (&rmSession));

    /* Establish communication session with the instrument */

    viCheckErr( viOpen (rmSession, resourceName, VI_NULL, VI_NULL, &instrSession));
    
    /* Configure VISA Formatted I/O */
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, 5000 ));
    viCheckErr( viSetBuf (instrSession, (ViUInt16) (VI_READ_BUF | VI_WRITE_BUF), 4000));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_WR_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_RD_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));

    checkErr( Rs_GetInfoFromResourceName (resourceName,
                                          (ViString)optionString,
                                          newResourceName,
                                          newOptionString,
                                          &isLogicalName));
    
    ATTRLIST(attrList,rsavpmg);  
    checkErr( Rs_SpecificDriverNew (instrSession, "rsavpmg", newOptionString, attrList));
    
    /* Build repeated capability table */
    REPCAPTABLE(repCapTable,rsavpmg);
    checkErr( Rs_BuildRepCapTable (instrSession, repCapTable));

    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_VISA_RM_SESSION, 0, &rmSession));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_IO_SESSION, 0, &instrSession));

    /* With remote control via Ethernet, remote control is not automatically
       set by means of a command. The instrument must be explicitly set to
       the REMOTE state, e.g. by sending the intf command &GTR
    */
    /*- Reset instrument ----------------------------------------------------*/
    if (resetDevice == VI_TRUE)
    {
        checkErr( rsavpmg_reset (instrSession));
    }
    else  /*- Send Default Instrument Setup ---------------------------------*/
        checkErr( rsavpmg_DefaultInstrSetup (instrSession));

    /*- Identification Query ------------------------------------------------*/
    if (IDQuery == VI_TRUE)
        {
        ViChar rdBuffer[RSAVPMG_IO_BUFFER_SIZE] = "";

        checkErr( Rs_GetAttribute (instrSession, VI_NULL, (ViAttr) RSAVPMG_ATTR_ID_QUERY_RESPONSE,
                                   0, RSAVPMG_IO_BUFFER_SIZE, rdBuffer));

        if (strncmp (rdBuffer, RSAVPMG_VALID_ID_RESPONSE_STRING, strlen(RSAVPMG_VALID_ID_RESPONSE_STRING)) != 0)
            {
            viCheckErr( VI_ERROR_FAIL_ID_QUERY);
            }
        }

    if (isLogicalName == VI_FALSE)
        {
        ViInt32 oldFlag = 0;

        checkErr (Rs_GetAttributeFlags (instrSession, RS_ATTR_IO_RESOURCE_DESCRIPTOR, &oldFlag));
        checkErr (Rs_SetAttributeFlags (instrSession, RS_ATTR_IO_RESOURCE_DESCRIPTOR, oldFlag & 0xfffb | 0x0010));
        checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_IO_RESOURCE_DESCRIPTOR, 0, newResourceName));
        }
    
    if (isLogicalName == VI_TRUE)
        checkErr( Rs_ApplyDefaultSetup (instrSession));

    /* Instrument session accessible by the end user is native VISA session,
       or when simulated virtual instrument (instrSession) session. */
    
Error:

    if (error < VI_SUCCESS)
    {
        ViSession   rmSess = VI_NULL;
        
        (void) Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_VISA_RM_SESSION, 0, (ViInt32) sizeof (rmSess), &rmSess);
        (void) Rs_Dispose (instrSession); 
        if (instrSession != 0)
            (void) viClose (instrSession);
        (void) viClose (rmSess);
    }
    else
    {
        *newInstrSession = instrSession;
    }
    
    return error;
    
}

/// HIFN  This function sets the value of a ViInt32 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpmg_SetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function sets the value of a ViInt64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpmg_SetAttributeViInt64 (ViSession instrSession,
                                                 ViChar channelName[],
                                                 ViAttr attributeId,
                                                 ViInt64 value)
{
	return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function sets the value of a ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpmg_SetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 value)
{
     return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function sets the value of a ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpmg_SetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViString value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/// HIFN  This function sets the value of a ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpmg_SetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function sets the value of a ViSession attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpmg_SetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function queries the value of a ViInt32 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViInt32 variable.
ViStatus _VI_FUNC rsavpmg_GetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViInt32), (ViInt32 *)value);
}

/// HIFN  This function queries the value of a ViInt64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViInt64 variable.
ViStatus _VI_FUNC rsavpmg_GetAttributeViInt64 (ViSession instrSession,
                                                 ViChar channelName[],
                                                 ViAttr attributeId,
                                                 ViInt64 *value)
{
	return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViInt64), (ViInt64 *)value);
}

/// HIFN  This function queries the value of a ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViReal64 variable.
ViStatus _VI_FUNC rsavpmg_GetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViReal64), (ViReal64 *)value);
}

/// HIFN  This function queries the value of a ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR bufSize/Pass the number of bytes in the ViChar array you specify for the 
/// HIPAR bufSize/Attribute Value parameter.  
/// HIPAR value/The buffer in which the function returns the current value of the 
/// HIPAR value/attribute.  The buffer must be of type ViChar and have at least as many 
/// HIPAR value/bytes as indicated in the Buffer Size parameter.
ViStatus _VI_FUNC rsavpmg_GetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViInt32 bufSize,
                                                 ViChar value[])
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            bufSize, value);
}

/// HIFN  This function queries the value of a ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViBoolean variable.
ViStatus _VI_FUNC rsavpmg_GetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViBoolean), (ViBoolean *)value);
}

/// HIFN  This function queries the value of a ViSession attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViSession variable.
ViStatus _VI_FUNC rsavpmg_GetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViSession), (ViSession *)value);
}

/// HIFN  This function checks the validity of a value you specify for a ViInt32 
/// HIFN  attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpmg_CheckAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViInt32 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function checks the validity of a value you specify for a ViInt64 
/// HIFN  attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpmg_CheckAttributeViInt64 (ViSession instrSession,
                                                   ViChar channelName[],
                                                   ViAttr attributeId,
                                                   ViInt64 value)
{
	return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpmg_CheckAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViReal64 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpmg_CheckAttributeViString (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViString value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpmg_CheckAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViBoolean value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViSession attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpmg_CheckAttributeViSession (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViSession value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN This function returns the repeated capability id(s) that belongs
/// HIFN to the attribute you specify. More then one id is listed in
/// HIFN comma separated string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Repeated Capability Id(s) parameter.
/// HIPAR repeatedCapabilityIds/The buffer in which the function returns the repeated capability id(s) belonging to the attribute.
ViStatus _VI_FUNC rsavpmg_GetAttributeRepeatedCapabilityIds (ViSession instrSession,
                                                            ViUInt32 attributeID,
                                                            ViInt32 bufferSize,
                                                            ViChar _VI_FAR repeatedCapabilityIds[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      repCapNameId[RS_MAX_MESSAGE_BUF_SIZE] = "";

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 3, "Buffer Size");

    viCheckErr (Rs_GetAttributeRepCapNameId (instrSession, attributeID, repCapNameId));

    strncpy (repeatedCapabilityIds, repCapNameId, RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIds[(bufferSize > RS_MAX_MESSAGE_BUF_SIZE) ?
        RS_MAX_MESSAGE_BUF_SIZE : bufferSize] = '\0';

Error:
    return error;
}

/// HIFN This function returns the repeated capability name(s) that
/// HIFN belongs to the attribute and single repeated capability id you
/// HIFN may specify. More then one name is listed in comma separated
/// HIFN string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR repeatedCapabilityId/If the attribute applies to a repeated capability such as a trace, this parameter defines the Id of the repeated capability on which to query repeated capability id name(s).
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Repeated Capability Id Name(s) parameter.
/// HIPAR repeatedCapabilityIdNames/The buffer in which the function returns the repeated capability id name(s) belonging to the attribute.
ViStatus _VI_FUNC rsavpmg_GetAttributeRepeatedCapabilityIdNames (ViSession instrSession,
                                                                ViUInt32 attributeID,
                                                                ViString repeatedCapabilityId,
                                                                ViInt32 bufferSize,
                                                                ViChar _VI_FAR repeatedCapabilityIdNames[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      repCapName[RS_MAX_MESSAGE_BUF_SIZE] = "";

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 4, "Buffer Size");

    viCheckErr (Rs_GetAttributeRepCapName (instrSession,
                                           attributeID,
                                           repeatedCapabilityId,
                                           repCapName));

    strncpy (repeatedCapabilityIdNames, repCapName, RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIdNames[(bufferSize >= RS_MAX_MESSAGE_BUF_SIZE) ?
        RS_MAX_MESSAGE_BUF_SIZE - 1 : bufferSize - 1] = '\0';

Error:
    return error;
}

/// HIFN  This function configures the MHL generator output.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR output/Selects the output signal for the measurement.
/// HIPAR HDCP/Sets the HDCP.
ViStatus _VI_FUNC rsavpmg_ConfigureMHLGeneratorOutput (ViSession instrSession,
                                                     ViInt32 board,
                                                     ViInt32 output,
                                                     ViBoolean HDCP)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpmg_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpmg_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPMG_ATTR_MHL_GENERATOR_OUTPUT, output), 3, "Output");
	
	viCheckParm(rsavpmg_SetAttributeViBoolean(instrSession, rep_cap,
        RSAVPMG_ATTR_MHL_GENERATOR_OUTPUT_HDCP, HDCP), 4, "HDCP");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function returns the MHL generator output parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR CBUS/Returns the status of CBUS connection.
/// HIPAR VBUS/Returns the VBUS load in mA.
ViStatus _VI_FUNC rsavpmg_QueryMHLGeneratorOutput (ViSession instrSession,
                                                 ViInt32 board, ViInt32 *CBUS,
                                                 ViReal64 *VBUS)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpmg_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpmg_GetAttributeViInt32(instrSession, rep_cap,
        RSAVPMG_ATTR_MHL_GENERATOR_CBUS_CONNECTION, CBUS), 3, "CBUS");
	
	viCheckParm(rsavpmg_GetAttributeViReal64(instrSession, rep_cap,
        RSAVPMG_ATTR_MHL_GENERATOR_VBUS_POWER_CABLE, VBUS), 4, "VBUS");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the MHL generator signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR signal/Selects the signal.
ViStatus _VI_FUNC rsavpmg_ConfigureMHLGeneratorSignal (ViSession instrSession,
                                                     ViInt32 board,
                                                     ViInt32 signal)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpmg_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpmg_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPMG_ATTR_MHL_GENERATOR_SIGNAL_PATTERN, signal), 3, "Signal");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function selects the operation to be performed on the MHL generator signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR operation/Selects the operation.
ViStatus _VI_FUNC rsavpmg_MHLGeneratorSignalOperation (ViSession instrSession,
                                                     ViInt32 board,
                                                     ViInt32 operation)
{
	ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsavpmg_invalidViInt32Range (operation, RSAVPMG_VAL_MHL_STOP, RSAVPMG_VAL_MHL_START), 3, "Operation");
	
    checkErr (rsavpmg_hardwareRepCap (instrSession, board, rep_cap));
	
	switch (operation)
    {
        case RSAVPMG_VAL_MHL_START:
	
		viCheckErr(rsavpmg_SetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPMG_ATTR_MHL_GENERATOR_SIGNAL_INIT, NULL));
												 
		
		break;
		
		case RSAVPMG_VAL_MHL_STOP:
	
		viCheckErr(rsavpmg_SetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPMG_ATTR_MHL_GENERATOR_STOP, NULL));
												 
		
		
		break;
	
	 default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER,3,"Operation");
            break;
			
    }			
	
	checkErr (rsavpmg_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the HDMI analyzer input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR state/returns the MHL generator signal state.
ViStatus _VI_FUNC rsavpmg_QueryMHLGeneratorSignalState (ViSession instrSession,
                                                      ViInt32 board,
                                                      ViInt32 *state)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpmg_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpmg_GetAttributeViInt32(instrSession, rep_cap,
        RSAVPMG_ATTR_MHL_GENERATOR_SIGNAL_STATUS, state), 3, "State");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function returns the MHL generator EDID (Extended Display Information Data).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR EDID/Returns the MHL generator EDID (Extended Display Information Data).
ViStatus _VI_FUNC rsavpmg_QueryMHLGeneratorEDID (ViSession instrSession,
                                               ViInt32 board, ViInt32 *EDID)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpmg_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpmg_GetAttributeViInt32(instrSession, rep_cap,
        RSAVPMG_ATTR_QUERY_MHL_GENERATOR_EXTENDED_DISPLAY_INFORMATION_DATA, EDID), 3, "EDID");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function returns the MHL generator CTS state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR state/Returns the MHL generator CTS state.
ViStatus _VI_FUNC rsavpmg_QueryMHLGeneratorCTSState (ViSession instrSession,
                                                   ViInt32 board, ViInt32 *state)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpmg_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpmg_GetAttributeViInt32(instrSession, rep_cap,
        RSAVPMG_ATTR_MHL_GENERATOR_CTS_STATUS, state), 3, "State");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function selects the operation to be performed on the MHL generator CTS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR operation/Selects the operation.
ViStatus _VI_FUNC rsavpmg_MHLGeneratorCTSOperation (ViSession instrSession,
                                                  ViInt32 board,
                                                  ViInt32 operation)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsavpmg_invalidViInt32Range (operation, RSAVPMG_VAL_MHL_STOP, RSAVPMG_VAL_MHL_START), 3, "Operation");
	
    checkErr (rsavpmg_hardwareRepCap (instrSession, board, rep_cap));
	
	switch (operation)
    {
        case RSAVPMG_VAL_MHL_START:
	
		viCheckErr(rsavpmg_SetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPMG_ATTR_MHL_GENERATOR_CTS_INIT, NULL));
												 
		
		break;
		
		case RSAVPMG_VAL_MHL_STOP:
	
		viCheckErr(rsavpmg_SetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPMG_ATTR_MHL_GENERATOR_STOP, NULL));
												 
		
		
		break;
	
	 default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER,3,"Operation");
            break;
			
    }			
	
	checkErr (rsavpmg_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the MHL generator CTS measurement progress.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR progress/Returns the MHL generator CTS measurement progress.
ViStatus _VI_FUNC rsavpmg_QueryMHLGeneratorCTSProgress (ViSession instrSession,
                                                      ViInt32 board,
                                                      ViReal64 *progress)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpmg_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpmg_GetAttributeViReal64(instrSession, rep_cap,
        RSAVPMG_ATTR_QUERY_MHL_GENERATOR_CTS_PROGRESS, progress), 3, "Progress");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error; 
}

/// HIFN  This function selects the MHL generator CTS test case.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR testCase/Selects the test case.
ViStatus _VI_FUNC rsavpmg_SelectMHLGeneratorCTSTestCase (ViSession instrSession,
                                                       ViInt32 board,
                                                       ViInt32 testCase)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpmg_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpmg_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPMG_ATTR_MHL_GENERATOR_CTS_TEST_CASE, testCase), 3, "Test Case");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function configures the MHL generator CTS Character Synchronization and Packet Types.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR testCase/Selects the test case.
/// HIPAR videoFormat/Selects the video format.
ViStatus _VI_FUNC rsavpmg_ConfigureMHLGeneratorCTSTMDSCoding (ViSession instrSession,
                                                            ViInt32 board,
                                                            ViInt32 testCase,
                                                            ViInt32 videoFormat)
{
     ViStatus    error = VI_SUCCESS;
    
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    viCheckParm (rsavpmg_invalidViInt32Range (board, 1, 8), 2, "Board");
    viCheckParm (rsavpmg_invalidViInt32Range (testCase, RSAVPMG_VAL_MHL_SINK_TEST_CSYNC, RSAVPMG_VAL_MHL_DONGLE_TEST_PACKET), 3, "Test Case");
	viCheckParm (rsavpmg_invalidViInt32Range (videoFormat, RSAVPMG_VAL_MHL_VIDEO_FORMAT_C2, RSAVPMG_VAL_MHL_VIDEO_FORMAT_C31), 4, "Video Format");
	
	switch (testCase)
		
	   {	
		  case RSAVPMG_VAL_MHL_SINK_TEST_CSYNC:
          case RSAVPMG_VAL_MHL_SINK_TEST_PACKET:			
		  case RSAVPMG_VAL_MHL_DONGLE_TEST_CSYNC:
		  case RSAVPMG_VAL_MHL_DONGLE_TEST_PACKET:
		  
			viCheckErr (viPrintf (instrSession, "CONF%ld:AVG:CTS:VFOR %s, %s", board, GenTestCaseArr[testCase], VideoFormatArr[videoFormat]));
			checkErr (rsavpmg_CheckStatus (instrSession));
		  break;
		  
		  default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER,3,"Test Case");
            break;
	   }
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the MHL generator CTS video modes Video Formats, Pixel Encoding and Video Quantization Ranges.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR testCase/Selects the test case.
/// HIPAR videoFormat/Selects the video format.
/// HIPAR pattern/Selects the pattern.
/// HIPAR pixelEncoding/Selects the pixel encoding.
ViStatus _VI_FUNC rsavpmg_ConfigureMHLGeneratorCTSVideoModes (ViSession instrSession,
                                                            ViInt32 board,
                                                            ViInt32 testCase,
                                                            ViInt32 videoFormat,
                                                            ViInt32 pattern,
                                                            ViInt32 pixelEncoding)
{
    ViStatus    error = VI_SUCCESS;
    
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    viCheckParm (rsavpmg_invalidViInt32Range (board, 1, 8), 2, "Board");
    viCheckParm (rsavpmg_invalidViInt32Range (testCase, RSAVPMG_VAL_MHL_SINK_TEST_VFORMAT, RSAVPMG_VAL_MHL_DONGLE_TEST_VQRANGE), 3, "Test Case");
    viCheckParm (rsavpmg_invalidViInt32Range (videoFormat, RSAVPMG_VAL_MHL_VIDEO_FORMAT_C1, RSAVPMG_VAL_MHL_VIDEO_FORMAT_C31), 3, "Video Format");
	viCheckParm (rsavpmg_invalidViInt32Range (pattern, RSAVPMG_VAL_MHL_PATTERN_TEST, RSAVPMG_VAL_MHL_PATTERN_RAMP), 5, "Pattern");
	viCheckParm (rsavpmg_invalidViInt32Range (pixelEncoding, RSAVPMG_VAL_MHL_PIXEL_ENCODING_RGB, RSAVPMG_VAL_MHL_PIXEL_ENCODING_Y444), 6, "Pixel Encoding");
	
	switch (testCase)
		
	   {											  
		  case RSAVPMG_VAL_MHL_SINK_TEST_VFORMAT:
		  case RSAVPMG_VAL_MHL_DONGLE_TEST_VFORMAT:	  
			  
			viCheckErr (viPrintf (instrSession, "CONF%ld:AVG:CTS:VFOR %s, %s", board, GenTestCaseArr[testCase], VideoFormatArr[videoFormat]));
			checkErr (rsavpmg_CheckStatus (instrSession));
			viCheckErr (viPrintf (instrSession, "CONF%ld:AVG:CTS:VFOR:PATT %s, %s", board, GenTestCaseArr[testCase], PatternArr[pattern]));
			checkErr (rsavpmg_CheckStatus (instrSession));
			
		  break;
		  
		  case RSAVPMG_VAL_MHL_SINK_TEST_PIXEL:
          case RSAVPMG_VAL_MHL_DONGLE_TEST_PIXEL:
          case RSAVPMG_VAL_MHL_SINK_TEST_VQRANGE:
		  case RSAVPMG_VAL_MHL_DONGLE_TEST_VQRANGE:			  
			  
			viCheckErr (viPrintf (instrSession, "CONF%ld:AVG:CTS:VFOR %s, %s", board, GenTestCaseArr[testCase], VideoFormatArr[videoFormat]));
            checkErr (rsavpmg_CheckStatus (instrSession));			  
			viCheckErr (viPrintf (instrSession, "CONF%ld:AVG:CTS:PIX %s, %s", board, GenTestCaseArr[testCase], PixelEncodingArr[pixelEncoding]));
            checkErr (rsavpmg_CheckStatus (instrSession));		
			
		  break;
		 
		  default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER,3,"Test Case");
            break;
	   }
	
	checkErr (rsavpmg_CheckStatus (instrSession));
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the MHL generator CTS audio tests IEC60958/IEC61937, Audio Clock Regeneration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR testCase/Selects the test case.
/// HIPAR videoFormat/Selects the video format.
/// HIPAR audioFormat/Selects the Audio Format.
/// HIPAR nValue/Selects the N Value.
ViStatus _VI_FUNC rsavpmg_ConfigureMHLGeneratorCTSAudioTests (ViSession instrSession,
                                                            ViInt32 board,
                                                            ViInt32 testCase,
                                                            ViInt32 videoFormat,
                                                            ViInt32 audioFormat,
                                                            ViInt32 nValue)
{
    ViStatus    error = VI_SUCCESS;
    
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    viCheckParm (rsavpmg_invalidViInt32Range (board, 1, 8), 2, "Board");
    viCheckParm (rsavpmg_invalidViInt32Range (testCase, RSAVPMG_VAL_MHL_SINK_TEST_IEC6, RSAVPMG_VAL_MHL_DONGLE_TEST_ACLOCK), 3, "Test Case");
    viCheckParm (rsavpmg_invalidViInt32Range (videoFormat, RSAVPMG_VAL_MHL_VIDEO_FORMAT_C2, RSAVPMG_VAL_MHL_VIDEO_FORMAT_C17), 3, "Video Format");
	viCheckParm (rsavpmg_invalidViInt32Range (audioFormat, RSAVPMG_VAL_MHL_AUDIO_FORMAT_P32, RSAVPMG_VAL_MHL_AUDIO_FORMAT_P48), 5, "Audio Format");
	viCheckParm (rsavpmg_invalidViInt32Range (nValue, RSAVPMG_VAL_MHL_N_VALUE_MIN, RSAVPMG_VAL_MHL_N_VALUE_MAX), 6, "N Value");
	
	switch (testCase)
		
	   {	
		  case RSAVPMG_VAL_MHL_SINK_TEST_IEC6:
		  case RSAVPMG_VAL_MHL_DONGLE_TEST_IEC6:	  
			  
			viCheckErr (viPrintf (instrSession, "CONF%ld:AVG:CTS:VFOR %s, %s", board, GenTestCaseArr[testCase], VideoFormatArr[videoFormat]));
			checkErr (rsavpmg_CheckStatus (instrSession));			
			viCheckErr (viPrintf (instrSession, "CONF%ld:AVG:CTS:AFOR %s, %s", board, GenTestCaseArr[testCase], AudioFormatArr[audioFormat]));
            checkErr (rsavpmg_CheckStatus (instrSession));						
			
		  break;
		  
		  case RSAVPMG_VAL_MHL_SINK_TEST_ACLOCK:
          case RSAVPMG_VAL_MHL_DONGLE_TEST_ACLOCK:
			  
			viCheckErr (viPrintf (instrSession, "CONF%ld:AVG:CTS:VFOR %s, %s", board, GenTestCaseArr[testCase], VideoFormatArr[videoFormat]));
			checkErr (rsavpmg_CheckStatus (instrSession));			  
			viCheckErr (viPrintf (instrSession, "CONF%ld:AVG:CTS:NVAL %s, %s", board, GenTestCaseArr[testCase], NValueArr[nValue]));
            checkErr (rsavpmg_CheckStatus (instrSession));						
			viCheckErr (viPrintf (instrSession, "CONF%ld:AVG:CTS:AFOR %s, %s", board, GenTestCaseArr[testCase], AudioFormatArr[audioFormat]));
            checkErr (rsavpmg_CheckStatus (instrSession));
			
		  break;
		  
		  default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER,3,"Test Case");
            break;
	   }
	
	checkErr (rsavpmg_CheckStatus (instrSession));
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the MHL generator CTS RCP Sub-Command RX or TX Test.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR testCase/Selects the test case.
/// HIPAR keyID/Sets the Key ID.
ViStatus _VI_FUNC rsavpmg_ConfigureMHLGeneratorCTSRCPSubCommandTest (ViSession instrSession,
                                                                   ViInt32 board,
                                                                   ViInt32 testCase,
                                                                   ViInt32 keyID)
{
 	ViStatus    error = VI_SUCCESS;
    
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    viCheckParm (rsavpmg_invalidViInt32Range (board, 1, 8), 2, "Board");
    viCheckParm (rsavpmg_invalidViInt32Range (testCase, RSAVPMG_VAL_MHL_SINK_TEST_RCPRX, RSAVPMG_VAL_MHL_DONGLE_TEST_RCPTX), 3, "Test Case");
	viCheckParm (rsavpmg_invalidViInt32Range (keyID, 0, 127), 4, "Key ID");
    
	viCheckErr (viPrintf (instrSession, "CONF%ld:AVG:CTS:KEY %s, %ld", board, GenTestCaseArr[testCase], keyID));
	
	checkErr (rsavpmg_CheckStatus (instrSession));
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the MHL analyzer CTS RAP and RAPK Sub-Command Tests.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR testCase/Selects the test case.
/// HIPAR OPcode/Selects the OP code.
ViStatus _VI_FUNC rsavpmg_ConfigureMHLGeneratorCTSRAPSubCommandTests (ViSession instrSession,
                                                                    ViInt32 board,
                                                                    ViInt32 testCase,
                                                                    ViInt32 OPcode)
{
    ViStatus    error = VI_SUCCESS;
    
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    viCheckParm (rsavpmg_invalidViInt32Range (board, 1, 8), 2, "Board");
    viCheckParm (rsavpmg_invalidViInt32Range (testCase, RSAVPMG_VAL_MHL_SINK_TEST_RCPRX, RSAVPMG_VAL_MHL_DONGLE_TEST_RCPTX), 3, "Test Case");
	viCheckParm (rsavpmg_invalidViInt32Range (OPcode, RSAVPMG_VAL_MHL_OPCODE_POLL, RSAVPMG_VAL_MHL_OPCODE_CONTENT_OFF), 4, "OP Code");
    
	viCheckErr (viPrintf (instrSession, "CONF%ld:AVG:CTS:RAP %s,%s\n", board, GenTestCaseArr[testCase], OPcodeArr[OPcode]));
	
	checkErr (rsavpmg_CheckStatus (instrSession));
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the MHL analyzer CTS RAP and RAPK Sub-Command Tests.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR testCase/Selects the test case.
/// HIPAR videoFormat/Selects the video format.
/// HIPAR _3DFormat/Sets the 3D format.
ViStatus _VI_FUNC rsavpmg_ConfigureMHLGeneratorCTS3DVideoTest (ViSession instrSession,
                                                             ViInt32 board,
                                                             ViInt32 testCase,
                                                             ViInt32 videoFormat,
                                                             ViInt32 _3DFormat)
{
    ViStatus    error = VI_SUCCESS;
    
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    viCheckParm (rsavpmg_invalidViInt32Range (board, 1, 8), 2, "Board");
    viCheckParm (rsavpmg_invalidViInt32Range (testCase, RSAVPMG_VAL_MHL_SINK_TEST_3DVFT, RSAVPMG_VAL_MHL_DONGLE_TEST_3DVFT), 3, "Test Case");
	viCheckParm (rsavpmg_invalidViInt32Range (videoFormat, RSAVPMG_VAL_MHL_VIDEO_FORMAT_C4, RSAVPMG_VAL_MHL_VIDEO_FORMAT_C32), 4, "Video Format");
	viCheckParm (rsavpmg_invalidViInt32Range (_3DFormat, RSAVPMG_VAL_MHL_3D_FORMAT_TOP_BOTTOM, RSAVPMG_VAL_MHL_3D_FORMAT_FRAME_SEQ), 5, "3D Format");
    
	viCheckErr (viPrintf (instrSession, "CONF%ld:AVG:CTS:VFOR %s,%s\n", board, GenTestCaseArr[testCase], VideoFormatArr[videoFormat]));
	checkErr (rsavpmg_CheckStatus (instrSession));
	viCheckErr (viPrintf (instrSession, "CONF%ld:AVG:CTS:VFOR:HD:EXT %s,%s", board, GenTestCaseArr[testCase], GenTDFormatArr[_3DFormat]));
	checkErr (rsavpmg_CheckStatus (instrSession));
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function selects the MHL generator CTS UCP Sub-Command Test - Receiving and Transmitting.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR testCase/Selects the test case.
/// HIPAR characterSequence/Sets the character sequence.
ViStatus _VI_FUNC rsavpmg_ConfigureMHLGeneratorCTSUCPSubCommandTests (ViSession instrSession,
                                                                    ViInt32 board,
                                                                    ViInt32 testCase,
                                                                    ViString characterSequence)
{
	ViStatus    error = VI_SUCCESS;
    
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    viCheckParm (rsavpmg_invalidViInt32Range (board, 1, 8), 2, "Board");
    viCheckParm (rsavpmg_invalidViInt32Range (testCase, RSAVPMG_VAL_MHL_SINK_TEST_UCPRX, RSAVPMG_VAL_MHL_DONGLE_TEST_UCPTX), 3, "Test Case");
	
	viCheckErr (viPrintf (instrSession, "CONF%ld:AVG:CTS:UCP %s, '%s'", board, GenTestCaseArr[testCase], characterSequence));
	
	checkErr (rsavpmg_CheckStatus (instrSession));
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function returns the MHL generator CTS test output report count.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR count/Returns the MHL generator CTS test output report count.
ViStatus _VI_FUNC rsavpmg_QueryMHLGeneratorCTSTestOutputReportCount (ViSession instrSession,
                                                                   ViInt32 board,
                                                                   ViInt32 *count)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpmg_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpmg_GetAttributeViInt32(instrSession, rep_cap,
        RSAVPMG_ATTR_MHL_GENERATOR_CTS_TEST_OUTPUT_REPORT_COUNT, count), 3, "Count");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function returns the MHL generator CTS test output report data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR index/Sets the index.
/// HIPAR arraySize/Sets the array size. 
/// HIPAR lineNumber/Returns line number referring to TestOutput scroll view. First line has #0
/// HIPAR referenceNumber/Returns reference number of text message.
/// HIPAR textMessage/Returns text message.
/// HIPAR optionalParameters/Returns the optional parameters.
ViStatus _VI_FUNC rsavpmg_QueryMHLGeneratorCTSTestOutputReport (ViSession instrSession,
                                                              ViInt32 board,
                                                              ViInt32 index,
															  ViInt32 arraySize,
                                                              ViInt32 *lineNumber,
                                                              ViInt32 *referenceNumber,
                                                              ViChar textMessage[],
                                                              ViInt32 optionalParameters[])
                                                              
{
    ViStatus error = VI_SUCCESS;
	ViChar   command[RSAVPMG_IO_BUFFER_SIZE] = "";
    ViUInt32 count = 0;
	ViChar   *p2buf  = NULL;
	ViChar   *p2data  = NULL;
	ViInt32 i = 0;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	viCheckParm (rsavpmg_invalidViInt32Range (board, 1, 8), 2, "Board");
	viCheckParm (rsavpmg_invalidViInt32Range (index, 0, 6), 3, "Index");
	
    sprintf (command, "READ%ld:AVGen:CTS:REP? %ld\n", board, index);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    
	checkErr (rsavpmg_CheckStatus (instrSession));
    
    p2buf = strtok (p2data, ",");
    if (lineNumber != NULL) 
	*lineNumber = atoi (p2buf);

    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        return VI_ERROR_INV_RESPONSE;
    if (referenceNumber != NULL) 
		sscanf (p2buf, "%x", &referenceNumber);
	
    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        return VI_ERROR_INV_RESPONSE;
	if (strspn (p2buf, "\"") != 0)
	{
		sscanf (p2buf, "\"%[^\"]", textMessage);
		p2buf = strtok (NULL, ",");
	}
	if (p2buf != NULL)
	{
		for (i=0;i<arraySize;i++)
    	{
		   	optionalParameters[i] = rsavpmg_atol (p2buf);
	   		p2buf = strtok (NULL, ",");
			if (p2buf == NULL) break;
    	}
	}

Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function resets the instrument to a known state and sends initialization commands to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
ViStatus _VI_FUNC rsavpmg_reset (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (viWrite (instrSession, (ViBuf) "*RST\n", 5, NULL));
	
	checkErr (rsavpmg_DefaultInstrSetup(instrSession));

Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;
}

/// HIFN This function resets the instrument to a known state and sends initialization commands to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR reference/Selects the reference of preset.
ViStatus _VI_FUNC rsavpmg_ResetSystem (ViSession instrSession, ViInt32 reference)
{
    ViStatus	error = VI_SUCCESS;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	switch (reference)
	{
		case RSAVPMG_VAL_RST_BASE:
			viCheckErr (rsavpmg_SetAttributeViString(instrSession, "",
    	    	RSAVPMG_ATTR_SYSTEM_RESET_BASE, NULL));
			break;
		case RSAVPMG_VAL_RST_ALL:
			viCheckErr (rsavpmg_SetAttributeViString(instrSession, "",
    	    	RSAVPMG_ATTR_SYSTEM_RESET_ALL, NULL));
			break;
		default:
			viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Reference");
			break;
	}
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error; 
}

/// HIFN The preset sets the parameters to values suitable for local/manual interaction, a reset to values suitable for remote operation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR reference/Selects the reference of preset.
ViStatus _VI_FUNC rsavpmg_Preset (ViSession instrSession, ViInt32 reference)
{
    ViStatus	error = VI_SUCCESS;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	switch (reference)
	{
		case RSAVPMG_VAL_RST_BASE:
			viCheckErr (rsavpmg_SetAttributeViString(instrSession, "",
    	    	RSAVPMG_ATTR_SYSTEM_PRESET_BASE, NULL));
			break;
		case RSAVPMG_VAL_RST_ALL:
			viCheckErr (rsavpmg_SetAttributeViString(instrSession, "",
    	    	RSAVPMG_ATTR_SYSTEM_PRESET_ALL, NULL));
			break;
		default:
			viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Reference");
			break;
	}
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error; 
}

/// HIFN This function runs the instrument's self test routine and returns the test result(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR testResult/Contains the value returned from the instrument self test.
/// HIPAR testMessage/Returns the self-test response string from the instrument.
ViStatus _VI_FUNC rsavpmg_self_test (ViSession instrSession, ViInt16 *testResult, 
                                      ViChar testMessage[])
{
    ViStatus    error   = VI_SUCCESS;
    ViInt32     result = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (testResult == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Test Result");
    if (testMessage == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Test Message");

    viCheckErr (rsavpmg_GetAttributeViInt32 (instrSession, NULL, RSAVPMG_ATTR_SELF_TEST_QUERY, &result));
    if (*testResult == 0)
        strcpy (testMessage, "Self-Test Passed.");
    else
        strcpy (testMessage, "Self-Test Failed.");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function returns the driver and instrument revisions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR driverRev/Returns the instrument driver software revision numbers in the form of a string.
/// HIPAR instrRev/Returns the instrument firmware revision numbers in the form of a string.
ViStatus _VI_FUNC rsavpmg_revision_query (ViSession instrSession, ViChar driverRev[], 
                                           ViChar instrRev[])
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (driverRev == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Driver Revision");
    if (instrRev == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Instrument Revision");

    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RSAVPMG_ATTR_SPECIFIC_DRIVER_REVISION,
                                        0, 256, driverRev));
    checkErr( Rs_GetAttribute (instrSession, "", RSAVPMG_ATTR_INSTRUMENT_FIRMWARE_REVISION,
                                        0, 256, instrRev));
    checkErr( rsavpmg_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function reads an error code and a message from the
/// HIFN instrument's error queue.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR errCode/Returns the error code read from the instrument's error queue.
/// HIPAR errMessage/Returns the error message string read from the instrument's error message queue.
ViStatus _VI_FUNC rsavpmg_error_query (ViSession instrSession, ViInt32 *errCode, 
                                        ViChar errMessage[])
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSAVPMG_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (errCode == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error Code");
    if (errMessage == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
        {
        viCheckErr (rsavpmg_GetAttributeViString( instrSession, "", RSAVPMG_ATTR_SYSTEM_ERROR, RSAVPMG_IO_BUFFER_SIZE, buffer));
        if  (sscanf (buffer, "%ld,\"%256[^\"]", errCode, errMessage) != 2)
            viCheckErr(RS_ERROR_UNEXPECTED_RESPONSE);

        }
    else
        {
            /* Simulate Error Query */
        *errCode = 0;
        strcpy (errMessage, "No error.");
        }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function converts a status code returned by an instrument driver function into a user-readable string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR errorCode/Pass the Status parameter that is returned from any of the instrument driver functions.
/// HIPAR errorMessage/Returns the user-readable message string that corresponds to the status code you specify.
ViStatus _VI_FUNC rsavpmg_error_message (ViSession instrSession, ViStatus errorCode,
                                          ViChar errorMessage[256])
{
    ViStatus    error   = VI_SUCCESS;

    static      RsStringValueTable errorTable =
        {
            RSAVPMG_ERROR_CODES_AND_MSGS,
            {VI_NULL, VI_NULL}
        };

    (void) Rs_LockSession(instrSession, VI_NULL);

        /* all VISA and RS error codes are handled as well as codes in the table */
    if (errorMessage == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    checkErr( Rs_GetSpecificDriverStatusDesc(instrSession, errorCode, errorMessage, errorTable));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function queries the number of the SCPI version, which is 
/// HIFN  relevant for the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR length/This control specifies the size of the buffer.
/// HIPAR systemVersion/Returns the number of the SCPI version, which is relevant for the 
/// HIPAR systemVersion/instrument.
ViStatus _VI_FUNC rsavpmg_SystemVersion(
	ViSession	instrSession,
	ViInt32		length,
	ViChar	  	systemVersion[]
)
{
    ViStatus error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsavpmg_GetAttributeViString (instrSession, "",
                                                RSAVPMG_ATTR_SYST_VERSION,
												length,
												systemVersion
                                                ), 2, "System Version");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function switches ON/OFF state checking of the instrument 
/// HIFN  (reading of the Standard Event Register and checking it for error). All 
/// HIFN  driver functions are using state checking. Switch this function to ON 
/// HIFN  when debug your application. For better bus throughput and instrument 
/// HIFN  performance switch it OFF. As default is state checking switched to ON!
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/This control selects ON or OFF instrument state checking.
ViStatus _VI_FUNC rsavpmg_setCheckStatus(
	ViSession	instrSession,
	ViBoolean	state
)
{
    ViStatus error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsavpmg_SetAttributeViBoolean (instrSession, "",
                                                RS_ATTR_QUERY_INSTRUMENT_STATUS,
												state
                                                ), 2, "State");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function retrieves and then clears the error information
/// HIFN for the session or the current execution thread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR errorCode/Returns the error code for the session or execution thread.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Description parameter.
/// HIPAR description/Returns the error description for the session or execution thread. If there is no description, the function returns an empty string.
ViStatus _VI_FUNC rsavpmg_GetError (ViSession instrSession, 
                                     ViStatus *errorCode, 
                                     ViInt32 bufferSize,
                                     ViChar description[])  
{
    ViStatus    error       = VI_SUCCESS;
    ViStatus    primary     = VI_SUCCESS,
                secondary   = VI_SUCCESS;
    ViChar      elaboration[RS_MAX_MESSAGE_BUF_SIZE]    = "";
    ViChar      errorMessage[RS_MAX_MESSAGE_BUF_SIZE]  = "";
    ViChar     *appendPoint = errorMessage;
    ViInt32     actualSize  = 0;
    ViBoolean   locked      = VI_FALSE;
    RsSessionPropertiesPtr  sessionProperties = Rs_ViSession (instrSession);

    /* Lock Session */
    checkErr( Rs_LockSession(instrSession, &locked));

    /* Test for nulls and acquire error data */
    if (bufferSize != 0)
    {
        if (errorCode == VI_NULL)
        {
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error");
        }
        if (sessionProperties == NULL)
        {
            primary = VI_ERROR_RSRC_NFOUND;
            secondary = 0;
        }
        else
        {
            checkErr( Rs_GetErrorInfo (instrSession, &primary, &secondary, elaboration));
        }
    }

    else
        {
            if (sessionProperties == NULL)
            {
                (void) Rs_UnlockSession (instrSession, &locked);
                return 1024;
            }
            else
            {
                checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_ERROR_ELABORATION, 0, 256, elaboration));
                checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_SECONDARY_ERROR, 0, (ViInt32) sizeof (secondary), &secondary));
                checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_PRIMARY_ERROR, 0, (ViInt32) sizeof (primary), &primary));
            }
        }

    /* Format data */
    if (primary != VI_SUCCESS)
        {
        ViChar msg[256] = "";
        checkErr( rsavpmg_error_message (instrSession, primary, msg));
        appendPoint += sprintf(appendPoint, "Primary Error: (Hex 0x%08X) %s\n", (unsigned int) primary, msg);
        }

    if (secondary != VI_SUCCESS)
        {
        ViChar msg[256] = "";
        checkErr( rsavpmg_error_message (instrSession, secondary, msg));
        appendPoint += sprintf(appendPoint, "Secondary Error: (Hex 0x%08X) %s\n", (unsigned int) secondary, msg);
        }

    if (strlen (elaboration) > 0)
        {
        sprintf(appendPoint, "Elaboration: %s", elaboration);
        }

    actualSize = (ViInt32) (strlen(errorMessage) + 1);

    /* Prepare return values */
    if (bufferSize == 0)
        {
        error = (ViStatus) actualSize;
        }
    else
        {
        if (bufferSize > 0)
            {
            if (actualSize > bufferSize)
                {
                error = actualSize;
                actualSize = bufferSize;
                }
            }
        if (description == VI_NULL)
            {
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Null address for Description");
            }
        memcpy(description, errorMessage, (size_t)(actualSize-1));
        description[actualSize-1] = '\0';
        }

    if (errorCode)
        {
        *errorCode = primary;
        }

Error:
    /* Unlock Session */
    (void) Rs_UnlockSession (instrSession, &locked);
    return error;
}

/// HIFN This function clears the error code and error description for
/// HIFN the session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
ViStatus _VI_FUNC rsavpmg_ClearError (ViSession instrSession)                                                        
{                                                                                                           
    return Rs_ClearErrorInfo (instrSession);                                                                             
}

/// HIFN This function writes a user-specified string to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
ViStatus _VI_FUNC rsavpmg_WriteInstrData (ViSession instrSession, ViString writeBuffer)
{
    ViStatus error = VI_SUCCESS;

    checkErr (Rs_WriteInstrData (instrSession, writeBuffer));
    
    checkErr (rsavpmg_CheckStatus (instrSession));
 
Error:   
    return error;
}

/// HIFN Document your function here.
/// HIFN You may use multiple lines for documentation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR numBytes/Pass the maximum number of bytes to read from the instruments.
/// HIPAR rdBuf/After this function executes, this parameter contains the data that was read from the instrument.
/// HIPAR bytesRead/Returns the number of bytes actually read from the instrument and stored in the Read Buffer.
ViStatus _VI_FUNC rsavpmg_ReadInstrData (ViSession instrSession, ViInt32 numBytes,
                                          ViChar rdBuf[], ViUInt32 *bytesRead)
{
    return Rs_ReadInstrData (instrSession, numBytes, rdBuf, bytesRead);
}

/// HIFN This function queries the ViBoolean value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViBoolean value.
ViStatus _VI_FUNC rsavpmg_QueryViBoolean (ViSession instrSession,
                                           ViString command, 
                                           ViBoolean *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%d", command, value));
    checkErr( rsavpmg_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function queries the ViInt32 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViInt32 value.
ViStatus _VI_FUNC rsavpmg_QueryViInt32 (ViSession instrSession,
                                         ViString command, 
                                         ViInt32 *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%ld", command, value));
    checkErr( rsavpmg_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function queries the ViReal64 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViReal64 value.
ViStatus _VI_FUNC rsavpmg_QueryViReal64 (ViSession instrSession,
                                          ViString command, 
                                          ViReal64 *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%le", command, value));
    checkErr( rsavpmg_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function queries the ViString value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Value parameter.
/// HIPAR value/Returns the string value.
ViStatus _VI_FUNC rsavpmg_QueryViString (ViSession instrSession,
                                          ViString command, 
                                          ViInt32 bufferSize,
                                          ViChar _VI_FAR value[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      *pbuffer = NULL;
    ViUInt32        bytesRead;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viPrintf (instrSession, "%s\n", command));
    viCheckErr(Rs_ReadDataUnknownLength (instrSession, &pbuffer, &bytesRead));  
    if (bytesRead>=(ViUInt32)bufferSize)
    {
        strncpy(value, pbuffer, (size_t) (bufferSize-1));
        value[bufferSize-1]='\0';
        error = (ViStatus) (bytesRead+1);
    } 
    else
    {
        strncpy(value, pbuffer, (size_t) bytesRead); 
        value[bytesRead]='\0';   
    }
    checkErr( rsavpmg_CheckStatus (instrSession)); 
Error:
    if (pbuffer) free (pbuffer);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function closes the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
ViStatus _VI_FUNC rsavpmg_close (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    checkErr( rsavpmg_RsClose (instrSession));

Error:

    (void) Rs_UnlockSession (instrSession, VI_NULL);
    (void) Rs_Dispose (instrSession);

    return error;
}

/// HIFN This function performs all of the drivers clean-up operations
/// HIFN except for closing the RS session.  This function is called by
/// HIFN rsavpmg_close or by an RS class driver.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpmg_init or 
/// HIPAR instrSession/rsavpmg_InitWithOptions function.
ViStatus _VI_FUNC rsavpmg_RsClose (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;
    ViSession   io      = VI_NULL;

    /* Do not lock here. The caller manages the lock. */

    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_IO_SESSION, 0, (ViInt32) sizeof (io), &io));

Error:

    (void) Rs_SetAttribute (instrSession, VI_NULL, RS_ATTR_IO_SESSION, 0, VI_NULL);
    
    if(io != 0)
        {
        ViSession   rmSession = VI_NULL;

        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_VISA_RM_SESSION, 0, (ViInt32) sizeof (rmSession), &rmSession));
        
        (void) viClose (io);
        (void) viClose (rmSession);
        }
    
    return error;
}

/*****************************************************************************
 *------------------- End Instrument Driver Source Code ---------------------*
 *****************************************************************************/
