/****************************************************************************
 *
 *  Rohde & Schwarz Audio Video Platform MHL Generator instrument driver include file
 *
 ****************************************************************************/

#ifndef __RSAVPMG_HEADER
#define __RSAVPMG_HEADER

#include "rsidr_core.h"
#include "rsavpmg_attributes.h"
#include "rsavpmg_utility.h"

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#if defined(__cplusplus) || defined(__cplusplus__)
extern "C" {
#endif

/*****************************************************************************
 *- Useful Macros & Constants -----------------------------------------------*
 *****************************************************************************/
#define RSAVPMG_SIMULATION_ID_QUERY      "Rohde-Schwarz,VNA,123456/789,1.23"  /* Default definition of ID Query Response for simulation */
#define RSAVPMG_VALID_ID_RESPONSE_STRING "Rohde&Schwarz"      /* Valid response for identification query */
#define RSAVPMG_SIMULATION_OPT_QUERY     ""      /* Simulated response for *OPT? command */

#define RSAVPMG_IO_BUFFER_SIZE           1024L   /* I/O buffer size */
#define RSAVPMG_OPC_TIMEOUT              5000L   /* Maximum time to wait for OPC in milliseconds */
#define IEEE_488_2_ERROR_MASK             0x4    /* 488.2 STB Bits */   
#define IEEE_488_2_ESE_MASK               0x3D    /* 488.2 Event Status Register (ESE) Bits */
#define RSAVPMG_NAN                 9.91e+37
#define RSAVPMG_NAN_VI_INT32             0x80000000     
/****************************************************************************
 *----------------- Instrument Driver Revision Information -----------------*
 ****************************************************************************/

/* Class Driver Identification */

#define RSAVPMG_CLASS_DRIVER_DESCRIPTION           ""
#define RSAVPMG_CLASS_DRIVER_PREFIX                ""
#define RSAVPMG_CLASS_DRIVER_VENDOR                ""
#define RSAVPMG_CLASS_DRIVER_REVISION              ""
#define RSAVPMG_CLASS_SPEC_MAJOR_VERSION           1L    /* Class specification major version */
#define RSAVPMG_CLASS_SPEC_MINOR_VERSION           0L    /* Class specification minor version */

/* Driver Identification */

#define RSAVPMG_SPECIFIC_DRIVER_DESCRIPTION        ""
#define RSAVPMG_SPECIFIC_DRIVER_PREFIX             "RSAVPMG"
#define RSAVPMG_SPECIFIC_DRIVER_LOCATOR            ""
#define RSAVPMG_SPECIFIC_DRIVER_VENDOR             ""
#define RSAVPMG_MAJOR_VERSION                      1L    /* Instrument driver major version          */
#define RSAVPMG_MINOR_VERSION                      0L    /* Instrument driver minor version          */
#define RSAVPMG_MINOR_MINOR_VERSION                0L    /* Instrument driver minor minor version    */

/* Driver Capabilities */          

#define RSAVPMG_SUPPORTED_INSTRUMENT_MODELS        ""    /* Instrument driver supported model(s)     */
#define RSAVPMG_GROUP_CAPABILITIES                 ""
#define RSAVPMG_FUNCTION_CAPABILITIES              ""

/****************************************************************************
 *---------------------------- Attribute Defines ---------------------------*
 ****************************************************************************/

    /*- Inherent Instrument Specific Attributes ----------------------------*/

        /* User Options */

#define RSAVPMG_ATTR_RANGE_CHECK                                           RS_ATTR_RANGE_CHECK                                     /* ViBoolean */
#define RSAVPMG_ATTR_QUERY_INSTRUMENT_STATUS                               RS_ATTR_QUERY_INSTRUMENT_STATUS                         /* ViBoolean */
#define RSAVPMG_ATTR_CACHE                                                 RS_ATTR_CACHE                                           /* ViBoolean */
#define RSAVPMG_ATTR_SIMULATE                                              RS_ATTR_SIMULATE                                        /* ViBoolean */
#define RSAVPMG_ATTR_RECORD_COERCIONS                                      RS_ATTR_RECORD_COERCIONS                                /* ViBoolean */
#define RSAVPMG_ATTR_INTERCHANGE_CHECK                                     RS_ATTR_INTERCHANGE_CHECK                               /* ViBoolean */
#define RSAVPMG_ATTR_SPY                                                   RS_ATTR_SPY                                             /* ViBoolean */
#define RSAVPMG_ATTR_USE_SPECIFIC_SIMULATION                               RS_ATTR_USE_SPECIFIC_SIMULATION                         /* ViBoolean */

        /* Class Driver Identification */

#define RSAVPMG_ATTR_CLASS_DRIVER_DESCRIPTION                              RS_ATTR_CLASS_DRIVER_DESCRIPTION                        /* ViString, read-only */
#define RSAVPMG_ATTR_CLASS_DRIVER_PREFIX                                   RS_ATTR_CLASS_DRIVER_PREFIX                             /* ViString, read-only */
#define RSAVPMG_ATTR_CLASS_DRIVER_VENDOR                                   RS_ATTR_CLASS_DRIVER_VENDOR                             /* ViString, read-only */
#define RSAVPMG_ATTR_CLASS_DRIVER_REVISION                                 RS_ATTR_CLASS_DRIVER_REVISION                           /* ViString, read-only */
#define RSAVPMG_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION           /* ViInt32,  read-only */
#define RSAVPMG_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION           /* ViInt32,  read-only */

        /* Driver Identification */

#define RSAVPMG_ATTR_SPECIFIC_DRIVER_DESCRIPTION                           RS_ATTR_SPECIFIC_DRIVER_DESCRIPTION                     /* ViString, read-only */
#define RSAVPMG_ATTR_SPECIFIC_DRIVER_PREFIX                                RS_ATTR_SPECIFIC_DRIVER_PREFIX                          /* ViString, read-only */
#define RSAVPMG_ATTR_SPECIFIC_DRIVER_LOCATOR                               RS_ATTR_SPECIFIC_DRIVER_LOCATOR                         /* ViString, read-only */
#define RSAVPMG_ATTR_SPECIFIC_DRIVER_VENDOR                                RS_ATTR_SPECIFIC_DRIVER_VENDOR                          /* ViString, read-only */
#define RSAVPMG_ATTR_SPECIFIC_DRIVER_REVISION                              RS_ATTR_SPECIFIC_DRIVER_REVISION                        /* ViString, read-only */
#define RSAVPMG_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION        /* ViInt32,  read-only */
#define RSAVPMG_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION        /* ViInt32,  read-only */

        /* Driver Capabilities */

#define RSAVPMG_ATTR_SUPPORTED_INSTRUMENT_MODELS                           RS_ATTR_SUPPORTED_INSTRUMENT_MODELS                     /* ViString, read-only */
#define RSAVPMG_ATTR_GROUP_CAPABILITIES                                    RS_ATTR_GROUP_CAPABILITIES                              /* ViString, read-only */
#define RSAVPMG_ATTR_FUNCTION_CAPABILITIES                                 RS_ATTR_FUNCTION_CAPABILITIES                           /* ViString, read-only */
#define RSAVPMG_ATTR_CHANNEL_COUNT                                         RS_ATTR_CHANNEL_COUNT                                   /* ViInt32,  read-only */

        /* Driver Setup */

#define RSAVPMG_ATTR_DRIVER_SETUP                                          RS_ATTR_DRIVER_SETUP                                    /* ViString            */

        /* Instrument Identification */

#define RSAVPMG_ATTR_INSTRUMENT_MANUFACTURER                               RS_ATTR_INSTRUMENT_MANUFACTURER                         /* ViString, read-only */
#define RSAVPMG_ATTR_INSTRUMENT_MODEL                                      RS_ATTR_INSTRUMENT_MODEL                                /* ViString, read-only */
#define RSAVPMG_ATTR_INSTRUMENT_FIRMWARE_REVISION                          RS_ATTR_INSTRUMENT_FIRMWARE_REVISION                    /* ViString, read-only */

        /* Advanced Session Information */

#define RSAVPMG_ATTR_IO_RESOURCE_DESCRIPTOR                                RS_ATTR_IO_RESOURCE_DESCRIPTOR                          /* ViString, read-only */
#define RSAVPMG_ATTR_LOGICAL_NAME                                          RS_ATTR_LOGICAL_NAME

/* ViString, read-only */

    /*- Instrument-Specific Attributes -------------------------------------*/

//#define RSAVPMG_ATTR_ID_QUERY_RESPONSE                                     (RS_SPECIFIC_PUBLIC_ATTR_BASE + 1L)                     /* ViString (Read Only)              */

/* BEGIN GENERATE */
#define RSAVPMG_ATTR_MHL_GENERATOR_STOP                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7
#define RSAVPMG_ATTR_MHL_GENERATOR_OUTPUT                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1
#define RSAVPMG_ATTR_MHL_GENERATOR_OUTPUT_HDCP                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2
#define RSAVPMG_ATTR_MHL_GENERATOR_CBUS_CONNECTION                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3
#define RSAVPMG_ATTR_MHL_GENERATOR_VBUS_POWER_CABLE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 4
#define RSAVPMG_ATTR_MHL_GENERATOR_SIGNAL_PATTERN                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5
#define RSAVPMG_ATTR_MHL_GENERATOR_SIGNAL_INIT                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6
#define RSAVPMG_ATTR_MHL_GENERATOR_SIGNAL_STATUS                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8
#define RSAVPMG_ATTR_QUERY_MHL_GENERATOR_EXTENDED_DISPLAY_INFORMATION_DATA  RS_SPECIFIC_PUBLIC_ATTR_BASE + 9
#define RSAVPMG_ATTR_MHL_GENERATOR_CTS_STATUS                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10
#define RSAVPMG_ATTR_MHL_GENERATOR_CTS_INIT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 11
#define RSAVPMG_ATTR_QUERY_MHL_GENERATOR_CTS_PROGRESS                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 13
#define RSAVPMG_ATTR_MHL_GENERATOR_CTS_TEST_CASE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 14
#define RSAVPMG_ATTR_MHL_GENERATOR_CTS_TEST_OUTPUT_REPORT_COUNT             RS_SPECIFIC_PUBLIC_ATTR_BASE + 15
#define RSAVPMG_ATTR_SELF_TEST_QUERY                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 16
#define RSAVPMG_ATTR_SYST_VERSION                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 17
#define RSAVPMG_ATTR_SYSTEM_RESET_ALL                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 18
#define RSAVPMG_ATTR_SYSTEM_RESET_BASE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 19
#define RSAVPMG_ATTR_SYSTEM_PRESET_ALL                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 20
#define RSAVPMG_ATTR_SYSTEM_PRESET_BASE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 21
#define RSAVPMG_ATTR_ID_QUERY_RESPONSE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 22
#define RSAVPMG_ATTR_SYSTEM_ERROR                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 23
#define RSAVPMG_ATTR_DISPLAY_UPDATE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 24

// rsavpmg_rngMHLGeneratorOutput
#define RSAVPMG_VAL_MHL_OUTPUT_1_3                                          0
#define RSAVPMG_VAL_MHL_OUTPUT_2_1                                          1

// rsavpmg_rngMHLGeneratorCBUSConnection
#define RSAVPMG_VAL_MHL_CBUS_WAIT_FOR_CONNECTION                            0
#define RSAVPMG_VAL_MHL_CBUS_CONNECTED                                      1

// rsavpmg_rngMHLGeneratorSignalPattern
#define RSAVPMG_VAL_MHL_SIGNAL_COLOR_BAR                                    0

// rsavpmg_rngMHLGeneratorSignalStatus
#define RSAVPMG_VAL_MHL_STATE_STOPPED                                       0
#define RSAVPMG_VAL_MHL_STATE_LOADING                                       1
#define RSAVPMG_VAL_MHL_STATE_RUNNING                                       2
#define RSAVPMG_VAL_MHL_STATE_TESTING                                       3
#define RSAVPMG_VAL_MHL_STATE_READY                                         4

// rsavpmg_rngQueryMHLGeneratorExtendedDisplayInformationData
#define RSAVPMG_VAL_MHL_STATE_ASCII                                         0
#define RSAVPMG_VAL_MHL_STATE_BINARY                                        1

// rsavpmg_rngMHLGeneratorCTSTestCase
#define RSAVPMG_VAL_MHL_SINK_TEST_CSYNC                                     0
#define RSAVPMG_VAL_MHL_SINK_TEST_PACKET                                    1
#define RSAVPMG_VAL_MHL_SINK_TEST_VFORMAT                                   2
#define RSAVPMG_VAL_MHL_SINK_TEST_PIXEL                                     3
#define RSAVPMG_VAL_MHL_SINK_TEST_VQRANGE                                   4
#define RSAVPMG_VAL_MHL_SINK_TEST_IEC6                                      5
#define RSAVPMG_VAL_MHL_SINK_TEST_ACLOCK                                    6
#define RSAVPMG_VAL_MHL_SINK_TEST_EDIT                                      7
#define RSAVPMG_VAL_MHL_SINK_TEST_DCRT                                      8
#define RSAVPMG_VAL_MHL_SINK_TEST_RCPRX                                     9
#define RSAVPMG_VAL_MHL_SINK_TEST_RCPTX                                     10
#define RSAVPMG_VAL_MHL_SINK_TEST_DSRT                                      11
#define RSAVPMG_VAL_MHL_SINK_TEST_RAPSC                                     12
#define RSAVPMG_VAL_MHL_SINK_TEST_3DVMS                                     13
#define RSAVPMG_VAL_MHL_SINK_TEST_3DVFT                                     14
#define RSAVPMG_VAL_MHL_SINK_TEST_UCPRX                                     15
#define RSAVPMG_VAL_MHL_SINK_TEST_UCPTX                                     16
#define RSAVPMG_VAL_MHL_DONGLE_TEST_CSYNC                                   17
#define RSAVPMG_VAL_MHL_DONGLE_TEST_PACKET                                  18
#define RSAVPMG_VAL_MHL_DONGLE_TEST_VFORMAT                                 19
#define RSAVPMG_VAL_MHL_DONGLE_TEST_PIXEL                                   20
#define RSAVPMG_VAL_MHL_DONGLE_TEST_VQRANGE                                 21
#define RSAVPMG_VAL_MHL_DONGLE_TEST_IEC6                                    22
#define RSAVPMG_VAL_MHL_DONGLE_TEST_ACLOCK                                  23
#define RSAVPMG_VAL_MHL_DONGLE_TEST_EDIT                                    24
#define RSAVPMG_VAL_MHL_DONGLE_TEST_DCRT                                    25
#define RSAVPMG_VAL_MHL_DONGLE_TEST_RCPRX                                   26
#define RSAVPMG_VAL_MHL_DONGLE_TEST_RCPTX                                   27
#define RSAVPMG_VAL_MHL_DONGLE_TEST_DSRT                                    28
#define RSAVPMG_VAL_MHL_DONGLE_TEST_RAPSC                                   29
#define RSAVPMG_VAL_MHL_DONGLE_TEST_3DVMS                                   30
#define RSAVPMG_VAL_MHL_DONGLE_TEST_3DVFT                                   31
#define RSAVPMG_VAL_MHL_DONGLE_TEST_UCPRX                                   32
#define RSAVPMG_VAL_MHL_DONGLE_TEST_UCPTX                                   33
/* END GENERATE */

/*****************************************************************************
 *- Hidden Attribute Declarations -------------------------------------------*
 *****************************************************************************/

#define RSAVPMG_ATTR_OPC_TIMEOUT               (RS_ATTR_OPC_TIMEOUT)           /* ViInt32 */
#define RSAVPMG_ATTR_IO_SESSION                (RS_ATTR_IO_SESSION)            /* ViSession */
#define RSAVPMG_ATTR_OPC_CALLBACK              (RS_ATTR_OPC_CALLBACK)          /* ViAddr */
#define RSAVPMG_ATTR_CHECK_STATUS_CALLBACK     (RS_ATTR_CHECK_STATUS_CALLBACK) /* ViAddr */

/****************************************************************************
 *---------------- Constant definition  ------------------------------------*
 ****************************************************************************/

#define RSAVPMG_VAL_MHL_PIXEL_ENCODING_RGB    0
#define RSAVPMG_VAL_MHL_PIXEL_ENCODING_Y422   1
#define RSAVPMG_VAL_MHL_PIXEL_ENCODING_Y444   2

#define RSAVPMG_VAL_MHL_STOP   0
#define RSAVPMG_VAL_MHL_START  1

#define RSAVPMG_VAL_MHL_3D_FORMAT_TOP_BOTTOM    0
#define RSAVPMG_VAL_MHL_3D_FORMAT_FRAME_SEQ     1
	
#define RSAVPMG_VAL_MHL_VIDEO_FORMAT_AUTO       0
#define RSAVPMG_VAL_MHL_VIDEO_FORMAT_C1         1
#define RSAVPMG_VAL_MHL_VIDEO_FORMAT_C2         2
#define RSAVPMG_VAL_MHL_VIDEO_FORMAT_C17        3
#define RSAVPMG_VAL_MHL_VIDEO_FORMAT_C4         4
#define RSAVPMG_VAL_MHL_VIDEO_FORMAT_C19        5
#define RSAVPMG_VAL_MHL_VIDEO_FORMAT_C5         6
#define RSAVPMG_VAL_MHL_VIDEO_FORMAT_C20        7
#define RSAVPMG_VAL_MHL_VIDEO_FORMAT_C32        8
#define RSAVPMG_VAL_MHL_VIDEO_FORMAT_C34        9
#define RSAVPMG_VAL_MHL_VIDEO_FORMAT_C6        10
#define RSAVPMG_VAL_MHL_VIDEO_FORMAT_C21       11
#define RSAVPMG_VAL_MHL_VIDEO_FORMAT_C16       12
#define RSAVPMG_VAL_MHL_VIDEO_FORMAT_C31       13
	
#define RSAVPMG_VAL_MHL_PATTERN_TEST           0
#define RSAVPMG_VAL_MHL_PATTERN_RAMP           1

#define	RSAVPMG_VAL_MHL_AUDIO_FORMAT_P32       0
#define RSAVPMG_VAL_MHL_AUDIO_FORMAT_P441      1
#define RSAVPMG_VAL_MHL_AUDIO_FORMAT_P48       2

#define	RSAVPMG_VAL_MHL_N_VALUE_MIN            0
#define RSAVPMG_VAL_MHL_N_VALUE_MAX            1

#define	RSAVPMG_VAL_MHL_OPCODE_POLL            0
#define RSAVPMG_VAL_MHL_OPCODE_CONTENT_ON      1
#define RSAVPMG_VAL_MHL_OPCODE_CONTENT_OFF     2

#define	RSAVPMG_VAL_RST_BASE  0
#define RSAVPMG_VAL_RST_ALL   1
/****************************************************************************
*---------------- Instrument Driver Function Declarations -----------------*
 ****************************************************************************/

ViStatus _VI_FUNC rsavpmg_init (ViRsrc resourceName, ViBoolean IDQuery,
                                ViBoolean resetDevice, ViSession* instrumentHandle);
ViStatus _VI_FUNC rsavpmg_InitWithOptions (ViRsrc resourceName, ViBoolean IDQuery,
                                           ViBoolean resetDevice,
                                           ViChar _VI_FAR optionString[],
                                           ViSession* instrumentHandle);
ViStatus _VI_FUNC rsavpmg_ConfigureMHLGeneratorOutput (ViSession instrumentHandle,
                                                       ViInt32 board,
                                                       ViInt32 output,
                                                       ViBoolean HDCP);
ViStatus _VI_FUNC rsavpmg_QueryMHLGeneratorOutput (ViSession instrumentHandle,
                                                   ViInt32 board, ViInt32* CBUS,
                                                   ViReal64* VBUS);
ViStatus _VI_FUNC rsavpmg_ConfigureMHLGeneratorSignal (ViSession instrumentHandle,
                                                       ViInt32 board,
                                                       ViInt32 signal);
ViStatus _VI_FUNC rsavpmg_MHLGeneratorSignalOperation (ViSession instrumentHandle,
                                                       ViInt32 board,
                                                       ViInt32 operation);
ViStatus _VI_FUNC rsavpmg_QueryMHLGeneratorSignalState (ViSession instrumentHandle,
                                                        ViInt32 board,
                                                        ViInt32* state);
ViStatus _VI_FUNC rsavpmg_QueryMHLGeneratorEDID (ViSession instrumentHandle,
                                                 ViInt32 board, ViInt32* EDID);
ViStatus _VI_FUNC rsavpmg_QueryMHLGeneratorCTSState (ViSession instrumentHandle,
                                                     ViInt32 board, ViInt32* state);
ViStatus _VI_FUNC rsavpmg_MHLGeneratorCTSOperation (ViSession instrumentHandle,
                                                    ViInt32 board,
                                                    ViInt32 operation);
ViStatus _VI_FUNC rsavpmg_QueryMHLGeneratorCTSProgress (ViSession instrumentHandle,
                                                        ViInt32 board,
                                                        ViReal64* progress);
ViStatus _VI_FUNC rsavpmg_SelectMHLGeneratorCTSTestCase (ViSession instrumentHandle,
                                                         ViInt32 board,
                                                         ViInt32 testCase);
ViStatus _VI_FUNC rsavpmg_ConfigureMHLGeneratorCTSTMDSCoding
             (ViSession instrumentHandle, ViInt32 board, ViInt32 testCase,
              ViInt32 videoFormat);
ViStatus _VI_FUNC rsavpmg_ConfigureMHLGeneratorCTSVideoModes
             (ViSession instrumentHandle, ViInt32 board, ViInt32 testCase,
              ViInt32 videoFormat, ViInt32 pattern, ViInt32 pixelEncoding);
ViStatus _VI_FUNC rsavpmg_ConfigureMHLGeneratorCTSAudioTests
             (ViSession instrumentHandle, ViInt32 board, ViInt32 testCase,
              ViInt32 videoFormat, ViInt32 audioFormat, ViInt32 nValue);
ViStatus _VI_FUNC rsavpmg_ConfigureMHLGeneratorCTSRCPSubCommandTest
             (ViSession instrumentHandle, ViInt32 board, ViInt32 testCase,
              ViInt32 keyID);
ViStatus _VI_FUNC rsavpmg_ConfigureMHLGeneratorCTSRAPSubCommandTests
             (ViSession instrumentHandle, ViInt32 board, ViInt32 testCase,
              ViInt32 OPcode);
ViStatus _VI_FUNC rsavpmg_ConfigureMHLGeneratorCTS3DVideoTest
             (ViSession instrumentHandle, ViInt32 board, ViInt32 testCase,
              ViInt32 videoFormat, ViInt32 _3DFormat);
ViStatus _VI_FUNC rsavpmg_ConfigureMHLGeneratorCTSUCPSubCommandTests
             (ViSession instrumentHandle, ViInt32 board, ViInt32 testCase,
              ViString characterSequence);
ViStatus _VI_FUNC rsavpmg_QueryMHLGeneratorCTSTestOutputReportCount
             (ViSession instrumentHandle, ViInt32 board, ViInt32* count);
ViStatus _VI_FUNC rsavpmg_QueryMHLGeneratorCTSTestOutputReport
             (ViSession instrumentHandle, ViInt32 board, ViInt32 index,
              ViInt32 arraySize, ViInt32* lineNumber, ViInt32* referenceNumber,
              ViChar _VI_FAR textMessage[], ViInt32 _VI_FAR optionalParameters[]);
ViStatus _VI_FUNC rsavpmg_reset (ViSession instrumentHandle);
ViStatus _VI_FUNC rsavpmg_ResetSystem (ViSession instrumentHandle, ViInt32 reference);
ViStatus _VI_FUNC rsavpmg_Preset (ViSession instrumentHandle, ViInt32 reference);
ViStatus _VI_FUNC rsavpmg_self_test (ViSession instrumentHandle,
                                     ViInt16* selfTestResult,
                                     ViChar _VI_FAR selfTestMessage[]);
ViStatus _VI_FUNC rsavpmg_revision_query (ViSession instrumentHandle,
                                          ViChar _VI_FAR instrumentDriverRevision[],
                                          ViChar _VI_FAR firmwareRevision[]);
ViStatus _VI_FUNC rsavpmg_error_query (ViSession instrumentHandle,
                                       ViInt32* errorCode,
                                       ViChar _VI_FAR errorMessage[]);
ViStatus _VI_FUNC rsavpmg_error_message (ViSession instrumentHandle,
                                         ViStatus errorCode,
                                         ViChar _VI_FAR errorMessage[]);
ViStatus _VI_FUNC rsavpmg_SystemVersion (ViSession instrumentHandle, ViInt32 length,
                                         ViChar _VI_FAR systemVersion[]);
ViStatus _VI_FUNC rsavpmg_setCheckStatus (ViSession instrumentHandle,
                                          ViBoolean state);
ViStatus _VI_FUNC rsavpmg_ConfigureDisplayUpdate (ViSession instrumentHandle,
                                                  ViBoolean displayUpdate);
ViStatus _VI_FUNC rsavpmg_GetError (ViSession instrumentHandle, ViStatus* code,
                                    ViInt32 bufferSize,
                                    ViChar _VI_FAR description[]);
ViStatus _VI_FUNC rsavpmg_ClearError (ViSession instrumentHandle);
ViStatus _VI_FUNC rsavpmg_WriteInstrData (ViSession instrumentHandle,
                                          ViChar _VI_FAR writeBuffer[]);
ViStatus _VI_FUNC rsavpmg_ReadInstrData (ViSession instrumentHandle,
                                         ViInt32 numberOfBytesToRead,
                                         ViChar _VI_FAR readBuffer[],
                                         ViUInt32* numBytesRead);
ViStatus _VI_FUNC rsavpmg_QueryViBoolean (ViSession instrumentHandle,
                                          ViString command, ViBoolean* value);
ViStatus _VI_FUNC rsavpmg_QueryViInt32 (ViSession instrumentHandle,
                                        ViString command, ViInt32* value);
ViStatus _VI_FUNC rsavpmg_QueryViReal64 (ViSession instrumentHandle,
                                         ViString command, ViReal64* value);
ViStatus _VI_FUNC rsavpmg_QueryViString (ViSession instrumentHandle,
                                         ViString command, ViInt32 bufferSize,
                                         ViChar _VI_FAR value[]);
ViStatus _VI_FUNC rsavpmg_SetAttributeViInt32 (ViSession instrumentHandle,
                                               ViChar _VI_FAR channelName[],
                                               ViAttr attributeID,
                                               ViInt32 attributeValue);
ViStatus _VI_FUNC rsavpmg_SetAttributeViInt64 (ViSession instrumentHandle,
                                               ViChar _VI_FAR repeatedCapabilityName[],
                                               ViAttr attributeID,
                                               ViInt64 attributeValue);
ViStatus _VI_FUNC rsavpmg_SetAttributeViReal64 (ViSession instrumentHandle,
                                                ViChar _VI_FAR channelName[],
                                                ViAttr attributeID,
                                                ViReal64 attributeValue);
ViStatus _VI_FUNC rsavpmg_SetAttributeViString (ViSession instrumentHandle,
                                                ViChar _VI_FAR channelName[],
                                                ViAttr attributeID,
                                                ViChar _VI_FAR attributeValue[]);
ViStatus _VI_FUNC rsavpmg_SetAttributeViBoolean (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViBoolean attributeValue);
ViStatus _VI_FUNC rsavpmg_SetAttributeViSession (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViSession attributeValue);
ViStatus _VI_FUNC rsavpmg_GetAttributeViInt32 (ViSession instrumentHandle,
                                               ViChar _VI_FAR channelName[],
                                               ViAttr attributeID,
                                               ViInt32* attributeValue);
ViStatus _VI_FUNC rsavpmg_GetAttributeViInt64 (ViSession instrumentHandle,
                                               ViChar _VI_FAR repeatedCapabilityName[],
                                               ViAttr attributeID,
                                               ViInt64* attributeValue);
ViStatus _VI_FUNC rsavpmg_GetAttributeViReal64 (ViSession instrumentHandle,
                                                ViChar _VI_FAR channelName[],
                                                ViAttr attributeID,
                                                ViReal64* attributeValue);
ViStatus _VI_FUNC rsavpmg_GetAttributeViString (ViSession instrumentHandle,
                                                ViChar _VI_FAR channelName[],
                                                ViAttr attributeID,
                                                ViInt32 bufferSize,
                                                ViChar _VI_FAR attributeValue[]);
ViStatus _VI_FUNC rsavpmg_GetAttributeViBoolean (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViBoolean* attributeValue);
ViStatus _VI_FUNC rsavpmg_GetAttributeViSession (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViSession* attributeValue);
ViStatus _VI_FUNC rsavpmg_CheckAttributeViInt32 (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViInt32 attributeValue);
ViStatus _VI_FUNC rsavpmg_CheckAttributeViInt64 (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViInt64 attributeValue);
ViStatus _VI_FUNC rsavpmg_CheckAttributeViReal64 (ViSession instrumentHandle,
                                                  ViChar _VI_FAR channelName[],
                                                  ViAttr attributeID,
                                                  ViReal64 attributeValue);
ViStatus _VI_FUNC rsavpmg_CheckAttributeViString (ViSession instrumentHandle,
                                                  ViChar _VI_FAR channelName[],
                                                  ViAttr attributeID,
                                                  ViChar _VI_FAR attributeValue[]);
ViStatus _VI_FUNC rsavpmg_CheckAttributeViBoolean (ViSession instrumentHandle,
                                                   ViChar _VI_FAR channelName[],
                                                   ViAttr attributeID,
                                                   ViBoolean attributeValue);
ViStatus _VI_FUNC rsavpmg_CheckAttributeViSession (ViSession instrumentHandle,
                                                   ViChar _VI_FAR channelName[],
                                                   ViAttr attributeID,
                                                   ViSession attributeValue);
ViStatus _VI_FUNC rsavpmg_GetAttributeRepeatedCapabilityIds
             (ViSession instrumentHandle, ViUInt32 attributeID, ViInt32 bufferSize,
              ViChar _VI_FAR repeatedCapabilityId_s[]);
ViStatus _VI_FUNC rsavpmg_GetAttributeRepeatedCapabilityIdNames
             (ViSession instrumentHandle, ViUInt32 attributeID,
              ViString repeatedCapabilityId, ViInt32 bufferSize,
              ViChar _VI_FAR repeatedCapabilityIdName_s[]);
ViStatus _VI_FUNC rsavpmg_close (ViSession instrumentHandle);

/****************************************************************************
 *------------------------ Error And Completion Codes ----------------------*
 ****************************************************************************/

#define RSAVPMG_WARN_MEASURE_UNCALIBRATED    (RS_CLASS_WARN_BASE + 0x0001L)
#define RSAVPMG_WARN_OVER_RANGE              (RS_CLASS_WARN_BASE + 0x0002L)

#define RSAVPMG_ERROR_MARKER_NOT_ENABLED     (RS_CLASS_ERROR_BASE + 0x0003L)
#define RSAVPMG_ERROR_NOT_DELTA_MARKER       (RS_CLASS_ERROR_BASE + 0x0002L)
#define RSAVPMG_ERROR_MAX_TIME_EXCEEDED      (RS_CLASS_ERROR_BASE + 0x0001L)

#define RSAVPMG_WARNMSG_MEASURE_UNCALIBRATED "Uncalibrated measurement"
#define RSAVPMG_WARNMSG_OVER_RANGE           "Measurement overrange"

#define RSAVPMG_ERRMSG_MARKER_NOT_ENABLED    "Marker Not Enabled"
#define RSAVPMG_ERRMSG_NOT_DELTA_MARKER      "Not Delta Marker"
#define RSAVPMG_ERRMSG_MAX_TIME_EXCEEDED     "Max Time Exceeded"

#define RSAVPMG_ERROR_CODES_AND_MSGS \
    {RSAVPMG_WARN_MEASURE_UNCALIBRATED,  RSAVPMG_WARNMSG_MEASURE_UNCALIBRATED},\
    {RSAVPMG_WARN_OVER_RANGE,            RSAVPMG_WARNMSG_OVER_RANGE},\
    {RSAVPMG_ERROR_MARKER_NOT_ENABLED,   RSAVPMG_ERRMSG_MARKER_NOT_ENABLED},\
    {RSAVPMG_ERROR_NOT_DELTA_MARKER,     RSAVPMG_ERRMSG_NOT_DELTA_MARKER},\
    {RSAVPMG_ERROR_MAX_TIME_EXCEEDED,    RSAVPMG_ERRMSG_MAX_TIME_EXCEEDED}

/****************************************************************************
 *---------------------------- End Include File ----------------------------*
 ****************************************************************************/

#if defined(__cplusplus) || defined(__cplusplus__)
}
#endif
#endif /* __RSAVPMG_HEADER */



