/****************************************************************************
 *
 *  Rohde & Schwarz Audio Video Platform Video Analyzer
 *
 ****************************************************************************/

#ifndef __RSAVPVA_HEADER
#define __RSAVPVA_HEADER

#include "rsidr_core.h"
#include "rsavpva_attributes.h"
#include "rsavpva_utility.h"

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#if defined(__cplusplus) || defined(__cplusplus__)
extern "C" {
#endif

/*****************************************************************************
 *- Useful Macros & Constants -----------------------------------------------*
 *****************************************************************************/
#define RSAVPVA_SIMULATION_ID_QUERY      "Rohde&Schwarz,R&S VTE,2115.7300k02/900000,1.45.0.0"  /* Default definition of ID Query Response for simulation */
#define RSAVPVA_VALID_ID_RESPONSE_STRING "Rohde&Schwarz"      /* Valid response for identification query */
#define RSAVPVA_SIMULATION_OPT_QUERY     ""      /* Simulated response for *OPT? command */

#define RSAVPVA_IO_BUFFER_SIZE           1024L   /* I/O buffer size */
#define RSAVPVA_OPC_TIMEOUT              5000L   /* Maximum time to wait for OPC in milliseconds */
#define IEEE_488_2_ERROR_MASK             0x4    /* 488.2 STB Bits */   
#define IEEE_488_2_ESE_MASK               0x3D    /* 488.2 Event Status Register (ESE) Bits */
#define RSAVPVA_NAN                 9.91e+37
#define RSAVPVA_NAN_VI_INT32             0x80000000     
/****************************************************************************
 *----------------- Instrument Driver Revision Information -----------------*
 ****************************************************************************/

/* Class Driver Identification */

#define RSAVPVA_CLASS_DRIVER_DESCRIPTION           ""
#define RSAVPVA_CLASS_DRIVER_PREFIX                ""
#define RSAVPVA_CLASS_DRIVER_VENDOR                ""
#define RSAVPVA_CLASS_DRIVER_REVISION              ""
#define RSAVPVA_CLASS_SPEC_MAJOR_VERSION           1L    /* Class specification major version */
#define RSAVPVA_CLASS_SPEC_MINOR_VERSION           0L    /* Class specification minor version */

/* Driver Identification */

#define RSAVPVA_SPECIFIC_DRIVER_DESCRIPTION        ""
#define RSAVPVA_SPECIFIC_DRIVER_PREFIX             "RSAVPVA"
#define RSAVPVA_SPECIFIC_DRIVER_LOCATOR            ""
#define RSAVPVA_SPECIFIC_DRIVER_VENDOR             ""
#define RSAVPVA_MAJOR_VERSION                      1L    /* Instrument driver major version          */
#define RSAVPVA_MINOR_VERSION                      0L    /* Instrument driver minor version          */
#define RSAVPVA_MINOR_MINOR_VERSION                0L    /* Instrument driver minor minor version    */

/* Driver Capabilities */          

#define RSAVPVA_SUPPORTED_INSTRUMENT_MODELS        ""    /* Instrument driver supported model(s)     */
#define RSAVPVA_GROUP_CAPABILITIES                 ""
#define RSAVPVA_FUNCTION_CAPABILITIES              ""

/****************************************************************************
 *---------------------------- Attribute Defines ---------------------------*
 ****************************************************************************/

    /*- Inherent Instrument Specific Attributes ----------------------------*/

        /* User Options */

#define RSAVPVA_ATTR_RANGE_CHECK                                           RS_ATTR_RANGE_CHECK                                     /* ViBoolean */
#define RSAVPVA_ATTR_QUERY_INSTRUMENT_STATUS                               RS_ATTR_QUERY_INSTRUMENT_STATUS                         /* ViBoolean */
#define RSAVPVA_ATTR_CACHE                                                 RS_ATTR_CACHE                                           /* ViBoolean */
#define RSAVPVA_ATTR_SIMULATE                                              RS_ATTR_SIMULATE                                        /* ViBoolean */
#define RSAVPVA_ATTR_RECORD_COERCIONS                                      RS_ATTR_RECORD_COERCIONS                                /* ViBoolean */
#define RSAVPVA_ATTR_INTERCHANGE_CHECK                                     RS_ATTR_INTERCHANGE_CHECK                               /* ViBoolean */
#define RSAVPVA_ATTR_SPY                                                   RS_ATTR_SPY                                             /* ViBoolean */
#define RSAVPVA_ATTR_USE_SPECIFIC_SIMULATION                               RS_ATTR_USE_SPECIFIC_SIMULATION                         /* ViBoolean */

        /* Class Driver Identification */

#define RSAVPVA_ATTR_CLASS_DRIVER_DESCRIPTION                              RS_ATTR_CLASS_DRIVER_DESCRIPTION                        /* ViString, read-only */
#define RSAVPVA_ATTR_CLASS_DRIVER_PREFIX                                   RS_ATTR_CLASS_DRIVER_PREFIX                             /* ViString, read-only */
#define RSAVPVA_ATTR_CLASS_DRIVER_VENDOR                                   RS_ATTR_CLASS_DRIVER_VENDOR                             /* ViString, read-only */
#define RSAVPVA_ATTR_CLASS_DRIVER_REVISION                                 RS_ATTR_CLASS_DRIVER_REVISION                           /* ViString, read-only */
#define RSAVPVA_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION           /* ViInt32,  read-only */
#define RSAVPVA_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION           /* ViInt32,  read-only */

        /* Driver Identification */

#define RSAVPVA_ATTR_SPECIFIC_DRIVER_DESCRIPTION                           RS_ATTR_SPECIFIC_DRIVER_DESCRIPTION                     /* ViString, read-only */
#define RSAVPVA_ATTR_SPECIFIC_DRIVER_PREFIX                                RS_ATTR_SPECIFIC_DRIVER_PREFIX                          /* ViString, read-only */
#define RSAVPVA_ATTR_SPECIFIC_DRIVER_LOCATOR                               RS_ATTR_SPECIFIC_DRIVER_LOCATOR                         /* ViString, read-only */
#define RSAVPVA_ATTR_SPECIFIC_DRIVER_VENDOR                                RS_ATTR_SPECIFIC_DRIVER_VENDOR                          /* ViString, read-only */
#define RSAVPVA_ATTR_SPECIFIC_DRIVER_REVISION                              RS_ATTR_SPECIFIC_DRIVER_REVISION                        /* ViString, read-only */
#define RSAVPVA_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION        /* ViInt32,  read-only */
#define RSAVPVA_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION        /* ViInt32,  read-only */

        /* Driver Capabilities */

#define RSAVPVA_ATTR_SUPPORTED_INSTRUMENT_MODELS                           RS_ATTR_SUPPORTED_INSTRUMENT_MODELS                     /* ViString, read-only */
#define RSAVPVA_ATTR_GROUP_CAPABILITIES                                    RS_ATTR_GROUP_CAPABILITIES                              /* ViString, read-only */
#define RSAVPVA_ATTR_FUNCTION_CAPABILITIES                                 RS_ATTR_FUNCTION_CAPABILITIES                           /* ViString, read-only */
#define RSAVPVA_ATTR_CHANNEL_COUNT                                         RS_ATTR_CHANNEL_COUNT                                   /* ViInt32,  read-only */

        /* Driver Setup */

#define RSAVPVA_ATTR_DRIVER_SETUP                                          RS_ATTR_DRIVER_SETUP                                    /* ViString            */

        /* Instrument Identification */

#define RSAVPVA_ATTR_INSTRUMENT_MANUFACTURER                               RS_ATTR_INSTRUMENT_MANUFACTURER                         /* ViString, read-only */
#define RSAVPVA_ATTR_INSTRUMENT_MODEL                                      RS_ATTR_INSTRUMENT_MODEL                                /* ViString, read-only */
#define RSAVPVA_ATTR_INSTRUMENT_FIRMWARE_REVISION                          RS_ATTR_INSTRUMENT_FIRMWARE_REVISION                    /* ViString, read-only */

        /* Advanced Session Information */

#define RSAVPVA_ATTR_IO_RESOURCE_DESCRIPTOR                                RS_ATTR_IO_RESOURCE_DESCRIPTOR                          /* ViString, read-only */
#define RSAVPVA_ATTR_LOGICAL_NAME                                          RS_ATTR_LOGICAL_NAME

/* ViString, read-only */

    /*- Instrument-Specific Attributes -------------------------------------*/

//#define RSAVPVA_ATTR_ID_QUERY_RESPONSE                                     (RS_SPECIFIC_PUBLIC_ATTR_BASE + 1L)                     /* ViString (Read Only)              */

/* BEGIN GENERATE */
#define RSAVPVA_ATTR_VIDEO_ANALYZER_TAB                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1
#define RSAVPVA_ATTR_VIDEO_ANALYZER_INPUT                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2
#define RSAVPVA_ATTR_VIDEO_ANALYZER_EXTERNAL_FREEZE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3
#define RSAVPVA_ATTR_VIDEO_ANALYZER_AVERAGE_COUNT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 4
#define RSAVPVA_ATTR_VIDEO_ANALYZER_COLOR_ANALOG                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5
#define RSAVPVA_ATTR_VIDEO_ANALYZER_COLOR_DIGITAL                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6
#define RSAVPVA_ATTR_QUERY_VIDEO_ANALYZER_CATALOG                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7
#define RSAVPVA_ATTR_QUERY_VIDEO_ANALYZER_INPUT_RESOLUTION                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8
#define RSAVPVA_ATTR_QUERY_VIDEO_ANALYZER_INPUT_STANDARD                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 9
#define RSAVPVA_ATTR_QUERY_VIDEO_ANALYZER_AVERAGING_LEVEL                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10
#define RSAVPVA_ATTR_QUERY_VIDEO_ANALYZER_DIGITAL_COLOR                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 11
#define RSAVPVA_ATTR_VIDEO_ANALYZER_SCOPE_VIEW                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 12
#define RSAVPVA_ATTR_VIDEO_ANALYZER_CURSOR_TYPE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 13
#define RSAVPVA_ATTR_VIDEO_ANALYZER_VECTOR_GAIN                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 14
#define RSAVPVA_ATTR_VIDEO_ANALYZER_VECTOR_PHASE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 15
#define RSAVPVA_ATTR_VIDEO_ANALYZER_WAVEFORM_CURSOR_CENTER                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 16
#define RSAVPVA_ATTR_VIDEO_ANALYZER_WAVEFORM_CURSOR_SPAN                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 17
#define RSAVPVA_ATTR_QUERY_VIDEO_ANALYZER_WAVEFORM_STATE                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 18
#define RSAVPVA_ATTR_SELF_TEST_QUERY                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 19
#define RSAVPVA_ATTR_SYST_VERSION                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 20
#define RSAVPVA_ATTR_SYSTEM_RESET_ALL                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 21
#define RSAVPVA_ATTR_SYSTEM_RESET_BASE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 22
#define RSAVPVA_ATTR_SYSTEM_PRESET_ALL                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 23
#define RSAVPVA_ATTR_SYSTEM_PRESET_BASE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 24
#define RSAVPVA_ATTR_ID_QUERY_RESPONSE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 25
#define RSAVPVA_ATTR_SYSTEM_ERROR                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 26
#define RSAVPVA_ATTR_DISPLAY_UPDATE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 27

// rsavpva_rngVideoAnalyzerInput
#define RSAVPVA_VAL_VIDEO_INPUT_VGA                                         0
#define RSAVPVA_VAL_VIDEO_INPUT_CCVS                                        1
#define RSAVPVA_VAL_VIDEO_INPUT_COMP                                        2
#define RSAVPVA_VAL_VIDEO_INPUT_HDMI                                        3
#define RSAVPVA_VAL_VIDEO_INPUT_MHL                                         4

// rsavpva_rngVideoAnalyzerColorAnalog
#define RSAVPVA_VAL_VIDEO_COLOR_RGB                                         0
#define RSAVPVA_VAL_VIDEO_COLOR_YPBPR                                       1

// rsavpva_rngVideoAnalyzerColorDigital
#define RSAVPVA_VAL_VIDEO_COLOR_YCBCR                                       1

// rsavpva_rngQueryVideoAnalyzerInputStandard
#define RSAVPVA_VAL_VIDEO_STANDARD_NONE                                     0
#define RSAVPVA_VAL_VIDEO_STANDARD_BGPAL                                    1
#define RSAVPVA_VAL_VIDEO_STANDARD_DKPAL                                    2
#define RSAVPVA_VAL_VIDEO_STANDARD_IPAL                                     3
#define RSAVPVA_VAL_VIDEO_STANDARD_MNTSC                                    4

// rsavpva_rngVideoAnalyzerScopeView
#define RSAVPVA_VAL_VIDEO_SCOPE_Y_GREEN                                     0
#define RSAVPVA_VAL_VIDEO_SCOPE_BLUE                                        1
#define RSAVPVA_VAL_VIDEO_SCOPE_RED                                         2
#define RSAVPVA_VAL_VIDEO_SCOPE_PARADE                                      3
#define RSAVPVA_VAL_VIDEO_SCOPE_OVERLAY                                     4
#define RSAVPVA_VAL_VIDEO_SCOPE_SYNC                                        5

// rsavpva_rngVideoAnalyzerCursorType
#define RSAVPVA_VAL_VIDEO_SCOPE_CURSOR_OFF                                  0
#define RSAVPVA_VAL_VIDEO_SCOPE_CURSOR_LEVEL                                1
#define RSAVPVA_VAL_VIDEO_SCOPE_CURSOR_SLOPE                                2

// rsavpva_rngVideoAnalyzerTab
#define RSAVPVA_VAL_VIDEO_ANALYZER_TAB_INPUT                                0
#define RSAVPVA_VAL_VIDEO_ANALYZER_TAB_MEASURE                              1
#define RSAVPVA_VAL_VIDEO_ANALYZER_TAB_SCOPE                                2
#define RSAVPVA_VAL_VIDEO_ANALYZER_TAB_VECTOR                               3
#define RSAVPVA_VAL_VIDEO_ANALYZER_TAB_AUTO                                 4
/* END GENERATE */

/*****************************************************************************
 *- Hidden Attribute Declarations -------------------------------------------*
 *****************************************************************************/

#define RSAVPVA_ATTR_OPC_TIMEOUT               (RS_ATTR_OPC_TIMEOUT)           /* ViInt32 */
#define RSAVPVA_ATTR_IO_SESSION                (RS_ATTR_IO_SESSION)            /* ViSession */
#define RSAVPVA_ATTR_OPC_CALLBACK              (RS_ATTR_OPC_CALLBACK)          /* ViAddr */
#define RSAVPVA_ATTR_CHECK_STATUS_CALLBACK     (RS_ATTR_CHECK_STATUS_CALLBACK) /* ViAddr */

/****************************************************************************
 *---------------- Constant definition  ------------------------------------*
 ****************************************************************************/


#define	RSAVPVA_VAL_VIDEO_SCALE_X              0
#define RSAVPVA_VAL_VIDEO_SCALE_Y_AMPLITUDE    1
#define RSAVPVA_VAL_VIDEO_SCALE_Y_GROUP_DELAY  2

#define RSAVPVA_VAL_TIME  0
#define RSAVPVA_VAL_PIXEL 1

#define RSAVPVA_VAL_VIDEO_SCOPE_TRACE_GREEN    0
#define RSAVPVA_VAL_VIDEO_SCOPE_TRACE_BLUE     1
#define RSAVPVA_VAL_VIDEO_SCOPE_TRACE_RED      2
#define RSAVPVA_VAL_VIDEO_SCOPE_TRACE_CCVS     3
#define RSAVPVA_VAL_VIDEO_SCOPE_CCVS     3
#define RSAVPVA_VAL_VIDEO_SCOPE_TRACE_SYNC     4

#define RSAVPVA_VAL_VECTOR_COLOR_BLACK    0
#define RSAVPVA_VAL_VECTOR_COLOR_BLUE     1
#define RSAVPVA_VAL_VECTOR_COLOR_CYAN     2
#define RSAVPVA_VAL_VECTOR_COLOR_GREEN    3
#define RSAVPVA_VAL_VECTOR_COLOR_MAGENTA  4
#define RSAVPVA_VAL_VECTOR_COLOR_RED      5
#define RSAVPVA_VAL_VECTOR_COLOR_WHITE    6
#define RSAVPVA_VAL_VECTOR_COLOR_YELLOW   7

#define RSAVPVA_VAL_VIDEO_TRACE_X   0
#define RSAVPVA_VAL_VIDEO_TRACE_Y   1

#define	RSAVPVA_VAL_RST_BASE   0
#define	RSAVPVA_VAL_RST_ALL    1
/****************************************************************************
*---------------- Instrument Driver Function Declarations -----------------*
 ****************************************************************************/

ViStatus _VI_FUNC rsavpva_init (ViRsrc resourceName, ViBoolean IDQuery,
                                ViBoolean resetDevice, ViSession* instrumentHandle);
ViStatus _VI_FUNC rsavpva_InitWithOptions (ViRsrc resourceName, ViBoolean IDQuery,
                                           ViBoolean resetDevice,
                                           ViChar _VI_FAR optionString[],
                                           ViSession* instrumentHandle);
ViStatus _VI_FUNC rsavpva_ConfigureVideoAnalyzer (ViSession instrumentHandle,
                                                  ViInt32 board, ViInt32 tab);
ViStatus _VI_FUNC rsavpva_ConfigureVideoAnalyzerInput (ViSession instrumentHandle,
                                                       ViInt32 board,
                                                       ViString input,
                                                       ViBoolean freeze,
                                                       ViInt32 average,
                                                       ViInt32 colorAnalog);
ViStatus _VI_FUNC rsavpva_QueryVideoAnalyzerCatalog (ViSession instrumentHandle,
                                                     ViInt32 board,
                                                     ViInt32 bufferSize,
                                                     ViChar _VI_FAR catalog[]);
ViStatus _VI_FUNC rsavpva_QueryVideoAnalyzerInput (ViSession instrumentHandle,
                                                   ViInt32 board,
                                                   ViInt32 bufferSize,
                                                   ViChar _VI_FAR resolution[],
                                                   ViInt32* standard,
                                                   ViReal64* averagingLevel,
                                                   ViInt32* color);
ViStatus _VI_FUNC rsavpva_QueryVideoAnalyzerFormat (ViSession instrumentHandle,
                                                    ViInt32 board,
                                                    ViInt32* TVMaxLine,
                                                    ViReal64* TVFrameRate,
                                                    ViReal64* lineWidth,
                                                    ViReal64* pixelClock);
ViStatus _VI_FUNC rsavpva_ConfigureVideoAnalyzerSinX_XScale
             (ViSession instrumentHandle, ViInt32 board, ViInt32 scale,
              ViReal64 leftBottom, ViReal64 rightTop);
ViStatus _VI_FUNC rsavpva_ConfigureVideoAnalyzerScopeView
             (ViSession instrumentHandle, ViInt32 board, ViInt32 scopeView);
ViStatus _VI_FUNC rsavpva_ConfigureVideoAnalyzerScopeXAxis
             (ViSession instrumentHandle, ViInt32 board, ViInt32 parameter,
              ViReal64 line, ViReal64 position, ViReal64 span);
ViStatus _VI_FUNC rsavpva_ConfigureVideoAnalyzerScopeYAxis
             (ViSession instrumentHandle, ViInt32 board, ViReal64 position,
              ViReal64 span);
ViStatus _VI_FUNC rsavpva_ConfigureVideoAnalyzerScopeCursor
             (ViSession instrumentHandle, ViInt32 board, ViInt32 cursor,
              ViInt32 cursorType);
ViStatus _VI_FUNC rsavpva_ConfigureVideoAnalyzerScopeCursorXAxis
             (ViSession instrumentHandle, ViInt32 board, ViInt32 cursor,
              ViInt32 parameter, ViReal64 line, ViReal64 position, ViReal64 span);
ViStatus _VI_FUNC rsavpva_QueryVideoAnalyzerScopeTrace (ViSession instrumentHandle,
                                                        ViInt32 board,
                                                        ViInt32 scopeView,
                                                        ViInt32 bufferSize,
                                                        ViReal64 _VI_FAR data[],
                                                        ViInt32* returnedValues);
ViStatus _VI_FUNC rsavpva_QueryVideoAnalyzerScopeCursor (ViSession instrumentHandle,
                                                         ViInt32 board,
                                                         ViInt32 cursor,
                                                         ViInt32 scopeView,
                                                         ViReal64* position,
                                                         ViReal64* slope);
ViStatus _VI_FUNC rsavpva_ConfigureVideoAnalyzerVectorCursor
             (ViSession instrumentHandle, ViInt32 board, ViInt32 parameter,
              ViInt32 line, ViReal64 position);
ViStatus _VI_FUNC rsavpva_ConfigureVideoAnalyzerVectorGainPhase
             (ViSession instrumentHandle, ViInt32 board, ViReal64 gain,
              ViReal64 phase);
ViStatus _VI_FUNC rsavpva_QueryVideoAnalyzerVectorColor (ViSession instrumentHandle,
                                                         ViInt32 board,
                                                         ViInt32 color,
                                                         ViReal64* luminanceAmplitude,
                                                         ViReal64* chrominanceAmplitude,
                                                         ViReal64* chrominancePhase);
ViStatus _VI_FUNC rsavpva_QueryVideoAnalyzerVectorCursor (ViSession instrumentHandle,
                                                          ViInt32 board,
                                                          ViReal64* luminanceAmplitude,
                                                          ViReal64* chrominanceAmplitude,
                                                          ViReal64* chrominancePhase);
ViStatus _VI_FUNC rsavpva_QueryVideoAnalyzerVectorCursorPhase (ViSession instrumentHandle,
                                                             ViInt32 board,
                                                             ViReal64 *vLuminanceAmplitudeN,
                                                             ViReal64 *vChrominanceAmplitudeN,
                                                             ViReal64 *vChrominancePhaseN,
                                                             ViReal64 *vLuminanceAmplitudeP,
                                                             ViReal64 *vChrominanceAmplitudeP,
                                                             ViReal64 *vChrominancePhaseP);
ViStatus _VI_FUNC rsavpva_QueryVideoAnalyzerVectorVideoFormat
             (ViSession instrumentHandle, ViInt32 board, ViInt32* TVMaxLine,
              ViReal64* TVFrameRate, ViReal64* lineWidth, ViReal64* pixelClock);
ViStatus _VI_FUNC rsavpva_QueryVideoAnalyzerVectorTrace (ViSession instrumentHandle,
                                                         ViInt32 board,
                                                         ViInt32 trace,
                                                         ViInt32 bufferSize,
                                                         ViReal64 _VI_FAR vectorTrace[],
                                                         ViReal64 _VI_FAR lineTrace[],
                                                         ViInt32* returnedValues);
ViStatus _VI_FUNC rsavpva_ConfigureVideoAnalyzerWaveformCursorCenterSpan (ViSession instrumentHandle,
                                                                          ViInt32 board,
                                                                          ViInt32 cursor,
                                                                          ViReal64 center,
                                                                          ViReal64 span);
ViStatus _VI_FUNC rsavpva_QueryVideoAnalyzerWaveformState
             (ViSession instrumentHandle, ViInt32 board, ViInt32* state);
ViStatus _VI_FUNC rsavpva_QueryVideoAnalyzerWaveformTrace
             (ViSession instrumentHandle, ViInt32 board, ViInt32 bufferSize,
              ViReal64 _VI_FAR data[], ViInt32* returnedValues);
ViStatus _VI_FUNC rsavpva_reset (ViSession instrumentHandle);
ViStatus _VI_FUNC rsavpva_Preset (ViSession instrumentHandle, ViInt32 reference);

ViStatus _VI_FUNC rsavpva_ResetSystem (ViSession instrumentHandle,
                                       ViInt32 reference);

ViStatus _VI_FUNC rsavpva_self_test (ViSession instrumentHandle,
                                     ViInt16* selfTestResult,
                                     ViChar _VI_FAR selfTestMessage[]);
ViStatus _VI_FUNC rsavpva_revision_query (ViSession instrumentHandle,
                                          ViChar _VI_FAR instrumentDriverRevision[],
                                          ViChar _VI_FAR firmwareRevision[]);
ViStatus _VI_FUNC rsavpva_error_query (ViSession instrumentHandle,
                                       ViInt32* errorCode,
                                       ViChar _VI_FAR errorMessage[]);
ViStatus _VI_FUNC rsavpva_error_message (ViSession instrumentHandle,
                                         ViStatus errorCode,
                                         ViChar _VI_FAR errorMessage[]);
ViStatus _VI_FUNC rsavpva_SystemVersion (ViSession instrumentHandle, ViInt32 length,
                                         ViChar _VI_FAR systemVersion[]);
ViStatus _VI_FUNC rsavpva_setCheckStatus (ViSession instrumentHandle,
                                          ViBoolean state);
ViStatus _VI_FUNC rsavpva_ConfigureDisplayUpdate (ViSession instrumentHandle,
                                                  ViBoolean displayUpdate);
ViStatus _VI_FUNC rsavpva_GetError (ViSession instrumentHandle, ViStatus* code,
                                    ViInt32 bufferSize,
                                    ViChar _VI_FAR description[]);
ViStatus _VI_FUNC rsavpva_ClearError (ViSession instrumentHandle);
ViStatus _VI_FUNC rsavpva_WriteInstrData (ViSession instrumentHandle,
                                          ViChar _VI_FAR writeBuffer[]);
ViStatus _VI_FUNC rsavpva_ReadInstrData (ViSession instrumentHandle,
                                         ViInt32 numberOfBytesToRead,
                                         ViChar _VI_FAR readBuffer[],
                                         ViUInt32* numBytesRead);
ViStatus _VI_FUNC rsavpva_QueryViBoolean (ViSession instrumentHandle,
                                          ViString command, ViBoolean* value);
ViStatus _VI_FUNC rsavpva_QueryViInt32 (ViSession instrumentHandle,
                                        ViString command, ViInt32* value);
ViStatus _VI_FUNC rsavpva_QueryViReal64 (ViSession instrumentHandle,
                                         ViString command, ViReal64* value);
ViStatus _VI_FUNC rsavpva_QueryViString (ViSession instrumentHandle,
                                         ViString command, ViInt32 bufferSize,
                                         ViChar _VI_FAR value[]);
ViStatus _VI_FUNC rsavpva_SetAttributeViInt32 (ViSession instrumentHandle,
                                               ViChar _VI_FAR channelName[],
                                               ViAttr attributeID,
                                               ViInt32 attributeValue);
ViStatus _VI_FUNC rsavpva_SetAttributeViInt64 (ViSession instrumentHandle,
                                               ViChar _VI_FAR repeatedCapabilityName[],
                                               ViAttr attributeID,
                                               ViInt64 attributeValue);
ViStatus _VI_FUNC rsavpva_SetAttributeViReal64 (ViSession instrumentHandle,
                                                ViChar _VI_FAR channelName[],
                                                ViAttr attributeID,
                                                ViReal64 attributeValue);
ViStatus _VI_FUNC rsavpva_SetAttributeViString (ViSession instrumentHandle,
                                                ViChar _VI_FAR channelName[],
                                                ViAttr attributeID,
                                                ViChar _VI_FAR attributeValue[]);
ViStatus _VI_FUNC rsavpva_SetAttributeViBoolean (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViBoolean attributeValue);
ViStatus _VI_FUNC rsavpva_SetAttributeViSession (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViSession attributeValue);
ViStatus _VI_FUNC rsavpva_GetAttributeViInt32 (ViSession instrumentHandle,
                                               ViChar _VI_FAR channelName[],
                                               ViAttr attributeID,
                                               ViInt32* attributeValue);
ViStatus _VI_FUNC rsavpva_GetAttributeViInt64 (ViSession instrumentHandle,
                                               ViChar _VI_FAR repeatedCapabilityName[],
                                               ViAttr attributeID,
                                               ViInt64* attributeValue);
ViStatus _VI_FUNC rsavpva_GetAttributeViReal64 (ViSession instrumentHandle,
                                                ViChar _VI_FAR channelName[],
                                                ViAttr attributeID,
                                                ViReal64* attributeValue);
ViStatus _VI_FUNC rsavpva_GetAttributeViString (ViSession instrumentHandle,
                                                ViChar _VI_FAR channelName[],
                                                ViAttr attributeID,
                                                ViInt32 bufferSize,
                                                ViChar _VI_FAR attributeValue[]);
ViStatus _VI_FUNC rsavpva_GetAttributeViBoolean (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViBoolean* attributeValue);
ViStatus _VI_FUNC rsavpva_GetAttributeViSession (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViSession* attributeValue);
ViStatus _VI_FUNC rsavpva_CheckAttributeViInt32 (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViInt32 attributeValue);
ViStatus _VI_FUNC rsavpva_CheckAttributeViInt64 (ViSession instrumentHandle,
                                                 ViChar _VI_FAR channelName[],
                                                 ViAttr attributeID,
                                                 ViInt64 attributeValue);
ViStatus _VI_FUNC rsavpva_CheckAttributeViReal64 (ViSession instrumentHandle,
                                                  ViChar _VI_FAR channelName[],
                                                  ViAttr attributeID,
                                                  ViReal64 attributeValue);
ViStatus _VI_FUNC rsavpva_CheckAttributeViString (ViSession instrumentHandle,
                                                  ViChar _VI_FAR channelName[],
                                                  ViAttr attributeID,
                                                  ViChar _VI_FAR attributeValue[]);
ViStatus _VI_FUNC rsavpva_CheckAttributeViBoolean (ViSession instrumentHandle,
                                                   ViChar _VI_FAR channelName[],
                                                   ViAttr attributeID,
                                                   ViBoolean attributeValue);
ViStatus _VI_FUNC rsavpva_CheckAttributeViSession (ViSession instrumentHandle,
                                                   ViChar _VI_FAR channelName[],
                                                   ViAttr attributeID,
                                                   ViSession attributeValue);
ViStatus _VI_FUNC rsavpva_GetAttributeRepeatedCapabilityIds
             (ViSession instrumentHandle, ViUInt32 attributeID, ViInt32 bufferSize,
              ViChar _VI_FAR repeatedCapabilityId_s[]);
ViStatus _VI_FUNC rsavpva_GetAttributeRepeatedCapabilityIdNames
             (ViSession instrumentHandle, ViUInt32 attributeID,
              ViString repeatedCapabilityId, ViInt32 bufferSize,
              ViChar _VI_FAR repeatedCapabilityIdName_s[]);
ViStatus _VI_FUNC rsavpva_close (ViSession instrumentHandle);



/****************************************************************************
 *------------------------ Error And Completion Codes ----------------------*
 ****************************************************************************/

#define RSAVPVA_WARN_MEASURE_UNCALIBRATED    (RS_CLASS_WARN_BASE + 0x0001L)
#define RSAVPVA_WARN_OVER_RANGE              (RS_CLASS_WARN_BASE + 0x0002L)

#define RSAVPVA_ERROR_MARKER_NOT_ENABLED     (RS_CLASS_ERROR_BASE + 0x0003L)
#define RSAVPVA_ERROR_NOT_DELTA_MARKER       (RS_CLASS_ERROR_BASE + 0x0002L)
#define RSAVPVA_ERROR_MAX_TIME_EXCEEDED      (RS_CLASS_ERROR_BASE + 0x0001L)

#define RSAVPVA_WARNMSG_MEASURE_UNCALIBRATED "Uncalibrated measurement"
#define RSAVPVA_WARNMSG_OVER_RANGE           "Measurement overrange"

#define RSAVPVA_ERRMSG_MARKER_NOT_ENABLED    "Marker Not Enabled"
#define RSAVPVA_ERRMSG_NOT_DELTA_MARKER      "Not Delta Marker"
#define RSAVPVA_ERRMSG_MAX_TIME_EXCEEDED     "Max Time Exceeded"

#define RSAVPVA_ERROR_CODES_AND_MSGS \
    {RSAVPVA_WARN_MEASURE_UNCALIBRATED,  RSAVPVA_WARNMSG_MEASURE_UNCALIBRATED},\
    {RSAVPVA_WARN_OVER_RANGE,            RSAVPVA_WARNMSG_OVER_RANGE},\
    {RSAVPVA_ERROR_MARKER_NOT_ENABLED,   RSAVPVA_ERRMSG_MARKER_NOT_ENABLED},\
    {RSAVPVA_ERROR_NOT_DELTA_MARKER,     RSAVPVA_ERRMSG_NOT_DELTA_MARKER},\
    {RSAVPVA_ERROR_MAX_TIME_EXCEEDED,    RSAVPVA_ERRMSG_MAX_TIME_EXCEEDED}

/****************************************************************************
 *---------------------------- End Include File ----------------------------*
 ****************************************************************************/



#if defined(__cplusplus) || defined(__cplusplus__)
}
#endif
#endif /* __RSAVPVA_HEADER */



